from qgis.core import Qgis

from qgis.PyQt.QtWidgets import QApplication

from qsphere.cswt_templates import (AUTHENTICATION_TEMPLATE, DELETE_METADATA_TEMPLATE,
                            GET_CAPABILITIES_TEMPLATE, GET_RECORDS,
                            GET_RECORD_BYID, INSERT_METADATA_TEMPLATE,
                            UPDATE_METADATA_TEMPLATE)
from qsphere.qsphere_tools import (sendMsg, appendNameSpace, getEncodingCar)

from base64 import b64encode
from datetime import datetime
from requests import (exceptions, HTTPError, post, session)

class cswtConnection(object):
    def __init__(self, parent, name, urlserver, urlauthserver): 
        self.parent = parent
        self.name = name
        self.service = "CSW"
        self.version = "2.0.2"
        self.server = urlserver

        self.geoNetworkVersion = 2100 if urlauthserver.endswith("j_spring_security_check") else 3000
        self.headers = {'Content-Type': 'application/xml'}
        self.session = None
        
        self.authserver = urlauthserver
        self.user = None
        self.password = None
        self.connected = False
        self.reporting = ""
        self.allReporting = ""

        self.dictCswt = {0:GET_CAPABILITIES_TEMPLATE, 1:INSERT_METADATA_TEMPLATE, 2:UPDATE_METADATA_TEMPLATE, -1:DELETE_METADATA_TEMPLATE , 6:GET_RECORDS, 8:GET_RECORD_BYID, 9:AUTHENTICATION_TEMPLATE}


        self.validXPaths = [appendNameSpace(self,'ows:ExceptionReport'),
                              appendNameSpace(self,'csw:Capabilities'),
                              appendNameSpace(self,'csw:DescribeRecordResponse'),
                              appendNameSpace(self,'csw:GetDomainResponse'),
                              appendNameSpace(self,'csw:GetRecordsResponse'),
                              appendNameSpace(self,'csw:GetRecordByIdResponse'),
                              appendNameSpace(self,'csw:HarvestResponse'),
                              appendNameSpace(self,'csw:TransactionResponse')
                             ]

        try : self.duration_info = self.parent.sliderInformation.value()
        except : self.duration_info = self.parent.duration_info
        finally : self.duration_info = 2

        try : self.duration_timeout = self.parent.sliderTimeOut.value()
        except : self.duration_timeout = self.parent.duration_timeout
        finally : self.duration_timeout = 5


    def defaultIsoNamespace(self):  return 'csw'
    def name(self): return self.name
    def service(self): return self.service
    def version(self): return self.version
    def server(self): return self.server
    def authserver(self): return self.authserver

    def user(self): return self.user 
    def password(self): return self.password
    
    def headers(self): return self.headers 
    def reporting(self): return self.reporting
    def allReporting(self): return self.allReporting
    def connected(self): return self.connected

    def closeSession(self):
        if self.connected :
            responseStatusCode = 200
            if self.session != None :
                response_close = self.session.post(url=self.authserver.replace("j_spring_security_check","j_spring_security_logout"), headers={'Connection': 'close'})
                responseStatusCode = response_close.status_code
                
            if responseStatusCode == 200 : 
                self.session = None
                self.user = None
                self.password = None           
                self.connected = False
        
    def initSession(self, user, password, initReporting) :
      
        self.allReporting = ""
        self.allReporting+= "CSW init session : %s\n" % (datetime.now().replace(microsecond=0).strftime("%d/%m/%Y %Hh:%Mm:%Ss"))
        self.user = user
        self.password = password
         
        if self.geoNetworkVersion == 2100 :
           self.session = session()
           params = {'username': '%s' % self.user, 'password': '%s' % self.password}
           try : responseCswtInitsession = self.session.post(self.authserver, data=params)
           except exceptions.ConnectionError as e:
                self.allReporting+= "%s %s" % (QApplication.translate("QSphere","Unable to establish connection to"), self.authserver)
                return None
               
        else :
           self.session = None
           paramsConnexion = "%s%s:%s" % (user, self.user, self.password)
           paramsConnexion = paramsConnexion.encode('utf-8')
           params = b64encode(paramsConnexion) 
           dataxml = AUTHENTICATION_TEMPLATE % (self.user, self.password)
           self.headers = {'Content-Type': 'application/xml','Accept': 'application/xml','Authorization': 'Basic %s' % (params)}
           try : responseCswtInitsession = post(self.authserver, dataxml, self.headers)
           except exceptions.ConnectionError as e:
                self.allReporting+= "%s %s\n%s : %s" % (QApplication.translate("QSphere","Unable to establish connection to"), self.authserver,
                                                        QApplication.translate("QSphere","HTTP error"), e)
                return None
 
        self.connected = True if responseCswtInitsession.status_code == 200 else False
        self.allReporting+= "%s : %s\n" % (self.authserver, QApplication.translate("QSphere","Successfull") if responseCswtInitsession.status_code== 200 else QApplication.translate("QSphere","Error"))

        if self.connected == False : responseCswtInitsession = None
        return responseCswtInitsession 


    def notAuthentified(self):
        xml = """<?xml version="1.0" encoding="UTF-8"?>
                    <ows:ExceptionReport xmlns:ows="http://www.opengis.net/ows" xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" version="1.0.0" xsi:schemaLocation=  "http://www.opengis.net/ows http://schemas.opengis.net/ows/1.0.0/owsExceptionReport.xsd">
                      <ows:Exception exceptionCode="NoApplicableCode">
                        <ows:ExceptionText>Cannot process transaction: User not authenticated.</ows:ExceptionText>
                      </ows:Exception>
                    </ows:ExceptionReport>"""


    def getCapabilities(self) :
        paramsXml = None 
        dataXml = self.dictCswt[0] % paramsXml if paramsXml != None else self.dictCswt[0]
        headers = {'Content-Type': 'application/xml'}
        self.session = session()
        response_getCapabilities = self.session.post(self.server.replace("csw-publication","csw"), data=dataXml, headers=headers)
        self.reporting = "%s\n%s\n" % (response_getCapabilities.status_code, response_getCapabilities.text)
        self.session = None
        return response_getCapabilities

    
    def metadataCswtPublication(self, useCase, paramsXml, initReporting) :
        if self.session == None and self.geoNetworkVersion == 2100 : return self.notAuthentified()
        self.reporting = ""
        lblUseCase = {1:"ADD RECORD", 2:"UPDATE RECORD", -1:"DELETE RECORD", 6:"GET RECORDS"}
        self.reporting+= "CSW Action %s : %s\n\n" % (lblUseCase[useCase], datetime.now().replace(microsecond=0).strftime("%d/%m/%Y %Hh:%Mm:%Ss"))
        dataXml = self.dictCswt[useCase] % paramsXml if paramsXml != None else self.dictCswt[useCase]

        if self.geoNetworkVersion == 2100 :
            try : responseCswtPublication = self.session.post(self.server, data=dataXml.encode("utf-8"), headers=self.headers)
            except exceptions.ConnectionError as e:
                self.allReporting+= "%s %s" % (QApplication.translate("QSphere","Unable to establish connection to"), self.server)
                return None                
        else :
            paramsConnexion = "%s:%s" % (self.user, self.password)
            paramsConnexion = paramsConnexion.encode('utf-8')
            params = b64encode(paramsConnexion)             
            headers = {'Content-Type': 'application/xml','Accept': 'application/xml','Authorization': 'Basic %s' % (params)}
            try : responseCswtPublication = post(self.server, data=dataXml, headers=headers)
            except HTTPError as e:
                msg = "%s %s\n%s : %s\n%s : %s " % (QApplication.translate("QSphere","Unable to establish connection to"), self.server,
                                                    QApplication.translate("QSphere","HTTP error"), e.code, QApplication.translate("QSphere","message"),
                                                    e.msg)
                self.allReporting+= msg
                return None
            

        if self.geoNetworkVersion == 2100 :
           encoding = getEncodingCar("", responseCswtPublication.text)
           try : self.reporting+= "%s\n%s\n" % (responseCswtPublication.status_code, responseCswtPublication.text.decode(encoding))
           except : self.reporting+= "%s\n%s\n" % (responseCswtPublication.status_code, responseCswtPublication.text)
        else :
           encoding = getEncodingCar("", responseCswtPublication.content)  
           try : self.reporting+= "%s\n%s\n" % (responseCswtPublication.status_code, responseCswtPublication.content.decode(encoding))
           except : self.reporting+= "%s\n%s\n" % (responseCswtPublication.status_code, responseCswtPublication.content)
           
        self.allReporting+= self.reporting

        return responseCswtPublication
        

    def analyseResponse(self, useCase, response):
        analyseResponse, cswtId = False, None
        if response == None :
            self.errAnalyseResponse(response)
            return (analyseResponse, cswtId)

        from xml.etree import ElementTree as ET
        from io import StringIO

        if self.geoNetworkVersion == 2100 : content = StringIO(response.text)
        else : content = StringIO(response.content)

        try : etree = ET.parse(content)
        except : return False, None
            
        self.root = etree.getroot()
	
        if self.root == None : return (False, None)
        if self.root.tag not in self.validXPaths : return (False, None)
       
        dictResponseCswt = {1:"csw:TransactionSummary/csw:totalInserted", 2:"csw:TransactionSummary/csw:totalUpdated", -1:"csw:TransactionSummary/csw:totalDeleted"}
        xPathNs = appendNameSpace(self, dictResponseCswt[useCase])

        rootElt = self.root.findall(xPathNs) 
        try :    
            if rootElt :
               analyseResponse = (int(rootElt[0].text)==1)
               if useCase == 1 :
                  xPathNs = appendNameSpace(self, "csw:InsertResult/csw:BriefRecord") 
                  try  : cswtId = self.root.findall("%s/identifier" % (xPathNs))[0].text
                  except : cswtId = None
            title = QApplication.translate("QSphere", "Information" )
            msg = QApplication.translate("QSphere","Transaction completely released with the server")
            if useCase == 1 and cswtId != None : msg+= "\n%s : %s" % (QApplication.translate("QSphere","Add record to the server with UUID"), cswtId)
            sendMsg(self.parent, title, "%s :\n%s" % (msg, response), Qgis.Info, self.duration_info)
            self.reporting+= "%s :\n%s" % (msg, response)
            self.allReporting+= "%s :\n%s" % (msg, response)
        except : self.errAnalyseResponse(response)

        return (analyseResponse, cswtId)

    def errAnalyseResponse(self, response):
        title = QApplication.translate("QSphere", "Warning" )
        msg = QApplication.translate("QSphere","No Querying possible with the server response")
        sendMsg(self.parent, title, "%s :\n%s" % (msg, response), Qgis.Warning, self.duration_info)
        self.reporting+= "%s :\n%s" % (msg, response)
        self.allReporting+= "%s :\n%s" % (msg, response)
