# -*- coding:utf-8 -*-
from qgis.core import (Qgis, QgsCoordinateReferenceSystem)
from qgis.gui import (QgsCollapsibleGroupBox, QgsDateTimeEdit)

from qgis.PyQt.QtCore import (Qt, QDate)
from qgis.PyQt.QtWidgets import (QApplication, QCalendarWidget, QCommandLinkButton, QComboBox, QGroupBox, QLabel,
                         QLineEdit, QTableView, QTableWidget, QTextEdit)

from qsphere.qsphere_objmaker import (myButton, myCheckBox, myComboBox, myPushButton, myPushButtonFunny, MySimpleWidgetLineEdit,
                                      MySimpleWidgetLineEditST, mySpinBox, myTableWidget, myTextEdit, MyWidgetLineEdit)
from qsphere.qsphere_tools import (addLineWidget, convertToUtf8, dateToString, dicoHasKey, getComboLayer, getTextWidget, markerText, reloadTableView,
                           reloadTableWidget, reinitDate, sendMsg, appendNameSpace, getWidget)

from codecs import open as openCodecs
from configparser import ConfigParser
from datetime import datetime
from io import StringIO
from os.path import (abspath, dirname)
from urllib.parse import quote as urllibquote
from urllib.parse import (unquote, urlparse)
from xml.etree import ElementTree as ET

   
#==================================================
# FUNCTIONS TO LOAD (IMPORT) AND SAVE / EXPORT QSP
#==================================================
def saveQsp(self, fileName, isHtml):

    Config = ConfigParser()

    i=0
    for mddcollapsibleGroupBox in self.mddcollapsibleGroupBox:

        collapsGroupChildren = mddcollapsibleGroupBox.children()
        
        for j in range(len(collapsGroupChildren)):
            objName = "%s" % (collapsGroupChildren[j].objectName())
            if not type(collapsGroupChildren[j]) in (QCommandLinkButton, QLabel, myPushButton, myPushButtonFunny) and not objName in ("", "collapseButton") :
               Config.add_section(objName)
               objClassName = "%s" % (collapsGroupChildren[j].metaObject().className())
               Config.set(objName,'Type',objClassName)
               if objClassName in ("QCalendarWidget", "QgsDateTimeEdit") : Config.set(objName,'Date', "%s" % (dateToString(self, collapsGroupChildren[j])))
               elif objClassName in ("QTextEdit", "MyTextEdit", "myTextEdit") : Config.set(objName,'Text','%s' % (collapsGroupChildren[j].toPlainText()))
               elif objClassName in ("QLineEdit", "MySimpleWidgetLineEdit", "MyWidgetLineEdit") : Config.set(objName,'Text', "%s" % (collapsGroupChildren[j].text()))
               elif objClassName in ("QComboBox", "MyComboBox", "myComboBox") :
                   Config.set(objName,'ItemCount',"%s" % collapsGroupChildren[j].model().rowCount())
                   for row in range(collapsGroupChildren[j].model().rowCount()): Config.set(objName,'Item_%s' % (row), "%s" % (collapsGroupChildren[j].itemText(row)))
                   Config.set(objName,'Index',"%s" % collapsGroupChildren[j].currentIndex())
               elif objClassName=="QGroupBox" :
                    groupBoxChildren = collapsGroupChildren[j].children()
                    for i in range(len(groupBoxChildren)):
                        child = groupBoxChildren[i]
                        childClassName = "%s" % (child.metaObject().className())
                        if childClassName == "QRadioButton" and child.isChecked(): Config.set(objName, "radio","%s" % (i))
                        elif childClassName in ("QComboBox", "MyComboBox", "myComboBox") : Config.set(objName,'Index', "%s" % child.currentIndex())
                        elif childClassName in ("QTableWidget", "MyTableWidget", "myTableWidget") :
                             Config.set(objName,'subtype', childClassName)
                             Config.set(objName,'zRows', "%s" % child.rowCount())
                             Config.set(objName,'zCols', "%s" % child.columnCount())
                             for row in range(child.rowCount()):
                                  widget = child.cellWidget(row, 0)
                                  Config.set(objName,'zRow_%s' % (row), "%s" % (widget.value()))
                        elif childClassName == "QTableView":
                             Config.set(objName,'subtype', childClassName)
                             items = ""
                             Config.set(objName,'ItemCount',"%s" % child.model().rowCount())
                             for row in range(child.model().rowCount()):
                                 values = ""
                                 for col in range(child.model().columnCount()): values+= "%s" % (child.model().item(row, col).text()) if values == "" else "|%s" % (child.model().item(row, col).text())
                                 Config.set(objName,'Item_%s' % (row), "%s" % values )
                                 item = child.model().item(row, 0)
                                 if item.checkState()== Qt.Checked: items+= "%s" % (row) if items=="" else "|%s" % (row)  
                             Config.set(objName,'Indexes',items)                                      
                   
               elif objClassName=="QTableView" :
                   items = ""
                   Config.set(objName,'ItemCount',"%s" % collapsGroupChildren[j].model().rowCount())
                   for row in range(collapsGroupChildren[j].model().rowCount()):
                       values = ""
                       for col in range(collapsGroupChildren[j].model().columnCount()): values+= "%s" % (collapsGroupChildren[j].model().item(row, col).text()) if values == "" else "|%s" % (collapsGroupChildren[j].model().item(row, col).text())
                       Config.set(objName,'Item_%s' % (row), "%s" % values)
                       item = collapsGroupChildren[j].model().item(row, 0)
                       if item.checkState()== Qt.Checked: items+= "%s" % (row) if items=="" else "|%s" % (row)  
                   Config.set(objName,'Indexes', "%s" % items)

               
               elif objClassName in ("QTableWidget", "MyTableWidget", "myTableWidget") :
                  Config.set(objName,'zRows',"%s" % collapsGroupChildren[j].rowCount())
                  Config.set(objName,'zCols',"%s" % collapsGroupChildren[j].columnCount())
                  for col in range(collapsGroupChildren[j].columnCount()):
                      obj = collapsGroupChildren[j].cellWidget(0, col)
                      if obj != None :
                          widgetClassName = collapsGroupChildren[j].cellWidget(0, col).metaObject().className()
                          Config.set(objName,'zWidget_%s' % (col), "%s|%s" % (widgetClassName, collapsGroupChildren[j].cellWidget(0, col).accessibleDescription()))
                      else: Config.set(objName,'zWidget_%s' % (col), "standard")   

                  for row in range(collapsGroupChildren[j].rowCount()):
                      line = ""
                      for col in range(collapsGroupChildren[j].columnCount()):
                          widget = collapsGroupChildren[j].cellWidget(row, col)
                          if widget != None :
                              if widget.metaObject().className() in ("QComboBox", "MyComboBox", "myComboBox") :
                                  if widget.isEditable() : line+= "%s" % widget.currentText() if line == "" else "|%s" % widget.currentText()  
                                  else : line+= "%s" % (widget.currentIndex()) if line == "" else "|%s" % (widget.currentIndex())
                              elif widget.metaObject().className() in ("QCheckBox", "MyCheckBox", "myCheckBox") : line+= "%s" % (widget.checkState()) if line == "" else "|%s" % (widget.checkState())
                              elif widget.metaObject().className()=="mySpinBox" : line+= "%s" % (widget.value()) if line == "" else "|%s" % (widget.value())
                              elif widget.metaObject().className() in ("QPushButton", "myButton") : line+= "action" if line == "" else "|action"
                              else : line+= "%s" % (widget.text()) if col==0 else "|%s" % (widget.text())
                          else:
                              cell = collapsGroupChildren[j].item(row, col)
                              if cell != None : line+= "%s" % (collapsGroupChildren[j].item(row, col).text()) if line == "" else  "|%s" % (collapsGroupChildren[j].item(row, col).text())
                              else : line+= "" if (line == "" and col == 0) else  "|"  

                      Config.set(objName,'zRow_%s' % (row), "%s" % line)
        i+= 1 
    
    if fileName != "" :
       with open(fileName, 'w', encoding="cp1252", errors="surrogateescape") as inFile : Config.write(inFile)
    else :
       if isHtml : 
           html = ""
           for section in Config.sections():    
              html+=  markerText(self,"[%s]\n" % section)
              for option in Config.options(section):
                  try : html+=  markerText(self, "%s = %s\n" % (option, Config.get(section, option).strip()))
                  except :
                      try : html+=  markerText(self, "%s = %s\n" % (option, Config.get(section, option)))
                      except : html+=  markerText(self, "%s =\n" % (option))
              html+= "<br>"  
       else :
           html = ""
           for section in Config.sections():    
              html+=  "[%s]\n" % section
              for option in Config.options(section):
                  try : html+= "%s = %s\n" % (option, Config.get(section, option).strip())
                  except :
                      try : html+=  "%s = %s\n" % (option, Config.get(section, option))
                      except : html+=  "%s =\n" % (option)
              html+= "\n"
              
       return html#.decoding("cp1252") #.ecoding("utf8")
    
def loadDataFromQsp(self, fileName, dataQsp):
    title = "Information"

    config = ConfigParser()
    
    if fileName :
       fileName = abspath(fileName)
       config.read_file(openCodecs(fileName, 'r','cp1252')) 
    else :
       from io import StringIO
       buffer = StringIO(dataQsp) 
       config.read_file(buffer) 
    
    sections = config.sections()
    bounds = len(sections)
    
    self.tabWidget.setVisible(False)
    self.progressBar.setVisible(True)

    msg1 = QApplication.translate("QSphere","Object ")

    dicoWidgetType = {"QTableWidget": myTableWidget, "MyTableWidget": myTableWidget, "myTableWidget": myTableWidget,
                "QCheckBox"   : myCheckBox,    "MyCheckBox"   : myCheckBox, "myCheckBox" : myCheckBox,
                "QTextEdit"   : myTextEdit,    "MyTextEdit"   : myTextEdit, "myTextEdit" : myTextEdit,
                "QLineEdit"   : QLineEdit,     "MySimpleWidgetLineEdit" : QLineEdit, "MyWidgetLineEdit" : QLineEdit,
                "QComboBox"    : QComboBox,    "MyComboBox" : myComboBox, "myComboBox" : myComboBox,
                "QTableView"  : QTableView,    "QCalendarWidget" : QgsDateTimeEdit, "QgsDateTimeEdit" : QgsDateTimeEdit, 
                "QGroupBox"   : QGroupBox, 
                }
    
    for i in range(bounds):
        section = sections[i]
        sectionType = config.get(section,'Type')
        self.progressBar.setValue(int(100 * i/bounds))

        if dicoHasKey(dicoWidgetType, sectionType) :    
            widgetClass = dicoWidgetType[sectionType]
            obj = self.findChild(widgetClass, section)

            if obj !=None :
                    if widgetClass == QgsDateTimeEdit :                         
                        infos = config.get(section,'Date')
                        infos = infos.rstrip()
                        if infos!="" and infos.find("-")!=-1:
                           eltsDate = infos.split("-")
                           try : obj.setDate(QDate(int(eltsDate[0]), int(eltsDate[1]), int(eltsDate[2])))
                           except :
                               try : obj.setSelectedDate(QDate(int(eltsDate[0]), int(eltsDate[1]), int(eltsDate[2])))
                               except : reinitDate(obj)
                        else: obj.showToday()

                    elif widgetClass in (QTextEdit, myTextEdit) :
                        infos = config.get(section,'Text')
                        infos = infos.rstrip()
                        if section != "coherence" : obj.setPlainText(infos)
                        else : self.updateConsistency(section, infos)

                    elif widgetClass in (QLineEdit, MySimpleWidgetLineEdit, MyWidgetLineEdit) :    
                        infos = config.get(section,'Text')
                        infos = infos.rstrip()
                        obj.setText(infos)

                    elif widgetClass in (QComboBox, myComboBox) :    
                        infos = config.get(section,'Index')
                        objIndex = int(infos.rstrip())
                        objInfos = config.get(section,'ItemCount')
                        objItemsCount = int(objInfos.rstrip())
                        obj.clear()
                        for k in range(objItemsCount):
                            item = config.get(section,'Item_%s' % (k))
                            item = item.replace("jeux", "séries") 
                            item = item.replace("Jeu", "Série")
                            obj.addItem(item)
                        obj.setCurrentIndex(objIndex)

                    elif widgetClass == QTableView : reloadTableView(self, config, obj, section, int(100 * i/bounds), int(100 * 1/bounds))
                         
                    elif widgetClass == QGroupBox :    
                        groupBoxChildren = obj.children()
                        infos = "%s" % (config.get(section, "radio"))
                        objIndex = int(infos.rstrip())
                        try : groupBoxChildren[objIndex].setChecked(True)
                        except : pass

                        for child in groupBoxChildren :
                            if child.metaObject().className()== "QTableView" : reloadTableView(self, config, child, section, int(100 * i/bounds), int(1/bounds))
                            elif child.metaObject().className() in ("QTableWidget", "MyTableWidget", "myTableWidget") :
                                child.clearContents()
                                for j in range(child.rowCount()): child.removeRow(0)
                                try : rows = int(config.get(section,'zrows'))
                                except :
                                    try : rows = int(config.get(section,'rows'))
                                    except : rows = 0
                                for j in range(rows):
                                     child.insertRow(j)
                                     infos = float(config.get(section,'zRow_%s' % (j)))
                                     addLineWidget(self, child, j,  0, 4, 0, infos)
                            elif child.metaObject().className() in ("QComboBox", "MyComboBox", "myComboBox") :
                                indexComboBox = int(config.get(section, "Index"))
                                child.setCurrentIndex(indexComboBox)

                    elif widgetClass in (QTableWidget, myTableWidget) :                        
                                reloadTableWidget(self, config, obj, section)
                                if obj.objectName() == "tablemotsclefsf":
                                   for i in range(obj.rowCount()):
                                       checkBox = obj.cellWidget(i, 1)
                                       for j in range(2,5):
                                           editLine = obj.cellWidget(i, j)
                                           try : editLine.setEnabled(checkBox.isChecked())
                                           except : pass

            else :
                msg2 = QApplication.translate("QSphere","'s properties not similar. Import data aborded.")
                sendMsg(self, title , "%s%s%s" % (msg1, section, msg2), Qgis.Warning, self.duration_warning)
        else :
            if dicoHasKey(dicoWidgetType, sectionType) :    
                msg2 = QApplication.translate("QSphere"," not find in the QSphere's collection.")
                sendMsg(self, title , "%s%s%s" % (msg1, section, msg2), Qgis.Warning, self.duration_warning)


def saveQmd(self, fileName):
    xml = ""
    xml+= '<!DOCTYPE qgis PUBLIC "http://mrcc.com/qgis.dtd" "UTF-8">\n' \
          '<qgis version="%s">\n' % Qgis.QGIS_VERSION

    layerSource = ""
    try :
        obj = self.ComboLayers
        if obj.currentIndex() > 0 : 
            layer = getComboLayer(self, obj, True)
            layerSource = layer.source()
    except : pass
    xml+= '\t<identifier>%s</identifier>\n' % layerSource

    obj = getWidget(self, "identificator")
    xml+= '\t<parentidentifier>%s</parentidentifier>\n' % obj.text()

    obj = getWidget(self, "langmetada")
    langageCode = obj.currentText()
    
    xml+= '\t<language>%s</language>\n' % langageCode.upper()

    obj = getWidget(self, "typedata")
    value = self.typeData[obj.currentIndex()]
    xml+= '\t<type>%s</type>\n' % value

    value, text = getTextWidget(self, "intitule", False)    
    xml+= '\t<title>%s</title>\n' % (convertToUtf8(value))

    value, text = getTextWidget(self, "resume", False)
    xml+= '\t<abstract>%s</abstract>\n' % (convertToUtf8(value))

    
    obj = getWidget(self, "tablecategories")
    rows = obj.model().rowCount() 
    xml+= '\t<keywords vocabulary="gmd:topicCategory">\n'
    for i in range(rows):
        item = obj.model().item(i, 0)
        if item.checkState()== Qt.Checked:
            xml+= '\t\t<keyword>%s</keyword>\n' % (makeCat(self, obj.model().item(i, 2).text()))
    xml+= '\t</keywords>\n'

    obj = getWidget(self, "tablemotsclefsf")
    listThemes = {}

    for i in range(obj.rowCount()):
        elementThesaurus = obj.cellWidget(i, 2).text()
        if elementThesaurus == "" : elementThesaurus = "None"
        if not dicoHasKey(listThemes, elementThesaurus): listThemes[elementThesaurus] = []
        listThemes[elementThesaurus].append(convertToUtf8(obj.cellWidget(i, 0).text()))
    
    for key in listThemes :
        xml+= '\t<keywords vocabulary="%s">\n' % key
        values = listThemes[key]
        for value in values :
            xml+= '\t\t<keyword>%s</keyword>\n' % value
        xml+= '\t</keywords>\n'
  
    xml+= '\t<fees></fees>\n'


    obj = getWidget(self, "groupedroits")
    if obj.children()[0].isChecked() :
        xml+= '\t<constraints type="use">%s</constraints>\n' % QApplication.translate("QSphere","No restriction for public access with INSPIRE")
    else :
        child = obj.children()[2]
        rows = child.model().rowCount()
        for i in range(rows):
            item = child.model().item(i, 0)
            if item.checkState()== Qt.Checked:
                value = child.model().item(i, 1).text()
                xml+= '\t<constraints type="use">%s</constraints>\n' % convertToUtf8(value)

    obj = getWidget(self, "licence")
    xml+= '\t<rights></rights>\n'
    xml+= '\t<license>%s</license>\n' % (convertToUtf8(obj.toPlainText()))


    obj = getWidget(self, "tablecarac")
    value = obj.currentText()    
    xml+= '\t<encoding>%s</encoding>\n' % value

    xml+= '\t<crs>\n'

    srs = QgsCoordinateReferenceSystem(3857)
    if srs :
        xml+= '\t\t<spatialrefsys>\n'
        xml+= '\t\t\t<proj4>%s</proj4>\n' % srs.toProj4()
        xml+= '\t\t\t<srsid>%s</srsid>\n' % srs.srsid()
        xml+= '\t\t\t<srid>%s</srid>\n' % srs.postgisSrid()
        xml+= '\t\t\t<authid>%s</authid>\n' % srs.authid()
        xml+= '\t\t\t<description>%s</description>\n' % srs.description()
        xml+= '\t\t\t<projectionacronym>%s</projectionacronym>\n' % srs.projectionAcronym()
        xml+= '\t\t\t<ellipsoidacronym>%s</ellipsoidacronym>\n' % srs.ellipsoidAcronym()
        xml+= '\t\t\t<geographicflag>%s</geographicflag>\n' % ("%s".lower() % srs.isGeographic())
        xml+= '\t\t</spatialrefsys>\n'

    
    obj = getWidget(self, "tablescr")
    for i in range(obj.rowCount()): 
            try :
                epsg = obj.cellWidget(i, 0).text().replace("EPSG:","")
                if epsg != "" :
                   id = long(epsg)
                   srs = QgsCoordinateReferenceSystem(id)

                   if srs :
                        xml+= '\t\t<spatialrefsys>\n'
                        xml+= '\t\t\t<proj4>%s</proj4>\n' % srs.toProj4()
                        xml+= '\t\t\t<srsid>%s</srsid>\n' % srs.srsid()
                        xml+= '\t\t\t<srid>%s</srid>\n' % srs.postgisSrid()
                        xml+= '\t\t\t<authid>%s</authid>\n' % srs.authid()
                        xml+= '\t\t\t<description>%s</description>\n' % srs.description()
                        xml+= '\t\t\t<projectionacronym>%s</projectionacronym>\n' % srs.projectionAcronym()
                        xml+= '\t\t\t<ellipsoidacronym>%s</ellipsoidacronym>\n' % srs.ellipsoidAcronym()
                        xml+= '\t\t\t<geographicflag>%s</geographicflag>\n' % ("%s".lower() % srs.isGeographic())
                        xml+= '\t\t</spatialrefsys>\n'
            except : pass    

    xml+= '\t</crs>\n'

    obj = getWidget(self, "tableemprises")
    xml+= '\t<extent>\n'
    xml+= '\t\t<spatial dimensions="2" miny="%s" maxz="" minz="" minx="%s" maxy="%s" maxx="%s" crs="ESPG:3857"/>\n' % (obj.cellWidget(0, 1).value(),
                                                                                                                       obj.cellWidget(0, 3).value(),
                                                                                                                       obj.cellWidget(0, 0).value(),
                                                                                                                       obj.cellWidget(0, 2).value()
                                                                                                                       )

    xml+= '\t\t<temporal>\n'
    obj = getWidget(self, "tableetenduetemporelle")
    for i in range(obj.rowCount()):
        dateWidgets = "%s" % (obj.cellWidget(i, 0).text())
        if dateWidgets.find("-- --") == -1 :
            dates = dateWidgets.split(" ")
            xml+= '\t\t\t<period>\n'
            xml+= '\t\t\t\t<start>%s</start>\n' % dates[0]
            xml+= '\t\t\t\t<end>%s</end>\n' % dates[1]
            xml+= '\t\t\t</period>\n'
    xml+= '\t\t</temporal>\n'
    xml+= '\t</extent>\n'

    obj = getWidget(self, "tableroles")
    for i in range(obj.rowCount()):
        role = self.dictListOfRules[obj.cellWidget(i, 0).currentIndex()]
        xml+= '\t<contact>\n'
        xml+= '\t\t<contactAddress>\n'
        xml+= '\t\t\t<type>postal</type>\n'
        xml+= '\t\t\t<address>%s</address>\n' % convertToUtf8(obj.cellWidget(i, 2).text()) if obj.cellWidget(i, 2).text().strip()!="" else ""
        xml+= '\t\t\t<city>%s</city>\n' % convertToUtf8(obj.cellWidget(i, 5).text()) if obj.cellWidget(i, 5).text().strip()!="" else ""
        xml+= '\t\t\t<administrativearea></administrativearea>\n'
        xml+= '\t\t\t<postalcode>%s</postalcode>\n' % convertToUtf8(obj.cellWidget(i, 4).text()) if obj.cellWidget(i, 4).text().strip()!="" else ""
        xml+= '\t\t\<country>%s</country>\n' % convertToUtf8(obj.cellWidget(i, 3).currentText().rstrip().lstrip()) if obj.cellWidget(i, 3).currentText().strip()!="" else ""
        xml+= '\t\t</contactAddress>\n'
        xml+= '\t\t<name>%s</name>\n' % convertToUtf8(obj.cellWidget(i, 1).text())
        xml+= '\t\t<organization>%s</organization>\n' % convertToUtf8(obj.cellWidget(i, 1).text())
        xml+= '\t\t<position></position>\n'
        xml+= '\t\t<voice>%s</voice>\n' % obj.cellWidget(i, 7).text() if obj.cellWidget(i, 7).text().strip()!="" else ""
        xml+= '\t\t<fax>%s</fax>\n' % obj.cellWidget(i, 8).text() if obj.cellWidget(i, 8).text().strip()!="" else ""
        xml+= '\t\t<email>%s</email>\n' % obj.cellWidget(i, 6).text() if obj.cellWidget(i, 6).text().strip()!="" else ""
        xml+= '\t\t<role>%s</role>\n' % role
        xml+= '\t</contact>\n'


    obj = getWidget(self, "tablelocalisator")
    xml+= '\t<links>\n'
    for i in range(obj.rowCount()):
      
      xml+= '\t\t<link name="%s" format="" mimeType="" size="" url="%s" description="" type="http"/>\n' % (convertToUtf8(obj.cellWidget(i, 1).text()), encodeUrlQueryString(obj.cellWidget(i, 0).text()))
    xml+= '\t</links>\n'

    xml+= '</qgis>\n'
    
    if fileName != None :
        with open(fileName, 'w', encoding="utf-8", errors="surrogateescape") as inFile : inFile.write(xml)
        return None
    else : return  xml.encode("utf-8")


    
#==============================
# FONCTION TO WRITE XML EXPORT       
#==============================
def exportDictionnaryToXml(self, fileName):

    xml = ''
    xml+= startXml(self)
    xml+= '<gmd:qspheredictionnary>\n'
    xml+= '<gco:CharacterString>dictionnary</gco:CharacterString>\n'
    xml+='</gmd:qspheredictionnary>\n'
    xml+= '<gmd:identificationInfo>\n'
    xml+= '<gmd:MD_DataIdentification>\n'
    obj = getWidget(self, "tablecontacts")
    for i in range(obj.rowCount()):
        value = self.dictListOfRules[obj.cellWidget(i, 0).currentIndex()]
        xml+= rulesInformations(self, obj, value, i, False, 0) 
    xml+= '</gmd:MD_DataIdentification>\n' 
    xml+= '</gmd:identificationInfo>\n'
    xml+= endXml(self)

    with open(fileName, 'w', encoding="utf-8", errors="surrogateescape") as inFile : inFile.write(xml)


def saveXml(self, fileName):
    xml = ''
    xml+= startXml(self)
    xml+= fileIdentifierInformation(self)
    xml+= fileLanguageInformation(self)
    xml+= characterSetCodeInformations(self, 1)
    xml+= hierarchyLevelInformations(self)
    xml+= contactInformations(self)
    xml+= dateStamp(self, 1, True, None)
    xml+= mddStandardNameInformation(self,)
    xml+= mddStandardVersionInformation(self)
    xml+= geographicalInformations(self)    
    xml+= spatialReferenceSystemInformations(self)
    xml+= identificationInformations(self)
    xml+= distributionInformations(self)
    xml+= qualityInformations(self)
    xml+= endXml(self)

    if fileName != None :
        with open(fileName, 'w', encoding="utf-8", errors="surrogateescape") as inFile : inFile.write(xml)
        return None
    else : return  xml.encode("utf-8")   
    

def saveXmlCatalog(self, fileName):
    xml = ''
    xml+= startXmlCatalog(self)
    xml+= nameCatalog(self)
    xml+= scopeCatalog(self)
    xml+= versionCatalog(self)
    xml+= dateCatalog(self)
    xml+= productorCatalog(self)
    xml+= fieldsMapCatalog(self)
    xml+= endXmlCatalog(self)

    with open(fileName, 'w', encoding="utf-8", errors="surrogateescape") as inFile : inFile.write(xml)
    
#------------------------------------
# FONCTION TO WRITE BLOCX XML EXPORT       
#------------------------------------
def startXmlCatalog(self):
    xml = ""
    obj = getWidget(self, "namelayer")
    xml+= '<?xml version="1.0" encoding="UTF-8"?>' \
                     '%s' \
                     '<gfc:FC_FeatureCatalogue uuid="CA_%s" xmlns:gfc="http://www.isotc211.org/2005/gfc" xmlns:gmd="http://www.isotc211.org/2005/gmd"' \
                     ' xmlns:gco="http://www.isotc211.org/2005/gco"' \
                     ' xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"' \
                     ' xsi:schemaLocation="http://www.isotc211.org/2005/gfc http://www.isotc211.org/2005/gfc/gfc.xsd">\n' % (xml, obj.text())
    return xml

def endXmlCatalog(self):
     xml = ''
     xml+= '</gfc:FC_FeatureCatalogue>'
     return xml

def nameCatalog(self):
     xml = ''
     value, text = getTextWidget(self, "namelayer", False)
     xml+= '\t<gfc:name>\n'
     xml+= '\t\t<gco:CharacterString>%s - %s</gco:CharacterString>\n' % (value,"Catalogue d'attributs")
     xml+= '\t</gfc:name>\n'
     return xml

def scopeCatalog(self):
    xml = ''
    obj = getWidget(self, "namelayer")
    xml+= '\t<gfc:scope>\n'
    xml+= '\t\t<gco:CharacterString>%s</gco:CharacterString>\n' % (obj.text())
    xml+= '\t</gfc:scope>\n'
    return xml

def versionCatalog(self):
    xml = ''
    xml+= '\t<gfc:versionNumber>\n'
    xml+= '\t\t<gco:CharacterString>NC</gco:CharacterString>\n'
    xml+= '\t</gfc:versionNumber>\n'
    return xml

def dateCatalog(self):
    xml = ''
    obj = getWidget(self, "datecredata")
    xml+= '\t<gfc:versionDate>\n'
    xml+= '\t\t<gco:Date>%s</gco:Date>\n' % (dateToString(self, obj))
    xml+= '\t</gfc:versionDate> \n'
    return xml

def productorCatalog(self):
    xml = ''
    obj = getWidget(self, "tableroles")
    for i in range(obj.rowCount()):
        value = self.dictListOfRules[obj.cellWidget(i, 0).currentIndex()]
        if value == "pointOfContact" : 
            xml+= '\t<gfc:producer>\n'
            xml+= responsiblePartiesInformations(self, obj, value, i, True, 2) 
            xml+= '\t</gfc:producer>\n'
            return xml
    
def fieldsMapCatalog(self):
    xml = ''
    xml+= '\t<gfc:featureType>\n'
    xml+= '\t\t<gfc:FC_FeatureType>\n'
    xml+= '\t\t\t<gfc:typeName><gco:LocalName>%s</gco:LocalName></gfc:typeName>\n' % (self.ComboLayers.currentText())
    xml+= '\t\t\t<gfc:definition><gco:CharacterString>%s</gco:CharacterString></gfc:definition>\n' % ("")

    obj = getWidget(self, "tabledico")
    for i in range(obj.rowCount()):
        xml+= '\t\t\t<gfc:carrierOfCharacteristics>\n'
        xml+= '\t\t\t\t<gfc:FC_FeatureAttribute>\n'
        xml+= '\t\t\t\t\t<gfc:memberName>\n'
        xml+= '\t\t\t\t\t\t<gco:LocalName>%s</gco:LocalName>\n' % (obj.item( i, 1 ).text())
        xml+= '\t\t\t\t\t</gfc:memberName>\n'
        xml+= '\t\t\t\t\t<gfc:definition><gco:CharacterString>%s</gco:CharacterString></gfc:definition>\n' % (obj.item( i, 5 ).text())
        xml+= '\t\t\t\t\t<gfc:cardinality>\n'
        xml+= '\t\t\t\t\t\t<gco:Multiplicity>\n'
        xml+= '\t\t\t\t\t\t\t<gco:range>\n'
        xml+= '\t\t\t\t\t\t\t\t<gco:MultiplicityRange>\n'
        xml+= '\t\t\t\t\t\t\t\t\t<gco:lower>\n'
        xml+= '\t\t\t\t\t\t\t\t\t\t<gco:Integer>1</gco:Integer>\n'
        xml+= '\t\t\t\t\t\t\t\t\t</gco:lower>\n'
        xml+= '\t\t\t\t\t\t\t\t\t<gco:upper>\n'
        xml+= '\t\t\t\t\t\t\t\t\t\t<gco:UnlimitedInteger isInfinite="false">1</gco:UnlimitedInteger>\n'
        xml+= '\t\t\t\t\t\t\t\t\t</gco:upper>\n'
        xml+= '\t\t\t\t\t\t\t\t</gco:MultiplicityRange>\n'
        xml+= '\t\t\t\t\t\t\t</gco:range>\n'
        xml+= '\t\t\t\t\t\t</gco:Multiplicity>\n'
        xml+= '\t\t\t\t\t</gfc:cardinality>\n'
        xml+= '\t\t\t\t\t<gfc:valueType>\n'
        xml+= '\t\t\t\t\t\t<gco:TypeName>\n'
        xml+= '\t\t\t\t\t\t\t<gco:aName>\n'
        xml+= '\t\t\t\t\t\t\t\t<gco:CharacterString>%s</gco:CharacterString>\n' % (obj.item( i, 2 ).text())
        xml+= '\t\t\t\t\t\t\t</gco:aName>\n'
        xml+= '\t\t\t\t\t\t</gco:TypeName>\n'
        xml+= '\t\t\t\t\t</gfc:valueType>\n'
        xml+= '\t\t\t\t</gfc:FC_FeatureAttribute>\n'
        xml+= '\t\t\t</gfc:carrierOfCharacteristics>\n'

    xml+= '\t\t</gfc:FC_FeatureType>\n'
    xml+= '\t</gfc:featureType>\n'

    return xml

 
def startXml(self):
    xml = ""
    xml = '<?xml version="1.0" encoding="UTF-8"?>\n' \
           '%s' \
           '<gmd:MD_Metadata xsi:schemaLocation="http://www.isotc211.org/2005/gmd http://schemas.opengis.net/iso/19139/20060504/gmd/gmd.xsd"' \
           ' xmlns:gmd="http://www.isotc211.org/2005/gmd"' \
           ' xmlns:gco="http://www.isotc211.org/2005/gco"' \
           ' xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"' \
           ' xmlns:gml="http://www.opengis.net/gml"' \
           ' xmlns:xlink="http://www.w3.org/1999/xlink">\n' % (xml)
    return xml

def fileIdentifierInformation(self):
    obj = getWidget(self, "identificator")
    xml = ""
    xml+= '\t<gmd:fileIdentifier>\n'
    xml+= '\t\t<gco:CharacterString>%s</gco:CharacterString>\n' % (obj.text()) 
    xml+= '\t</gmd:fileIdentifier>\n'
    return xml

def fileLanguageInformation(self):
    obj = getWidget(self, "langmetada")
    value = obj.currentText()
    xml = ""
    xml+= langsInformations(self, value, 1)
    return xml

def characterSetCodeInformations(self, nbTab):
    tab=""
    for k in range(nbTab): tab+="\t"
    if nbTab == 1 : value = "utf8"
    else :
        obj = getWidget(self, "tablecarac")
        value = obj.currentText()
    #value = "MD_CharacterSetCode_%s" % value
    xml = ''    
    xml+= '%s<gmd:characterSet>\n' % (tab)
    xml+= '%s\t<gmd:MD_CharacterSetCode codeList="http://www.isotc211.org/2005/resources/codeList.xml#MD_CharacterSetCode" codeListValue="MD_CharacterSetCode_%s">%s</gmd:MD_CharacterSetCode>\n' % (tab, value, value) 
    xml+= '%s</gmd:characterSet>\n' % (tab)
    return xml

def hierarchyLevelInformations(self):
    obj = getWidget(self, "typedata")
    value = self.typeData[obj.currentIndex()]
    xml = ''
    xml+= '\t<gmd:hierarchyLevel>\n'
    xml+= '\t\t<gmd:MD_ScopeCode codeList="http://standards.iso.org/ittf/PubliclyAvailableStandards/ISO_19139_Schemas/resources/codelist/gmxCodelists.xml#MD_ScopeCode" codeListValue="%s">%s</gmd:MD_ScopeCode>\n' % (value, value)
    xml+= '\t</gmd:hierarchyLevel>\n'
    return xml

def contactInformations(self):
    obj = getWidget(self, "tableroles")
    xml = ''
    for i in range(obj.rowCount()):
        value = self.dictListOfRules[obj.cellWidget(i, 0).currentIndex()]
        if value == "pointOfContact" :
           xml+= rulesInformations(self, obj, value, i, True, 1)
           return xml

def dateStamp(self, nbTab, isValue, obj):
    tab=""
    for k in range(nbTab): tab+="\t"
    xml = ''
    xml+= '%s<gmd:dateStamp>\n' % (tab)
    if isValue :
        value, text = getTextWidget(self, "datemetada", False)
        xml+= '%s\t<gco:Date>%s</gco:Date>\n' % (tab, value)
    else :
        if obj != None : xml+= '%s\t<gco:Date>%s</gco:Date>\n' % (tab, dateToString(self, obj))
        else : xml+= '%s\t<gco:Date>%s</gco:Date>\n' % (tab, datetime.now().strftime("%d/%m/%Y %Hh%Mm%Ss"))
        
    xml+= '%s</gmd:dateStamp>\n' % (tab)
    return xml

def mddStandardNameInformation(self):
    xml = ''
    xml+= '\t<gmd:metadataStandardName>\n'
    xml+= '\t\t<gco:CharacterString>ISO19115</gco:CharacterString>\n' #ISO 19115
    xml+= '\t</gmd:metadataStandardName>\n'
    return xml

def mddStandardVersionInformation(self):
    xml = ''
    xml+= '\t<gmd:metadataStandardVersion>\n'
    xml+= '\t\t<gco:CharacterString>2003/Cor.1:2006</gco:CharacterString>\n' #2003 Cor.1:2006
    xml+= '\t</gmd:metadataStandardVersion>\n'
    return xml

def spatialReferenceSystemInformations(self):
    obj = getWidget(self, "tablescr")
    xml = ''
    for i in range(obj.rowCount()): 
        epsg = obj.cellWidget(i, 0).text()
        xml+= '\t<gmd:referenceSystemInfo>\n'
        xml+= '\t\t<gmd:MD_ReferenceSystem>\n'
        xml+= '\t\t\t<gmd:referenceSystemIdentifier>\n'        
        xml+= '\t\t\t\t<gmd:RS_Identifier>\n'

        infos = []
        if epsg.find(":") != -1 :
            tempo = epsg.split(":")
            if len(tempo) == 2 : infos = tempo

        if infos != [] :
            codeSpaceEpsg = infos[0]
            codeEpsg = infos[1] #int(epsg.replace("EPSG:",""))
        else :
            codeSpaceEpsg = "EPSG" 
            codeEpsg = epsg
        
        xml+= '\t\t\t\t\t<gmd:code>\n'
        xml+= '\t\t\t\t\t\t<gco:CharacterString>%s</gco:CharacterString>\n' % codeEpsg
        xml+= '\t\t\t\t\t</gmd:code>\n'
        xml+= '\t\t\t\t\t<gmd:codeSpace>\n'
        xml+= '\t\t\t\t\t\t<gco:CharacterString>%s</gco:CharacterString>\n' % codeSpaceEpsg
        xml+= '\t\t\t\t\t</gmd:codeSpace>\n'
        xml+= '\t\t\t\t</gmd:RS_Identifier>\n'
        xml+= '\t\t\t</gmd:referenceSystemIdentifier>\n'
        xml+= '\t\t</gmd:MD_ReferenceSystem>\n'
        xml+= '\t</gmd:referenceSystemInfo>\n'
    return xml

def identificationInformations(self):
    xml = ''
    xml+= '\t<gmd:identificationInfo>\n'
    xml+= '\t\t<gmd:MD_DataIdentification>\n'
    #CITATION BLOC
    xml+= '\t\t\t<gmd:citation>\n'
    xml+= '\t\t\t\t<gmd:CI_Citation>\n'
    value, text = getTextWidget(self, "intitule", False)
    xml+= '\t\t\t\t\t<gmd:title>\n'
    xml+= '\t\t\t\t\t\t<gco:CharacterString>%s</gco:CharacterString>\n' % (convertToUtf8(value))
    xml+= '\t\t\t\t\t</gmd:title>\n'
    
    obj = getWidget(self, "tabledatepubdata")
    for i in range(obj.rowCount()):
        value = obj.cellWidget(i, 0).text()
        dateType = getTypeDate(2, False) 
        if value.find("--") == -1 : xml+= datesInformations(self, value, dateType, 5)
    
    for i in range(0, 2):
        dateWidget = getTypeDate(i, True) 
        dateType = getTypeDate(i, False)
        value, text = getTextWidget(self, dateWidget, False)
        if value.find("--") == -1 : xml+= datesInformations(self, value, dateType, 5)

    obj = getWidget(self, "identificator")
    xml+= '\t\t\t\t\t<gmd:identifier>\n'
    xml+= '\t\t\t\t\t\t<gmd:MD_Identifier>\n'
    xml+= '\t\t\t\t\t\t\t<gmd:code>\n'
    xml+= '\t\t\t\t\t\t\t\t<gco:CharacterString>%s</gco:CharacterString>\n' % (convertToUtf8(obj.text()))
    xml+= '\t\t\t\t\t\t\t</gmd:code>\n'
    xml+= '\t\t\t\t\t\t</gmd:MD_Identifier>\n'
    xml+= '\t\t\t\t\t</gmd:identifier>\n'
    xml+= '\t\t\t\t</gmd:CI_Citation>\n'
    xml+= '\t\t\t</gmd:citation>\n'
    #End CITATION BLOC

    #ABSTRACT BLOC
    value, text = getTextWidget(self, "resume", False)
    xml+= '\t\t\t<gmd:abstract>\n'
    xml+= '\t\t\t\t<gco:CharacterString>%s</gco:CharacterString>\n' % (convertToUtf8(value))
    xml+= '\t\t\t</gmd:abstract>\n'
    #End ABSTRACT BLOC

    #ROLES BLOC
    obj = getWidget(self, "tableroles")
    for i in range(obj.rowCount()):
        value = self.dictListOfRules[obj.cellWidget(i, 0).currentIndex()]
        if value != "pointOfContact" : xml+= rulesInformations(self, obj, value, i, False, 3) 
    #End ROLES BLOC

    #BLOC INSPIRE optional keywords 
    obj = getWidget(self, "tablemotsclefsf")
    for i in range(obj.rowCount()):
        xml+= '\t\t\t<gmd:descriptiveKeywords>\n'
        xml+= '\t\t\t\t<gmd:MD_Keywords>\n'
        xml+= '\t\t\t\t\t<gmd:keyword>\n'
        xml+= '\t\t\t\t\t\t<gco:CharacterString>%s</gco:CharacterString>\n' % (convertToUtf8(obj.cellWidget(i, 0).text()))
        xml+= '\t\t\t\t\t</gmd:keyword>\n'

        elementThesaurus = obj.cellWidget(i, 2).text()
        
        if obj.cellWidget(i, 1).isChecked():
            xml+= '\t\t\t\t\t<gmd:thesaurusName>\n'
            xml+= '\t\t\t\t\t\t<gmd:CI_Citation>\n'
            xml+= '\t\t\t\t\t\t\t<gmd:title>\n'
            xml+= '\t\t\t\t\t\t\t\t<gco:CharacterString>%s</gco:CharacterString>\n' % (convertToUtf8(elementThesaurus))
            xml+= '\t\t\t\t\t\t\t</gmd:title>\n'

            dateValue = obj.cellWidget(i, 3).text()
            if dateValue.find("--")==-1:
                dateType = getTypeDate(obj.cellWidget(i, 4).currentIndex(), False)
                xml+= datesInformations(self, dateValue, dateType, 7) 
               
            xml+= '\t\t\t\t\t\t</gmd:CI_Citation>\n'
            xml+= '\t\t\t\t\t</gmd:thesaurusName>\n'
    
        xml+= '\t\t\t\t</gmd:MD_Keywords>\n'
        xml+= '\t\t\t</gmd:descriptiveKeywords>\n'
    #End BLOC INSPIRE optional keywords

    #BLOC INSPIRE required keywords
    obj = getWidget(self, "tablemotsclefso")
    for i in range(obj.rowCount()):
        xml+= '\t\t\t<gmd:descriptiveKeywords>\n'
        xml+= '\t\t\t\t<gmd:MD_Keywords>\n'
        xml+= '\t\t\t\t\t<gmd:keyword>\n'
        xml+= '\t\t\t\t\t\t<gco:CharacterString>%s</gco:CharacterString>\n' % (convertToUtf8(obj.cellWidget(i, 1).currentText()))
        xml+= '\t\t\t\t\t</gmd:keyword>\n'
        xml+= '\t\t\t\t\t<gmd:thesaurusName>\n'
        xml+= '\t\t\t\t\t\t<gmd:CI_Citation>\n'
        xml+= '\t\t\t\t\t\t\t<gmd:title>\n'
        xml+= '\t\t\t\t\t\t\t\t<gco:CharacterString>%s</gco:CharacterString>\n' % (convertToUtf8(obj.cellWidget(i, 0).currentText()))
        xml+= '\t\t\t\t\t\t\t</gmd:title>\n'

        try : xml+= datesInformations(self, self.dateListOfThesaurus[obj.cellWidget(i, 0).currentIndex()], "publication", 7)
        except : xml+= datesInformations(self, datetime.now().strftime("%d/%m/%Y"), "publication", 7) 

        xml+= '\t\t\t\t\t\t</gmd:CI_Citation>\n'
        xml+= '\t\t\t\t\t</gmd:thesaurusName>\n'
        xml+= '\t\t\t\t</gmd:MD_Keywords>\n'
        xml+= '\t\t\t</gmd:descriptiveKeywords>\n'
    #End BLOC INSPIRE required keywords   

    #BLOC resourceConstraints
    xml+= '\t\t\t<gmd:resourceConstraints>\n'
    xml+= '\t\t\t\t<gmd:MD_LegalConstraints>\n'
    obj = getWidget(self, "licence")
    xml+= '\t\t\t\t\t<gmd:useLimitation>\n'
    xml+= '\t\t\t\t\t\t<gco:CharacterString>%s</gco:CharacterString>\n' % (convertToUtf8(obj.toPlainText()))
    xml+= '\t\t\t\t\t</gmd:useLimitation>\n'
    obj = getWidget(self, "groupedroits")

    if obj.children()[0].isChecked() :
        xml+= '\t\t\t\t\t<gmd:accessConstraints>\n'
        xml+= '\t\t\t\t\t\t<gmd:MD_RestrictionCode codeList="http://standards.iso.org/ittf/PubliclyAvailableStandards/ISO_19139_Schemas/resources/Codelist/ML_gmxCodelists.xml#MD_RestrictionCode" codeListValue="otherRestrictions">otherRestrictions</gmd:MD_RestrictionCode>\n'
        xml+= '\t\t\t\t\t</gmd:accessConstraints>\n'
        xml+= '\t\t\t\t\t<gmd:otherConstraints>\n'
        xml+= '\t\t\t\t\t\t<gco:CharacterString>%s</gco:CharacterString>\n' % (QApplication.translate("QSphere","No restriction for public access with INSPIRE"))
        xml+= '\t\t\t\t\t</gmd:otherConstraints>\n'
    else :
        xml+= '\t\t\t\t\t<gmd:accessConstraints>\n'
        xml+= '\t\t\t\t\t\t<gmd:MD_RestrictionCode codeList="http://standards.iso.org/ittf/PubliclyAvailableStandards/ISO_19139_Schemas/resources/Codelist/ML_gmxCodelists.xml#MD_RestrictionCode" codeListValue="otherRestrictions">otherRestrictions</gmd:MD_RestrictionCode>\n'
        xml+= '\t\t\t\t\t</gmd:accessConstraints>\n'
        child = obj.children()[2]
        rows = child.model().rowCount()
        for i in range(rows):
            item = child.model().item(i, 0)
            if item.checkState()== Qt.Checked:
                value = child.model().item(i, 1).text()
                xml+= '\t\t\t\t\t<gmd:otherConstraints>\n'
                xml+= '\t\t\t\t\t\t<gco:CharacterString>%s</gco:CharacterString>\n' % (convertToUtf8(value))
                xml+= '\t\t\t\t\t</gmd:otherConstraints>\n'

    xml+= '\t\t\t\t</gmd:MD_LegalConstraints>\n'
    xml+= '\t\t\t</gmd:resourceConstraints>\n'
    #End BLOC resourceConstraints

    #BLOC spatialRepresentationType - not yet implemented (vector, raster, other ? ...)
    """
    xml+= '<gmd:spatialRepresentationType>\n'
    xml+= '<gmd:MD_SpatialRepresentationTypeCode codeList="http://librairies.ign.fr/geoportail/resources/CodeLists.xml#MD_SpatialRepresentationTypeCode" codeListValue="vector">vector</gmd:MD_SpatialRepresentationTypeCode>\n'
    xml+= '</gmd:spatialRepresentationType>\n'
    """

    #BLOC Spatial Resolution
    obj = getWidget(self, "grouperesolutionscale")
    child = obj.children()[2]
    for i in range(child.rowCount()):
        widget = child.cellWidget(i, 0)
        if obj.children()[0].isChecked() :
            xml+= '\t\t\t<gmd:spatialResolution>\n'
            xml+= '\t\t\t\t<gmd:MD_Resolution>\n'
            xml+= '\t\t\t\t\t<gmd:equivalentScale>\n'
            xml+= '\t\t\t\t\t\t<gmd:MD_RepresentativeFraction>\n'
            xml+= '\t\t\t\t\t\t\t<gmd:denominator>\n'
            xml+= '\t\t\t\t\t\t\t\t<gco:Integer>%s</gco:Integer>\n' % (int(widget.value()))
            xml+= '\t\t\t\t\t\t\t</gmd:denominator>\n'
            xml+= '\t\t\t\t\t\t</gmd:MD_RepresentativeFraction>\n'
            xml+= '\t\t\t\t\t</gmd:equivalentScale>\n'
            xml+= '\t\t\t\t</gmd:MD_Resolution>\n'
            xml+= '\t\t\t</gmd:spatialResolution>\n'
        else:
            unitMeasure = "%s" % (obj.children()[3].currentText())
            xml+= '\t\t\t<gmd:spatialResolution>\n'
            xml+= '\t\t\t\t<gmd:MD_Resolution>\n'
            xml+= '\t\t\t\t\t<gmd:distance>\n'
            xml+= '\t\t\t\t\t\t<gco:Distance uom="%s">%s</gco:Distance>\n' % (unitMeasure, widget.value())
            xml+= '\t\t\t\t\t</gmd:distance>\n'
            xml+= '\t\t\t\t</gmd:MD_Resolution>\n'
            xml+= '\t\t\t</gmd:spatialResolution>\n'
    #End BLOC Spatial Resolution

    #BLOC Language for the resource
    obj = getWidget(self, "tablelangues")
    rows = obj.model().rowCount()
    for i in range(rows):
        item = obj.model().item(i, 0)
        if item.checkState()== Qt.Checked:
           value = obj.model().item(i, 0).text()
           xml+= langsInformations(self, value, 3) 
    #End BLOC Language for the resource

    xml+= characterSetCodeInformations(self, 3) 


    #BLOC INSPIRE category(/ISO)
    obj = getWidget(self, "tablecategories")
    rows = obj.model().rowCount() 
    
    for i in range(rows):
        item = obj.model().item(i, 0)
        if item.checkState()== Qt.Checked:
            xml+= '\t\t\t<gmd:topicCategory>\n'
            xml+= '\t\t\t\t<gmd:MD_TopicCategoryCode>%s</gmd:MD_TopicCategoryCode>\n' % (makeCat(self, obj.model().item(i, 2).text()))
            xml+= '\t\t\t</gmd:topicCategory>\n'
    #End BLOC INSPIRE category(/ISO)

    #BLOC Extents
    xml+= '\t\t\t<gmd:extent>\n'
    xml+= '\t\t\t\t<gmd:EX_Extent>\n'
    obj = getWidget(self, "tableemprises")
    xml+= '\t\t\t\t\t<gmd:description>\n'
    xml+= '\t\t\t\t\t\t<gco:CharacterString>Liste des emprises</gco:CharacterString>\n'
    xml+= '\t\t\t\t\t</gmd:description>\n'
    for i in range(obj.rowCount()):
        xml+= '\t\t\t\t\t<gmd:geographicElement>\n'
        xml+= '\t\t\t\t\t\t<gmd:EX_GeographicBoundingBox>\n'
        xml+= '\t\t\t\t\t\t\t<gmd:westBoundLongitude>\n'
        xml+= '\t\t\t\t\t\t\t\t<gco:Decimal>%s</gco:Decimal>\n' % (obj.cellWidget(i, 2).value())
        xml+= '\t\t\t\t\t\t\t</gmd:westBoundLongitude>\n'
        xml+= '\t\t\t\t\t\t\t<gmd:eastBoundLongitude>\n'
        xml+= '\t\t\t\t\t\t\t\t<gco:Decimal>%s</gco:Decimal>\n' % (obj.cellWidget(i, 3).value())
        xml+= '\t\t\t\t\t\t\t</gmd:eastBoundLongitude>\n'
        xml+= '\t\t\t\t\t\t\t<gmd:southBoundLatitude>\n'
        xml+= '\t\t\t\t\t\t\t\t<gco:Decimal>%s</gco:Decimal>\n' % (obj.cellWidget(i, 1).value())
        xml+= '\t\t\t\t\t\t\t</gmd:southBoundLatitude>\n'
        xml+= '\t\t\t\t\t\t\t<gmd:northBoundLatitude>\n'
        xml+= '\t\t\t\t\t\t\t\t<gco:Decimal>%s</gco:Decimal>\n' % (obj.cellWidget(i, 0).value())
        xml+= '\t\t\t\t\t\t\t</gmd:northBoundLatitude>\n'   
        xml+= '\t\t\t\t\t\t</gmd:EX_GeographicBoundingBox>\n'
        xml+= '\t\t\t\t\t</gmd:geographicElement>\n'
       
    obj = getWidget(self, "tableetenduetemporelle")
    for i in range(obj.rowCount()):
        dateWidgets = "%s" % (obj.cellWidget(i, 0).text())
        if dateWidgets.find("-- --") == -1 :
            dates = dateWidgets.split(" ")
            xml+= '\t\t\t\t\t<gmd:temporalElement>\n'
            if obj.rowCount() == 1 :  xml+= '\t\t\t\t\t\t<gmd:EX_TemporalExtent>\n'
            else : xml+= '\t\t\t\t\t\t<gmd:EX_TemporalExtent gml:id="tp00%s">\n' % (i+1) 
            xml+= '\t\t\t\t\t\t\t<gmd:extent>\n'
            xml+= '\t\t\t\t\t\t\t\t<gml:TimePeriod xsi:type="gml:TimePeriodType">\n'
            xml+= '\t\t\t\t\t\t\t\t\t<gml:beginPosition>%s</gml:beginPosition>\n' % (dates[0]) 
            xml+= '\t\t\t\t\t\t\t\t\t<gml:endPosition>%s</gml:endPosition>\n'  % (dates[1])
            xml+= '\t\t\t\t\t\t\t\t</gml:TimePeriod>\n'
            xml+= '\t\t\t\t\t\t\t</gmd:extent>\n'
            xml+= '\t\t\t\t\t\t</gmd:EX_TemporalExtent>\n'
            xml+= '\t\t\t\t\t</gmd:temporalElement>\n'
    xml+= '\t\t\t\t</gmd:EX_Extent>\n'
    xml+= '\t\t\t</gmd:extent>\n'
    #End BLOC Extents
    
    xml+= '\t\t</gmd:MD_DataIdentification>\n'
    xml+= '\t</gmd:identificationInfo>\n'

    return xml


def distributionInformations(self):
    xml = ''                     
    xml+= '\t<gmd:distributionInfo>\n'
    xml+= '\t\t<gmd:MD_Distribution>\n'
    obj = getWidget(self, "tableformats")
    for i in range(obj.rowCount()): xml+= formatsInformations(self, obj, i, 3)
   
    xml+= '\t\t\t<gmd:transferOptions>\n'
    xml+= '\t\t\t\t<gmd:MD_DigitalTransferOptions>\n'
    obj = getWidget(self, "tablelocalisator")
    for i in range(obj.rowCount()):
        xml+= '\t\t\t\t\t<gmd:onLine>\n'
        xml+= '\t\t\t\t\t\t<gmd:CI_OnlineResource>\n'
        xml+= '\t\t\t\t\t\t\t<gmd:linkage>\n'
        xml+= '\t\t\t\t\t\t\t\t<gmd:URL>%s</gmd:URL>\n' % (encodeUrlQueryString(obj.cellWidget(i, 0).text()))
        xml+= '\t\t\t\t\t\t\t</gmd:linkage>\n'
        xml+= '\t\t\t\t\t\t\t<gmd:protocol>\n'
        xml+= '\t\t\t\t\t\t\t\t<gco:CharacterString>%s</gco:CharacterString>\n' % (getProtocole(obj.cellWidget(i, 0).text()))
        xml+= '\t\t\t\t\t\t\t</gmd:protocol>\n'
        xml+= '\t\t\t\t\t\t\t<gmd:name>\n'
        xml+= '\t\t\t\t\t\t\t\t<gco:CharacterString>%s</gco:CharacterString>\n' % (convertToUtf8(obj.cellWidget(i, 1).text()))
        xml+= '\t\t\t\t\t\t\t</gmd:name>\n'
        xml+= '\t\t\t\t\t\t</gmd:CI_OnlineResource>\n'
        xml+= '\t\t\t\t\t</gmd:onLine>\n'
    xml+= '\t\t\t\t</gmd:MD_DigitalTransferOptions>\n'
    xml+= '\t\t\t</gmd:transferOptions>\n'
    xml+= '\t\t</gmd:MD_Distribution>\n'
    xml+= '\t</gmd:distributionInfo>\n'
    return xml

def qualityInformations(self):
    xml = ''
    obj = getWidget(self, "typedata")
    value = self.typeData[obj.currentIndex()] 
    
    xml+= '\t<gmd:dataQualityInfo>\n'
    xml+= '\t\t<gmd:DQ_DataQuality>\n'
    xml+= '\t\t\t<gmd:scope>\n'
    xml+= '\t\t\t\t<gmd:DQ_Scope>\n'
    xml+= '\t\t\t\t\t<gmd:level>\n'
    xml+= '\t\t\t\t\t\t<gmd:MD_ScopeCode codeList="http://standards.iso.org/ittf/PubliclyAvailableStandards/ISO_19139_Schemas/resources/codelist/gmxCodelists.xml#MD_ScopeCode" codeListValue="%s">%s</gmd:MD_ScopeCode>\n' % (value, value)
    xml+= '\t\t\t\t\t</gmd:level>\n'
    xml+= '\t\t\t\t</gmd:DQ_Scope>\n'
    xml+= '\t\t\t</gmd:scope>\n'

    obj = getWidget(self, "tablespecifications")

    if obj.rowCount()> 0 : xml+= '\t\t\t<gmd:report>\n'
    for i in range(obj.rowCount()):
        specialValue = obj.cellWidget(i, 0).text() 
        if specialValue != "" :
            xml+= '\t\t\t\t<gmd:DQ_DomainConsistency xsi:type="gmd:DQ_DomainConsistency_Type">\n'
            xml+= '\t\t\t\t\t<gmd:result>\n'
            xml+= '\t\t\t\t\t\t<gmd:DQ_ConformanceResult xsi:type="gmd:DQ_ConformanceResult_Type">\n'
            xml+= '\t\t\t\t\t\t\t<gmd:specification>\n'
            xml+= '\t\t\t\t\t\t\t\t<gmd:CI_Citation>\n'
            xml+= '\t\t\t\t\t\t\t\t\t<gmd:title>\n'
            xml+= '\t\t\t\t\t\t\t\t\t\t<gco:CharacterString>%s</gco:CharacterString>\n' % (specialValue)
            xml+= '\t\t\t\t\t\t\t\t\t</gmd:title>\n'

            dateValue = obj.cellWidget(i, 1).text()
            if dateValue.find("--") == -1 :
                dateType = getTypeDate(obj.cellWidget(i, 2).currentIndex(), False) 
                xml+= datesInformations(self, dateValue, dateType, 9) 

            xml+= '\t\t\t\t\t\t\t\t</gmd:CI_Citation>\n'
            xml+= '\t\t\t\t\t\t\t</gmd:specification>\n'
            xml+= '\t\t\t\t\t\t\t<gmd:explanation>\n'
            xml+= '\t\t\t\t\t\t\t\t<gco:CharacterString>See the referenced specification</gco:CharacterString>\n'
            xml+= '\t\t\t\t\t\t\t</gmd:explanation>\n'
            
            checkConformity = obj.cellWidget(i, 3).currentIndex()
            if checkConformity == 0 : value = "true"
            elif checkConformity : value = "false"
            else : value = ""
            if value != "" :
                xml+= '\t\t\t\t\t\t\t<gmd:pass>\n'
                xml+= '\t\t\t\t\t\t\t\t<gco:Boolean>%s</gco:Boolean>\n' % (value)
                xml+= '\t\t\t\t\t\t\t</gmd:pass>\n'
            else: xml+= '\t\t\t\t\t\t\t<gmd:pass/>\n'   
            
            xml+= '\t\t\t\t\t\t</gmd:DQ_ConformanceResult>\n'
            xml+= '\t\t\t\t\t</gmd:result>\n'
            xml+= '\t\t\t\t</gmd:DQ_DomainConsistency>\n'
    if obj.rowCount()> 0 : xml+= '\t\t\t</gmd:report>\n'

    obj = getWidget(self, "genealogie")
    value = obj.toPlainText()
    xml+= '\t\t\t<gmd:lineage>\n'
    xml+= '\t\t\t\t<gmd:LI_Lineage>\n'
    xml+= '\t\t\t\t\t<gmd:statement>\n'
    xml+= '\t\t\t\t\t\t<gco:CharacterString>%s</gco:CharacterString>\n' % convertToUtf8(value)
    xml+= '\t\t\t\t\t</gmd:statement>\n'
    xml+= '\t\t\t\t</gmd:LI_Lineage>\n'
    xml+= '\t\t\t</gmd:lineage>\n'  

    xml+= '\t\t</gmd:DQ_DataQuality>\n'
    xml+= '\t</gmd:dataQualityInfo>\n'

    return xml

def geographicalInformations(self):
    xml = ''
    obj = getWidget(self, "coherence")
    if obj.toPlainText() != "" :
        import ast
        try : mydict = ast.literal_eval("%s" % (obj.toPlainText()))
        except : mydict = None

        if mydict != None and type(mydict)==dict:
            topologyLevelCode = True if dicoHasKey(mydict, 'TopologyLevelCode') and mydict['TopologyLevelCode']!= 'unknow' else False
            geometricObjTypeCode = True if dicoHasKey(mydict, 'GeometricObjectTypeCode') and mydict['GeometricObjectTypeCode']!= 'unknow' else False

            if topologyLevelCode or geometricObjTypeCode :
                    xml+= '\t<gmd:spatialRepresentationInfo>\n'
                    xml+= '\t\t<gmd:MD_VectorSpatialRepresentation>\n'
                    if topologyLevelCode :
                        xml+= '\t\t\t<gmd:topologyLevel>\n'
                        xml+= '\t\t\t\t<gmd:MD_TopologyLevelCode codeList="http://standards.iso.org/ittf/PubliclyAvailableStandards/ISO_19139_Schemas/resources/Codelist/ML_gmxCodelists.xml#MD_TopologyLevelCode" codeListValue="%s">%s</gmd:MD_TopologyLevelCode>\n' % (mydict['TopologyLevelCode'], mydict['TopologyLevelCode'])
                        xml+= '\t\t\t</gmd:topologyLevel>\n'
                    if geometricObjTypeCode :    
                        xml+= '\t\t\t<gmd:geometricObjects>\n'
                        xml+= '\t\t\t\t<gmd:MD_GeometricObjects>\n'
                        xml+= '\t\t\t\t\t<gmd:geometricObjectType>\n'
                        xml+= '\t\t\t\t\t\t<gmd:MD_GeometricObjectTypeCode codeList="http://standards.iso.org/ittf/PubliclyAvailableStandards/ISO_19139_Schemas/resources/Codelist/ML_gmxCodelists.xml#MD_GeometricObjectTypeCode" codeListValue="%s">%s</gmd:MD_GeometricObjectTypeCode>\n' % (mydict['GeometricObjectTypeCode'],mydict['GeometricObjectTypeCode'])
                        xml+= '\t\t\t\t\t</gmd:geometricObjectType>\n'
                        xml+= '\t\t\t\t</gmd:MD_GeometricObjects>\n'
                        xml+= '\t\t\t</gmd:geometricObjects>\n'
                    xml+= '\t\t</gmd:MD_VectorSpatialRepresentation>\n'
                    xml+= '\t</gmd:spatialRepresentationInfo>\n'
    return xml               

def endXml(self):
    xml = ''                 
    xml+= '</gmd:MD_Metadata>'
    return xml
    
#-----------------------------
# PRIMARY BLOC FOR XML EXPORT       
#----------------------------- 
def formatsInformations(self, obj, i, nbTab):
    tab=""
    xml = ''
    for k in range(nbTab): tab+="\t"    
    if obj.cellWidget(i, 0).styleSheet() == "background-color:#AEEE00;" :
        xml+= '%s<gmd:distributionFormat>\n' % (tab)
        xml+= '\t%s<gmd:MD_Format>\n' % (tab)
        xml+= '\t\t%s<gmd:name>\n' % (tab)
        xml+= '\t\t\t%s<gco:CharacterString>%s</gco:CharacterString>\n' % (tab, obj.cellWidget(i, 0).text())
        xml+= '\t\t%s</gmd:name>\n' % (tab)
        xml+= '\t\t%s<gmd:version>\n' % (tab)
        xml+= '\t\t\t%s<gco:CharacterString>%s</gco:CharacterString>\n' % (tab, obj.cellWidget(i, 2).text())
        xml+= '\t\t%s</gmd:version>\n' % (tab)
        xml+= '\t%s</gmd:MD_Format>\n' % (tab)
        xml+= '%s</gmd:distributionFormat>\n' % (tab)
    return xml
    
def langsInformations(self, value, nbTab):
    tab=""
    xml = ''
    for k in range(nbTab): tab+="\t"
    xml+= '%s<gmd:language>\n' % (tab)
    xml+= '%s<gmd:LanguageCode codeList="http://www.loc.gov/standards/iso639-2/" codeListValue="%s">%s</gmd:LanguageCode>\n' % ("%s%s" % (tab, "\t"),value, value)
    xml+= '%s</gmd:language>\n' % (tab)
    return xml

def datesInformations(self, value, zType, nbTab):
    tab=""
    xml = ''
    for k in range(nbTab): tab+="\t"
    xml+= '%s<gmd:date>\n' % (tab)
    xml+= '\t%s<gmd:CI_Date>\n' % (tab)
    xml+= '\t\t%s<gmd:date>\n' % (tab)
    xml+= '\t\t\t%s<gco:Date>%s</gco:Date>\n' % (tab, value)
    xml+= '\t\t%s</gmd:date>\n' % (tab)
    xml+= '\t\t%s<gmd:dateType>\n' % (tab)
    xml+= '\t\t\t%s<gmd:CI_DateTypeCode codeList="http://standards.iso.org/ittf/PubliclyAvailableStandards/ISO_19139_Schemas/resources/Codelist/ML_gmxCodelists.xml#CI_DateTypeCode" codeListValue="%s">%s</gmd:CI_DateTypeCode>\n' % (tab, zType, zType)
    xml+= '\t\t%s</gmd:dateType>\n' % (tab)
    xml+= '\t%s</gmd:CI_Date>\n' % (tab)
    xml+= '%s</gmd:date>\n' % (tab)  

    return xml                     

def rulesInformations(self, obj, value, i, isPointOfContact, nbTab):
    tab, k, xml = "", 0, ''
    for k in range(nbTab): tab+="\t"    
    xml+= '%s<gmd:contact>\n' % (tab) if isPointOfContact else '%s<gmd:pointOfContact>\n' % (tab)
    xml+= responsiblePartiesInformations(self, obj, value, i, isPointOfContact, (k+1))
    xml+= '%s</gmd:contact>\n' % (tab) if isPointOfContact else '%s</gmd:pointOfContact>\n' % (tab)
    return xml

def responsiblePartiesInformations(self, obj, value, i, isPointOfContact, nbTab):
    tab=""
    xml = ''
    for k in range(nbTab): tab+="\t"     
    xml+= '%s<gmd:CI_ResponsibleParty>\n' % (tab)
    xml+= '\t%s<gmd:organisationName>\n' % (tab)
    xml+= '\t\t%s<gco:CharacterString>%s</gco:CharacterString>\n' % (tab, convertToUtf8(obj.cellWidget(i, 1).text())) 
    xml+= '\t%s</gmd:organisationName>\n' % (tab)
    xml+= '\t%s<gmd:contactInfo>\n' % (tab)
    xml+= '\t\t%s<gmd:CI_Contact>\n' % (tab)
    xml+= '\t\t\t%s<gmd:phone>\n' % (tab)
    xml+= '\t\t\t\t%s<gmd:CI_Telephone>\n' % (tab)
    xml+= '\t\t\t\t\t%s<gmd:voice>\n' % (tab)
    xml+= '\t\t\t\t\t\t%s<gco:CharacterString>%s</gco:CharacterString>\n' % (tab, obj.cellWidget(i, 7).text() if obj.cellWidget(i, 7).text().strip()!="" else "")
    xml+= '\t\t\t\t\t%s</gmd:voice>\n' % (tab)
    xml+= '\t\t\t\t\t%s<gmd:facsimile>\n' % (tab)
    xml+= '\t\t\t\t\t\t%s<gco:CharacterString>%s</gco:CharacterString>\n' % (tab, obj.cellWidget(i, 8).text() if obj.cellWidget(i, 8).text().strip()!="" else "")
    xml+= '\t\t\t\t\t%s</gmd:facsimile>\n' % (tab)
    xml+= '\t\t\t\t%s</gmd:CI_Telephone>\n' % (tab)
    xml+= '\t\t\t%s</gmd:phone>\n' % (tab)
    xml+= '\t\t\t%s<gmd:address>\n' % (tab)
    xml+= '\t\t\t\t%s<gmd:CI_Address>\n' % (tab)
    xml+= '\t\t\t\t\t%s<gmd:deliveryPoint>\n' % (tab)
    xml+= '\t\t\t\t\t\t%s<gco:CharacterString>%s</gco:CharacterString>\n' % (tab, convertToUtf8(obj.cellWidget(i, 2).text()) if obj.cellWidget(i, 2).text().strip()!="" else "")
    xml+= '\t\t\t\t\t%s</gmd:deliveryPoint>\n' % (tab)
    xml+= '\t\t\t\t\t%s<gmd:city>\n' % (tab)
    xml+= '\t\t\t\t\t\t%s<gco:CharacterString>%s</gco:CharacterString>\n' % (tab, convertToUtf8(obj.cellWidget(i, 5).text()) if obj.cellWidget(i, 5).text().strip()!="" else "")
    xml+= '\t\t\t\t\t%s</gmd:city>\n' % (tab)
    xml+= '\t\t\t\t\t%s<gmd:postalCode>\n' % (tab)
    xml+= '\t\t\t\t\t\t%s<gco:CharacterString>%s</gco:CharacterString>\n' % (tab, convertToUtf8(obj.cellWidget(i, 4).text()) if obj.cellWidget(i, 4).text().strip()!="" else "00000")
    xml+= '\t\t\t\t\t%s</gmd:postalCode>\n' % (tab)
    xml+= '\t\t\t\t\t%s<gmd:country>\n' % (tab)
    xml+= '\t\t\t\t\t\t%s<gco:CharacterString>%s</gco:CharacterString>\n' % (tab, convertToUtf8(obj.cellWidget(i, 3).currentText().rstrip().lstrip()) if obj.cellWidget(i, 3).currentText().strip()!="" else "")
    xml+= '\t\t\t\t\t%s</gmd:country>\n' % (tab)
    xml+= '\t\t\t\t\t%s<gmd:electronicMailAddress>\n' % (tab)
    xml+= '\t\t\t\t\t\t%s<gco:CharacterString>%s</gco:CharacterString>\n' % (tab, obj.cellWidget(i, 6).text() if obj.cellWidget(i, 6).text().strip()!="" else "")
    xml+= '\t\t\t\t\t%s</gmd:electronicMailAddress>\n' % (tab)
    xml+= '\t\t\t\t%s</gmd:CI_Address>\n' % (tab)
    xml+= '\t\t\t%s</gmd:address>\n' % (tab)
    xml+= '\t\t\t%s<gmd:onlineResource>\n' % (tab)
    xml+= '\t\t\t\t%s<gmd:CI_OnlineResource>\n' % (tab)
    xml+= '\t\t\t\t\t%s<gmd:linkage>\n' % (tab) #8 devient 9
    xml+= '\t\t\t\t\t\t%s<gmd:URL>%s</gmd:URL>\n' % (tab, encodeUrlQueryString(obj.cellWidget(i, 9).text()) if obj.cellWidget(i, 9).text().strip()!="" else "")
    xml+= '\t\t\t\t\t\t%s</gmd:linkage>\n' % (tab)
    xml+= '\t\t\t\t%s</gmd:CI_OnlineResource>\n' % (tab)
    xml+= '\t\t\t%s</gmd:onlineResource>\n'  % (tab)   
    xml+= '\t\t%s</gmd:CI_Contact>\n' % (tab)
    xml+= '\t%s</gmd:contactInfo>\n' % (tab)
    xml+= '\t%s<gmd:role>\n' % (tab)
    xml+= '\t\t%s<gmd:CI_RoleCode codeList="http://standards.iso.org/ittf/PubliclyAvailableStandards/ISO_19139_Schemas/resources/Codelist/ML_gmxCodelists.xml#CI_RoleCode" codeListValue="%s">%s</gmd:CI_RoleCode>\n' % (tab, value, value)
    xml+= '\t%s</gmd:role>\n' % (tab)
    xml+= '%s</gmd:CI_ResponsibleParty>\n' % (tab)
    return xml
    
#-------------------------------
# FONCTIONs TO WRITE XML EXPORT       
#-------------------------------
def getProtocole(url) :
    if url == None : return ""

    try : 
        elts = urlparse(url)
        protocole = ""
        if elts.scheme != "" : return (elts.scheme)
        else : return ""
    except : return ""     
    
def encodeUrlQueryString(url):  
    if url == None : return ""

    try : 
        elts = urlparse(url)
        urlEncoded = ""
        if elts.scheme != "" : urlEncoded+= "%s://" % (elts.scheme)
        else : return urlEncoded
       
        if elts.netloc != "" and not elts.port in (None,"", "None") : urlEncoded+= "%s:%s" % (elts.netloc, elts.port)
        elif elts.netloc != "" and elts.port in (None,"", "None") : urlEncoded+= "%s" % (elts.netloc)
        else : return urlEncoded
        
        if elts.path != "" : urlEncoded+= "%s" % (elts.path)
        if elts.query != "" : urlEncoded+= "?%s" % (urllibquote(elts.query))
        if elts.fragment != "" : urlEncoded+= "#%s" % (elts.fragment)
        
        return urlEncoded

    except :
        return url

def getTypeDate(index, key):
    if index > 2 : index = 0
    dates = (("datecredata", "creation"), ("daterevdata","revision"), ("datepubdata","publication"))
    return dates[index][0] if key else dates[index][1]
    
def makeCat(self, text):  
    temp = text.split("(")
    cat = temp[1].replace(")", "")
    return "%s" % (convertToUtf8(cat))

def writeInFile(inFile, msg):
    if inFile != None :  inFile.write(msg)
    
def writeInFileWithoutEncoding(inFile, msg):
    if inFile != None : inFile.write(msg)

def closeFile(inFile):     
    if inFile != None : inFile.close()
