# -*- coding:utf-8 -*- 
from qgis.core import Qgis
from qgis.gui import QgsDateTimeEdit

from qgis.PyQt.QtCore import (QCoreApplication, QFileInfo, QLocale, QSettings, Qt, QTranslator)
from qgis.PyQt.QtGui import QIcon
from qgis.PyQt.QtWidgets import (QAction, QApplication, QCalendarWidget, QCheckBox,
                         QComboBox, QCommandLinkButton, QDialog, QFileDialog, QGroupBox,
                         QLabel, QLineEdit, QMdiArea, QMenu, QPushButton, QRadioButton, QSizePolicy, QTabBar,
                         QTabWidget, QTableView, QTableWidget, QTableWidgetItem, QTextEdit,
                         QToolBar, QToolButton, QWidget)

from qsphere.ui import (dlgAbout, dlgContacts, dlgMetadata, dlgOptions, dlgSgbdr, loadDlgNavigator)
from qsphere.qsphere_objmaker import (myAction, myPushButton, myTableWidget, myTextEdit, toolSearchFilesDialog)
from qsphere.qsphere_tools import (getIsoCodes, getOptions, getThemeIcon, getUiLangIndex, loadData, loadDefautValue,
                           getListCountries, getHelp, getListEncoders, getListFormats, getListLangs,
                           getListTemporalSystem, getListTypeRessources, windowIcon, windowProperties)
from qsphere.ui_mainwindow import myMdiWindow
from qsphere.ui_editorxml import xmlEditor

from os.path import dirname
import sys
if sys.version_info[:2] >= (3, 4): from importlib import reload
else: from imp import reload


class qspherePlugin(object):
  
  def __init__(self, iface):
    self.name = "qsphere"
    self._objectName = "qsphere"
    self.version = "5.5.0"
    self.iface = iface
    self.qspherePlugin = self
    self.mainWindow = None
    self.webNavigator = None
    self.dlgSgbdr = None
    self.dialogSearchFiles = None
    self.dialogXmlEditor = None

    overrideLocale = QSettings().value("locale/overrideFlag", False)
    localeFullName = QLocale.system().name() if not overrideLocale else QSettings().value("locale/userLocale")
    self.localLangage = localeFullName[0:2]
    self.localePath = "%s/i18n/qsphere_%s.qm" % (dirname(__file__), self.localLangage)
    self.localeFullName = localeFullName

    reload(sys)
    try : sys.setdefaultencoding('iso-8859-1')
    except : pass

    self.languesDico = getIsoCodes('ISO-639-2_utf-8.txt')
    self.langue, self.languageIndex = getUiLangIndex(self)
    self.dicoLangs = ("fr", "en", "it", "de", "es", "fi")
    self.indexLang = self.dicoLangs.index(self.localLangage) if self.localLangage in self.dicoLangs else 0
    self.toolBar = None
    self.translator = QTranslator()

    self.libelles = {
                         0:  ("tab1", QWidget, "Identification", "Identification data (title, abstract,\ntype, encoding ...)", "identification.png", ""),
                         1:  ("titletab1", QLabel, "Data identification : ","Data identification ", None, ""),
                         2:  ("intitule", QLineEdit, "Title : ", "Title of spatial data sets and services ", None, "file:intitule_%s.html"),
                         3:  ("resume", myTextEdit, "Abstract : ", "Abstract of the dataset describing the resource ", None, "file:resume_%s.html"),
                         4:  ("typedata", QComboBox, "Resource type : ", "Resource type ", None, "file:type_ressource_%s.html"),
                         5:  ("tablelocalisator", myTableWidget, "Locator (s) for the resource : ", "Link(s) for the resource ", None, "file:localisateur_%s.html"),
                         6:  ("tablelocalisator_c0", None, "URL", "", None, ""),
                         7:  ("tablelocalisator_c1", None, "Name", "", None, ""),
                         8:  ("identificator", QLineEdit, "Resource Identifier : ", "Identifier for the resource ", None, "file:identificateur_%s.html"),
                         9:  ("tablelangues", QTableView, "Language(s) for the resource : ", "List of language(s) for the resource : ", None, "file:langue_%s.html"),
                         10: ("tableformats", myTableWidget, "Format(s) : ", "Different formats of spatial data sets and services ", None, "file:format_fichier_%s.html"),
                         11: ("tableformats_c0", None, "Format", "", None, ""),
                         12: ("tableformats_c1", None, "", "", None, ""),
                         13: ("tableformats_c2", None, "Version", "", None, ""),
                         14: ("tablecarac", QComboBox, "Encoding : ", "Character encoding of spatial data sets and services ", None, "file:encodage_%s.html"),
                         15: ("tab2", QWidget, "Classification & Keywords","Classification & Keywords", "classification.png", "file:categorie_thematique_%s.html"),
                         16: ("titletab2_1", QLabel, "Classification of spatial data and services : ", "Data classification and mapping services ", None, ""),
                         17: ("tablecategories", QTableView, "Thematic categories : ", "Thematic categories\n(INSPIRE Annexes - INSPIRE Themes - ISO classes ) ", None, "file:categorie_thematique_%s.html"),
                         18: ("titletab2_2", QLabel, "Keywords : ", "Keywords ", None, ""),
                         19: ("tablemotsclefso", myTableWidget, "Keywords from INSPIRE<br>Data themes & repositories : ", "List of keywords required (INSPIRE Themes) ", None, "file:mots_clefs_%s.html"),
                         20: ("tablemotsclefso_c0", None, "Thematic", "", None, ""),
                         21: ("tablemotsclefso_c1", None, "Keyword", "", None, ""),
                         22: ("tablemotsclefsf", myTableWidget, "<i>Free</i> keywords : ", "List of optional keywords (INSPIRE Themes) ", None, "file:mots_clefs_%s.html"),
                         23: ("tablemotsclefsf_c0", None, "Keyword", "", None, ""),
                         24: ("tablemotsclefsf_c1", None, "", "", None, ""),
                         25: ("tablemotsclefsf_c2", None, "Controlled vocabulary ", "", None, ""),
                         26: ("tablemotsclefsf_c3", None, "Date", "", None, ""),
                         27: ("tablemotsclefsf_c4", None, "Date type", "", None, ""),
                         28: ("tab3", QWidget, "Geographic", "Geographical location", "location.png", None, ""),
                         29: ("titletab3_1", QLabel, "Location : ", "Information for the location (data required) ", None, ""),
                         30: ("subtitletab3_1", QLabel, "Bouding box : ", "Information for the location (Bouding box) ", None, ""),
                         31: ("tableemprises", myTableWidget, "", "List of the extents of spatial data sets and services ", None, "file:emprise_%s.html"),
                         32: ("tableemprises_c0", None, "North lat.", "", None, ""),
                         33: ("tableemprises_c1", None, "South lat.", "", None, ""),
                         34: ("tableemprises_c2", None, "West long.", "", None, ""),
                         35: ("tableemprises_c3", None, "East long.", "", None, ""),
                         36: ("tableemprises_c4", None, "Extent", "", None, ""),
                         37: ("subtitletab3_2", QLabel, "Coordinate reference system(s) : ", "Information about coordinate reference system(s) (data required) ", None, ""),
                         38: ("tablescr", myTableWidget, "SRS : " ,"List of reference system(s) of spatial data sets and services ", None, "file:systeme_de_reference_%s.html"),
                         39: ("tablescr_c0", None, "Selected CRS", "", None, ""),
                         40: ("tablescr_c1", None, "", "", None, ""),
                         41: ("tab4", QWidget, "Temporal", "Dates temporal referencing", "temporality.png", ""),
                         42: ("titletab4_1", QLabel, "Temporal informations : ", "Temporal informations ", None, ""),
                         43: ("tableetenduetemporelle", myTableWidget, "Temporal extent : ", "Information about temporal extent (optional data)", None, "file:etendue_temporelle_%s.html"),
                         44: ("tableetenduetemporelle_c0", None, " Start <-> End ", "", None, ""),
                         45: ("tableetenduetemporelle_c1", None, "", "", None, ""),
                         46: ("tabledatepubdata", myTableWidget, "Date of publication : ", "Date of publication of spatial data sets and services ", None, "file:dates_reference_%s.html"),
                         47: ("tabledatepubdata_c0", None, "Date", "", None, ""),
                         48: ("tabledatepubdata_c1", None, "", "", None, ""),
                         49: ("datecredata", QgsDateTimeEdit, "Date of creation : ", "Date of creation of spatial data sets and services ", None, "file:dates_reference_%s.html"),
                         50: ("daterevdata", QgsDateTimeEdit, "Date of the last revision : ", "Date of the last revision of spatial data sets and services ", None, "file:dates_reference_%s.html"),
                         51: ("sysreftemp", QComboBox, "Temporal reference system : ", "Temporal reference system ", None, "file:systeme_de_reference_temporel_%s.html"),
                         52: ("tab5", QWidget, "Quality, Validity and Conformity", "Quality, Validity and Conformity", "quality.png", ""),
                         53: ("titletab5_1", QLabel, "Quality & validity : ", "Quality & validity", None, ""),
                         54: ("subtitletab5_1", QLabel, "Lineage : ", "Lineage of spatial data sets and services ", None, ""),
                         55: ("genealogie", myTextEdit, "Lineage for the data : ", "Lineage of spatial data sets and services ", None, "file:genealogie_%s.html"),
                         56: ("subtitletab5_2", QLabel, "Spatial resolution : ", "Spatial resolution of spatial data sets and services ", None, ""),
                         57: ("grouperesolutionscale", QGroupBox, "Equivalent scale : ", "Equivalent scale of spatial data sets and services ", None, "file:resolution_spatiale_%s.html"),
                         58: ("coherence", myTextEdit, "Topological consistency : ", "Topological consistency of spatial data sets and services (data not required) ", None, "file:coherence_topologique_%s.html"),
                         59: ("subtitletab5_3", QLabel, "Compliance : ", "Compliance of spatial data sets and services ", None, ""),
                         60: ("tablespecifications", myTableWidget, "Compliance specifications : ", "List of compliance specifications of spatial data sets and services ", None, "file:conformite_specifications_%s.html"),
                         61: ("tablespecifications_c0", None, "Specification", "", None, ""),
                         62: ("tablespecifications_c1", None, "Date", "", None, ""),
                         63: ("tablespecifications_c2", None, "Date type", "", None, ""),
                         64: ("tablespecifications_c3", None, "Class", "", None, ""),
                         65: ("tab6", QWidget, "Constraints", "Access and uses", "rights_and_uses.png", ""),
                         66: ("titletab6_1", QLabel, "Constraint related to access and use : ", "Constraint related to access and use ", None, ""),
                         67: ("groupedroits", QGroupBox, "Conditions applying...\nto access and use : ", "Conditions applying to access and use ", None, "file:contraintes_%s.html"),
                         68: ("licence", myTextEdit, "Conditions licensing applying...\nto access and use : ", "Licensing conditions applying to access and use ", None, "file:licence_%s.html"),
                         69: ("tab7", QWidget, "Responsible party", "Responsible party", "responsible.png", ""),
                         70: ("titletab7_1", QLabel, "Responsible party : ", "Organisations reponsible for the etablishment, management, maintenance and distribution of spatial data sets and services ", None, ""),
                         71: ("tableroles", myTableWidget, "", "Responsible party role ", None, "file:organisme_responsable_%s.html"),
                         72: ("tableroles_c0", None, "Role", "", None, ""),
                         73: ("tableroles_c1", None, "Organization name", "", None, ""),
                         74: ("tableroles_c2", None, "Address", "", None, ""),
                         75: ("tableroles_c3", None, "Country", "", None, ""),
                         76: ("tableroles_c4", None, "Zip code", "", None, ""),
                         77: ("tableroles_c5", None, "City", "", None, ""),
                         78: ("tableroles_c6", None, "E-mail", "", None, ""),
                         79: ("tableroles_c7", None, "Phone", "", None, ""),
                         80: ("tableroles_c8", None, "Fax", "", None, ""),
                         81: ("tableroles_c9", None, "URL", "", None, ""),
                         82: ("tab8", QWidget, "Metadata", "Metadata", "metadata.png", ""),
                         83: ("titletab8_1", QLabel, "Informations about Metadata : ", "Informations about Metadata ", None, ""),
                         84: ("datemetada", QgsDateTimeEdit, "Metadata date : ", "Metadata date ", None, "file:date_metadonnee_%s.html"),
                         85: ("langmetada", QComboBox, "Metadata language : ", "Metadata language ", None, "file:langue_metadonnee_%s.html"),
                         86: ("tab9", QWidget, "QGIS metadata", "Metadata by QGIS", "qgis.png", ""),
                         87: ("titletab9_1", QLabel, "Additional Information QGIS QSPHERE : ", "Additional Information QGIS QSPHERE : ", None, ""),
                         88: ("namelayer", QLineEdit, "Name layer : ", "Name Layer ", None, "file:catalogue_attributaire_%s.html"),
                         89: ("typelayer", QLineEdit, "Type layer : ", "Type Layer ", None, "file:catalogue_attributaire_%s.html"),
                         90: ("metadata", myTextEdit, "", "Generic metadata QGIS for geographical layer (vector or raster) ", None, "file:catalogue_attributaire_%s.html"),
                         91: ("tabledico", myTableWidget, "", "Fields for the query layer (vector) ", None, "file:catalogue_attributaire_%s.html"),
                         92: ("tabledico_c0", None, "Id", "", None, ""),
                         93: ("tabledico_c1", None, "Name", "", None, ""),
                         94: ("tabledico_c2", None, "Type", "", None, ""),
                         95: ("tabledico_c3", None, "Length", "", None, ""),
                         96: ("tabledico_c4", None, "Precision", "", None, ""),
                         97: ("tabledico_c5", None, "Comment", "", None, ""),
                         100:("helptopic", myPushButton, "", "Get informations for this topic", "info.png", ""),
                         101:("actionsmetadata", myPushButton, "", "Actions for the metadata...", "actions.png", ""),
                         102:("callwizard", myPushButton, "", "Call the wizards ...", "wizards_form.png", "")
                         }

    self.formats = getListFormats(self)
    self.langs = getListLangs(self)
    self.listCodecs = getListEncoders(self)
    self.listTemporalSystem = getListTemporalSystem(self)
    self.listTypeRessources = getListTypeRessources(self)
    self.listCountries, self.indexCountry, self.listCountriesCode = getListCountries(self)

    self.paramsLineWidget = {
                                  "tablescr" : [(0, 3, "EPSG:3857"), (1, -1, "")],
                                  "tableformats" : [(0, 6, ""), ( 1, -1, ""), (-1, 0, "")],
                                  "tableroles" : [(2, 0, 6),(-1, 0, ""), (-1, 0, ""), (2, 5, self.indexCountry), (0, 2, ""), (-1, 0, ""), (0, 0, ""), (-1, 0, ""), (-1, 0, ""), (0, 4, "")],
                                  "tablelocalisator" : [(0, 4, ""), (-1, 0, "")],
                                  "tableemprises" : [(3, 0, -1), (3, 0, -1), (3, 0, -1), (3, 0, -1), (5, 0, "")],
                                  "tableetenduetemporelle" :[(0, 1, "")],
                                  "tabledatepubdata": [(0, 5, "")],
                                  "tablemotsclefso" : [(2, 1, 0), (2, -1, (0,0))],
                                  "tablemotsclefsf" : [(-1, 0, ""), (6, 0, 0), (-1, 0, ""), (0, 5, ""), (2, 2, 0)],
                                  "tablespecifications" : [(-1, 0, ""), (0, 5, ""), (2, 2, 0), (2, 3, 0)],
                                  "tableechelles" : [(4, 0, -1)],
                                  "tabledico" : [(-1, 0, ""), (-1, 0, ""), (-1, 0, ""), (-1, 0, ""), (-1, 0, ""), (-1, 0, "")]
                                  }


    
    self.changeLang(False)

  def objectName(self): return self._objectName
  def changeLang(self, toReload):
        overrideLocale = QSettings().value("locale/overrideFlag", False)
        localeFullName = QLocale.system().name() if not overrideLocale else QSettings().value("locale/userLocale", "")
        self.localePath = "%s/i18n/qsphere_%s.qm" % (dirname(__file__), self.dicoLangs[self.indexLang])
        self.listOfRules, self.dictListOfRules = loadData(self, "file:/ressources/roles_%s.csv" % (self.dicoLangs[self.indexLang]), "roles")

        self.typeData = getListTypeRessources(self) 

        if QFileInfo(self.localePath).exists():
             self.translator.load(self.localePath)
             QCoreApplication.installTranslator(self.translator)

        if toReload : self.translate()

        if self.mainWindow : self.mainWindow.translate(True)

        if self.webNavigator : pass
        if self.dlgSgbdr : pass
        if self.dialogSearchFiles : pass
        if self.dialogXmlEditor : pass


  def initGui(self):
    self.menu = QMenu("QSphere")
    self.menu.setObjectName("mQsphereMenu")
    self.menu.setIcon(QIcon(getThemeIcon("qsphere.png")))

    getOptions(self)

    self.barInfo = self.iface.messageBar()
    self.barInfo.setSizePolicy( QSizePolicy.Minimum, QSizePolicy.Fixed )
    self.barInfo.setGeometry(0, 0, self.iface.mapCanvas().width()-10, 60)

    try: self.QgisVersion =  Qgis.QGIS_VERSION_INT 
    except: self.QgisVersion = Qgis.qgisVersion[0]

    if self.toolBar == None :
      if self.QgisVersion < 20400 : self.toolBar = self.iface.addToolBar("QSphere");
      else : self.toolBar = QToolBar("QSphere"); self.iface.addToolBar(self.toolBar);
      self.toolBar.setObjectName("mQsphereToolBar")   

    self.mainWindowAction = myAction("mActionMainWindow", "", getThemeIcon("qspheremainui.png"), self.iface.mainWindow())
    self.mainDatabase = myAction("mActionMainDatabase", "", getThemeIcon("layers-database.png"), self.iface.mainWindow())
    self.mainNavigator = myAction("mActionMainNavigator", "", getThemeIcon("navigatorweb.png"), self.iface.mainWindow())
    self.mainToolSearchFiles = myAction("mActionToolSearchFiles", "", getThemeIcon("toolssearchfiles.png"), self.iface.mainWindow())
    self.mainXmlEditor = myAction("mActionXmlEditor", "", getThemeIcon("editxml.png"), self.iface.mainWindow())
    self.mainContacts = myAction("mActionContacts", "", getThemeIcon("contact.png"), self.iface.mainWindow())
    self.options = myAction("mActionOptions", "", getThemeIcon("qsphereoptions.png"), self.iface.mainWindow())
    self.about = myAction("mActionAbout", "", getThemeIcon("about.png"), self.iface.mainWindow())

    icon = getThemeIcon("tools.png")
    self.actionTools = myAction("mActionTools", "", icon, self.iface.mainWindow())

    self.menuTools = QMenu("QSphereTools")
    self.menuTools.setIcon(QIcon(icon))


    self.translate()

    self.menuTools.addAction(self.mainDatabase)
    self.menuTools.addAction(self.mainNavigator)
    self.menuTools.addAction(self.mainXmlEditor)    
    self.menuTools.addSeparator()
    self.menuTools.addAction(self.mainToolSearchFiles)
    self.menuTools.addAction(self.mainContacts)
    self.menuTools.addSeparator()
    self.menuTools.addAction(self.options)


    self.menu.addAction(self.mainWindowAction)
    self.menu.addSeparator()
    self.menu.addMenu(self.menuTools)
    self.menu.addSeparator()    
    self.menu.addAction(self.about)

    menuBar = self.iface.mainWindow().menuBar()
    menu = menuBar
    for child in menuBar.children():
        if child.objectName()== "mPluginMenu" : menu =  child; break;
    menu.addMenu(self.menu)
    

    self.toolBar.addAction(self.mainWindowAction)

    self.toolBar.addSeparator()

    tools = QMenu(menuBar)
    tools.addAction(self.mainDatabase)
    tools.addAction(self.mainNavigator)
    tools.addAction(self.mainXmlEditor)
    tools.addSeparator()
    tools.addAction(self.mainToolSearchFiles)
    tools.addAction(self.mainContacts)
    tools.addSeparator()
    tools.addAction(self.options)

    self.actionTools.setMenu(tools)

    self.toolBar.addAction(self.actionTools)
    self.toolBar.addSeparator()
    self.toolBar.addAction(self.about)

    self.mainWindowAction.triggered.connect(self.getMainWindow)
    self.mainDatabase.triggered.connect(self.getUiDataBase)
    self.mainNavigator.triggered.connect(self.getWebViewer)
    self.mainToolSearchFiles.triggered.connect(self.getToolSearchFiles)
    self.mainXmlEditor.triggered.connect(self.getXmlEditor)
    self.mainContacts.triggered.connect(self.getContacts)
    self.options.triggered.connect(self.getUiOptions)
    self.about.triggered.connect(self.getAbout)


  def translate(self):
      self.menuTools.setTitle("%s ..." % QApplication.translate("QSphere","Tools"))
      
      self.mainWindowAction.setText(QApplication.translate("QSphere","Open MainWindow"))
      self.actionTools.setText("%s ..." % QApplication.translate("QSphere","Tools"))
      self.mainNavigator.setText(QApplication.translate("QSphere","Web navigator ..."))
      self.mainDatabase.setText("%s ..." % QApplication.translate("QSphere","Databases"))

      self.mainXmlEditor.setText("%s ..." % QApplication.translate("QSphere","Edit XML"))
      self.mainToolSearchFiles.setText("%s ..." % QApplication.translate("QSphere","Open search files tool"))
      self.mainContacts.setText("%s" % QApplication.translate("QSphere","Contacts manager ..."))
      self.options.setText(QApplication.translate("QSphere","Call the options GUI ..."))
      self.about.setText(QApplication.translate("QSphere","About ..."))

      self.listOfConformities = loadDefautValue(self, "file:/ressources/conformity_default_%s.csv"  % (self.dicoLangs[self.indexLang]))
      self.listOfRules, self.dictListOfRules = loadData(self, "file:/ressources/roles_%s.csv" % (self.dicoLangs[self.indexLang]), "roles") 
      self.listOfThesaurus , self.dateListOfThesaurus = loadData(self, "file:/ressources/thesaurus.csv", "thesaurus")
      
      self.listTypeDates = [QApplication.translate("QSphere","Date of creation"),
                            QApplication.translate("QSphere","Date of last revision"),
                            QApplication.translate("QSphere","Date of publication"),]
      self.listDegres = [QApplication.translate("QSphere","Conformant"),
                         QApplication.translate("QSphere","Not conformant"),
                         QApplication.translate("QSphere","Not evaluated")]

      self.tablesWidget = {"tablelocalisator" : ((190, 200),
                              (QApplication.translate("QSphere", self.libelles[6][2]),
                               QApplication.translate("QSphere", self.libelles[7][2])) ),
                           "tableformats" : ((100, 40, 80),
                              (QApplication.translate("QSphere", self.libelles[11][2]),
                               QApplication.translate("QSphere", self.libelles[12][2]),
                               QApplication.translate("QSphere", self.libelles[13][2])) ),
                           "tableemprises" : ((140, 140, 140, 140, 80),
                              (QApplication.translate("QSphere", self.libelles[32][2]),
                               QApplication.translate("QSphere", self.libelles[33][2]),
                               QApplication.translate("QSphere", self.libelles[34][2]),
                               QApplication.translate("QSphere", self.libelles[35][2]),
                               QApplication.translate("QSphere", self.libelles[36][2])) ),
                           "tablescr" : ((120, 40),
                              (QApplication.translate("QSphere", self.libelles[39][2]),
                               QApplication.translate("QSphere", self.libelles[40][2])) ),
                           "tableetenduetemporelle" : ((140),
                              (QApplication.translate("QSphere", self.libelles[44][2]),
                               QApplication.translate("QSphere", self.libelles[45][2])) ),
                           "tabledatepubdata" : ( (100),
                              (QApplication.translate("QSphere", self.libelles[47][2]),
                               QApplication.translate("QSphere", self.libelles[48][2])) ),
                            "tablemotsclefso" : ((200, 300),
                              (QApplication.translate("QSphere", self.libelles[20][2]),
                               QApplication.translate("QSphere", self.libelles[21][2])) ),
                           "tablemotsclefsf" : ((150, 25, 140, 80, 90),
                              (QApplication.translate("QSphere", self.libelles[23][2]),
                               QApplication.translate("QSphere", self.libelles[24][2]),
                               QApplication.translate("QSphere", self.libelles[25][2]),
                               QApplication.translate("QSphere", self.libelles[26][2]),
                               QApplication.translate("QSphere", self.libelles[27][2])) ),
                            "tablespecifications" : ((240, 80, 100, 100),
                              (QApplication.translate("QSphere", self.libelles[61][2]),
                               QApplication.translate("QSphere", self.libelles[62][2]),
                               QApplication.translate("QSphere", self.libelles[63][2]),
                               QApplication.translate("QSphere", self.libelles[64][2]))),
                           "tableroles" : ((100, 80, 80, 80, 60, 80, 80, 80, 80, 80),
                              (QApplication.translate("QSphere", self.libelles[72][2]),
                               QApplication.translate("QSphere", self.libelles[73][2]),
                               QApplication.translate("QSphere", self.libelles[74][2]),
                               QApplication.translate("QSphere", self.libelles[75][2]),
                               QApplication.translate("QSphere", self.libelles[76][2]),
                               QApplication.translate("QSphere", self.libelles[77][2]),
                               QApplication.translate("QSphere", self.libelles[78][2]),
                               QApplication.translate("QSphere", self.libelles[79][2]),
                               QApplication.translate("QSphere", self.libelles[80][2]),
                               QApplication.translate("QSphere", self.libelles[81][2])) ),
                           "tabledico" : ((50, 150, 70, 60, 50, 150),
                              (QApplication.translate("QSphere", self.libelles[92][2]),
                               QApplication.translate("QSphere", self.libelles[93][2]),
                               QApplication.translate("QSphere", self.libelles[94][2]),
                               QApplication.translate("QSphere", self.libelles[95][2]),
                               QApplication.translate("QSphere", self.libelles[96][2]),
                               QApplication.translate("QSphere", self.libelles[97][2])) ),
                           "tableechelles" : ((70, ),
                               ("",""))}
       
  def getAbout(self): self.dialogAbout = dlgAbout(self); self.dialogAbout.exec_();

  def getContacts(self): self.dialogContacts = dlgContacts(self, False); self.dialogContacts.exec_();

  def getMainWindow(self):
      if not self.mainWindow : self.mainWindow = myMdiWindow(self.qspherePlugin)
      self.mainWindow.setWindowState(Qt.WindowMaximized)
      self.mainWindow.show()

  def getToolSearchFiles(self):
      if self.dialogSearchFiles == None :
              self.dialogSearchFiles = toolSearchFilesDialog(self)
              windowIcon(self.dialogSearchFiles, "toolssearchfiles.png")
      self.dialogSearchFiles.show()
      
  def getUiDataBase(self):
      title = "%s : %s" % (QApplication.translate("QSphere", "Databases"), QApplication.translate("QSphere", "Resources").lower())
      if not self.dlgSgbdr : self.dlgSgbdr = dlgSgbdr(self.iface, self, 1, title)
      self.dlgSgbdr.show(); self.dlgSgbdr.dockSql.show();
      if self.mainWindow : self.mainWindow.dlgSgbdr = self.dlgSgbdr

  def getUiOptions(self): self.dialogOptions = dlgOptions(self.iface, self); self.dialogOptions.exec_();
  
  def getWebViewer(self, url):
      if not self.webNavigator : self.webNavigator = loadDlgNavigator(self, self.iface, "#blank", False, [], self.localLangage, self, None, "navigatorweb.png", False)
      self.webNavigator.setWindowState(Qt.WindowMaximized)
      self.webNavigator.show()
      self.webNavigator.activateWindow()
      
  def getXmlEditor(self):
      if self.dialogXmlEditor == None : 
                self.dialogXmlEditor = xmlEditor(None, "", self.qspherePlugin)
                windowIcon(self.dialogXmlEditor, "editxml.png")
                windowProperties(self.dialogXmlEditor, self)
                self.dialogXmlEditor.setWindowTitle(self.dialogXmlEditor.rootWindowTitle)
                self.dialogXmlEditor.setModal(False)
      self.dialogXmlEditor.show()
      
  def unload(self):
      menuBar = self.iface.mainWindow().menuBar()
      child = None
      for child in menuBar.children():
          if child.objectName()== "mPluginMenu" :
             break
      if child : child.removeAction(self.menu.menuAction())
      self.toolBar.parentWidget().removeToolBar(self.toolBar)

