# -*- coding:utf-8 -*-
from qgis.core import (Qgis, QgsCoordinateReferenceSystem, QgsCoordinateTransform, QgsPoint, QgsProject, QgsWkbTypes)
from qgis.gui import (QgsCollapsibleGroupBox, QgsDateTimeEdit, QgsMessageBar)

from psycopg2 import (connect, pool as psycopg2pool, OperationalError)
from psycopg2.extensions import (ISOLATION_LEVEL_AUTOCOMMIT, POLL_OK, POLL_WRITE, POLL_READ)

from qgis.PyQt.QtCore import (QDate, QDateTime, QLocale, QMetaObject, QObject, QRegExp, QSizeF, QUrl, Qt)
from qgis.PyQt.QtWidgets import (QAbstractItemView, QAction, QApplication, QCalendarWidget, QCheckBox, QComboBox, QCommandLinkButton,
                                 QDialog, QFileDialog, QGroupBox, QHBoxLayout, QLabel, QLineEdit, QListView, QMenu, QPushButton,
                                 QRadioButton, QScrollArea, QSizePolicy, QTabBar, QTabWidget, QTableView, QTableWidget, QTableWidgetItem,
                                 QTextEdit, QToolBar, QToolButton, QWidget)
from qgis.PyQt.QtGui import (QBrush, QColor, QFont, QIcon, QImage, QKeySequence, QLinearGradient, QMovie,
                             QPainter, QPixmap, QStandardItem, QStandardItemModel, QTextDocument, QTextOption)

from qgis.PyQt.QtPrintSupport import (QPrintPreviewDialog, QPrinter)

from configparser import ConfigParser
from os.path import (abspath, basename, dirname, exists, join, splitext)


from qsphere.importexport import (closeFile, saveXmlCatalog, saveXml, saveQmd, loadDataFromQsp, saveQsp, writeInFileWithoutEncoding) 
from qsphere.qsphere_tools import (addLineWidget, analyzeInfosObj, badNetwork, changeButtonIcon, changeSizeCols, cleanHtml, colorizeWords, 
                                   isFloatNumber, isIntNumber, countItems, correctPath, countCaractere, dateToString, dicoHasKey, fileRessourceExist, fixeLabelsFileDialog,
                                   fileNameWithExtension, getComboLayer, getComboLayerIndex, getEpsg, getExtension, getHelp, getIndex,
                                   getListLanguesIndex, getListLayers, getListTemporalSystem, getListTypeRessources, getListXslt, getOptions,
                                   getRandowId, getThemeIcon, getTypeLayer, getWidget,initSizeCols, isCorrectLayer, isFloatNumber, isMailUrl,
                                   loadFile, multiGetRequest, msgBadFileXml, notEmptyElts, notEmptyList, patternZipPostalCode, propertiesDoubleSpinBox,
                                   reinitDate, sendMsg, setTextWidget, sortDic, testXmlData, toolTipCalendar, windowIcon, windowProperties)
from qsphere.qsphere_objmaker import (dlgAuthCswt, dlgCrs, dlgDataFormat, dlgTopology, myButton, myCheckBox, myComboBox,  myLabel, movieCtrl, myPushButton,
                                      myPushButtonFunny, myScintilla, MySimpleWidgetLineEdit, MySimpleWidgetLineEditST, mySpinBox, myTableWidget, myTextEdit,
                                      MyWidgetLineEdit, printerSphere)

from qsphere.xmlisoparser import xmlIsoParser

from datetime import datetime
from PyQt5.QtXmlPatterns import QXmlQuery
 
from qgis.PyQt.Qsci import (QsciLexerXML, QsciScintilla, QsciScintillaBase)
from PyQt5.QtWebKitWidgets import (QWebInspector, QWebPage, QWebView)
from PyQt5.QtWebKit import QWebSettings

from webbrowser import open as webbrowseropen
from urllib.parse import (quote, unquote, unquote_plus)


class uiDlgMetadata(object):
    def __init__(self):
        self.connexion = None
    
    def setupUi(self):
        self.blocProgressBar = self.parent.blocProgressBar 
        self.progressBar = self.parent.progressBar
        self.blocProgressBarText = self.parent.blocProgressBarText 
        self.blocProgressBarButton = self.parent.blocProgressBarButton 
        
        windowIcon(self, "qspheremainui.png")
        sizeW, sizeHeight, self.minwidth, self.minheight = 1100, 680, 800, 680
        self.minWidth, self.minHeight = sizeW, sizeHeight
        self.InitDir, self.extension = "", "qsp"
        self.listImportCategories = []
        self.badNetwork = None 
        
        self.libelles = self.qspherePlugin.libelles
        self.paramsLineWidget = self.qspherePlugin.paramsLineWidget

        self.tableListLayers, self.comboLayersCurrentLayerId = {}, 0
        self.rootWindowTitle = "%s :" % QApplication.translate("QSphere","metadata")

        self.translateTablesWidget()
        getOptions(self)
        
        self.InitName = ""
        self.typeData = self.parent.typeData 
        self.listUnitsMesure = ["m", "km", "foot", "inch", "mile", "yard"]
        self.restore_tabledico = None
        self.ToolTipLangue = ""

        self.setWindowFlags(Qt.Window | Qt.FramelessWindowHint  | Qt.WindowCloseButtonHint)
        self.setAttribute(Qt.WA_NoSystemBackground)
        self.setAttribute(Qt.WA_TranslucentBackground)
        self.setAttribute(Qt.WA_PaintOnScreen)

        self.originalWidth = self.minwidth
        self.setMinimumSize(self.minwidth, self.minheight)
        self.resize(self.minwidth, self.minheight)
        
        self.tabWidget = QScrollArea(self) 
        self.tabWidget.setObjectName("tabWidget")
        self.tabWidget.setVerticalScrollBarPolicy(Qt.ScrollBarAlwaysOn)

        self.theGroup = QGroupBox(self)

        self.editor = myScintilla(self,"xmlWidget", True, (10, 10, 780, 610)) 
        
        self.webWidget = QWebView(self)
        self.webWidget.setGeometry(10, 10, 780, 610) 
        self.webWidget.setObjectName("viewHtml")
        self.webWidget.settings().setAttribute(QWebSettings.JavaEnabled, True)
        self.webWidget.settings().setAttribute(QWebSettings.PluginsEnabled, True)
        self.webWidget.settings().setAttribute(QWebSettings.AutoLoadImages, True)
        self.webWidget.settings().setAttribute(QWebSettings.LocalContentCanAccessRemoteUrls, True)
        self.webWidget.settings().setAttribute(QWebSettings.JavascriptEnabled, True)
        self.webWidget.settings().setAttribute(QWebSettings.DeveloperExtrasEnabled, True)
        self.webWidget.settings().setAttribute(QWebSettings.LinksIncludedInFocusChain, True)
        self.webWidget.setRenderHints(QPainter.Antialiasing | QPainter.SmoothPixmapTransform)
        self.webWidget.page().setLinkDelegationPolicy(QWebPage.DelegateAllLinks)
        self.webWidget.page().setForwardUnsupportedContent(True)

        self.webWidget.page().linkClicked.connect(self.getLinkedUrl)
        self.webWidget.setContextMenuPolicy(Qt.CustomContextMenu)
        self.webWidget.customContextMenuRequested.connect(self.context_menu_requested)

        myPathIconvRightArrow = getThemeIcon("rightarrow.png")
        myPathIconvLeftArrow = getThemeIcon("leftarrow.png")

        labelWidth, labelHeight = 60, 25
        self.sizeWidth, self.sizeHeight = 200, 25
        self.latitudeExtent, self.longitudeExtent = 180, 360
       
        i = 1
        self.listObj = []

        posH, indexButton, self.mddcollapsibleGroupBox, self.libCollapsibleGroupBox, self.heights =  5, 0, [], [], []
        
        for key in (self.libelles.keys()):
            if self.libelles[key][1]== QWidget :

                 self.progressBar.setValue(int(100 * i/9))               
 
                 collapsibleGroupBox = QgsCollapsibleGroupBox(self.theGroup)
                 collapsibleGroupBox.setStyleSheet("QgsCollapsibleGroupBox{background-color:qlineargradient(x1: 0, y1: 0, x2: 0.5, y2: 0.5, stop: 0 %s, stop: 1 %s); border:1px;}" % (self.startColor, self.stopColor))
                                                
                 collapsibleGroupBox.setObjectName(self.libelles[key][0])
                 collapsibleGroupBox.setTitle("%s : " % (QApplication.translate("QSphere",self.libelles[key][2])))
                 
                 posH+= 15

                 self.libCollapsibleGroupBox.append(self.libelles[key][2])
                 self.mddcollapsibleGroupBox.append(collapsibleGroupBox)
                 
                 setattr(self, self.libelles[key][0], collapsibleGroupBox)

                 collapsibleGroupBox.collapsedStateChanged.connect(self.changePosGroups)

                 makeTabWidgets(self, collapsibleGroupBox, self.libelles[key][4], i, labelWidth, labelHeight, self.sizeWidth, self.sizeHeight)
                 i+= 1


        kingSize = 0
        for elt in self.heights : kingSize = max(kingSize, elt)

        for mddcollapsibleGroupBox in self.mddcollapsibleGroupBox:
            mddcollapsibleGroupBox.setMinimumHeight(kingSize)
            mddcollapsibleGroupBox.setMaximumHeight(kingSize)
        
        self.tabWidget.setWidget(self.theGroup)

        self.barInfo = QgsMessageBar(self)
        self.barInfo.setSizePolicy( QSizePolicy.Minimum, QSizePolicy.Fixed )
        self.barInfo.setGeometry(0, 0, sizeW-10, 90)

        self.changeMode = self.parent.changeMode 
        self.histoIDChangeMode = 0
        self.childswindows = []

        self.barInfo = self.parent.barInfo 
        self.movieCtrl = self.parent.movieCtrl

        self.ToolBarActions = QToolBar(self)
        self.ToolBarActions.setObjectName("ToolBarActions")
        self.ToolBarActions.setAllowedAreas(Qt.TopToolBarArea | Qt.BottomToolBarArea)
        self.ToolBarActions.setGeometry(5, sizeHeight-68, self.tabWidget.width(), 68)

        self.lblComboLayers = QLabel()
        self.lblComboLayers.setGeometry(10, 10, 150, labelHeight)
        self.lblComboLayers.setAlignment(Qt.AlignRight)
        self.lblComboLayers.setObjectName("lblComboLayers")

        self.model = QStandardItemModel()
        self.view = QTableView()
        self.view.setModel( self.model )
        self.view.horizontalHeader().setVisible(False)
        self.view.verticalHeader().setVisible(False)
        self.view.setColumnHidden(1, True)
        self.view.verticalHeader().setDefaultSectionSize(20)
        self.view.setStyleSheet("QTableView {min-width: 400px; max-width: 500px; selection-background-color: qlineargradient(x1: 0, y1: 0, x2: 0.5, y2: 0.5, stop: 0 #FFFFFF, stop: 1 #000000);}")
        self.view.setShowGrid(False)

        self.ComboLayers = QComboBox()
        self.ComboLayers.setObjectName("ComboLayers")
        self.ComboLayers.setFixedSize(150, labelHeight)
        self.ComboLayers.setGeometry(170, sizeHeight-30, 150, labelHeight)
        self.ComboLayers.setModel(self.model)
        self.ComboLayers.setView(self.view)

        self.listXslt = getListXslt(self)
        self.lblPrimaryXslt = QLabel()
        self.primaryXslt = QComboBox()
        self.primaryXslt.addItems(self.listXslt)

        test, value = fileRessourceExist(self, "xml/xsl/transformation_%s.xsl" % (self.localLangage))

        if value != "" : value = basename(value)
        index = self.listXslt.index(value) if value in (self.listXslt) else 0
        self.primaryXslt.setCurrentIndex(index)

        self.findprevious = myPushButton()
        self.findprevious.initPushButton(48, 24, 5, 5, "findprevious", "", QApplication.translate("QSphere", "Find previous"), True, getThemeIcon("leftarrow.png"), 48, 24, True)
        self.findprevious.setShortcut(QKeySequence("F2"))

        self.searchbutton = myPushButton()
        self.searchbutton.initPushButton(24, 24, 5, 5, "searchbutton", "", QApplication.translate("QSphere", "Search word"), True, getThemeIcon("find.png"), 24, 24, True)
        self.searchbutton.setShortcut(QKeySequence("Ctrl+F"))

        self.findnext = myPushButton()
        self.findnext.initPushButton(48, 24, 5, 5, "findnext", "", QApplication.translate("QSphere", "Find next"), True, getThemeIcon("rightarrow.png"), 48, 24, True)
        self.findnext.setShortcut(QKeySequence("F3"))

        self.actionLblComboLayers = self.ToolBarActions.addWidget(self.lblComboLayers)
        self.actionComboLayers = self.ToolBarActions.addWidget(self.ComboLayers)
        
        self.actionLblPrimaryXSLT = self.ToolBarActions.addWidget(self.lblPrimaryXslt)
        self.actionPrimaryXSLT = self.ToolBarActions.addWidget(self.primaryXslt)
        self.actionfindprevious = self.ToolBarActions.addWidget(self.findprevious)
        self.actionsearchbutton = self.ToolBarActions.addWidget(self.searchbutton)
        self.actionfindnext = self.ToolBarActions.addWidget(self.findnext)

        self.ToolBarActions.addSeparator()
 
        self.doChangeMode(0)

        self.ComboLayers.currentIndexChanged.connect(self.getMetaData)
        self.primaryXslt.currentIndexChanged.connect(self.passChangeMode)
        self.findprevious.clicked.connect(self.findPrevious)
        self.searchbutton.clicked.connect(self.findText)
        self.findnext.clicked.connect(self.findNext)
        self.actionfindprevious.setEnabled(False)
        self.actionfindnext.setEnabled(False)
        self.editor.marginClicked.connect(self.on_margin_clicked)

        self.iface.mapCanvas().layersChanged.connect(self.UpdateListComboLayers)
        try : self.iface.mapCanvas().currentLayerChanged.connect(self.UpdateListComboLayers)
        except : pass
        self.iface.newProjectCreated.connect(self.UpdateListComboLayers)
        
        QMetaObject.connectSlotsByName(self)

        self.translateFixedWidgets()
        self.UpdateListComboLayers()
        self.iface.newProjectCreated.connect(self.UpdateListComboLayers)
        self.iface.projectRead.connect(self.UpdateListComboLayers)
        
        self.countLangues()
        self.IsVisibleViewXMLButton()

        obj = self.findChild(myTableWidget, "tableemprises")
        if obj != None : obj.selectRow(0)        
        self.drawExtent()
      

    def changePosGroups(self):
         posH, newHeight = 0, 0
         for mddcollapsibleGroupBox in self.mddcollapsibleGroupBox:
             mddcollapsibleGroupBox.setMaximumWidth(self.width()-50)
             mddcollapsibleGroupBox.setMinimumWidth(self.width()-50)                         
             mddcollapsibleGroupBox.setGeometry(5, posH, self.width()-50, mddcollapsibleGroupBox.height())
             posH+= mddcollapsibleGroupBox.height() if not mddcollapsibleGroupBox.isCollapsed() else 15
             newHeight+= 15 + mddcollapsibleGroupBox.height() if not mddcollapsibleGroupBox.isCollapsed() else 15
         self.tabWidget.setGeometry(10, 10, self.width()-20, self.height()-100)
         self.theGroup.setGeometry(10, 10, self.width()-20, max(newHeight+20, self.height()+20))
      

    def resizeEvent(self, event):
         self.changePosGroups()
         self.ToolBarActions.setGeometry(10,  self.height()-80, self.width(), 50)
         self.editor.setGeometry(10, 10, self.width()-20, self.height()-100)
         self.webWidget.setGeometry(10, 10, self.width()-20,self.height()-100)

         for listItems in self.listObj:
             Obj = listItems[0]
             if listItems[1]== []:
                Obj.setGeometry(Obj.x(), Obj.y(), self.tabWidget.width()-Obj.x()-60, Obj.height())
                if type(Obj) == QTableView :
                    nbCols = Obj.model().columnCount()
                    newSizeCol = float(Obj.width()/nbCols)-10
                    for i in range(nbCols): Obj.setColumnWidth(i, newSizeCol)
                         
             elif type(listItems[1]) == tuple :
                    supObj = listItems[1][0]
                    supObj.setGeometry(supObj.x(), supObj.y(), self.tabWidget.width()-supObj.x()-20, supObj.height())
                    Obj.setGeometry(Obj.x(), Obj.y(), supObj.width()-Obj.x()-20, Obj.height())
                    if type(Obj) == QTableView :
                       nbCols = Obj.model().columnCount()
                       newSizeCol = float(supObj.width()/nbCols)-10
                       for i in range(nbCols): Obj.setColumnWidth(i, newSizeCol)
                
             else :
                Obj.setGeometry(Obj.x(), Obj.y(), self.tabWidget.width()-Obj.x()-60, Obj.height())
                if type(Obj) == myTableWidget :
                  if dicoHasKey(self.tablesWidget, Obj.objectName()):
                     self.dims = self.tablesWidget[Obj.objectName()][0]
                     prorata = (self.width()-70)/float(Obj.originalWidth)
                     changeSizeCols(self, Obj, Obj.dims, prorata)
                    
                for button in listItems[1]:
                    delta = 0 if button.width() == 24 else 15
                    button.setGeometry(self.tabWidget.width()-55-delta, button.y(), button.width(), button.height())


    def on_margin_clicked(self, nmargin, nline, modifiers):
        if self.editor.markersAtLine(nline) != 0: self.editor.markerDelete(nline, self.arrowMarkerNum)
        else: self.editor.markerAdd(nline, self.arrowMarkerNum)
    
    def isBfind(self, bfind):          
        if not bfind:
           title = QApplication.translate("QSphere", "Information")
           msg = QApplication.translate("QSphere","Not found")
           sendMsg(self, title , "%s :<br><u><i>%s</i></u>" % (msg, self.editor.find_text), Qgis.Info, self.duration_info)
         
    def enableFindPrevNextButton(self):
        self.findprevious.setEnabled(not self.editor.find_text=="")
        self.findnext.setEnabled(not self.editor.find_text=="")
    
    def findText(self):
        ret = self.editor.findDialog.exec_()
        self.editor.findDialog.lineEdit.setFocus()
        if self.editor.findDialog.ret > 0 :
          self.editor.find_text = self.editor.findDialog.getFindText()
          self.enableFindPrevNextButton()

    def findNext(self):
        line, index = self.editor.getCursorPosition()
        bfind = self.editor.findFirst(self.editor.find_text, True, self.editor.findDialog.isCaseSensitive(), self.editor.findDialog.isWholeWord(), True, True, line, index)
        self.isBfind(bfind)
        self.enableFindPrevNextButton()

    def findPrevious(self):
        line, index = self.editor.getCursorPosition()
        index -= len(self.editor.find_text)
        bfind = self.editor.findFirst(self.editor.find_text, True, self.editor.findDialog.isCaseSensitive(), self.editor.findDialog.isWholeWord(), True, False, line, index)
        self.isBfind(bfind)
        self.enableFindPrevNextButton()


    def printGraphicalMetadata(self):
                editor = QTextDocument()
                editor.clear()
                editor.setPageSize(QSizeF(printer.pageRect().size()))

                states = []
                for mddcollapsibleGroupBox in self.mddcollapsibleGroupBox:
                     states.append(mddcollapsibleGroupBox.isCollapsed())
                     mddcollapsibleGroupBox.setCollapsed(False)
                kingSize = 0
                for elt in self.heights : kingSize = max(kingSize, elt)
                
                antHeight = self.theGroup.height()
                sAntHeight = self.height()
                kingSize = kingSize* len(self.mddcollapsibleGroupBox)
                self.theGroup.setGeometry(self.theGroup.x(), self.theGroup.y(), self.theGroup.width(), kingSize)
                self.setGeometry(self.x(), self.y(), self.width(), kingSize)
                pix = self.grab(self.rect())

                i = 0
                for mddcollapsibleGroupBox in self.mddcollapsibleGroupBox:
                     mddcollapsibleGroupBox.setCollapsed(states[i])
                     i+= 1

                self.theGroup.setGeometry(self.theGroup.x(), self.theGroup.y(), self.theGroup.width(), antHeight)
                self.setGeometry(self.x(), self.y(), self.width(), sAntHeight)


                
    def printMetadata(self):
        idbutton = self.changeMode.id(self.changeMode.checkedButton())

        if idbutton in (0, 1) :
            if idbutton == 0 :
                editor = QWebView()
                editor.setHtml(saveQsp(self, "", True)) 
            else :     
                editor = QWebView()
                editor.setHtml(colorizeWords(self.editor.text()))

        try : 
            fileName = self.windowTitle().replace(self.rootWindowTitle, "") if self.windowTitle() != self.rootWindowTitle and self.windowTitle().find("~")== -1 else self.windowTitle()
        except : fileName = self.rootWindowTitle()

        if idbutton in (0, 1) : obj = editor
        elif idbutton == 2 : obj = self.webWidget
        else : return

        orientation = QPrinter.Landscape if idbutton == 1 else  QPrinter.Portrait
        title = "%s : %s" % (QApplication.translate("QSphere", "Print current file"), fileName)

        dlgPrint = printerSphere(self, obj, title, orientation)

        

    def context_menu_requested(self, point):
        self.imageurl = None
        context_menu = QMenu()
        page = self.webWidget.page()
        frame = page.currentFrame()
        if unicode(self.webWidget.selectedText()):
            menuIcon = getThemeIcon("copier.png")
            text = (QApplication.translate("QSphere", "Copy"))
            self.copy = context_menu.addAction(QIcon(menuIcon), text)
            self.toCopy = unicode(self.webWidget.selectedText())
            self.copy.triggered.connect(self.makeToCopy)

        hit_test = frame.hitTestContent(point)
        
        if unicode(hit_test.linkUrl().toString()):
            
            menuIcon = getThemeIcon("copierlink.png")
            text = (QApplication.translate("QSphere", "Copy link"))
            self.copyURL = context_menu.addAction(QIcon(menuIcon), text)
            self.toCopyUrl = unicode(hit_test.linkUrl().toString())
            self.copyURL.triggered.connect(self.makeToCopyUrl)

            context_menu.addSeparator()

            text = QApplication.translate("QSphere", "Go url")
            self.openUrl = context_menu.addAction(QIcon(menuIcon), text)
            self.toOpenUrl = unicode(hit_test.linkUrl().toString())
            self.openUrl.triggered.connect(self.makeOpenUrl)

            if not isMailUrl(self.toOpenUrl) :

                text = QApplication.translate("QSphere", "Open link in my default navigator")
                self.openUrlExt = context_menu.addAction(QIcon(menuIcon), text)
                self.openUrlExt.triggered.connect(self.makeOpenUrl)


        context_menu.addSeparator()
           
        menuIcon = getThemeIcon("selectall.png")
        text = (QApplication.translate("QSphere", "Select all"))
        self.selall = QAction(QIcon(menuIcon), text, self)
        context_menu.addAction(self.selall)
        self.selall.triggered.connect(self.selectAll)

        context_menu.exec_(self.webWidget.mapToGlobal(point))     


    def getLinkedUrl(self, url):
        if isMailUrl(url.toString()) :
            from os import startfile
            startfile(url.toString())
        else :    
            try : self.parent.loadNavigator(url.toString())
            except : pass
        
    def makeOpenUrl(self):
        if isMailUrl(self.toOpenUrl) :
            from os import startfile
            startfile(self.toOpenUrl)
        else :    
            if self.sender()== self.openUrlExt :
                try : webbrowseropen(self.toOpenUrl)
                except :
                    title = QApplication.translate("QSphere","Information")
                    msg = QApplication.translate("QSphere","Try to open Web navigator for https protocol ...")
                    sendMsg(self, title , msg, Qgis.Warning, 3)
            elif self.sender()== self.openUrl :
                self.parent.loadNavigator(self.toOpenUrl)
   
    def makeToCopy(self): QApplication.clipboard().setText(self.toCopy)
    def makeToCopyUrl(self): QApplication.clipboard().setText(self.toCopyUrl)
    def makeToPaste(self): self.webWidget.page().triggerAction(QWebPage.Paste)
    def selectAll(self): self.webWidget.page().triggerAction(QWebPage.SelectAll)

    def passChangeMode(self):
        idbutton = self.changeMode.id(self.changeMode.checkedButton())
        self.doChangeMode(idbutton) 

    def analyzeData(self):
   
        dataValues = []

        for mddcollapsibleGroupBox in self.mddcollapsibleGroupBox:

            if mddcollapsibleGroupBox.objectName() != "tab9" :
                nbProperties, nbFill = 0, 0
                
                collapsGroupChildren = mddcollapsibleGroupBox.children()
                for j in range(len(collapsGroupChildren)):
                    objName = "%s" % (collapsGroupChildren[j].objectName())
                    if not type(collapsGroupChildren[j]) in (QCommandLinkButton, QLabel, myPushButton, myPushButtonFunny) and not objName in ("", "collapseButton") :

                       objClassName = collapsGroupChildren[j].metaObject().className()

                       if objClassName in ("QCalendarWidget", "QgsDateTimeEdit") :
                           if dateToString(self, collapsGroupChildren[j]) != "--" : nbFill+= 1 ; 
                           nbProperties+= 1
                       elif objClassName in ("QTextEdit", "MyTextEdit", "myTextEdit") :
                           if collapsGroupChildren[j].toPlainText() != "" : nbFill+= 1
                           nbProperties+= 1
                       elif objClassName in ("QLineEdit", "MySimpleWidgetLineEdit", "MyWidgetLineEdit") :
                           if collapsGroupChildren[j].text() != "" : nbFill+= 1
                           nbProperties+= 1
                       elif objClassName in ("QComboBox", "MyComboBox", "myComboBox") : pass
                       elif objClassName=="QGroupBox" :
                            groupBoxChildren = collapsGroupChildren[j].children()
                            for i in range(len(groupBoxChildren)):
                                child = groupBoxChildren[i]
                                childClassName = "%s" % (child.metaObject().className())
                                if childClassName == "QRadioButton" : pass
                                elif childClassName in ("QComboBox", "MyComboBox", "myComboBox") : pass
                                elif childClassName in ("QTableWidget", "MyTableWidget", "myTableWidget") :
                                     for row in range(child.rowCount()):
                                          widget = child.cellWidget(row, 0)
                                          if widget.value() != 0 : nbFill+= 1 ; 
                                          nbProperties+= 1
                                elif childClassName == "QTableView":
                                     for row in range(child.model().rowCount()):
                                         item = child.model().item(row, 0)
                                         if item.checkState()== Qt.Checked: nbFill+= 1; break;  
                                     nbProperties+= 1
                           
                       elif objClassName=="QTableView" :
                           for row in range(collapsGroupChildren[j].model().rowCount()):
                               item = collapsGroupChildren[j].model().item(row, 0)
                               if item.checkState()== Qt.Checked: nbFill+= 1; break;  
                           nbProperties+= 1 

                       
                       elif objClassName in ("QTableWidget", "MyTableWidget", "myTableWidget") :
                          for row in range(collapsGroupChildren[j].rowCount()):
                              for col in range(collapsGroupChildren[j].columnCount()):
                                  widget = collapsGroupChildren[j].cellWidget(row, col)
                                  if widget != None :
                                      if widget.metaObject().className() in ("QComboBox", "MyComboBox", "myComboBox") :
                                          if widget.isEditable() :
                                             if widget.currentText() != "" : nbFill+= 1
                                          else : nbFill+= 1
                                          nbProperties+= 1
                                      elif widget.metaObject().className() in ("QCheckBox", "MyCheckBox", "myCheckBox",
                                                                               "mySpinBox", "QPushButton", "myButton") : pass
                                      else :
                                          if widget.text() != "" : nbFill+= 1
                                          nbProperties+= 1
                                  else:
                                      cell = collapsGroupChildren[j].item(row, col)
                                      if cell != None :
                                         if collapsGroupChildren[j].item(row, col).text() != "" : nbFill+= 1
                                         nbProperties+= 1

                if nbProperties == 0 : nbProperties = 1
                dataValues.append(nbFill/nbProperties)

        return tuple(dataValues) 

            
    def doChangeMode(self, idbutton):
        container = (self.tabWidget, self.editor, self.webWidget)
        dims = len(container)

        self.actionLblComboLayers.setVisible(idbutton == 0)
        self.actionComboLayers.setVisible(idbutton == 0)        
        self.actionfindprevious.setVisible(idbutton == 1)
        self.actionsearchbutton.setVisible(idbutton == 1)
        self.actionfindnext.setVisible(idbutton == 1)        
        self.actionLblPrimaryXSLT.setVisible(idbutton == 2)
        self.actionPrimaryXSLT.setVisible(idbutton == 2)
        
        for i in range(dims):
            if i == idbutton : container[i].setVisible(True)
            else : container[i].setVisible(False)

        if self.editor.isVisible() :
           dataXml = saveXml(self, None)
           self.editor.setText(dataXml.decode("utf-8"))

        if self.webWidget.isVisible() :
           dataXml = saveXml(self, None)
           fileName = join(dirname(__file__),"xml/tempo.xml")
           
           inFile = open(fileName, "w")
           writeInFileWithoutEncoding(inFile, dataXml.decode("utf8"))
           closeFile(inFile)

           baseUrl = ""
           qry = QXmlQuery(QXmlQuery.XSLT20)
           test, XSL = fileRessourceExist(self, "xml/xsl/%s" % self.primaryXslt.currentText() ) 
           if XSL!= "" : baseUrl = QUrl.fromLocalFile(XSL)
           qry.setFocus(QUrl.fromLocalFile(fileName)) 
           qry.setQuery(QUrl.fromLocalFile(XSL))

           HTML = qry.evaluateToString() 

           if HTML is None :
              HTML = QApplication.translate("QSphere", "File parsing XSLT error !<br>Invalid XSLT or XML file !")
              HTML = "<div align='center'><img src='file:///%s'><br><br>%s</div>" % (getThemeIcon("parsing_error.gif"), HTML)
           else : HTML = cleanHtml(HTML)

           self.webWidget.setHtml(HTML, baseUrl)
           
               

    def clickHelp(self): getHelp(self)

    def translateTablesWidget(self):
        self.listOfConformities = self.qspherePlugin.listOfConformities
        self.listOfRules, self.dictListOfRules = self.qspherePlugin.listOfRules, self.qspherePlugin.dictListOfRules
        self.listOfThesaurus , self.dateListOfThesaurus = self.qspherePlugin.listOfThesaurus , self.qspherePlugin.dateListOfThesaurus
        self.listTypeDates = self.qspherePlugin.listTypeDates
        self.listDegres = self.qspherePlugin.listDegres
        self.tablesWidget = self.qspherePlugin.tablesWidget

    def translateFixedWidgets(self):

        i = 0
        for collapsibleGroupBox in self.mddcollapsibleGroupBox :
            collapsibleGroupBox.setTitle("%s : " % (QApplication.translate("QSphere", self.libCollapsibleGroupBox[i])))
            i+= 1        

        fileName = self.windowTitle().replace(self.rootWindowTitle, "")
        elt0 = QApplication.translate("QSphere","project").title()
        elt1 = QApplication.translate("QSphere","metadata")
        self.rootWindowTitle = "%s :" % (elt1.title())

        self.setWindowTitle("%s%s" % (self.rootWindowTitle, fileName))
        self.setToolTip("%s%s" % (self.rootWindowTitle, fileName))
        self.extensionsNames = {"qsp": "%s QSphere (*.qsp)" % elt0, "xml" : "eXtensible Markup Language %s (*.xml)" %  elt1,
                                "qmd" : "QGis %s (*.qmd)" % elt1}
        self.lblComboLayers.setText("\n %s" % QApplication.translate("QSphere","Query a layer : "))
        self.lblComboLayers.setToolTip(QApplication.translate("QSphere","Query a layer : "))
        self.ComboLayers.setToolTip(self.lblComboLayers.toolTip())
        self.ComboLayers.setItemText(0, QApplication.translate("QSphere","No layer"))

        self.lblPrimaryXslt.setText(" %s : " % QApplication.translate("QSphere", "Primary XSLT"))

        
        
    def changeLang(self):

        obj = self.findChild(myTableWidget, "tablemotsclefso")
        if obj != None :     
               data = {}
               for row in range(obj.rowCount()):
                  datarow = [] 
                  for col in range(obj.columnCount()):
                      ObjWidget = obj.cellWidget(row, col)
                      datarow.append(ObjWidget.currentIndex())
                  data[row] = datarow

        self.qspherePlugin.changeLang(True)
        lang = self.qspherePlugin.dicoLangs[self.qspherePlugin.indexLang]
        self.indexLang = self.qspherePlugin.indexLang

        i, bounds = 0, len(self.libelles.keys())
        
        self.progressBar.setValue(0)
        self.tabWidget.setVisible(False)
        self.blocProgressBar.setVisible(True)
        self.blocProgressBarText.setText("%s ... :\n %s" % (QApplication.translate("QSphere","Translation into the language"),
                                                        self.languesDico[lang]['english'].upper()))
        self.blocProgressBarButton.setIcon(QIcon(getThemeIcon("../ressources/images/%s.png"% self.languesDico[lang]['bibliographic'])))

        windowProperties(self, self.parent)
        self.translateTablesWidget()
        
        for key in self.libelles.keys() :
            i+= 1
            self.progressBar.setValue(int(100 * i/bounds))

            nameObj, typeObj, textObj, toolTipObj = self.libelles[key][0], self.libelles[key][1], self.libelles[key][2], self.libelles[key][3]

            if typeObj != None :
                obj = self.findChild(typeObj, nameObj)
                
                if obj != None :
                   text = QApplication.translate("QSphere", toolTipObj)
                   if text == "" : text = toolTipObj 
                   try : obj.setToolTip(text)
                   except: pass

                   nameObjTable = "" 
                   if type(obj) == myTableWidget : 
                      isShowWarning = self.showWarning
                      self.showWarning = False 
                      translations, nameObjTable, targetObj = [], nameObj, obj
                     
                      for col in range(obj.columnCount()) :
                          objKey = key+(col+1)
                          if dicoHasKey(self.libelles, objKey) : translations.append(QApplication.translate("QSphere", self.libelles[objKey][2]))
                      if translations!= []: obj.setHorizontalHeaderLabels(translations)

                      for row in range(obj.rowCount()):
                         for col in range(obj.columnCount()):
                             ObjWidget = obj.cellWidget(row, col)
                          
                             if ObjWidget != None :
                                 
                               if type(ObjWidget) in (QPushButton, myComboBox, myCheckBox):
                                   indexWidget = 0 
                                   if type(ObjWidget) == myComboBox : indexWidget = ObjWidget.currentIndex()

                                   if ObjWidget.objectName().startswith("tableroles_combobox") :
                                      test = False 
                                      if not ObjWidget.isEditable() :
                                         test = True
                                         ObjWidget.setEditable(True)                                       
                                      ObjWidget.clear()
                                      ObjWidget.addItems(self.listOfRules)
                                      if test : ObjWidget.setEditable(False)

                                   elif ObjWidget.objectName().startswith("tablemotsclefsf_combobox") :
                                      test = False
                                      if not ObjWidget.isEditable() :
                                         test = True   
                                         ObjWidget.setEditable(True)
                                      ObjWidget.clear()
                                      ObjWidget.addItems(self.listTypeDates)
                                      if test : ObjWidget.setEditable(False)
                                      

                                   elif ObjWidget.objectName().startswith("tablespecifications_combobox") :
                                        ObjWidget.clear()
                                        if ObjWidget.objectName().endswith("2") : ObjWidget.addItems(self.listTypeDates)
                                        elif ObjWidget.objectName().endswith("3") : ObjWidget.addItems(self.listDegres)
                                    
                                   if type(ObjWidget) == myComboBox : ObjWidget.setCurrentIndex(indexWidget)

                                   elif ObjWidget.objectName().startswith("tablemotsclefsf_action") :   
                                      ObjWidget.setToolTip(QApplication.translate("QSphere", "From controlled Vocabulary "))
                                      
                                   elif ObjWidget.objectName().startswith("tableformats_action") :
                                      ObjWidget.setToolTip(QApplication.translate("QSphere","Call the QSphere Formats Dialog box"))

                                   elif ObjWidget.objectName().startswith("tablescr_action") : 
                                      ObjWidget.setToolTip(QApplication.translate("QSphere","Call the SRS QGIS Dialog box"))

                                   elif ObjWidget.objectName().startswith("tableemprises_action") :   
                                      ObjWidget.setToolTip(QApplication.translate("QSphere","View with WMS service the current extent"))

                      self.showWarning = isShowWarning

                   if type(obj) == QgsCollapsibleGroupBox :
                       mddcollapsibleGroupBox = self.findChild(QgsCollapsibleGroupBox, nameObj)
                       text = QApplication.translate("QSphere", textObj)
                       toolTip = QApplication.translate("QSphere", toolTipObj)
                       if mddcollapsibleGroupBox :
                            mddcollapsibleGroupBox.setTitle(text)

                   if type(obj) == QWidget :
                       index = int(nameObj.replace("tab",""))
                       text = QApplication.translate("QSphere", textObj)
                       toolTip = QApplication.translate("QSphere", toolTipObj)
                       self.tabWidget.setTabText(i-1, text)
                       self.tabWidget.setTabToolTip(i-1, toolTip)
                    
                   if type(obj) == QGroupBox :
                      if nameObj == "groupedroits" :
                         valObj = ((QApplication.translate("QSphere","No restriction for public access in INSPIRE"),"norestriction_droits",25000.00),
                                      (QApplication.translate("QSphere","With restriction for public access in INSPIRE (Directive 2007/2/CE)"),"restriction_droits",2.0)) 
                         for oi in range(len(valObj)):
                             radioButton = self.findChild(QRadioButton, valObj[oi][1]) 
                             if radioButton : radioButton.setText(valObj[oi][0])
                         table = self.findChild(QTableView, "tablegroupedroits")
                         if table : CleanSheetTable(self, table, "file:280:contraintes_%s.csv:1:0" % (self.qspherePlugin.dicoLangs[self.qspherePlugin.indexLang]), None)
                            

                   if type(obj) == QComboBox and nameObj in ("typedata", "sysreftemp") :
                       indexWidget = obj.currentIndex()
                       obj.clear()
                       if nameObj == "typedata" :
                           self.listTypeRessources = getListTypeRessources(self)
                           obj.addItems(self.listTypeRessources)
                       if nameObj == "sysreftemp" :
                           self.listTemporalSystem = getListTemporalSystem(self)
                           obj.addItems(self.listTemporalSystem)
                       obj.setCurrentIndex(indexWidget)

                   if type(obj) == QTableView  :
                       if obj.objectName() == "tablecategories" : CleanSheetTable(self, obj, "file:300:categories_thematiques_%s.csv:1:0" % (self.qspherePlugin.dicoLangs[self.qspherePlugin.indexLang]), "tablemotsclefso")
                
                obj = self.findChild(QLabel, "lbl%s" % (nameObj))
                if obj != None :
                   try :
                       if textObj!= "" :
                          text = QApplication.translate("QSphere", textObj)
                          if text == "" : text = textObj
                          if nameObj.startswith("titletab") : text = ">> %s" % (text)
                          obj.setText(text)
                          obj.setAccessibleDescription(text)
                   except: pass
                   if nameObjTable != "" :
                      countItems(self, nameObjTable, targetObj)

                      obj = self.findChild(myPushButton, "ActionsButton_%s" % (nameObj))
                      if obj != None :
                         obj.setToolTip(QApplication.translate("QSphere", self.libelles[101][3]))
                         contextMnuMDDActions(self, self.tabWidget, targetObj, nameObj, 6, obj, "") 

                      obj = self.findChild(myPushButton, "OpenWizard_%s_%s" % (6, nameObj))
                      if obj != None : obj.setToolTip(QApplication.translate("QSphere", self.libelles[102][3]))              
                      
                obj = self.findChild(myPushButton, "help_%s" % (nameObj))
                if obj != None : obj.setToolTip(QApplication.translate("QSphere", self.libelles[100][3]))


        obj = self.findChild(QRadioButton, "resolution_scale")       
        if obj != None : obj.setText("%s%s" % (QApplication.translate("QSphere", self.libelles[57][2])," 1/"))
        obj = self.findChild(QRadioButton, "resolution_pixel")
        if obj != None : obj.setText(QApplication.translate("QSphere","Equivalent scale in unit of measure "))

        if data != {}:
           obj = self.findChild(myTableWidget, "tablemotsclefso")
           if obj != None :
              obj.clearContents()
              for j in range(obj.rowCount()): obj.removeRow(0)
              for row in data :
                  for col in range(len(data[row])):
                      if col == 0 : makeLine(self, obj, False, False, -1, True)
                      ObjWidget = obj.cellWidget(row, col)
                      if ObjWidget != None :
                          ObjWidget.setCurrentIndex(data[row][col])
                          if data[row][0]== 0 : ObjWidget.setEnabled(False)

        self.translateFixedWidgets()

        if self.qspherePlugin.mainWindow != None : self.qspherePlugin.mainWindow.translate(True)

        idbutton = self.changeMode.id(self.changeMode.checkedButton())
        if idbutton == 2 : self.doChangeMode(idbutton)  
 
        self.progressBar.setValue(0)
        self.tabWidget.setVisible(True)
        self.blocProgressBar.setVisible(False) 
         

    def reject(self): self.killWindows()

    def killWindows(self):
        try :
            for key, obj in self.isFromDownload.d.items() :
                if self == obj :
                   self.isFromDownload.d.pop(key)
                   if self.isFromDownload.d == {} : self.isFromDownload.parent = None
                   break
            if self.isFromDownload.d != {} :
                for key, obj in self.isFromDownload.d.items() :    
                    self.isFromDownload.parent = self.isFromDownload.d[key]
                    break
        except : pass    
        
        for child in self.childswindows :
            if child != None:
                try : child.reject()
                except : pass
        self.reject()

    def UpdateListComboLayers(self):
        self.ComboLayers.blockSignals(True)
        self.modelLayers = QStandardItemModel()
        if self.ComboLayers.model().item(self.ComboLayers.currentIndex(),1)!= None : self.comboLayersCurrentLayerId = self.ComboLayers.model().item(self.ComboLayers.currentIndex()).data(0) 
        maxdim = getListLayers(self, self.tableListLayers, self.model, self.ComboLayers, False)
        indexLayer = getComboLayerIndex(self, self.ComboLayers, "%s" % (self.comboLayersCurrentLayerId))
        self.ComboLayers.setCurrentIndex(indexLayer)
        self.getMetaData()
        self.ComboLayers.blockSignals(False)
        if self.iface.mapCanvas().currentLayer()!= None:
            self.iface.mapCanvas().currentLayer().nameChanged.connect(self.UpdateListComboLayers)
            self.iface.mapCanvas().currentLayer().legendChanged.connect(self.UpdateListComboLayers)

        self.view.horizontalHeader().setDefaultSectionSize(maxdim*8)
        self.view.setStyleSheet("QTableView {min-width: %spx; max-width: %spx; selection-background-color: qlineargradient(x1: 0, y1: 0, x2: 0.5, y2: 0.5, stop: 0 #FFFFFF, stop: 1 #000000);}" % (maxdim*8, (maxdim+1)*8))

        
    def callInitSizeCols(self):  callInitSizeCols(self)
        
    def isVisibleRestoreDico(self, zCond) : self.restore_tabledico.setEnabled(zCond)

    def IsVisibleViewXMLButton(self):
        if self.restore_tabledico != None : self.restore_tabledico.setEnabled(False)
        if self.windowTitle() != self.rootWindowTitle and self.windowTitle().find("~")== -1 :
            fileName = self.windowTitle().replace(self.rootWindowTitle, "")

    def geoLocalisator(self): geoLocalisator(self)

    def initComments(self): initComments(self)

    def getMetaData(self):
        if self.ComboLayers.currentIndex()==-1 : return
        
        crs, northLatitude, southLatitude, westLongitude, eastLongitude = "EPSG:4326", 51.9, 41.36, -5.79, 9.56
        topologyConsistency = "{'TopologyLevelCode':'unknow', 'GeometricObjectTypeCode':'unknow'}"

        obj = self.findChild(myTableWidget, "tabledico")
        if obj == None : return
        obj.clearContents()
        onlyEnabledColor = QBrush(QColor(240,240,240))
        while obj.rowCount()> 0 : obj.removeRow(0)

        setTextWidget(self, "namelayer", "", False)
        setTextWidget(self, "typelayer", "", False)
        setTextWidget(self, "metadata", "", False)
        
        if self.ComboLayers.currentIndex() > 0 :
           layer = getComboLayer(self, self.ComboLayers, True)
           self.comboLayersCurrentLayerId = self.ComboLayers.model().item(self.ComboLayers.currentIndex()).data(0) 
           if not layer : return            

           typeLayer = getTypeLayer(self, layer.type()) 
           if typeLayer == "vector" :
               topologyType = {QgsWkbTypes.Point: "{'TopologyLevelCode':'geometryOnly', 'GeometricObjectTypeCode':'point'}",
                               QgsWkbTypes.LineString  : "{'TopologyLevelCode':'topology1D', 'GeometricObjectTypeCode':'curve'}",
                               QgsWkbTypes.Polygon : "{'TopologyLevelCode':'abstract', 'GeometricObjectTypeCode':'surface'}"}
               key = layer.geometryType()+1
               if dicoHasKey(topologyType, key) : topologyConsistency = topologyType[key]
           
           extent = layer.extent()
           transformer = transformerDefinition(layer, 4326)

           westLongitude, northLatitude = 0, 0
           eastLongitude, southLatitude = 0, 0

           point = transformer.transform (extent.xMinimum(), extent.yMaximum(), QgsCoordinateTransform.ForwardTransform)
           westLongitude, northLatitude = point.x(), point.y()

           point = transformer.transform (extent.xMaximum(), extent.yMinimum(), QgsCoordinateTransform.ForwardTransform)
           eastLongitude, southLatitude = point.x(), point.y()

           if isCorrectLayer(layer, True) and obj :
               provider = layer.dataProvider()
               fields = provider.fields()
               i = 0
               DicoModel = QStandardItemModel()
               for j in range(len(fields)):
                   itemId = QTableWidgetItem()
                   itemName = QTableWidgetItem()
                   itemTypeName = QTableWidgetItem()
                   itemLength = QTableWidgetItem()
                   itemPrecision = QTableWidgetItem()
                   itemComment = QTableWidgetItem()
                                     
                   obj.insertRow(i)
                   
                   itemId.setText("%s" % (i))
                   obj.setItem(i,0,itemId)
                   obj.item( i, 0 ).setFlags(Qt.ItemIsSelectable|Qt.ItemIsEnabled)
                   obj.item( i, 0 ).setBackground(onlyEnabledColor)
                                      
                   itemName.setText("%s" % (fields[j].name()))
                   obj.setItem(i,1,itemName)
                   obj.item( i, 1 ).setFlags(Qt.ItemIsSelectable|Qt.ItemIsEnabled)
                   obj.item( i, 1 ).setBackground(onlyEnabledColor)
                   
                   itemTypeName.setText("%s" %(fields[j].typeName()))
                   obj.setItem(i,2,itemTypeName)
                   obj.item( i, 2 ).setFlags(Qt.ItemIsSelectable|Qt.ItemIsEnabled)
                   obj.item( i, 2 ).setBackground(onlyEnabledColor)
                   
                   itemLength.setText("%s" %(fields[j].length()))
                   obj.setItem(i,3,itemLength)
                   obj.item( i, 3 ).setFlags(Qt.ItemIsSelectable|Qt.ItemIsEnabled)
                   obj.item( i, 3 ).setBackground(onlyEnabledColor)
                   
                   itemPrecision.setText("%s" %(fields[j].precision()))
                   obj.setItem(i,4,itemPrecision)
                   obj.item( i, 4 ).setFlags(Qt.ItemIsSelectable|Qt.ItemIsEnabled)
                   obj.item( i, 4 ).setBackground(onlyEnabledColor)
                   
                   itemComment.setText("%s" %(fields[j].comment()))
                   obj.setItem(i,5,itemComment)
                   obj.item( i, 5 ).setFlags(Qt.ItemIsSelectable|Qt.ItemIsEnabled|Qt.ItemIsEditable|Qt.ItemIsDragEnabled|Qt.ItemIsDropEnabled)

                   i+= 1


           obj = self.findChild(myTableWidget, "tableemprises")
           if obj :
                extentIndex = getIndex(obj)
           
                item = obj.cellWidget(extentIndex,2)
                if item == None : return
                item.setValue(westLongitude)
                positionStartX = int(item.value())+180

                item = obj.cellWidget(extentIndex,3)
                item.setValue(eastLongitude)
                positionEndX = int(item.value())+180
                   
                item = obj.cellWidget(extentIndex,0)
                item.setValue(northLatitude)
                positionStartY = 90-int(item.value())

                item = obj.cellWidget(extentIndex,1)
                item.setValue(southLatitude)
                positionEndY = 90-int(item.value())


           obj = self.findChild(myTableWidget, "tablescr")
           if obj :
                extentIndex = getIndex(obj)
                item = obj.cellWidget(extentIndex,0)
                if item == None : return
                item.setText(layer.crs().authid())
                
           setTextWidget(self, "namelayer", layer.name(), False)
           setTextWidget(self, "typelayer", typeLayer, False)
           setTextWidget(self, "metadata", layer.htmlMetadata(), False)
           setTextWidget(self, "scr", layer.crs().authid(), False)
           setTextWidget(self, "coherence", topologyConsistency, False)

           if exists(layer.source()):
               self.InitDir = "%s" % (dirname(layer.source()))
               self.InitDir = correctPath(self.InitDir.replace("\\","/"))
               self.InitName = splitext(basename(layer.source()))[0]
           else :
               self.InitName = layer.name()


    def OpenFormTable(self, Obj):
        if Obj == None :
            title = QApplication.translate("QSphere","Warning")
            msg = QApplication.translate("QSphere","No object focused !")
            sendMsg(self.parent, title , "<u><i>%s</i></u>" % msg, Qgis.Warning, self.duration_warning)   
            return
        
        if type(Obj) != myTableWidget :
            try :
                table = Obj.parent().parent()
                if type(table)== myTableWidget : Obj = table
            except :
                title = QApplication.translate("QSphere","Warning")
                msg = QApplication.translate("QSphere","Place focus into a table composant !")
                sendMsg(self.parent, title , "<u><i>%s</i></u>" % msg, Qgis.Warning, self.duration_warning)    
                return

        if type(Obj) == myTableWidget :
            if Obj.rowCount()==0: makeLine(self, Obj, True, False, -1, True) 
            dialog = formWidgetDialog(Obj, self)
            dialog.exec_()
        
    def addLine(self): addLine(self)
    
    def CheckTable(self, obj):
        for i in range(obj.rowCount()):
            if i in (0, 1) :
                obj.cellWidget(i,0).setEnabled(False)
                if i == 0 : obj.cellWidget(i,0).setCurrentIndex(2)
                else : obj.cellWidget(i,0).setCurrentIndex(6)
            else :
                obj.cellWidget(i,0).setEnabled(True)
                if obj.cellWidget(i,0).currentIndex() in (2,6) : obj.cellWidget(i,0).setCurrentIndex(1)

        
    def moveLineUp(self):
        nameObj = "%s" % (self.sender().objectName()) 
        moveLineUpProcess(self, nameObj)

    def moveLineDown(self):
        nameObj = "%s" % (self.sender().objectName()) 
        moveLineDownProcess(self, nameObj)

    def deleteLine(self):
        nameObj = "%s" % (self.sender().objectName()) 
        deleteLineProcess(self, nameObj, True)

    def deleteCurrentLine(self):
        nameObj = "%s" % (self.sender().objectName()) 
        deleteLineProcess(self, nameObj, False)

    def newSheet(self):
        self.setWindowTitle(self.rootWindowTitle)
        self.setToolTip(self.rootWindowTitle)
        obj = None

        mylang = QLocale.languageToString( self.langue )
        for key in self.languesDico.keys():
            if self.languesDico[key]['english'] == mylang :
               mycodelang = self.languesDico[key]['bibliographic']
               break

        i, bounds = 0, len(self.mddcollapsibleGroupBox)
        
        for mddcollapsibleGroupBox in self.mddcollapsibleGroupBox:
              objs = mddcollapsibleGroupBox.children()
              for j in range(len(objs)):
                obj = objs[j]
                classObjName = "%s" % (obj.metaObject().className())

                self.progressBar.setValue(int(100 * i/bounds))
                i+= 1

                if classObjName=="myButton" : pass
                elif classObjName in ("QCalendarWidget", "QgsDateTimeEdit") : reinitDate(obj)

                elif classObjName in ("QTextEdit", "MyTextEdit", "myTextEdit") :
                    if obj.isEnabled(): obj.setPlainText("")
                    else :
                        if obj.objectName()== "coherence" : obj.setPlainText("{'TopologyLevelCode':'unknow', 'GeometricObjectTypeCode':'unknow'}")
                    if obj.objectName() == "licence" : obj.setPlainText("%s" % (QApplication.translate("QSphere","Open Licence")))
                elif classObjName in ("QLineEdit", "MySimpleWidgetLineEdit", "MyWidgetLineEdit", "MySimpleWidgetLineEditST") : 
                    obj.setText("") if obj.objectName() != "identificator" else obj.setText("%s-%s-%s" % (self.localLangage.upper(), datetime.now().year, getRandowId(self)))
                elif classObjName in ("QComboBox", "MyComboBox", "myComboBox") :
                    iIndex = 0
                    obj.clear()
                    if obj.objectName() == "typedata" : obj.insertItems(0, self.listTypeRessources)
                    elif obj.objectName()== "sysreftemp" :
                       obj.insertItems(0, self.listTemporalSystem)
                       iIndex = 1
                    elif obj.objectName()== "tablecarac" :
                       obj.insertItems(0, self.listCodecs)
                       iIndex = self.listCodecs.index("utf8")
                    elif obj.objectName()== "langmetada" :
                       iIndex = getListLanguesIndex(self, obj, mylang)
                    obj.setCurrentIndex(iIndex)

                elif classObjName=="QGroupBox" :
                    children = obj.children()
                    children[0].setChecked(True)
                    table = children[2]
                    if table.metaObject().className() in ("QTableWidget", "MyTableWidget", "myTableWidget") :
                        table.clearContents()
                        for j in range(table.rowCount()): table.removeRow(0)
                        makeLine(self, table, False, True, -1, True)
    
                    elif table.metaObject().className() == "QTableView" :
                        if obj.objectName() == "tablegroupedroits" : CleanSheetTable(self, obj, "file:280:contraintes_%s.csv:1:0" % (self.qspherePlugin.dicoLangs[self.qspherePlugin.indexLang]), None)
                            

                    if len(children) > 3 :    
                        comboBox = children[3]
                        comboBox.setCurrentIndex(0)
    
                elif classObjName=="QTableView" :
                     if obj.objectName() == "tablelangues" :
                            for i in range(len(self.langs)):
                                item = obj.model().item(i, 0)
                                language = self.langs[i]
                   
                     for k in range(obj.model().rowCount()):
                         item = obj.model().item(k, 0)
                         item.setCheckState(Qt.Unchecked)
                         if obj.objectName() == "tablelangues" and item.text().startswith(self.localLangage): item.setCheckState(Qt.Checked)

                elif classObjName in ("QTableWidget", "MyTableWidget", "myTableWidget") :
                    obj.clearContents()
                    for j in range(obj.rowCount()): obj.removeRow(0)
                    makeLine(self, obj, False, True,-1, True)
                    if obj.objectName() == "tableemprises" : self.drawExtent()
                    
        self.countLangues()
        self.IsVisibleViewXMLButton()
        idbutton = self.changeMode.id(self.changeMode.checkedButton())
        self.doChangeMode(idbutton)    

    def saveData(self):
        if self.windowTitle() != self.rootWindowTitle and self.windowTitle().find("~")== -1 :
            fileName = self.windowTitle().replace(self.rootWindowTitle, "")
            title = QApplication.translate("QSphere","Information")
            if fileName!="": self.makeSave(fileName, True, title)
            else : self.saveDataAs(0)
        else : self.saveDataAs(0)

    def saveDataAs(self, index):
        extension = self.extension
        if self.windowTitle() != self.rootWindowTitle and self.windowTitle().find("~")== 0 :
            fileName = self.windowTitle().replace(self.rootWindowTitle, "")
            self.InitName = basename(fileName) 
            extension = getExtension(fileName)
            extension = extension.replace(".","")

        if index == 2 : title, isActiveProject = QApplication.translate("QSphere","Save as a copy ..."), False
        else : title, isActiveProject = QApplication.translate("QSphere","Save as a file"), True

        dlgFile = QFileDialog(self, title)
        dlgFile.setNameFilters([items[1] for items in self.extensionsNames.items()])
        dlgFile.selectNameFilter(self.extensionsNames[extension])
        dlgFile.setViewMode(QFileDialog.Detail)
        dlgFile.setDirectory(self.InitDir)
        dlgFile.setAcceptMode(QFileDialog.AcceptSave)

        fixeLabelsFileDialog(self, dlgFile, 1, True)
        
        dlgFile.selectFile(self.InitName)
        if dlgFile.exec_():
            fileName = fileNameWithExtension(self, dlgFile.selectedFiles()[0], dlgFile.selectedNameFilter())
            self.makeSave(fileName, isActiveProject, title)
            self.connexion = None

        
    def makeSave(self, fileName, isActiveProject, title):

        extension = ""
        
        if fileName.startswith("pgmdd:") :
           extension = "pgsql"
           if self.connexion != None :

              dataXml = saveXml(self, None)
              dataXml = dataXml.decode("utf-8").replace("'", "''")
              
              fileName = self.windowTitle().replace(self.rootWindowTitle, "")
              obj = self.findChild(QLineEdit, "intitule")
              title = obj.text().replace("'", "''") if obj != None else "No result"
              obj = self.findChild(myTextEdit, "resume")
              abstract = obj.toPlainText().replace("'", "''") if obj != None else "No result"
              obj = self.findChild(QLineEdit, "identificator")
              identificator = obj.text() if obj != None else "-1"

              layer = getComboLayer(self, self.ComboLayers, True)
              comboLayersCurrentLayerId = self.ComboLayers.model().item(self.ComboLayers.currentIndex()).data(0) 
              if not layer :  datasource, parentid = None, -1
              else :
                 typedatasource = getTypeLayer(self, layer.type())
                 datasource = layer.source()              

              idData = fileName.split("id=")[1]

              try : 
                  self.connexion.set_isolation_level(ISOLATION_LEVEL_AUTOCOMMIT)
                  cur = self.connexion.cursor()
                  req = """SELECT id FROM qsp_metadata WHERE id=%s""" % idData
                  cur.execute(req)
                  records = cur.fetchall()

                  if records[0]==0:
                     sendMsg(self, title , "%s :<br><u><i>%s</i></u> No present" % ("Error >> PostgreSQL ?", idData), Qgis.Warning, self.duration_warning)
                     self.setWindowTitle(self.rootWindowTitle)
                     self.connexion = None
                  else :
                      try : 
                          req = """UPDATE qsp_metadata SET
                                   title = '%s',
                                   abstract = '%s',
                                   identificator = '%s',
                                   metadata = '%s',
                                   datasource = '%s',
                                   parentid  = '%s'
                                   WHERE id = %s;
                              """ % (title, abstract, identificator, dataXml, datasource, fileName, idData)
                          cur.execute(req)
                          sendMsg(self, title , "%s :<br><u><i>%s</i></u>" % ("Successfull >> PostgreSQL ?", fileName), Qgis.Info, self.duration_info)
                      except :
                          if cur is not None : cur.rollback()
              except :
                  sendMsg(self, title , "%s :<br><u><i>%s</i></u>" % ("Error >> PostgreSQL ?", fileName), Qgis.Info, self.duration_info)
               
        else :
            extension = getExtension(fileName)
            
            if extension == ".qsp" : saveQsp(self, fileName, False)
            elif extension == ".xml" : saveXml(self, fileName)
            elif extension == ".qmd" : saveQmd(self, fileName) 
 
            msg = QApplication.translate("QSphere","The metadata record was saved as")
            sendMsg(self, title , "%s :<br><u><i>%s</i></u>" % (msg, fileName), Qgis.Info, self.duration_info)

        if isActiveProject :
           if extension != "" :
               fileName = "%s%s" % (self.rootWindowTitle, fileName)
               self.setWindowTitle(fileName)
               self.setToolTip(fileName)
  

    def getDialogViewerFromAnother(self, url, dlgAnother): self.parent.loadNavigator(url)          
           
    def saveXmlCatalog(self):
        obj = self.findChild(myTableWidget, "tabledico")
        title = QApplication.translate("QSphere","Warning")
        if obj.rowCount()==0 :
            msg =  QApplication.translate("QSphere","No information for fields map.")
            obj.setFocus()
            sendMsg(self, title , "%s" % (msg), Qgis.Warning, self.duration_warning)
            return

        title = QApplication.translate("QSphere","Export Fields Map from QSphere record in XML")
        dlgFile = QFileDialog(self, title)
        dlgFile.setDefaultSuffix("xml")
        dlgFile.setNameFilters([items[1] for items in self.extensionsNames.items() if items[0]=="xml"])
        dlgFile.selectNameFilter(self.extensionsNames["xml"])
        obj = self.findChild(QLineEdit, "namelayer")
        nameLayer = obj.text() if obj else ""
        dlgFile.selectFile("cat_%s.xml" % (nameLayer))
        dlgFile.setViewMode(QFileDialog.Detail)
        dlgFile.setDirectory(self.InitDir)
        dlgFile.setAcceptMode(QFileDialog.AcceptSave)

        fixeLabelsFileDialog(self, dlgFile, 1, True)        
        
        if dlgFile.exec_():
           fileName = fileNameWithExtension(self, dlgFile.selectedFiles()[0], dlgFile.selectedNameFilter())
           saveXmlCatalog(self, fileName)
           title = QApplication.translate("QSphere","Information")
           msg =  QApplication.translate("QSphere","The Field Map was exported as :\n") 
           sendMsg(self, title , "%s<u><i>%s</i></u>" % (msg, fileName), Qgis.Info, self.duration_info)    


    def reloadUi(self, fileName, index):
        if fileName != None :
            self.InitDir = dirname(fileName)
            fileName = "%s%s" % (self.rootWindowTitle, fileName)
            self.setWindowTitle(fileName)
            self.setToolTip(fileName)            
        self.countLangues()
        self.progressBar.setValue(0)
        self.blocProgressBar.setVisible(False)
        self.IsVisibleViewXMLButton()

        idbutton = self.changeMode.id(self.changeMode.checkedButton())
        self.doChangeMode(idbutton)

    def reLoadMetadata(self):
        if self.windowTitle() != self.rootWindowTitle and self.windowTitle().find("~")== -1 :    
            fileName = self.windowTitle().replace(self.rootWindowTitle, "")
            extension = getExtension(fileName)
            if extension == "" and fileName.startswith("pgmdd:"): child.mddFomDataBase(fileName)
            else :  
                if extension == ".qsp" : self.loadQsp(fileName, None)
                elif extension == ".xml" : self.loadXml(fileName)
                elif extension == ".qmd" : self.loadQmd(fileName)

            if self.ComboLayers.currentIndex()>0 :
               
               self.getMetaData()
               if self.autoComment : self.initComments()
               self.isVisibleRestoreDico(extension == ".qsp")

                
    def mddFomDataBase(self, fileName):
        paramConnexion = fileName.replace("pgmdd:","").split(" ")
        self.connexion = None
        dialog = dlgAuthCswt("", "", "%s" % paramConnexion, True)
        windowIcon(dialog, "sgbdr/pucebase.png")
        if dialog.exec_():
            from psycopg2 import (connect, pool as psycopg2pool, OperationalError)
            from psycopg2.extensions import (ISOLATION_LEVEL_AUTOCOMMIT, POLL_OK, POLL_WRITE, POLL_READ)
            
            dns = "%s %s  user=%s password=%s %s" % (paramConnexion[0], paramConnexion[2], dialog.authInfos()[0], dialog.authInfos()[1], paramConnexion[1])

            try : self.connexion = connect(dns)
            except OperationalError : return

            if self.connexion :

                self.connexion.set_isolation_level(ISOLATION_LEVEL_AUTOCOMMIT)
                cur = self.connexion.cursor()
                req = "SELECT metadata from qsp_metadata WHERE %s;" % paramConnexion[5]
                cur.execute(req)
                records = cur.fetchall()

                if records :   
                    dataXml = unquote(records[0][0]).replace("''", "'")
                    from io import StringIO as bufferData
                    #from qsphere.xmlisoparser import xmlIsoParser
                    buffer = bufferData(dataXml)
                    isoParser = xmlIsoParser(None, buffer, 'MEDDE', 'fr') 
                    isoParser.createISOdataStructure(True)
                    if isoParser.getTagDictionnary() :
                       self.reloadDataFromXml(isoParser, None, 0)
                       self.setWindowTitle(fileName)
                   
        
        
    def loadQsp(self, fileName, dataQSP):
        index = 0
        try :
            loadDataFromQsp(self, None, dataQSP) if dataQSP else loadDataFromQsp(self, fileName, None)
            self.checkSelectKeyWords()
            self.reloadUi(None, index) if dataQSP else self.reloadUi(fileName, index)
            return True
        except : return False   

    def loadQmd(self, fileName):
        isoParser = xmlIsoParser(fileName, None, 'QMD', self.localLangage)
        isoParser.createQMDdataStructure(True)
        self.reloadDataFromQmd(isoParser, fileName, 0)
        return True

    def reloadDataFromQmd(self, isoParser, fileName, index):
        orderKeys = ("intitule", "resume", "typedata", "tablelocalisator", "identificator", "tablelangues",
                     "tableemprises", "tablecategories:0", "tablemotsclefsf", "tablescr", "tableetenduetemporelle", "tableroles")

        widgetValues = {"intitule": isoParser.title , "resume" : isoParser.abstract, "typedata" : isoParser.type, "tablelocalisator" : isoParser.localisators,
                        "identificator" : isoParser.parentidentifier, "tablelangues" : isoParser.language,
                        "tableemprises" : isoParser.tableemprises, "tablecategories:0" : isoParser.codecategories, "tablemotsclefsf" : isoParser.keywordsF,
                        "tablescr" : isoParser.tablescr, 
                        "tableetenduetemporelle" : isoParser.tableetenduetemporelle, "tableroles" : isoParser.pointsofcontact}

        
        self.tabWidget.setVisible(False)
        self.blocProgressBar.setVisible(True)
        bounds, i = len(orderKeys), 0
        for i in range(bounds):
            key = orderKeys[i]
            self.updateWidget(key, widgetValues[key], isoParser)
            self.progressBar.setValue(int(100 * i/bounds))
            i+= 1

        self.reloadUi(fileName, 0)

        
    def loadXml(self, fileName):

        #from qsphere.xmlisoparser import xmlIsoParser
        if self.autoCorrect == True : 
            f = open(fileName, "r")

            error = False
            try : bufferXML = f.read()
            except : error = True

            f.close()

            if error : msgBadFileXml(self); return False;

            from io import StringIO

            if bufferXML.find("<fra:FRA_DataIdentification gco:isoType=\"gmd:MD_DataIdentification\">")!=-1:
               bufferXML = bufferXML.replace("xsi:schemaLocation=\"http://www.cnig.gouv.fr/2005/fra http://eden.ign.fr/xsd/fra/20060106/fra/fra.xsd\"", "xsi:schemaLocation=\"http://www.isotc211.org/2005/gmx http://schemas.opengis.net/iso/19139/20070417/gmx/gmx.xsd\"")
               bufferXML = bufferXML.replace("<fra:FRA_DataIdentification gco:isoType=\"gmd:MD_DataIdentification\">", "<gmd:MD_DataIdentification>")
               bufferXML = bufferXML.replace("</fra:FRA_DataIdentification>", "</gmd:MD_DataIdentification>")
               bufferXML = bufferXML.replace("<gco:CharacterString/>", "<gco:CharacterString> </gco:CharacterString>")

               if self.byStream == True :
                   self.listImportCategories = []
                   isoParser = xmlIsoParser(None, StringIO(bufferXML) , 'MEDDE', self.localLangage)
                   if isoParser == None :
                       msgBadFileXml(self)
                       return               
                   isoParser.createISOdataStructure(True)

                   if isoParser.getTagDictionnary() : self.reloadDataFromXml(isoParser, None, 0)
                   return

               elif self.byFile == True :
                   baseName = splitext(basename(fileName))[0]
                   extension = getExtension(baseName)
                   baseName = baseName.replace(extension, "")
                   fileName = "%s%s_qsphere_transformation.xml" % (self.InitDirByFile,)
                   inFile = open(fileName, "w")
                   inFile.write(bufferXML)
                   inFile.close()

        self.listImportCategories = []
        isoParser = xmlIsoParser(fileName, None, 'MEDDE', self.localLangage)

        cond = isoParser.getTagDictionnary()
        if not cond :
           msg = "%s<br>%s" % (QApplication.translate("QSphere","XML dictionnary file for QSphere !"),
                               QApplication.translate("QSphere","Open or add this file with the dictionnary interface (list of contacts) !"))
           sendMsg(self, "information" , msg, Qgis.Warning, self.duration_warning) 
           return                

        self.reloadDataFromXml(isoParser, fileName, 0)

        return True
    

    def reloadDataFromXml(self, isoParser, fileName, index):
        self.newSheet()
        if isoParser == None :
               msgBadFileXml(self)
               return False
        
        isoParser.createISOdataStructure(True)
        
        orderKeys = ("intitule", "resume", "typedata", "tablelocalisator", "identificator", "tablelangues", "tableformats", "tablecarac",
                      "tablecategories", "tablecategories:0", "tablemotsclefsf", "tablescr", "tableemprises", "tableetenduetemporelle", "groups:dates",
                      "genealogie", "coherence", "grouperesolutionscale", "tablespecifications", "groupedroits", "licence", "tableroles",
                      "datemetada", "langmetada")

        widgetValues = {"intitule": isoParser.title , "resume" : isoParser.abstract, "typedata": isoParser.typedata, "tablelocalisator" : isoParser.localisators,
                         "identificator" : isoParser.UUID, "tablelangues" : isoParser.languesjdd, "tableformats" : isoParser.formatsjdd, "tablecarac": isoParser.tablecaracjdd,
                         "tablecategories" : isoParser.categories, "tablecategories:0" : isoParser.codecategories, "tablemotsclefsf" : isoParser.keywordsF,
                         "tablescr" : isoParser.scr, "tableemprises" : isoParser.boundingboxcoordinates,
                         "tableetenduetemporelle" : isoParser.tableetenduetemporelle, "groups:dates" : isoParser.dates,
                         "genealogie" : isoParser.genealogie, "coherence" : isoParser.coherence, "grouperesolutionscale" : isoParser.scalesEC,
                         "tablespecifications": isoParser.conformities, "groupedroits" : isoParser.accessconstraints, "licence" : isoParser.legalconstraints,
                         "tableroles" : isoParser.pointsofcontact,
                         "datemetada" : (isoParser.datemdd, isoParser.datetmdd ), "langmetada" : isoParser.languemdd
                         }

        self.tabWidget.setVisible(False)
        self.blocProgressBar.setVisible(True)
        bounds, i = len(orderKeys), 0
        for i in range(bounds):
            key = orderKeys[i]
            self.updateWidget(key, widgetValues[key], isoParser)
            self.progressBar.setValue(int(100 * i/bounds))
            i+= 1

        self.reloadUi(fileName, index)
        return True


    def updateWidget(self, widgetKey, isoXml, isoParser):
        if not (isoXml) : return
        
        if widgetKey.startswith("groups:"):
           groupTarget = widgetKey.split(":")[1]
           groups = {"dates": {"c" : "datecredata", "r" : "daterevdata", "p" : "tabledatepubdata"}}
           group = groups[groupTarget]

           if groupTarget == "dates" :
               obj = self.findChild(myTableWidget, "tabledatepubdata")
               self.cleanAllObj(obj, True)
               i=0
               for elt in isoXml :
                   if type(elt) == dict :
                       if dicoHasKey(elt, 'type') :
                           if not elt['type'] == "" : 
                               typeObj = elt['type'][0:1]
                               try : obj = getWidget(self, group[typeObj])
                               except : return    
                               if typeObj in ("c", "r") : 
                                   infosDate = elt['date'].split("-")
                                   
                                   if len(infosDate) == 3 :
                                       try : obj.setDate(QDate(int(infosDate[0]), int(infosDate[1]), int(infosDate[2]) ))
                                       except :
                                           try : obj.setSelectedDate(QDate(int(infosDate[0]), int(infosDate[1]), int(infosDate[2]) ))
                                           except : pass
                                   elif len(infosDate) == 2 :
                                       try : obj.setDate(QDate(int(infosDate[0]), int(infosDate[1]), 1 ))
                                       except :
                                           try : obj.setSelectedDate(QDate(int(infosDate[0]), int(infosDate[1]), 1 ))
                                           except : pass
                               elif typeObj == "p" :
                                    makeLine(self, obj, True, False,-1, True)
                                    obj.cellWidget(i, 0).setText(elt['date'])
                                    i+= 1
        elif widgetKey == "coherence" : self.updateConsistency(widgetKey, isoXml)                     
        elif widgetKey == "tablespecifications" : self.updateSpecifications(widgetKey, isoXml)                    
        elif widgetKey == "tablemotsclefsf" : self.updateTableOptionalKeyWords(widgetKey, isoXml, isoParser)
        elif widgetKey == "tableetenduetemporelle": self.updateTemporalityExtents(widgetKey, isoXml, isoParser)
        elif widgetKey == "groupedroits": self.updateUserRights(widgetKey, isoXml, isoParser)
        elif widgetKey == "tableroles" : self.updateTableRoles(widgetKey, isoXml)
        elif widgetKey.startswith("tablecategories:") : self.updateTableCategories(widgetKey, isoXml)
        elif widgetKey == "grouperesolutionscale":
            if (isoParser.scalesEC[0]==[]) and (isoParser.scalesUM[0]==[]): return
            obj = self.findChild(QGroupBox, "grouperesolutionscale")
            children = obj.children()


            if (isoParser.scalesUM[0]!=[]) :
                children[1].setChecked(True)
                infos = isoParser.scalesUM[0]
                try : units = isoParser.UnitsScalesUM[0][0]['uom']
                except : units = "m" 
                children[3].setEnabled(True)
                index = self.listUnitsMesure.index(units) if units in (self.listUnitsMesure) else 0
                children[3].setCurrentIndex(index) 
            else :
                children[0].setChecked(True)
                infos = isoParser.scalesEC[0]
                children[3].setEnabled(False)

            children[2].clearContents()
            for j in range(children[2].rowCount()): children[2].removeRow(0)
            for j in range(len(infos)):
                children[2].insertRow(j)
                value = float(infos[j]) if isFloatNumber(infos[j]) or isIntNumber(infos[j]) else 0.0
                addLineWidget(self, children[2], j,  0, 4, 0, value)
     
        else :
            obj = getWidget(self, widgetKey)

            if not obj : return
            if obj.metaObject().className() == "QLineEdit" :
                if (isoXml[0])!= []: obj.setText(isoXml[0][0])
                
            elif obj.metaObject().className() in ("QCalendarWidget", "QgsDateTimeEdit") :
                 dateValue = ""
                 for elt in isoXml :
                     if elt[0]!=[] : dateValue = elt[0][0]
                     
                 if dateValue != "":
                     sep = ""    
                     if dateValue.find("T")!=-1 : sep = "T" 
                     elif dateValue.find(" ")!=-1 : sep = " "
                     if sep!="" : dateValue = dateValue.split(sep)[0] 
                     infosDate = dateValue.split("-")
                     try : obj.setDate(QDate(int(infosDate[0]), int(infosDate[1]), int(infosDate[2])))
                     except :
                         try : obj.setSelectedDate(QDate(int(infosDate[0]), int(infosDate[1]), int(infosDate[2]) ))
                         except : reinitDate(obj)
             
            elif obj.metaObject().className() in ("QTextEdit", "MyTextEdit", "myTextEdit") :
                if (isoXml[0])!= []:
                    try :  obj.setText(isoXml[0][0])
                    except :  obj.setPlainText(isoXml[0][0])
                
            elif obj.metaObject().className() == "QTableView" :
                   if obj.objectName() == "tablecategories": refcol, isoXml[0] = 1, [item.capitalize() for item in isoXml[0] if item!= None]
                   elif obj.objectName() == "tablelangues": refcol, isoXml[0] = 0, [item.lower() for item in isoXml[0] if item!= None]
                   else : refcol = 0
                   for i in range(obj.model().rowCount()):
                       if obj.model().item(i, refcol).text() in isoXml[0]:
                          obj.model().item(i, 0).setCheckState(Qt.Checked)
                          self.listImportCategories.append(obj.model().item(i, refcol).text())
                       else : obj.model().item(i, 0).setCheckState(Qt.Unchecked)
                       
            elif obj.metaObject().className() in ("QComboBox", "MyComboBox", "myComboBox") :
                 try : value = isoXml[0][0]
                 except : value = isoXml[0]
                 
                 if value == [] : return
                 if obj.objectName() == "tablecarac" :
                     if value == None :
                         try : value = isoParser.tablecaracmdd[0][0]
                         except : value = None
                     if value != None :
                         if value.find("MD_CharacterSetCode_")!=-1: value = value.replace("MD_CharacterSetCode_","")
                     else : value == "None"    
                     index = self.listCodecs.index(value) if value in (self.listCodecs) else 0
                     obj.setCurrentIndex(index)
                 if obj.objectName() == "typedata" :
                     index = self.typeData.index(value) if value in (self.typeData) else 0
                     obj.setCurrentIndex(index)
                 else :
                     index = obj.findText(value) if obj.findText(value) !=-1 else 0
                     obj.setCurrentIndex(index)
                     
            elif obj.metaObject().className() in ("QTableWidget", "MyTableWidget", "myTableWidget") :
                 self.cleanAllObj(obj, True)
                 rows = len(isoXml)

                 if obj.objectName() == "tablelocalisator" :
                    if isoXml == "None" : return
                    try :
                        if rows == 2 and isoXml[1]== [] :
                             rows = len(isoXml[0])
                             for i in range(rows):
                                 makeLine(self, obj, True, False,-1, True)
                                 try : obj.cellWidget(i, 0).setText("%s" % (isoXml[0][i]))
                                 except : pass
                             return
                    except : pass        

                 
                 if obj.objectName() == "tablescr" : 
                    rows = len(isoXml) 
                    orderByRows = True
                    if type(isoXml[0]) != dict :
                       if len(isoXml[0]) > 1 : rows = len(isoXml[0]); orderByRows = False
                 
                 for i in range(rows):
                     if obj.objectName() == "tableemprises":
                         if type(isoXml[i])==dict:
                             makeLine(self, obj, True, False,-1, True)
                             obj.cellWidget(i, 0).setValue(float(isoXml[i]['north']))
                             obj.cellWidget(i, 1).setValue(float(isoXml[i]['south']))
                             obj.cellWidget(i, 2).setValue(float(isoXml[i]['west']))
                             obj.cellWidget(i, 3).setValue(float(isoXml[i]['east']))
                     elif  obj.objectName() == "tableformats":
                         if type(isoXml[i])==dict:
                             makeLine(self, obj, True, False,-1, True)
                             obj.cellWidget(i, 0).setText("%s" % (isoXml[i]['name']))
                             obj.cellWidget(i, 2).setText("%s" % (isoXml[i]['version']))
                         elif type(isoXml[i])== list :
                             self.updateTableFormats(obj, isoXml[i])
                             return
                     elif obj.objectName() == "tablelocalisator" :
                         if type(isoXml[i])== dict :
                             if (isoXml[i]['url']!= None and isoXml[i]['url']!=' ' and isoXml[i]['name']!= None) :
                                 makeLine(self, obj, True, False,-1, True)
                                 if isoXml[i]['url']!= None : obj.cellWidget(i, 0).setText("%s" % (unquote(isoXml[i]['url']))) 
                                 if isoXml[i]['name']!= None : obj.cellWidget(i, 1).setText("%s" % (unquote_plus(isoXml[i]['name'])))
                         else :
                             makeLine(self, obj, True, False,-1, True)
                             try : obj.cellWidget(i, 0).setText("%s" % (isoXml[0][i]))
                             except : pass
                     elif obj.objectName() == "tablescr" :
                         cond = False
                         text = ""

                         if not orderByRows :
                            try :
                                if isoXml[0][i] != "" : text = "%s:%s" % (isoXml[0][i], isoXml[1][i])
                            except : 
                                   try : text = "%s:%s" % (isoXml[0][i], isoXml[2][i]) 
                                   except : break 
                         else :     
                            if type(isoXml[i]) == dict :
                                if dicoHasKey(isoXml[0], 'crs') :
                                    text = isoXml[i]['crs']
                                    if text != "" and text.find(":")!=-1 : text = text.split(":")[1]
                                elif (dicoHasKey(isoXml[i], 'authid')) :
                                    if isoXml[i]['authid'].find(":") != -1 : text = "EPSG:%s" % isoXml[i]['id']
                                    else : text = "%s:%s" % (isoXml[i]['authid'], isoXml[i]['id'])
                                else : text = ""                                              
                            else :
                                 try :
                                    if isoXml[0][i] != "" : text = "%s:%s" % (isoXml[0][i], isoXml[1][i])
                                 except : 
                                       try : text = "%s:%s" % (isoXml[0][i], isoXml[2][i]) 
                                       except : break

                         makeLine(self, obj, True, False,-1, True)

                         if text != "" :
                            try :
                                cond = QgsCoordinateReferenceSystem().createFromSrid(long(text))
                                text = "EPSG:%s" % text
                            except :
                                 try : cond = QgsCoordinateReferenceSystem().createFromUserInput(text)
                                 except : cond = False
                            if cond : obj.cellWidget(i, 0).setText("%s" % (text))
                         else : 
                             if obj.rowCount()==0 : obj.cellWidget(i, 0).setText("EPSG:3857")

                 if obj.objectName() == "tableemprises":
                    obj.setToolTip("%s" % (obj.rowCount()-1))
                    self.drawExtent()



    def updateTableFormats(self, obj, values):
        for k in range(len(values)):
            makeLine(self, obj, True, False,-1, True)
            obj.cellWidget(k, 0).setText("%s" % (values[k]))

    def updateConsistency(self, widgetKey, isoXml):
        obj = self.findChild(myTextEdit, "coherence")
        import ast
        try : consistencyDict = ast.literal_eval("%s" % (isoXml))
        except : consistencyDict = None
       
        if consistencyDict!= None :
           if type(consistencyDict)== dict :
               if dicoHasKey(consistencyDict, 'TopologyLevelCode') and dicoHasKey(consistencyDict, 'GeometricObjectTypeCode'): obj.setPlainText("{'TopologyLevelCode':'%s', 'GeometricObjectTypeCode':'%s'}" % (consistencyDict['TopologyLevelCode'],consistencyDict['GeometricObjectTypeCode']))
               else : obj.setPlainText("{'TopologyLevelCode':'unknow', 'GeometricObjectTypeCode':'unknow'}")
           elif type(consistencyDict)== list :
               if len(consistencyDict)== 2 and type(consistencyDict[0])==list and type(consistencyDict[1])==list:
                  try : topologyLevelCode = consistencyDict[0][0]
                  except : topologyLevelCode = 'unknow'
                  try : geometryTypeCode = consistencyDict[1][0]
                  except: geometryTypeCode = 'unknow'
                  obj.setPlainText("{'TopologyLevelCode':'%s', 'GeometricObjectTypeCode':'%s'}" % (topologyLevelCode, geometryTypeCode)) 
               else : obj.setPlainText("{'TopologyLevelCode':'unknow', 'GeometricObjectTypeCode':'unknow'}")
        else : obj.setPlainText("{'TopologyLevelCode':'unknow', 'GeometricObjectTypeCode':'unknow'}")
        

    def updateUserRights(self, widgetKey, isoXml, isoParser):
        if type(isoXml[0])==list:
            if isoXml[0]==[]: return
            if isoXml[0][0] == "otherRestrictions" : 
                counter=0
                obj = getWidget(self, widgetKey)
                constraints = isoParser.otherconstraints[0]
                child = obj.children()[2]
                model = child.model()
                for j in range(len(constraints)):
                    cond, index = self.isConstraintIn(constraints[j], model)
                    if cond :
                       child.model().item(index, 0).setCheckState(Qt.Checked)
                       counter+= 1
                if counter > 0 : obj.children()[1].setChecked(True)

    def isConstraintIn(self, constraintText, model):
        cond, index = False, -1
        #First, check the model ...
        for i in range(model.rowCount()):
            if constraintText.find(model.item(i, 0).text())!=-1 or  constraintText.find(model.item(i, 1).text())!=-1  : return True, i
        #Second test, on list INSPIRE
        constraints = ("(a) the confidentiality of the proceedings of public authorities, where such confidentiality is provided for by law.",
                          "(b) international relations, public security or national defence.",
                          "(c) the course of justice, the ability of any person to receive a fair trial or the ability of a public authority to conduct an enquiry of a criminal or disciplinary nature",
                          "(d) the confidentiality of commercial or industrial information, where such confidentiality is provided for by national or Community law to protect a legitimate economic interest, including  the  public  interest  in  maintaining  statistical confidentiality and tax secrecy.",
                          "(e) intellectual property rights",
                          "(f) the confidentiality of personal data and/or files relating to a natural person where that person has not consented to the disclosure of  the information to the public, where such confidentiality is provided for by national or Community law.",
                          "(g) the interests or protection of any person who supplied the information requested on a voluntary basis without being under, or capable of being put under, a legal obligation to do so, unless that person has consented to the release of the information concerned.",
                          "(h) the protection of the environment to which such information relates, such as the location of rare species.")
        for i in range(len(constraints)):
            if constraints[i].find(constraintText)!=-1 or constraintText.find(constraints[i])!=-1 : return True, i
        return cond, index


    def updateSpecifications(self, widgetKey, isoXml):
        dims = len(isoXml)
        obj = self.findChild(myTableWidget, widgetKey)
        if not obj : return
        self.cleanAllObj(obj, True)
        for i in range(dims):
            if isoXml[i]==[]:pass
            else :
                if type(isoXml[i])==dict:
                    makeLine(self, obj, True, False,-1, True)
                    try :
                        obj.cellWidget(i, 0).setText(isoXml[i]['text'])
                        obj.cellWidget(i, 1).setText(isoXml[i]['date'])
                        try : index = ("creation","revision","publication").index(isoXml[i]['typedate'].lower())
                        except : index = 0
                        if index == -1 : index = 0
                        obj.cellWidget(i, 2).setCurrentIndex(index)

                        if type(isoXml[i]['conformity'])== str :
                           index = 0 if isoXml[i]['conformity'].lower() == 'true' else 1 
                        else : index = 2                
                        obj.cellWidget(i, 3).setCurrentIndex(index)
                    except : pass

    def updateTemporalityExtents(self, widgetKey, isoXml, isoParser):
        obj = self.findChild(myTableWidget, widgetKey)
        if not obj : return
        self.cleanAllObj(obj, True)
        
        if type(isoXml[0])== dict :
           dims = len(isoXml) 
           for i in range(dims):
               makeLine(self, obj, True, False,-1, True)
               line = obj.rowCount()-1

               dateStart, dateEnd = "", ""
               try : dateStart = isoXml[i]['start']
               except : pass

               try : dateEnd = isoXml[i]['end']
               except : pass                      

               if dateStart.find("T")!=-1 : dateStart = dateStart.split("T")[0]
               if dateEnd.find("T")!=-1 : dateEnd = dateEnd.split("T")[0]
               
               obj.cellWidget(line, 0).setText("%s %s" % (dateStart, dateEnd))
        else :
           if len(isoXml[0])>0: 
              for i in range(0, len(isoXml[0]), 2):
                   makeLine(self, obj, True, False,-1, True)
                   line = obj.rowCount()-1

                   dateStart, dateEnd = "", ""
                   try : dateStart = isoXml[0][i]
                   except :
                       try : dateStart = isoXml[0][0]
                       except : pass

                   try : dateEnd = isoXml[0][i+1]
                   except :
                       try : dateEnd = isoXml[1][0]
                       except : pass                      

                   if dateStart.find("T")!=-1 : dateStart = dateStart.split("T")[0]
                   if dateEnd.find("T")!=-1 : dateEnd = dateEnd.split("T")[0]
            
                   obj.cellWidget(line, 0).setText("%s %s" % (dateStart, dateEnd))    


    def updateTableOptionalKeyWords(self, widgetKey, isoXml, isoParser):
        dims = len(isoXml)
        obj = self.findChild(myTableWidget, widgetKey)
        if not obj : return
        
        if isoXml == 'None' :
           isoXml = isoParser.keywordsFNC
           dims = len(isoXml[0])

           for i in range(dims):
               if type(isoXml[0]) == list:
                 if isoXml[0][i]!=None :
                     if isoXml[0][i]!="" and isoXml[0][i].capitalize() not in (self.listImportCategories) :
                        makeLine(self, obj, True, False,-1, True)
                        line = obj.rowCount()-1

                        obj.cellWidget(line, 0).setText("%s" % (isoXml[0][i]))
                        obj.cellWidget(line, 1).setChecked(False)
                        
                        for k in range(2,5):
                            itemEditLine = obj.cellWidget(line, k)
                            try : itemEditLine.setEnabled(False)
                            except : pass

        else :
            keys = {'keyword':0,'thesaurus':2,'date':3,'typedate':4}

            if isoXml == [{}]: return
            
            for i in range(dims):
                if type(isoXml[i])== dict :
                   if isoXml[i]['keyword']!=None : 
                       if isoXml[i]['keyword'].capitalize() not in (self.listImportCategories): 
                           makeLine(self, obj, True, False,-1, True)
                           line = obj.rowCount()-1

                           for key in isoXml[i] :
                               indexKey = keys[key]
                               if obj.cellWidget(line, indexKey).metaObject().className() in ("QComboBox", "MyComboBox", "myComboBox") :
                                  try : index = ("creation","revision","publication").index(isoXml[i][key])
                                  except : index = 0
                                  if index == -1 : index = 0
                                  obj.cellWidget(line, indexKey).setCurrentIndex(index)
                               else :
                                   obj.cellWidget(line, indexKey).setText("%s" % (isoXml[i][key]))

                               if indexKey == 2 :
                                  if not isoXml[i][key] in ("", None) : obj.cellWidget(line, 1).setChecked(True)
                                  else : obj.cellWidget(line, 1).setChecked(False)
  
                                       
                else :
                     if type(isoXml[i])== list and isoXml[i]!= []:
                         line = 0
                         for keyword in isoXml[i] :
                            if i == 0 : makeLine(self, obj, True, False,-1, True)
                            if obj.cellWidget(line, i).metaObject().className() in ("QComboBox", "MyComboBox", "myComboBox") :
                               try : index = ("creation","revision","publication").index(keyword)
                               except : index = 0
                               if index == -1 : index = 0
                               obj.cellWidget(line, i).setCurrentIndex(index)
                            else : obj.cellWidget(line, i).setText(keyword)
                            line+= 1
                        
                      

    def updateTableCategories(self, widgetKey, isoXml):
        if isoXml!=[[]]:
           obj = self.findChild(QTableView, widgetKey.split(":")[0])
           if not obj : return
           categories = isoXml[0]

           for i in range(len(categories)):
               category = categories[i]
               for k in range(obj.model().rowCount()):
                   if obj.model().item(k, 2).text().find("(%s)" % (category))!= -1 :
                       if obj.model().item(k, 0).checkState() == Qt.Unchecked :
                          obj.model().item(k, 0).setCheckState(Qt.Checked) 
                       break
             

    def updateTableRoles(self, widgetKey, isoXml):
         
         tableWidget = self.findChild(myTableWidget, widgetKey)
         if not tableWidget : return

         isoXml = sorted(isoXml)

         firstOwner, findPointOfContact = False, False
         if isoXml == [[]]: return
         for contact in isoXml :

               if notEmptyList(contact) :
             
                   if contact[0] == None :
                      if not firstOwner : row, firstOwner = 0, True
                      else : row, findPointOfContact = 1, True
                   elif contact[0].lower() == "pointofcontact" and not findPointOfContact : row, findPointOfContact = 1, True
                   else :
                       if contact[0].lower() == "owner" and not firstOwner : row, firstOwner = 0, True
                       else :        
                          makeLine(self, tableWidget, True, False,-1, True)
                          row = tableWidget.rowCount()-1
     
                   for i in range(len(contact)):
                       index = 0
                       if tableWidget.cellWidget(row, i).metaObject().className() in ("QComboBox", "MyComboBox", "myComboBox") :
                             if i == 0:
                                 zIndexes = [keyitem for keyitem, item in self.dictListOfRules.items() if item == contact[i]] 
                                 if zIndexes!=[] : index = zIndexes[0]
                             else : index = tableWidget.cellWidget(row, i).findText(contact[i].capitalize())
                             if index == -1 : index = 0
                             if row not in (0, 1) : tableWidget.cellWidget(row, i).setCurrentIndex(index) 
                       else : tableWidget.cellWidget(row, i).setText("%s" % (contact[i]))


    def changeScalesType(self):
        self.sender().parent().children()[3].setEnabled(self.sender().parent().children()[1].isChecked())
        if self.sender().parent().children()[0].isChecked():
           toolTip = self.sender().parent().children()[0].toolTip() 
           cible = 1 
        else :
           toolTip = self.sender().parent().children()[1].toolTip() 
           cible = 0 
        objValues = None

        if toolTip.find("|")!=-1: objValues = toolTip.split("|")
        else:
            if isFloatNumber(toolTip): valObj = float(toolTip)
            else : valObj = 0.0
        obj = self.sender().parent().children()[2] 
        row = obj.rowCount()
        saveObjValues = ""
        for i in range(row):
            widget = obj.cellWidget(i, 0)
            separator = "" if saveObjValues == "" else "|"
            saveObjValues+= "%s%s" % (separator, widget.value())
            
            if self.sender().parent().children()[0].isChecked(): propertiesDoubleSpinBox(self, widget, 0, 0, 10000000, 1000)
            else: propertiesDoubleSpinBox(self, widget, 4, 0, 1000, 10)
            
            if objValues == None : widget.setValue(valObj)
            else :
                if i <= (len(objValues)-1): widget.setValue(float(objValues[i]))
                else : widget.setValue(float(objValues[len(objValues)-1]))

        if saveObjValues!= "" : self.sender().parent().children()[cible].setToolTip("%s" % (saveObjValues))
            
    def changeRightsType(self): self.sender().parent().children()[2].setEnabled(self.sender().parent().children()[1].isChecked())

    def testUrl(self): testUrl(self)
    def goToLocalisator(self): goToLocalisator(self)

    def CallSelectorConformityInfos(self):
        obj = self.findChild(myTextEdit, "coherence")
        dialog = dlgTopology(obj.toPlainText())
        if dialog.exec_(): obj.setPlainText("%s" % (dialog.infosTopology()))
        
    def CallQgsProjectionSelector(self):
         obj = self.findChild(myTableWidget, "tablescr")
         index = int(self.sender().objectName().split("_")[2])
         itemEditLine = obj.cellWidget(index, 0)
         changeButtonIcon(self, self.sender(),"projectionactivate.png", 18, 16)
         self.parent.myDock.setVisible(False)
         dialog = dlgCrs(self, itemEditLine.text())
         windowIcon(dialog, "projection.png")
         dialog.setWindowTitle(QApplication.translate("QSphere","Coordinate Reference System Selector"))
         if dialog.exec_(): itemEditLine.setText("%s" % (dialog.crs().authid()))
         self.parent.myDock.setVisible(True)          
         changeButtonIcon(self, self.sender(),"projection.png", 18, 16)

    def CallFormatSelector(self):
        obj = self.findChild(myTableWidget, "tableformats")
        index = int(self.sender().objectName().split("_")[2])
        itemEditLine = obj.cellWidget(index, 0)
        changeButtonIcon(self, self.sender(),"formatactivate.png", 18, 16)
        dialog = dlgDataFormat(self.formats, itemEditLine.text())
        windowIcon(dialog, "format.png")
        if dialog.exec_(): itemEditLine.setText("%s" % (dialog.format()))
        changeButtonIcon(self, self.sender(),"format.png", 18, 16)

    def checkRules(self): checkRules(self)
    def updatePatternZipPostalCode(self): updatePatternZipPostalCode(self)
    def listOfValues(self): listOfValues(self)

    def getToolTipCalendar(self):
        calendarWidget = self.findChild(QgsDateTimeEdit, "%s" % (self.sender().objectName()))
        toolTip = toolTipCalendar(self, calendarWidget)

    def setAccessibility(self, checked): setAccessibility(self, checked)

    def checkSelectKeyWords(self):
        obj = self.findChild(QTableView, "tablecategories")
        targetObj = self.findChild(myTableWidget, "tablemotsclefso")
        for row in range(obj.model().rowCount()):
            item = obj.model().item(row, 1)
            item = obj.model().item(row, 0)
            if item.checkState() == Qt.Checked : 
                makeLine(self, targetObj, True, False,-1, True)
                targetItem = targetObj.cellWidget(targetObj.rowCount()-1, 0)
                targetItem.setEnabled(True)
                targetItem.setCurrentIndex(0)
                targetItem.setEnabled(False)
                targetItem = targetObj.cellWidget(targetObj.rowCount()-1, 1)
                for i in range(targetItem.count()):
                    if targetItem.itemText(i).upper() == item.text().upper():
                       targetItem.setCurrentIndex(i)
                       targetItem.setEnabled(False)
                       break

                
    def selectKeyWord(self, item):
        obj = self.findChild(QTableView, "tablecategories")
        itemModel = obj.model().item(item.row(), 1)
        targetObj = self.findChild(myTableWidget, "tablemotsclefso")

        if item.checkState() == Qt.Checked :
            makeLine(self, targetObj, True, False, -1, True)
            targetItem = targetObj.cellWidget(targetObj.rowCount()-1, 0)
            targetItem.setEnabled(True)
            targetItem.setCurrentIndex(0)
            targetItem.setEnabled(False)
            targetItem = targetObj.cellWidget(targetObj.rowCount()-1, 1)
            for i in range(targetItem.count()):
                if targetItem.itemText(i).upper() == itemModel.text().upper():
                   targetItem.setCurrentIndex(i)
                   targetItem.setEnabled(False)
                   break
        else:
            for line in range(targetObj.rowCount()):
                targetItem = targetObj.cellWidget(line, 1)
                if targetItem == None : break
                if targetItem.currentText().upper() == itemModel.text().upper():
                   targetObj.removeRow(line)
                   break
        countItems(self, "tablemotsclefso", targetObj)

    def majItemLangues(self, item): self.countLangues()
        
    def countLangues(self):
        lbl = self.findChild(QLabel, "lbltablelangues")
        if not lbl : return
        obj = self.findChild(QTableView, "tablelangues")
        if not obj : return
        rows, counter, listlangues = obj.model().rowCount(), 0, ""
        for i in range(rows):
            if obj.model().item(i, 0).checkState()== Qt.Checked:
                counter+= 1
                newLine = "<br>" if counter % 4 == 0 else ""
                listlangues+= "<b>%s</b> %s" % (obj.model().item(i, 0).text(), newLine)
        roothLbl = QApplication.translate("QSphere","Language(s) for the resource : ")
        lbl.setText("(<b><u>%s</u></b>) %s<br>%s" % (counter, roothLbl, listlangues))


    def cleanAllObj(self, obj, doCount):
        if not obj : return
        obj.clearContents()
        for i in range(obj.rowCount()): obj.removeRow(0)
        if doCount : countItems(self, obj.objectName(), obj)        

    def doDessCadre(self, currentRow, currentColumn, previousRow, previousColumn):
        if currentRow!= previousRow : self.drawExtent()
                    
    def drawExtent(self):
        longitudeBound = int(self.longitudeExtent/2)
        latitudeBound = int(self.latitudeExtent/2)
        extentIndex = -1

        if self.sender()==None or self.sender().objectName()=="" or self.sender().objectName() == "mActionMetadata" :
           obj = self.findChild(myTableWidget, "tableemprises")
           extentIndex = getIndex(obj)
        else:
           nameObj = "%s" % (self.sender().objectName())
           obj = self.findChild(myTableWidget, "tableemprises")

           if obj != None : extentIndex = obj.currentRow()

        if extentIndex == -1: return
        item = obj.cellWidget(extentIndex, 0)        
        if item == None : return
       
        item = obj.cellWidget(extentIndex, 2)
        positionStartX = int(item.value())+ longitudeBound

        item = obj.cellWidget(extentIndex, 3)
        positionEndX = int(item.value())+longitudeBound
       
        item = obj.cellWidget(extentIndex, 0)
        positionStartY = latitudeBound-int(item.value())

        item = obj.cellWidget(extentIndex, 1)
        positionEndY = latitudeBound-int(item.value())

        self.cadre.setGeometry(positionStartX, positionStartY, (positionEndX - positionStartX), (positionEndY - positionStartY))
        self.latitudeAxe.setGeometry(positionStartX + int((positionEndX - positionStartX)/2), 0, 1, self.latitudeExtent)
        self.longitudeAxe.setGeometry(0, positionStartY - int((positionStartY - positionEndY)/2), self.longitudeExtent, 1)


    def addDragDropLine(self, Obj, data):
        makeLine(self, Obj, True, False, -1, True)
        widget = Obj.cellWidget(Obj.rowCount()-1, 0)
        if not widget : return
        if widget.metaObject().className() == "mySpinBox" :
           try : widget.setValue(float(data))
           except :
               try : widget.setValue(int(data))
               except : pass
        elif widget.metaObject().className() in ("QComboBox", "MyComboBox", "myComboBox") :
             originalIndex = widget.currentIndex()
             index = widget.findText("%s" % (data))
             if index == -1 and originalIndex == -1 : index = 0
             else : index = originalIndex
             widget.setCurrentIndex(index)             
        else : widget.setText("%s" % (data))

def CleanSheetTable(self, table, dataFile, linkedTable): 
     items = []
     for k in range(table.model().rowCount()):
         item = table.model().item(k, 0)
         if item.checkState()== Qt.Checked: items.append(k)
     table.clearSpans()

     if linkedTable != None : 
        listKeyWords = self.findChild(myTableWidget, linkedTable)
        if listKeyWords :
           listKeyWords.clearContents()
           listKeyWords.setRowCount(0)

     self.tableModel = QStandardItemModel(self)
     sizeWidth, cols, line = loadFile(self, table, table.objectName(), dataFile, 3, self.tableModel)
     table.setModel(self.tableModel)
     for k in range(len(items)):
         item = table.model().item(items[k], 0)
         item.setCheckState(Qt.Checked)


def checkRules(self):
    if self.showWarning :
        obj = self.sender()
        objParent = self.findChild(myTableWidget, "tableroles")
        if obj.currentIndex() == 6 and objParent.rowCount() > 2 :
           title = QApplication.translate("QSphere", "Warning" ) 
           msg = QApplication.translate("QSphere"," One PointOfContact ! Please ...")
           try : sendMsg(self, title, msg, Qgis.Warning, self.duration_warning)
           except : pass
           obj.setCurrentIndex(1)

def addLine(self):

    objName = "%s" % (self.sender().objectName())

    infos = objName.split("_")
    rooth = infos[0]
    typeObj = int(infos[1])
    targetWidget = infos[2]
    
    if targetWidget.startswith("groupe"): obj = self.findChild(QGroupBox, targetWidget)
    else : obj = self.findChild(myTableWidget, targetWidget)
   
    if obj == None : return
    if obj.metaObject().className() == "QGroupBox" : obj = getWidget(self, "tableechelles")

    row = -1 if rooth != "addAfter" else (obj.currentRow()+1)
    makeLine(self, obj, True, False, row, True)

    if self.objectName() == "dlgMetadata" : renameAllWidgets(self, obj)
    obj.selectRow(row)
    obj.setFocus()



def makeLine(self, obj, drawResult, init, row, isCount):
    collection = []
    objWidgets = []

    key = obj.objectName()
    if not dicoHasKey(self.paramsLineWidget, key): return
    if row == -1 : row = obj.rowCount()

    params = self.paramsLineWidget[key]
    
    if obj.objectName() in ("tablescr", "tableformats", "tablelocalisator", "tableemprises", "tabledatepubdata", "tableechelles") :
       obj.insertRow(row)
       for i in range(len(params)):
           widget = addLineWidget(self, obj, row, i, params[i][0], params[i][1], params[i][2])
           objWidgets.append(widget)
       collection.append(objWidgets)    

       if obj.objectName() == "tableemprises" :
          obj.setToolTip("%s" % (row))
          if drawResult and self.objectName() == "dlgMetadata" : self.drawExtent()

    elif obj.objectName() in ("tableetenduetemporelle", "tablemotsclefso", "tablemotsclefsf", "tablespecifications") :
       if not init :
           getCheckBox = False
           obj.insertRow(row)
           for i in range(len(params)):
               widget = addLineWidget(self, obj, row, i, params[i][0], params[i][1], params[i][2])
               objWidgets.append(widget)
               if params[i][0] == 6 and obj.objectName() == "tablemotsclefsf" :
                   getCheckBox = True
                   widgetCheckBox, defaultValue = widget, params[i][2]

           if getCheckBox :
              if type(defaultValue)!= int : defaultValue = 2
              widgetCheckBox.setCheckState(defaultValue)
              widgetCheckBox.toggled.emit(False)
               
           collection.append(objWidgets)

       if obj.objectName() == "tablespecifications" :
           if init : 
              for k in range(len(self.listOfConformities)):
                  obj.insertRow(row)
                  for i in range(len(params)):
                      widget = addLineWidget(self, obj, row, i, params[i][0], params[i][1], params[i][2])
                      objWidgets.append(widget)
                  collection.append(objWidgets)
                      
                  for j in range(4):
                      itemEditLine = obj.cellWidget(row, j)
                      if itemEditLine.metaObject().className() in ("QLineEdit", "MySimpleWidgetLineEdit", "MyWidgetLineEdit", "MySimpleWidgetLineEditST"): itemEditLine.setText(self.listOfConformities[k][j]) 
                      elif itemEditLine.metaObject().className() in ("QComboBox", "MyComboBox", "myComboBox"):
                          try : itemEditLine.setCurrentIndex(int(self.listOfConformities[k][j]))
                          except : itemEditLine.setCurrentIndex(0)
           
    
    elif obj.objectName() == "tableroles" :
       obj.insertRow(row)
       if row == 0 : value = 2
       elif row == 1 : value = 6
       else : value = 1

       for i in range(len(params)):
           if i== 0 : widget = addLineWidget(self, obj, row, i, params[i][0], params[i][1], value)
           else : widget =  addLineWidget(self, obj, row, i, params[i][0], params[i][1], params[i][2])
           objWidgets.append(widget)
       collection.append(objWidgets)
       
       if init :
          row = obj.rowCount()
          obj.insertRow(row)
          for i in range(len(params)):
              if i== 0 and row == 1: widget = addLineWidget(self, obj, row, i, params[i][0], params[i][1], 6)
              else : widget = addLineWidget(self, obj, row, i, params[i][0], params[i][1], params[i][2])
              objWidgets.append(widget)
          collection.append(objWidgets)
                
    if isCount : countItems(self, obj.objectName(), obj)

    return objWidgets

def deleteLineProcess(self, nameObj, lastLine):
   
    if nameObj == "" : return
    infos = nameObj.split("_")
    if countCaractere(nameObj, "_", False, False)!=2 : return
    typeObj = int(infos[1])
    targetWidget = infos[2]
    
    obj = self.findChild(QGroupBox, targetWidget) if targetWidget.startswith("groupe") else self.findChild(myTableWidget, targetWidget)    
    
    title = QApplication.translate("QSphere","Information")

    if obj == None : return
    
    if typeObj !=8 :      
        if obj.rowCount()==0 and self.showWarning :
           msg = QApplication.translate("QSphere","Nothing to delete.")
           sendMsg(self, title , msg, Qgis.Warning, self.duration_warning)
           return

    if targetWidget == "tableroles" :
       minimumRows, msg = 2, QApplication.translate("QSphere","Two reference information must be defined !") 
    else :
       if targetWidget in("tablemotsclefso", "tablemotsclefsf", "tableetenduetemporelle", "tablespecifications", "tableformats") : minimumRows, msg = 0, QApplication.translate("QSphere","Information must be set at least !")
       else : minimumRows, msg = 1, QApplication.translate("QSphere","Information must be set at least !")
    
    if typeObj == 8: obj = obj.children()[2]
    if obj.rowCount() > minimumRows :
        if lastLine : index = (obj.rowCount()-1)
        else :
           if obj.rowCount()>0 : index = 0 if obj.currentRow()== -1 else obj.currentRow()
        if targetWidget == "tableroles" :
           if index < 2 and self.showWarning :
              msg = QApplication.translate("QSphere","Can delete the current line.")
              sendMsg(self, title , msg, Qgis.Warning, self.duration_warning)
              return
        if targetWidget == "tablemotsclefso" and self.showWarning :
              if obj.cellWidget(index, 0).isEnabled()==False and obj.cellWidget(index, 1).isEnabled()==False :
                  msg = "%s<br>%s" % (QApplication.translate("QSphere","Can delete the current line."),
                                      QApplication.translate("QSphere","Use thematic selection to delete this line."))
                  sendMsg(self, title , msg, Qgis.Warning, self.duration_warning)
                  return
            
        obj.removeRow(index) 
        if targetWidget == "tableemprises":
           obj.setToolTip("%s" % (obj.currentRow())) 
           if self.objectName() == "dlgMetadata" : self.drawExtent()
    else:
        if self.showWarning :
            msg = QApplication.translate("QSphere","Can not delete the last line.")+ "<br>%s" % (msg)
            sendMsg(self, title , msg, Qgis.Warning, self.duration_warning)
    countItems(self, targetWidget, obj)
    renameAllWidgets(self, obj)
    obj.selectRow(obj.currentRow()) 
    obj.setFocus()


def moveLineUpProcess(self, nameObj):
    if nameObj == "" : return
    infos = nameObj.split("_")
    if countCaractere(nameObj, "_", False, False)!= 2 : return
    typeObj = int(infos[1])
    targetWidget = infos[2]
    
    if targetWidget.startswith("groupe"): obj = self.findChild(QGroupBox, targetWidget)
    else : obj = self.findChild(myTableWidget, targetWidget)
    
    if obj == None : return
    
    if obj.metaObject().className() == "QGroupBox" :
       targetWidget = "tableechelles"
       obj = getWidget(self, targetWidget)
    if type(obj ) != myTableWidget : return
    if obj.rowCount() == 0 : return
   
    row = obj.currentRow()
    if row > 0:
        isShowWarning = self.showWarning
        if self.objectName() == "dlgMetadata" : self.showWarning = False
        NewCollectionObjWidget = makeLine(self, obj, True, False, row-1, False)
        
        for i in range(obj.columnCount()):
           OriginalWidget = (obj.cellWidget(row+1,i))
           if OriginalWidget == None : OriginalWidget = (obj.item(row+1,i))
           NewObjWidget = NewCollectionObjWidget[i]
           if OriginalWidget.metaObject().className() in ("QLineEdit", "MySimpleWidgetLineEdit", "MyWidgetLineEdit", "MySimpleWidgetLineEditST") : NewObjWidget.setText(OriginalWidget.text())
           elif OriginalWidget.metaObject().className() in ("QCheckBox", "MyCheckBox","myCheckBox" ) : NewObjWidget.setChecked(OriginalWidget.isChecked())
           elif OriginalWidget.metaObject().className() in ("QComboBox", "MyComboBox", "myComboBox") :
                  if targetWidget!= "tableroles" or i != 0 : NewObjWidget.setCurrentIndex(OriginalWidget.currentIndex())
           elif OriginalWidget.metaObject().className() == "mySpinBox" : NewObjWidget.setValue(OriginalWidget.value())     
        
        obj.selectRow(row-1)
        obj.removeRow(row+1)
        if targetWidget == "tableroles" : self.CheckTable(obj)          
        if self.objectName() == "dlgMetadata" : self.showWarning = isShowWarning            
        renameAllWidgets(self, obj)

           
def moveLineDownProcess(self, nameObj):
    if nameObj == "" : return
    infos = nameObj.split("_")
    if countCaractere(nameObj, "_", False, False)!= 2 : return
    rooth = infos[0]
    typeObj = int(infos[1])
    targetWidget = infos[2]

    if targetWidget.startswith("groupe"): obj = self.findChild(QGroupBox, targetWidget)
    else : obj = self.findChild(myTableWidget, targetWidget)    

    if obj == None : return
    
    if obj.metaObject().className() == "QGroupBox" :
       targetWidget = "tableechelles"
       obj = getWidget(self, targetWidget)
    if type(obj ) != myTableWidget : return
    if obj.rowCount() == 0 : return

    row = obj.currentRow()
    if row == -1 : row = 0
    bounds = obj.rowCount()-1 if rooth == "moveDown" else obj.rowCount()
    if row < 0 and rooth == "CloneLine" : return

    if row < bounds :
        rowcible = (row+2) if rooth == "moveDown" else (row+1)
        isShowWarning = self.showWarning
        if self.objectName() == "dlgMetadata" : self.showWarning = False
        NewCollectionObjWidget = makeLine(self, obj, True, False, rowcible, False) 
        for i in range(obj.columnCount()):
           OriginalWidget = obj.cellWidget(row,i)
           NewObjWidget = NewCollectionObjWidget[i]
           if OriginalWidget == None : OriginalWidget = obj.item(row,i)
           if OriginalWidget.metaObject().className() in ("QLineEdit", "MySimpleWidgetLineEdit", "MyWidgetLineEdit", "MySimpleWidgetLineEditST") : NewObjWidget.setText(OriginalWidget.text())
           elif OriginalWidget.metaObject().className() in ("QCheckBox", "MyCheckBox", "myCheckBox") : NewObjWidget.setChecked(OriginalWidget.isChecked())
           elif OriginalWidget.metaObject().className() in ("QComboBox", "MyComboBox", "myComboBox") :
                  if targetWidget!= "tableroles" or i != 0 : NewObjWidget.setCurrentIndex(OriginalWidget.currentIndex())
           elif OriginalWidget.metaObject().className() == "mySpinBox" : NewObjWidget.setValue(OriginalWidget.value())                      
       
        if rooth == "moveDown" :
            obj.selectRow(rowcible)
            obj.removeRow(row)
        if targetWidget == "tableroles" : self.CheckTable(obj)          
        if self.objectName() == "dlgMetadata" : self.showWarning = isShowWarning
        renameAllWidgets(self, obj)


def renameAllWidgets(self, obj):
    for row in range(obj.rowCount()):
        for col in range(obj.columnCount()):
            widget = obj.cellWidget(row, col)
            name = widget.objectName()
            if countCaractere(name, "_", False, False) == 2 :
               name = name.split("_")
               fullName = "%s_%s_%s" % (name[0], name[1], row)
            elif countCaractere(name, "_", False, False) == 3 :
               name = name.split("_") 
               fullName = "%s_%s_%s_%s" % (name[0], name[1], row, col)
            else : fullName = name   
            widget.setObjectName(fullName) 
    countItems(self, obj.objectName(), obj)


def callInitSizeCols(self): 
      nameObj = self.sender().objectName().split("_")[2]
      obj = self.findChild(myTableWidget, nameObj)
      if dicoHasKey(self.tablesWidget, nameObj):
         dim = self.tablesWidget[nameObj][0]
         initSizeCols(self, obj, dim)

         
def listOfValues(self):
    if self.sender().objectName() == "" : return
    rootObjName = self.sender().objectName().split("_")[0]
    comboBoxRow = int(self.sender().objectName().split("_")[2])
    indexThesaurus = self.sender().currentIndex()

    obj = self.findChild(myTableWidget, rootObjName)
    if obj != None :
        widget = obj.cellWidget(comboBoxRow, 1)
        if widget :
           widget.clear()
           sizeWidth, cols, line = loadFile(self, widget, "", "file:200:thesaurus_%s_%s.csv:0:0" % (indexThesaurus, self.qspherePlugin.dicoLangs[self.qspherePlugin.indexLang]), 1, None) 

def geoLocalisator(self): 
    if self.objectName() == "dlgMetadata" : self.drawExtent()
    obj = self.findChild(myTableWidget, "tableemprises")
    if obj == None : return
    row = int(self.sender().objectName().split("_")[2]) 
    if row==-1 : obj.selectRow(0)
    else : obj.selectRow(row)
    
    emprise = []
    for j in range(obj.columnCount()-1): emprise.append(obj.cellWidget(obj.currentRow(),j).value())
    
    if self.badNetwork == None : self.badNetwork = badNetwork("http://server.arcgisonline.com/")
    
    changeButtonIcon(self, self.sender(),"voiractivate.png", 24, 24)
    self.parent.myDock.setVisible(False)
 
    from qsphere.qsphere_objmaker import geoLocalisatorDialog
    d = geoLocalisatorDialog(self, emprise, obj, obj.currentRow(), True, False, self.badNetwork)
    d.exec_()
    self.parent.myDock.setVisible(True)
    changeButtonIcon(self, self.sender(),"voir.png", 18, 16)

def setAccessibility(self, checked):
    if self.sender()==None or self.sender().objectName()=="": return
    else:
       nameObj = "%s" % (self.sender().objectName())
       obj = self.findChild(myTableWidget, "tablemotsclefsf")
       if nameObj.find("tablemotsclefsf_action_")!=-1: row = int(nameObj.split("_")[2])
       else: row = getIndex(obj)

       for col in range(2, 5):
           itemEditLine = obj.cellWidget(row, col)
           try : itemEditLine.setEnabled(checked)
           except : pass

def updatePatternZipPostalCode(self): patternZipPostalCode(self, self.sender())


def testUrl(self):

    nameTableWidget = self.sender().objectName().split("_")[2]
    tableWidget = self.findChild(myTableWidget, nameTableWidget)
    
    if tableWidget :
        
        col = 0 if nameTableWidget == "tablelocalisator" else tableWidget.columnCount()-1
        self.movieCtrl.startMovie()
        
        if tableWidget.currentRow()==-1: tableWidget.selectRow(0)
        widget = tableWidget.cellWidget(tableWidget.currentRow(), col)
        url = widget.text() 

        if url != "" :
            sites = [('%s' % (url), 'GET', {'Content-Type': 'application/xml'}, 'None')] 
            try : isValidUrl = multiGetRequest(self.movieCtrl.movie, self.movieCtrl, sites, self.duration_timeout, 0)[0][1]
            except : isValidUrl = None
        else : isValidUrl = None    

        if isValidUrl != None :
           title = QApplication.translate("QSphere", "Information" ) 
           textForValidUrl = QApplication.translate("QSphere", "Valid")
           icon = Qgis.Info
           duration = self.duration_info
        else :
           title = QApplication.translate("QSphere", "Warning" ) 
           textForValidUrl = QApplication.translate("QSphere", "Not valid")
           icon = Qgis.Warning
           duration = self.duration_warning
        msg = "%s [%s %s]:<br>%s<br>%s." % (QApplication.translate("QSphere", "Current URL"),
                                         QApplication.translate("QSphere", "row"),
                                         tableWidget.currentRow(), url, textForValidUrl)
        sendMsg(self, title, msg, icon, duration)
        self.movieCtrl.stopMovie()

def initComments(self): 
     if self.windowTitle() != self.rootWindowTitle and self.windowTitle().find("~")== -1 :
        fileName = self.windowTitle().replace(self.rootWindowTitle, "")
        if fileName!="":
            config = ConfigParser()
            config.read(fileName)
            
            obj = self.findChild(myTableWidget, "tabledico")
            if obj == None : return
            
            test, rows, cols = analyzeInfosObj(self, config, obj, "tabledico")
            if rows == 0 : return

            bounds = obj.rowCount() if obj.rowCount()<= rows else rows
            
            for i in range(bounds):
                try : infos = config.get("tabledico",'zRow_%s' % (i))
                except : infos = ""
                classWidgetValues = infos.split("|")
                if len(classWidgetValues)== cols :
                   widget = obj.cellWidget(i, 1)
                   fieldName =  widget.text() if widget != None else obj.item(i, 1).text()
                   if fieldName == classWidgetValues[1]: 
                       itemComment = QTableWidgetItem()
                       itemComment.setText("%s" %(classWidgetValues[cols-1]))
                       obj.setItem(i,5,itemComment)
                       obj.item( i, 5 ).setFlags(Qt.ItemIsSelectable | Qt.ItemIsEnabled | Qt.ItemIsEditable | Qt.ItemIsDragEnabled | Qt.ItemIsDropEnabled)
                else : pass 

def goToLocalisator(self):

    isModal = False if self.objectName()=="dlgMetadata" else True

    nameTableWidget = self.sender().objectName().split("_")[2]
    tableWidget = self.findChild(myTableWidget, nameTableWidget)
    
    if tableWidget :
        
        col = 0 if nameTableWidget == "tablelocalisator" else tableWidget.columnCount()-1  
    
        if tableWidget.currentRow()==-1: tableWidget.selectRow(0)
        widget = tableWidget.cellWidget(tableWidget.currentRow(), col)
        
        if widget.styleSheet()== "background-color:#AEEE00;" :
           from qsphere.ui import loadDlgNavigator
           d = loadDlgNavigator(self, self.iface,  widget.text(), False, [], self.localLangage, self, None, "navigatorweb.png", isModal)
           if not isModal : self.childswindows.append(d)
        else :
           title = QApplication.translate("QSphere", "Warning" ) 
           msg = QApplication.translate("QSphere","The URL you entered is incomplete or invalid ! Please correct the line : ")
           try : sendMsg(self, title, "%s%s" % (msg, tableWidget.currentRow()), Qgis.Warning, self.duration_warning)
           except : pass    

def makeTabWidgets(self, tab, iconTab, index, labelWidth, labelHeight, sizeWidth, sizeHeight):
        referentSizeWidth, referentSizeHeight = sizeWidth, sizeHeight
  
        if index == 1 :
            widgets = {
                       "B_Intitule" : (self.width()-260,0,"intitule", 2, 5, (), True, False),
                       "C_Resume" : (self.width()-260,100, "resume", 3, 0, (),  True, False),
                       "D_Type" : (self.width()-260, 0, "typedata", 4, 1, self.listTypeRessources, True, False),
                       "E_Localisator" : (self.width()-260, 15, "tablelocalisator", 5, 6, (), True, False),
                       "F_Identificator" : (300, 0, "identificator", 8, 5, (), True, False),
                       "G2_lang" : (self.width()-260, 0, "tablelangues", 9, 3, "txt:", True, False),
                       "H_format" : (240, 12, "tableformats", 10, 6, (), True, True),
                       "I_carac" : (100, 0, "tablecarac", 14, 1, self.listCodecs, True, True) 
                      }
        elif index == 2 :
             widgets = {
                        "B_Categorie" : (self.width()-260, 100,"tablecategories", 15, 3, "file:300:categories_thematiques_%s.csv:1:0" % (self.qspherePlugin.dicoLangs[self.qspherePlugin.indexLang]), True, False),
                        "D_MotsClefsO" : (self.width()-260, 18,"tablemotsclefso", 19, 6, (), True, False),
                        "E_MotsClefsF" : (self.width()-260, 18,"tablemotsclefsf", 22, 6, (), True, False)
                       }
   
        elif index == 3 :
             widgets = {
                        "B_TITLE" : (0, 0,"subtitletab3_1", 30, -2, (), True, False),
                        "C_TableSCR" :(self.width()-260, 20,"tableemprises", 31, 6, (), True, False),
                        "D_TITLE" : (0, 0,"subtitletab3_2", 37, -2, (), True, False),
                        "E_TableScr" :(self.width()-635, 0,"tablescr", 38, 6, "EPSG:3857", True, False)
                       }

             mapPreview = QLabel(tab)

             mapPreview.setMinimumSize(360, 180)
             mapPreview.setMaximumSize(360, 180)
             mapPreview.setGeometry(self.width()-380, 310, 360, 180)
             icon = getThemeIcon("world.png")
             carIcon = QImage(icon) 
             mapPreview.setPixmap(QPixmap.fromImage(carIcon))

             self.cadre = myLabel(mapPreview)
             self.latitudeAxe = myLabel(mapPreview)
             self.longitudeAxe = myLabel(mapPreview)

        elif index == 4 :
            widgets = {
                       "B_Etendue" : (160, 15,"tableetenduetemporelle", 43, 6, (), True, True),
                       "C_DatePub" : (120, 15, "tabledatepubdata", 46, 6, (), True, False),
                       "D_DateCre" : (80, 0, "datecredata", 49, 4, (), True, False),
                       "E_DateRev" : (80, 0, "daterevdata", 50, 4, (), True, False),
                       "F_SysRefTemp" : (0, 0, "sysreftemp", 51, 1, self.listTemporalSystem, True, False) 
                      }
        elif index == 5 :
            widgets = {
                       "B_TITLE" : (0, 0,"subtitletab5_1", 54, -2, (), True, False),
                       "C_Genealogie" : (self.width()-260, 100,"genealogie", 55, 0, (),  True, False),
                       "D_TITLE" : (0, 0,"subtitletab5_2", 56, -2, (), True, False),
                       "E_Echelle" : (400, 120, "grouperesolutionscale", 57, 8, (("%s 1/" % QApplication.translate("QSphere", self.libelles[57][2]),"resolution_scale",25000.00),(QApplication.translate("QSphere","Equivalent scale in unit of measure "),"resolution_pixel",2.0)), True, False),
                       "F_Coherence" : (self.width()-300, 0,"coherence", 58, 0, "{'TopologyLevelCode':'unknow', 'GeometricObjectTypeCode':'unknow'}", False, False),
                       "G_TITLE" : (0, 0,"subtitletab5_3", 59, -1, (), True, False),
                       "H_Conformite" : (self.width()-260, 20,"tablespecifications", 60, 6, (), True, False)
                      }


        elif index == 6 :
            widgets = {
                       "B_droits" : (self.width()-260, self.height()-190,"groupedroits", 67, 8,
                                     ((QApplication.translate("QSphere","No restriction for public access in INSPIRE"),"norestriction_droits",25000.00),
                                      (QApplication.translate("QSphere","With restriction for public access in INSPIRE (Directive 2007/2/CE)"),"restriction_droits",2.0)), True, False),
                       "C_Licence" : (self.width()-260, 50,"licence", 68, 0, "%s" % ((QApplication.translate("QSphere","Open Licence"))), True, False)
                      }
   
        elif index == 7 :
            widgets = {
                       "B_TableRole" :(self.width()-260, 60,"tableroles", 71, 6, (), True, False) 
                      }
            
        elif index == 8 :
            widgets = {
                       "B_DateMetadata" : (80, 0, "datemetada", 83, 4, (), True, False),
                       "C_LangMetadata" : (80, 0, "langmetada", 84, 1, self.languesDico, True, False) 
                      }
        elif index == 9 :
            widgets = {
                       "B_NameLayer" : (self.width()-260,0,"namelayer", 87, 5, (), False, False),
                       "C_TypeLayer" : (200,0,"typelayer", 88, 5, (), False, False),
                       "D_Metadata" : (self.width()-260, 150,"metadata", 89, 0, (), True, False),
                       "E_TableDico" :(self.width()-260, 45,"tabledico", 90, 6, (), True, False)
                       }
        else : return


        posX, posY = 200, 20 
        
        keys = sortDic(widgets)
        for j in range(len(keys)):    
            key = keys[j]
            listObjProperties = widgets[key]
            sizeWidth =  listObjProperties[0] if listObjProperties[0] != 0 else referentSizeWidth
            sizeHeight =  listObjProperties[1] if listObjProperties[1] != 0 else referentSizeHeight
            nameObj, indexObj = listObjProperties[2], listObjProperties[3]
            typeObj, valObj   = listObjProperties[4], listObjProperties[5]
            isEnabled, isVertical = listObjProperties[6], listObjProperties[7]
            corPosY = makeWidget(self, tab, nameObj, typeObj, valObj, posX, posY, sizeWidth, sizeHeight, referentSizeHeight, isEnabled, isVertical, indexObj)
            if isVertical : posX+= sizeWidth + 200
            else:
               posY+= 30 if corPosY == 0 else corPosY
               posX = 200


        
        iconTab = getThemeIcon(iconTab)
        iconTab = QImage(abspath(iconTab).replace("\\", "/"))
        lblImgTab = QLabel(tab)
        lblImgTab.setPixmap(QPixmap.fromImage(iconTab))
        lblImgTab.setGeometry(5, 20, 32, 32)

        objs = tab.children()
        maxPosY, maxHeight, maxWidth = 0, 0, 0 
        for i in range(len(objs)-1, 0, -1) :
            obj = objs[i]
            maxPosY = max(maxPosY, obj.y())
          
        for i in range(len(objs)-1, 0, -1) :
            obj = objs[i]
            if obj.y() == maxPosY :
                maxHeight = max(maxHeight, obj.height())

        self.heights.append(maxPosY+maxHeight)
            

def makeWidget(self, tab, objName, objType, objValue, coordX, coordY, objWidth, objHeight, referentHeight, objIsEnable, objIsVertical, objIndex):
        lblObj = QLabel(tab)
        lblObj.setObjectName("lbl%s" % (objName))

        corPosY = 0

        tooltipObj = QApplication.translate("QSphere", self.libelles[objIndex][3]) if dicoHasKey(self.libelles, objIndex) else ""
        txtObj = QApplication.translate("QSphere", self.libelles[objIndex][2]) if dicoHasKey(self.libelles, objIndex) else ""
        url = self.libelles[objIndex][5]

        if objType == -2 : 
            lblObj.setGeometry(10,coordY+5,self.width(),objHeight)
            if objType == -1 :
                lblObj.setAlignment(Qt.AlignLeft)
                lblObj.setStyleSheet("QLabel { background-color : #5D5D5D; color : white; }")
                txtObj = ">> %s" % (txtObj)
            else :
                lblObj.setAlignment(Qt.AlignCenter)
                lblObj.setStyleSheet("QLabel { background-color : #CCCCCC; color : black; }")
            lblObj.setText(txtObj)
            corPosY = objHeight+10 
        else :
            if objName=="tablelangues" : lblObj.setGeometry(coordX-170,coordY+5, 170, objHeight*4)
            else : lblObj.setGeometry(coordX-170,coordY+5, 170,self.sizeHeight*2)
            lblObj.setAlignment(Qt.AlignRight)
            lblObj.setText(txtObj)
            lblObj.setWordWrap(True)
            lblObj.setAccessibleDescription(txtObj)
        
        if objType == 0:
            obj = myTextEdit(tab)
            obj.initTextEdit(180, 25, 10, 50, objName, True, True, False, False, True)
            obj.setWordWrapMode(QTextOption.WordWrap)
            typeInfo = "%s" % (type(objValue))
            if typeInfo == "<type 'str'>" : obj.setText("%s" % (objValue))
            corPosY = objHeight+10   
                
        elif objType == 1:
            obj = QComboBox(tab)
            itemtarget = 0 if objName != "sysreftemp" else 1
            if type(objValue) == str :
               if objValue.find("file:")!=-1 :
                   objWidth, cols, line = loadFile(self, obj, objName, objValue, objType, None)
               else :
                   obj.addItem("%s" % (objValue))
                   objWidth = int(objWidth / 2)
            elif type(objValue) == tuple :
               obj.insertItems (0, objValue)
               objWidth = int(objWidth / 2)
            elif type(objValue) == dict :    
                if objName == "langmetada" : 
                    mylang = QLocale.languageToString(self.langue)
                    itemtarget = getListLanguesIndex(self, obj, mylang)
            elif type(objValue) == list :
                if objName == "tablecarac" :
                    objValue.sort()
                    obj.insertItems (0, objValue)
                    itemtarget = objValue.index("utf8")
            obj.setCurrentIndex(itemtarget)

        
        elif objType == 3:
            self.tableModel = QStandardItemModel(self)
            obj = QTableView(tab)
            if self.sizeWidth == objWidth : objWidth =  self.width()-coordX - 330
            line, cols = 0, 1

            if type(objValue) == str :
                if objValue.startswith("file:"): 
                   oldHeight = objHeight
                   objHeight, cols, line = loadFile(self, obj, objName, objValue, objType, self.tableModel)
                   if oldHeight != referentHeight: objHeight = oldHeight
                elif objValue.startswith("txt:"):
                   data_icons = "%s/ressources/images/" % dirname(__file__).replace("\\","/")
                   cols, i = 2, 0
                   mylans = sortDic(self.languesDico)
                   mylang = QLocale.languageToString( self.langue )
                   for elt in mylans :
                        if len(elt)==3 :
                            language_codeiso = elt
                            language_name = self.languesDico[elt]['french']
                            for j in range(cols):
                                icon = QIcon("%s%s.png" % (data_icons, language_codeiso))
                                item = QStandardItem() if j == 0 else QStandardItem(icon, "")
                                text = language_codeiso if j == 0 else language_name
                                item.setText(text)
                                if j == 0 :
                                   item.setCheckable(True)
                                   item.setCheckState(Qt.Unchecked)
                                   if  objName == "tablelangues" and self.languesDico[elt]['english'] == mylang : item.setCheckState(Qt.Checked)
                                item.setEditable(False)    
                                self.tableModel.setItem(i,j,item)
                            i+= 1
                objHeight = objHeight + 70

            for i in range(cols): obj.setColumnWidth(i, int(objWidth/cols)-10)
            obj.horizontalHeader().setDefaultSectionSize(int(objWidth/cols)-10)
            if objName == "tabledroits" : obj.verticalHeader().setDefaultSectionSize(60)
            obj.horizontalHeader().setVisible(False)
            obj.setDragDropMode(QAbstractItemView.DragOnly)
            obj.verticalHeader().setVisible(False) 
            obj.setModel(self.tableModel)

            if objName=="tablelangues": obj.model().itemChanged.connect(self.majItemLangues) 
            
            corPosY =  objHeight + 10


        elif objType == 4:
            obj = QgsDateTimeEdit(tab)
            obj.setStyleSheet("QgsDateTimeEdit {color: white; background-color:qlineargradient(x1:0, y1:0, x2:0, y2:1, stop: 0 #cccccc, stop: 1 #5D5D5D);}"
                              "QMenu { font-size:12px; width: 100px; left: 16px; background-color:qlineargradient(x1:0, y1:0, x2:0, y2:1, stop: 0 #cccccc, stop: 1 #5D5D5D);}"
                              "QToolButton {icon-size: 20px, 20px;background-color: qlineargradient(x1:0, y1:0, x2:0, y2:1, stop: 0 #cccccc, stop: 1 #5D5D5D); height: 20px; width: 140px;}"
                              "QAbstractItemView {selection-background-color: rgb(255, 174, 0);}"
                              "QToolButton::menu-arrow {}"
                              "QToolButton::menu-button {}"
                              "QToolButton::menu-indicator{width: 5px;}"
                              "QToolButton::menu-indicator:pressed,"
                              "QToolButton::menu-indicator:open{top:10px; left: 20px;}"
                              "QListView {background-color:white;}"
                              "subcontrol-position: top right; width:50px; border-image: url(icons:arrow_up_n.png);}"
                              "border-width: 1px; width:25px;}")
            obj.setDisplayFormat("yyyy-MM-dd")
            obj.setCalendarPopup(True)
            obj.setAllowNull(False)
            obj.setEnabled(True) 
            obj.setAlignment(Qt.AlignCenter)
            reinitDate(obj)
            obj.dateChanged.connect(self.getToolTipCalendar)
            tooltipObj = toolTipCalendar(self, obj)

            objHeight = 30 
            corPosY = objHeight + 10 
            objWidth = objWidth 

            try : obj.setAccessibleDescription(url)
            except : pass             

        elif objType in(5, 50):
            if objType == 5 : obj = QLineEdit(tab)
            elif objType == 50 :
                 obj = MyWidgetLineEdit(tab)
                 text = objValue.split(":")
                 objType = int(text[0])
                 obj.initType(objType)
                 if objType == 1: obj.setInputMask("9999-99-99 9999-99-99;X")
                 elif objType == 2: obj.setInputMask("99999;X")
                 elif objType == 5: obj.setInputMask("9999-99-99;X")
                 objValue = text[1]
                 obj.setAlignment(Qt.AlignCenter)
                 obj.textChanged.connect(self.checkRegExp)
            obj.setText("") if objName != "identificator" else obj.setText("%s-%s-%s" % (self.localLangage.upper(), datetime.now().year, getRandowId(self)))


        elif objType == 6 : 
            if dicoHasKey(self.tablesWidget, objName):
                obj = tableWidgetMaker(self, objName, tab, True)
                if objName == "tableemprises" : obj.currentCellChanged.connect(self.doDessCadre) 
                objHeight = (objHeight * 7) + 30
                corPosY = coordY + objHeight if objName not in("tabledico", "tableformats", "tablelocalisator", "tableemprises", "tablemotsclefso", "tablemotsclefsf", "tableroles") else objHeight + 10

                try : obj.setAccessibleDescription(url)
                except : pass 


        elif objType == 8:
            obj = QGroupBox(tab)
            yypos = 5
            for i in range(len(objValue)):
                properties = objValue[i]
                radioButton = QRadioButton(obj)
                radioButton.setObjectName(properties[1])
                radioButton.setText("%s" % (properties[0]))
                if objName == "grouperesolutionscale" : toolTip = "%s" % (properties[2])
                else : toolTip = QApplication.translate("QSphere","Constraints")
                radioButton.setToolTip(toolTip)
                radioButton.setGeometry(5,yypos,540,25)
                if i == 0 :  radioButton.setChecked(True)
                if objName == "grouperesolutionscale" : radioButton.clicked.connect(self.changeScalesType) 
                elif objName == "groupedroits" : radioButton.toggled.connect(self.changeRightsType) 
                yypos+= 25
                
                try : radioButton.setAccessibleDescription(url)
                except : pass                
                
            if objName == "grouperesolutionscale": 
               groupWidget = myTableWidget(1,1, obj)
               groupWidget.refDialog = self
               groupWidget.setObjectName("tableechelles")
               groupWidget.setStyleSheet("""QTableWidget {selection-background-color: rgb(50, 125, 180); }""")
               groupWidget.horizontalHeader().setVisible(False)
               groupWidget.verticalHeader().setVisible(False)
               groupWidget.setGeometry(240,0,140,objHeight-10)
               addLineWidget(self, groupWidget, 0,  0, 4, 0, -1)

               try : groupWidget.setAccessibleDescription(url)
               except : pass
                
            elif objName == "groupedroits":
               self.tableModel = QStandardItemModel(self)
               groupWidget = QTableView(obj)
               nameTable = "table%s" % (objName)
               groupWidget.setObjectName(nameTable)
               line, cols = 0, 1
               heightOfSubWidgets, cols, line = loadFile(self, groupWidget, nameTable, "file:280:contraintes_%s.csv:1:0" % (self.qspherePlugin.dicoLangs[self.qspherePlugin.indexLang]), 3, self.tableModel)
               groupWidget.setGeometry(10,60, (objWidth-20), objHeight-70)
               if cols  == 0 : return objHeight + 10
               try : self.listObj.append([groupWidget, (obj, "")])
               except : pass 
               
               for i in range(cols): groupWidget.setColumnWidth(i, int((objWidth-20)/cols)-10)
               groupWidget.horizontalHeader().setDefaultSectionSize(int((objWidth-20)/cols)-10)
               groupWidget.verticalHeader().setDefaultSectionSize(60)
               groupWidget.horizontalHeader().setVisible(False) 
               groupWidget.verticalHeader().setVisible(False) 
               groupWidget.setModel(self.tableModel)
               groupWidget.setEnabled(False)

               try : groupWidget.setAccessibleDescription(url)
               except : pass

            if objName == "grouperesolutionscale": 
                comboBoxWidget = QComboBox(obj)
                comboBoxWidget.addItems(self.listUnitsMesure) 
                comboBoxWidget.setGeometry(100,60, 80,25)
                comboBoxWidget.setObjectName("unitsofscale")
                comboBoxWidget.setEnabled(False)

                try : comboBoxWidget.setAccessibleDescription(url)
                except : pass
            corPosY = objHeight + 10
    
        if objType == 11:
           obj = QCommandLinkButton(tab)
           obj.setText(tooltipObj)
           obj.setToolTip(url)
           objWidth = 1.5 * objWidth
           objHeight = 1.5 * objHeight


        if objType not in (1, 4, 8, 11)  : listButtonsObj = []


        if objType != -1 and  objType != -2 :
            if txtObj == "" and objType != 11:
               coordX = 20
               objWidth+= 180

            obj.setGeometry(coordX, coordY, objWidth, objHeight)
            if objType != 11 : obj.setToolTip(tooltipObj)
            obj.setObjectName(objName)
            if type(obj) == myTableWidget : obj.originalWidth = objWidth
            obj.setAccessibleDescription("Informations blocs XML INSPIRE")
            obj.setEnabled(objIsEnable)

            if objType in (6,8) and objName != "groupedroits" :
                
                icon = getThemeIcon(self.libelles[101][4]) 
                toolTip = QApplication.translate("QSphere", self.libelles[101][3])
                button = myPushButtonFunny(tab) 
                button.initPushButton(40, 24, (obj.width()+ obj.x()-5), coordY, "ActionsButton_%s" % (objName), "", toolTip, True, icon, 40, 24, True)
            
                contextMnuMDDActions(self, tab, obj, objName, objType, button, url)

                if objType not in (1, 4, 8, 11)  : listButtonsObj.append(button)

                if objType != 8 and not objName in ("tablescr", "tableetenduetemporelle", "tabledatepubdata", "tabledico") :

                   icon = getThemeIcon(self.libelles[102][4]) 
                   toolTip = QApplication.translate("QSphere", self.libelles[102][3])
                   button = myPushButton(tab) 
                   button.initPushButton(40, 24, (obj.width()+ obj.x()-5), coordY + 30, "OpenForm_%s" % ( objName), "", toolTip, True, icon, 40, 24, True)

                   if objType not in (1, 4, 8, 11)  : listButtonsObj.append(button)

            
            if objName == "coherence" and objType == 0 : 
               button = myButton(tab)
               toolTip = QApplication.translate("QSphere","Call the topology conformity dialog box")
               button.initButton(35, 25, (obj.width()+ obj.x()-35), coordY, "topo_%s" % objName, "...", toolTip, True, True)
               button.clicked.connect(self.CallSelectorConformityInfos)

               if objType not in (1, 4, 8, 11)  : listButtonsObj.append(button)
              
        try : obj.setAccessibleDescription(url)
        except : pass

        if objType not in (1, 4, 8, 11)  :
           if objType == -2 : obj = lblObj
           if objName not in ("tableformats", "tablescr", "tableetenduetemporelle", "tabledatepubdata"):
               try :self.listObj.append([obj, listButtonsObj])
               except : pass 

        font = QFont()
        font.setPointSize(8) 
        font.setWeight(8)
        try : obj.setFont(font)
        except : pass
                
        if objIsVertical : corPosY = 0
        if objName == "tableemprises" : obj.setToolTip("0")
        
        return corPosY

def tableWidgetMaker(dialog, nameObj, tab, isMakeLine):
    dims = dialog.tablesWidget[nameObj][0]
    headers, i = [], 0
    objKey = "%s_c%s" % (nameObj, i)

    for key, values in dialog.libelles.items() : 
        if objKey in values :
            headers.append(QApplication.translate("QSphere", values[2]))
            i+= 1
            objKey = "%s_c%s" % (nameObj, i)

    cols = 1 if type(dims)== int else len(dims)
    Obj = myTableWidget(0, cols, tab)
    Obj.refDialog = dialog
    Obj.setObjectName(nameObj)
    Obj.setStyleSheet("QTableWidget {selection-background-color: rgb(50, 125, 180); }")
    Obj.dims = dims
    initSizeCols(dialog, Obj, dims)

    if isMakeLine : makeLine(dialog, Obj, False, True,-1, True) 
    if headers!= []: Obj.setHorizontalHeaderLabels(headers)
    Obj.verticalHeader().setVisible(False)
    Obj.setTabKeyNavigation(False)

    return Obj

def contextMnuMDDActions(self, tab, Obj, nameObj, typeObj, buttonAction, url):

    contextMenu = QMenu()

    if typeObj==6 and nameObj=="tabledico" :
           contextMenu.addSeparator() 
           
           icon = getThemeIcon("attributrestore.png")
           text = QApplication.translate("QSphere","Reload the comments from Project") 
           action = QAction(QIcon(icon), text, self)
           action.setObjectName("restore_%s" % (nameObj))
           action.setEnabled(False)
           contextMenu.addAction(action)
           self.restore_tabledico = action
           action.triggered.connect(self.initComments)

           contextMenu.addSeparator()
           
           icon = getThemeIcon("tab_sizecol.png")
           text = QApplication.translate("QSphere","Original size for the columns")
           action = QAction(QIcon(icon), text, self)
           action.setObjectName("Size_%s_%s" % (typeObj, nameObj))
           contextMenu.addAction(action)
           action.triggered.connect(self.callInitSizeCols)

    if typeObj in(6, 8, 9) and not nameObj in ("tabledico", "groupedroits") :

       contextMenu.addSeparator()

       icon = getThemeIcon("tab_addline.png")
       text = QApplication.translate("QSphere","Add a Line")
       if self.objectName() not  in ("ViewerTableWidget", "formTableWidget") : text = "%s%s" % (text, "F11".rjust(60-len(text)))
       action = QAction(QIcon(icon), text, self)
       action.setObjectName("add_%s_%s" % (typeObj, nameObj))
       if self.objectName()  in ("ViewerTableWidget", "formTableWidget") : action.setShortcut(QKeySequence("F11"))
       contextMenu.addAction(action)
       action.triggered.connect(self.addLine)

       icon = getThemeIcon("tab_addaftercurrentline.png")
       text = QApplication.translate("QSphere","Add a Line after the current line")
       action = QAction(QIcon(icon), text, self)
       action.setObjectName("addAfter_%s_%s" % (typeObj, nameObj))
       if self.objectName()in ("ViewerTableWidget", "formTableWidget") : action.setShortcut(QKeySequence("Ctrl++"))
       contextMenu.addAction(action)
       action.triggered.connect(self.addLine)

       icon = getThemeIcon("tab_clonecurrentline.png")
       text = QApplication.translate("QSphere","Duplicate the current line")
       action = QAction(QIcon(icon), text, self)
       action.setObjectName("Clone_%s_%s" % (typeObj, nameObj))
       if self.objectName() in ("ViewerTableWidget", "formTableWidget") : action.setShortcut(QKeySequence("Ctrl+C"))
       contextMenu.addAction(action)
       action.triggered.connect(self.moveLineDown)

       contextMenu.addSeparator()

       icon = getThemeIcon("tab_delcurrentline.png")
       text = QApplication.translate("QSphere","Delete the current line")
       action = QAction(QIcon(icon), text, self)
       action.setObjectName("delC_%s_%s" % (typeObj, nameObj))
       if self.objectName() in ("ViewerTableWidget", "formTableWidget") : action.setShortcut(QKeySequence("Ctrl+X"))
       contextMenu.addAction(action)
       action.triggered.connect(self.deleteCurrentLine)

       icon = getThemeIcon("tab_delline.png")
       text = QApplication.translate("QSphere","Delete the last line")
       if self.objectName() not  in ("ViewerTableWidget", "formTableWidget") : text = "%s%s" % (text, "F12".rjust(60-len(text)))
       action = QAction(QIcon(icon), text, self)
       action.setObjectName("del_%s_%s" % (typeObj, nameObj))
       if self.objectName() in ("ViewerTableWidget", "formTableWidget") : action.setShortcut(QKeySequence("F12"))
       contextMenu.addAction(action)
       action.triggered.connect(self.deleteLine)

       contextMenu.addSeparator()

       icon = getThemeIcon("tab_linemoveup.png")
       text = QApplication.translate("QSphere","Move the line up")
       action = QAction(QIcon(icon), text, self)
       action.setObjectName("moveUp_%s_%s" % (typeObj, nameObj))
       if self.objectName() in ("ViewerTableWidget", "formTableWidget") : action.setShortcut(QKeySequence.MoveToPreviousPage)
       contextMenu.addAction(action)
       action.triggered.connect(self.moveLineUp)

       icon = getThemeIcon("tab_linemovedown.png")
       text = QApplication.translate("QSphere","Move the line down")
       action = QAction(QIcon(icon), text, self)
       action.setObjectName("moveDown_%s_%s" % (typeObj, nameObj))
       if self.objectName() in ("ViewerTableWidget", "formTableWidget") : action.setShortcut(QKeySequence.MoveToNextPage)
       contextMenu.addAction(action)
       action.triggered.connect(self.moveLineDown)

       if self.objectName() == "dlgMetadata" :
           
           contextMenu.addSeparator()

           if type(Obj) != QGroupBox :    
               icon = getThemeIcon("tab_sizecol.png")
               text = QApplication.translate("QSphere","Original size for the columns")
               action = QAction(QIcon(icon), text, self)
               action.setObjectName("Size_%s_%s" % (typeObj, nameObj))
               contextMenu.addAction(action)
               action.triggered.connect(self.callInitSizeCols)

       if nameObj in ("tablelocalisator", "tableroles"): 
           contextMenu.addSeparator() 
           
           icon = getThemeIcon("urlvalid.png")
           text = QApplication.translate("QSphere","Test the URL") 
           action = QAction(QIcon(icon), text, self)
           action.setObjectName("testURL_%s_%s" % (typeObj, nameObj))
           action.setToolTip(url)
           contextMenu.addAction(action)
           action.triggered.connect(self.testUrl)

           icon = getThemeIcon("urllocalisator.png")
           text = QApplication.translate("QSphere","View the localisator")
           action = QAction(QIcon(icon), text, self)
           action.setObjectName("URL_%s_%s" % (typeObj, nameObj))
           action.setToolTip(url)
           contextMenu.addAction(action)
           action.triggered.connect(self.goToLocalisator)
       
    buttonAction.setMenu(contextMenu)

    if typeObj == 6 : Obj.fixeMenu(contextMenu)
    elif typeObj==8 and nameObj=="grouperesolutionscale" :
        subObject = getWidget(tab, "tableechelles")
        if subObject : subObject.fixeMenu(contextMenu)
    


def transformerDefinition(layer, proj4Definition):
    destinationCrs = QgsCoordinateReferenceSystem()
    destinationCrs.createFromId(proj4Definition)
    sourceCrs = layer.crs()
    transformer = QgsCoordinateTransform()
    transformer.setSourceCrs(sourceCrs)
    transformer.setDestinationCrs(destinationCrs)
    return transformer


#======================================
# TEST GENERIC VIEWER FOR QTableWidget
#======================================
class formWidgetDialog(QDialog):
      def  __init__(self, TableWidget, parent):
         QDialog.__init__(self)
         self.setWindowTitle(QApplication.translate("QSphere","Wizard Table Widget Composer"))
         self.setObjectName("formTableWidget")

         self.parent = parent
         self.showWarning = self.parent.showWarning
         self.badNetwork = self.parent.badNetwork
         
         windowIcon(self, "wizform.png")
         windowProperties(self, self.parent)
         
         self.iface = self.parent.iface
         self.paramsLineWidget = self.parent.paramsLineWidget
         self.tablesWidget = self.parent.tablesWidget
         self.duration_warning = self.parent.duration_warning
         self.duration_info = self.parent.duration_info
         self.duration_timeout = self.parent.duration_timeout
         self.listOfThesaurus = self.parent.listOfThesaurus
         self.dateListOfThesaurus = self.parent.dateListOfThesaurus
         self.listTypeDates = self.parent.listTypeDates
         self.listDegres = self.parent.listDegres


         self.lblTableWidget = QLabel(self)
         self.lblTableWidget.setObjectName("lbl%s" % (TableWidget.objectName()))

         self.tableWidget = TableWidget

         if self.tableWidget.objectName()=="tableemprises" : self.disconnectObj(self.tableWidget)
         
         self.barInfo = QgsMessageBar(self)
         self.barInfo.setSizePolicy( QSizePolicy.Minimum, QSizePolicy.Fixed )

         if dicoHasKey(self.tablesWidget, self.tableWidget.objectName()):
             self.dims = self.tablesWidget[self.tableWidget.objectName()][0]
             self.libelles = self.tablesWidget[self.tableWidget.objectName()][1]
             self.paramsWidget = self.paramsLineWidget[self.tableWidget.objectName()]

         try : dimAll = (max(5, len(self.dims))+1)*35
         except :
             try : dimAll = max(70, self.dims)
             except : dimAll = 70

         self.minWidth, self.minHeight = 500, dimAll
         self.setMinimumSize(self.minWidth, self.minHeight)
         self.setMaximumSize(self.minWidth * 1.5,self.minHeight)
         self.setWindowFlags(Qt.WindowMaximizeButtonHint  | Qt.WindowCloseButtonHint)
         
         row = self.tableWidget.currentRow()
         if row == -1 : row = 0
         self.widgets = []

         key = self.tableWidget.objectName()
         
         for column in range(self.tableWidget.columnCount()):
             referentWidget = self.tableWidget.cellWidget(row, column)

             if referentWidget != None :

                 if key =="tablemotsclefso" and type(referentWidget) == myComboBox and referentWidget.isEnabled() and column == 1 :
                     referentParentWidget = self.tableWidget.cellWidget(row, column-1)
                     self.widget = addLineWidget(self, self.tableWidget, -1,  column,  self.paramsWidget[column][0], self.paramsWidget[column][1], (referentParentWidget.currentIndex(), referentWidget.currentIndex()))
                 else :
                     self.widget = addLineWidget(self, self.tableWidget, -1,  column,  self.paramsWidget[column][0], self.paramsWidget[column][1], self.paramsWidget[column][2])
                     
                 self.widget.setObjectName("%s_%s_%s" % (self.tableWidget.objectName(), row, column))
                 
                 if self.widget != None :
                    if type(self.widget) not in (myPushButton, QPushButton, myCheckBox, QCheckBox) :
                        lbl = QLabel(self)
                        lbl.setObjectName("lbl%s" % (referentWidget.objectName()))
                        lbl.setGeometry(10, 10+(column*30), 140, 30)
                        if self.libelles[column] != "" : lbl.setText("%s : " % self.libelles[column])
                        else :
                            try : lbl.setText("%s : " % QApplication.translate("QSphere", referentWidget.objectName().split("_")[0] ))
                            except : lbl.setText("%s : " % QApplication.translate("QSphere", "Unknow" ))
                        
                        lbl.setAlignment(Qt.AlignRight)
                        lbl.setStyleSheet("border: 2px solid grey; border-radius: 5px; color: white; background-color: #5d5d5d; font-family : Times New Roman, Times, serif; font-size : 8pt; text-align: center;")
                    self.widget.setGeometry(150, 10+(column*30), 300, 30)
                    self.setValue(referentWidget, self.widget)
                    self.widgets.append(self.widget)

             else :
                  if key == "tabledico" :
                     referentWidget = self.tableWidget.item(row, column)

                     lbl = QLabel(self)
                     lbl.setObjectName("LblQTableWidgetItem%s" % column)
                     lbl.setGeometry(10, 10+(column*30), 140, 30)
                     lbl.setText("%s :" % self.libelles[column])
                     lbl.setAlignment(Qt.AlignRight)
                     lbl.setStyleSheet("border: 2px solid grey; border-radius: 5px; color: white; background-color: #5d5d5d; font-family : Times New Roman, Times, serif; font-size : 8pt; text-align: center;")
                     
                     self.widget = addLineWidget(self, self.tableWidget, -1,  column,  self.paramsWidget[column][0], self.paramsWidget[column][1], self.paramsWidget[column][2])
                     self.widget.setGeometry(150, 10+(column*30), 300, 30)
                     self.setValue(referentWidget, self.widget)
                     self.widgets.append(self.widget)                     
                      

         for obj in self.widgets :
             if type(obj) == myCheckBox :
                self.setAccessibility(obj.isChecked())
                break

        
         icon = getThemeIcon("actions.png")
         toolTip = QApplication.translate("QSphere","Actions for the metadata...")
         self.buttonActions = myPushButton(self) 
         self.buttonActions.initPushButton(40, 24, 0, 0, "ActionsButton_%s" % (TableWidget.objectName()), "", toolTip, True, icon, 40, 24, True)

         contextMnuMDDActions(self, 0, self.tableWidget, "%s" % (self.tableWidget.objectName()), 6, self.buttonActions, "")

         self.saveButton = myPushButton(self)
         self.saveButton.initPushButton(100, 25, self.minWidth - 250, self.minHeight - 45, "saveButton", "", QApplication.translate("QSphere", "Save"), True, getThemeIcon("sendtotable.png"), 100, 25, True)
         self.saveButton.setShortcut(QKeySequence("Ctrl+S"))

         self.closeButton = QPushButton(self)
         self.closeButton.setObjectName("closeButton")
         self.closeButton.setText(QApplication.translate("QSphere", "Close"))
         self.closeButton.setShortcut(QKeySequence("Ctrl+Q"))

         self.groupStyleSheet = """QGroupBox {background-color: qlineargradient(x1: 0, y1: 0, x2: 0, y2: 1, stop: 0 #E0E0E0, stop: 1 #FFFFFF);""" \
                                   """border: 2px solid gray; border-radius: 5px; margin-top: 1ex; })"""  

         self.groupBoxActionsMeta = QGroupBox(self)
         self.groupBoxActionsMeta.setStyleSheet(self.groupStyleSheet)
         self.groupBoxActionsMeta.setGeometry(10, self.minHeight - 50, 220, 40)

         toolTip = QApplication.translate("QSphere","Move down")
         self.moveDown = myPushButton(self.groupBoxActionsMeta) 
         self.moveDown.initPushButton(24, 24, 10, 10, "moveDown", "", toolTip, True, getThemeIcon("movedown.png"), 24, 24, True)
         self.moveDown.setAutoRepeat(True)
         self.moveDown.setShortcut(QKeySequence.MoveToNextLine)

         self.lblPosition = QLabel(self.groupBoxActionsMeta)
         self.lblPosition.setGeometry(40, 10, 100, 25)
         self.lblPosition.setText("%s :" % (QApplication.translate("QSphere","Current contact")))
        
         toolTip = QApplication.translate("QSphere","Move up")
         self.moveUp = myPushButton(self.groupBoxActionsMeta) 
         self.moveUp.initPushButton(24, 24, 180, 10, "moveUp", "", toolTip, True, getThemeIcon("moveup.png"), 24, 24, True)
         self.moveUp.setAutoRepeat(True)
         self.moveUp.setShortcut(QKeySequence.MoveToPreviousLine)
       
         self.lblCurrentItem = QLabel(self.groupBoxActionsMeta)
         self.lblCurrentItem.setGeometry(140, 10, 40, 25)


         self.movieCtrl = movieCtrl(self, "sablier.gif")

         self.barInfo = QgsMessageBar(self)
         self.barInfo.setSizePolicy( QSizePolicy.Minimum, QSizePolicy.Fixed )

         self.saveButton.clicked.connect(self.sendDataToTable)
         self.closeButton.clicked.connect(self.close)
         self.moveUp.clicked.connect(self.moveSelectionUp)
         self.moveDown.clicked.connect(self.moveSelectionDown)
         
         self.tableWidget.selectRow(0) if self.tableWidget.currentRow()==-1 else self.tableWidget.selectRow(self.tableWidget.currentRow())
         self.row = self.tableWidget.currentRow()
         self.rowCount = self.tableWidget.rowCount()
         if self.tableWidget.objectName() == "tableemprises" : self.majIconButton(True)
         self.getCurrentItem()

      def close(self):
          self.connectObj(self.tableWidget)
          if self.tableWidget.objectName() == "tableemprises" : self.majIconButton(False)
          self.reject()

      def majIconButton(self, cond):
         from qsphere.qsphere_tools import changeButtonIcon
         try :
             icon = "voir.png" if not cond else "voiractivate.png"
             button = self.tableWidget.cellWidget(self.row, 4)
             changeButtonIcon(self, button, icon, 24, 24) 
         except : pass
         
      def disconnectObj(self, tableWidget):
          for row in range(self.tableWidget.rowCount()):
              for column in range(self.tableWidget.columnCount()):
                      referentWidget = self.tableWidget.cellWidget(row, column)
                      if type(referentWidget) == mySpinBox : referentWidget.removeEventFilter(referentWidget) 
          
      def connectObj(self, tableWidget):
          for row in range(self.tableWidget.rowCount()):
              for column in range(self.tableWidget.columnCount()):
                      referentWidget = self.tableWidget.cellWidget(row, column)
                      if type(referentWidget) == mySpinBox : referentWidget.installEventFilter(referentWidget)
          
      
      def resizeEvent(self, ev):
         posX = ev.size().width()
         for obj in self.widgets :  obj.setGeometry(obj.x(), obj.y(), posX-200, 30)
         self.buttonActions.setGeometry(posX - 50, 10, 40, 24)
         self.closeButton.setGeometry(posX - 140, self.minHeight - 45, 100, 25)
         self.saveButton.setGeometry(posX - 250, self.minHeight - 45, 100, 25)

      def loadWebViewer(self):
          from qsphere.ui import loadDlgNavigator
          d = loadDlgNavigator(self, self.iface, "#blank", False, [], self.localLangage, self, None, "navigatorweb.png", True)

      def clickHelp(self): getHelp(self)

      def testUrl(self): testUrl(self.parent)
      
      def goToLocalisator(self):
          from qsphere.ui import loadDlgNavigator
          col = 0 if self.tableWidget.objectName() == "tablelocalisator" else tableWidget.columnCount()-1 
          d = loadDlgNavigator(self, self.iface, self.widgets[col].text(), False, [], self.localLangage, self, None, "navigatorweb.png", True)
          
      def addLine(self):
          addLine(self.parent)
          self.refreshData()
          
      def moveLineUp(self): 
          moveLineUpProcess(self.parent, "%s" % (self.sender().objectName()))
          self.refreshData()
          
      def moveLineDown(self):
          moveLineDownProcess(self.parent, "%s" % (self.sender().objectName()))
          self.refreshData()
          
      def deleteLine(self):
          if self.tableWidget.rowCount()> 1 :
              deleteLineProcess(self.parent, "%s" % (self.sender().objectName()), True)
              self.refreshData()

      def deleteCurrentLine(self):
          if self.tableWidget.rowCount()> 1 :
              deleteLineProcess(self.parent, "%s" % (self.sender().objectName()), False)
              self.refreshData()
      
      def getCurrentItem(self):  self.lblCurrentItem.setText("%s / %s" % (self.row+1, self.tableWidget.rowCount()))

      def refreshData(self):
          self.row = self.tableWidget.currentRow()
          self.tableWidget.selectRow(self.row)
          self.loadData()
          self.getCurrentItem()
      
      def moveSelectionUp(self):
        self.majIconButton(False)
        if self.row > 0 : self.row+=  - 1
        else :  self.row = self.tableWidget.rowCount()-1       
        self.tableWidget.selectRow(self.row)
        self.refreshData()
        self.majIconButton(True)
                  
      def moveSelectionDown(self):
        self.majIconButton(False)
        if self.row <  (self.tableWidget.rowCount()-1) : self.row+= 1
        else : self.row = 0
        self.tableWidget.selectRow(self.row)
        self.refreshData()
        self.majIconButton(True)

      def initComments(self): initComments(self.parent)     

      def callInitSizeCols(self): callInitSizeCols(self.parent) 

      def loadData(self):
          try :
              for column in range(self.tableWidget.columnCount()):
                  referentWidget = self.tableWidget.cellWidget(self.row, column)  if self.tableWidget.objectName()!= "tabledico" else self.tableWidget.item(self.row, column)
                  if referentWidget != None : self.setValue(referentWidget, self.widgets[column])
          except : pass  

      def sendDataToTable(self):
          try :
              column = 0
              for obj in self.widgets :
                  targetObj = self.tableWidget.cellWidget(self.row, column) if self.tableWidget.objectName()!= "tabledico" else self.tableWidget.item(self.row, column)
                  if targetObj!= None and obj != None : self.setValue(obj, targetObj)
                  column+= 1
          except : pass        
          
      def setValue(self, originalObj, targetObj):
          try : targetObj.setEnabled(originalObj.isEnabled())
          except :
              try : targetObj.setEnabled(not (originalObj.flags()== Qt.ItemIsSelectable|Qt.ItemIsEnabled))
              except : pass
          if type(originalObj) == myCheckBox :
              targetObj.setCheckState(originalObj.checkState())
          try : targetObj.setValue(originalObj.value())
          except :
              try : targetObj.setText(originalObj.text())
              except :
                      try : targetObj.setCurrentIndex(originalObj.currentIndex())
                      except : pass
                              
      def listOfValues(self):
            if self.sender().objectName() == "" : return
            indexThesaurus = self.sender().currentIndex()
            i = 0
            for obj in self.widgets :
                if self.sender() == obj : break
                i+= 1
            try :     
                widget = self.widgets[i+1]
                if widget != None :
                   widget.clear()
                   sizeWidth, cols, line = loadFile(self, widget, "", "file:200:thesaurus_%s_%s.csv:0:0" % (indexThesaurus, self.parent.qspherePlugin.dicoLangs[self.parent.qspherePlugin.indexLang]), 1, None) 
            except : pass
            
      def setAccessibility(self, checked):
          for obj in self.widgets :
              if self.sender() == obj :
                 obj.setCheckState(self.sender().checkState())
                 break          
          for i in range(2,5):
               try : self.widgets[i].setEnabled(checked)
               except : pass
               
      def updatePatternZipPostalCode(self):
          i = 0
          for obj in self.widgets :
              if self.sender() == obj :
                 break
              i+= 1
          try : obj = self.widgets[i+1]
          except : obj = None
          if obj==None: return
          targetKey = self.sender().currentText()
          oldText = obj.text()
          if dicoHasKey(self.listCountriesCode, targetKey):
                obj.setInputMask("")
                obj.setInputMask(self.listCountriesCode[targetKey][1])
                obj.regex = QRegExp(r"%s" % (self.listCountriesCode[targetKey][0]), Qt.CaseSensitive)
          else :
                obj.setInputMask("")
                obj.setInputMask("XXXxxxxxxx;X")
                obj.regex = QRegExp(r"(^+[a-zA-Z_0-9\s]{3,10}$)", Qt.CaseSensitive)
          obj.setText(oldText)      

          
      def geoLocalisator(self):
          emprise = []
          for i in range(0,4): emprise.append(self.widgets[i].value())
          
          changeButtonIcon(self, self.sender(),"voiractivate.png", 24, 24)
          if self.qspherePlugin.mainWindow : self.qspherePlugin.mainWindow.myDock.setVisible(False)
          
          if self.badNetwork == None : 
              self.badNetwork = badNetwork("http://server.arcgisonline.com/")
              self.parent.badNetwork = self.badNetwork
          from qsphere.qsphere_objmaker import geoLocalisatorDialog    
          d = geoLocalisatorDialog(self, emprise, self.tableWidget, self.tableWidget.currentRow(), False, True, self.badNetwork)
          d.exec_()

          if self.qspherePlugin.mainWindow : self.qspherePlugin.mainWindow.myDock.setVisible(True)
          changeButtonIcon(self, self.sender(),"voir.png", 18, 16)  

      def checkRules(self): pass
          
      def CheckTable(self, obj):
            for i in range(obj.rowCount()):
                if i in (0, 1) :
                    obj.cellWidget(i,0).setEnabled(False)
                    if i == 0 : obj.cellWidget(i,0).setCurrentIndex(2)
                    else : obj.cellWidget(i,0).setCurrentIndex(6)
                else :
                    obj.cellWidget(i,0).setEnabled(True)
                    if obj.cellWidget(i,0).currentIndex() in (2,6) : obj.cellWidget(i,0).setCurrentIndex(1)

      def CallFormatSelector(self):
          i = 0  
          for obj in self.widgets :
              if self.sender() == obj : break
              i+= 1

          obj = self.widgets[i-1]
          changeButtonIcon(self, self.sender(),"formatactivate.png", 18, 16)
          dialog = dlgDataFormat(self.formats, obj.text())
          windowIcon(dialog, "format.png")
          if dialog.exec_(): obj.setText("%s" % (dialog.format()))
          changeButtonIcon(self, self.sender(),"format.png", 18, 16)

