from qgis.core import (Qgis, QgsDataSourceUri, QgsMapLayer, QgsProject, QgsRasterLayer, QgsRectangle, QgsStyle, QgsVectorLayer)
from qgis.gui import (QgsCollapsibleGroupBox, QgsCodeEditorSQL, QgsColorDialog, QgsFilterLineEdit, QgsMapCanvas, QgsRendererPropertiesDialog, QgsRendererRasterPropertiesWidget,
                      QgsMapToolPan, QgsMapToolZoom, QgsMessageBar) 

from qgis.PyQt.QtCore import (QObject, QEvent, QPoint, QSizeF, Qt, QTime, QUrl)
from qgis.PyQt.QtGui import (QColor, QCursor, QFont, QFontMetrics, QIcon, QImage, QKeySequence, QMovie, QPainter, QPixmap, QBrush,
                             QRegion, QStandardItemModel, QTextDocument, QTextCursor)
from qgis.PyQt.QtPrintSupport import (QPrintPreviewDialog, QPrinter)
from qgis.PyQt.Qsci import (QsciAPIs, QsciScintilla)
from qgis.PyQt.QtWidgets import (QAbstractItemView, QAction, QApplication, QCheckBox, QComboBox, QDialog, QDialogButtonBox, QDesktopWidget, QDockWidget,
                                 QFileDialog, QFrame, QGroupBox, QHBoxLayout, QInputDialog, QLCDNumber, QLabel, QLineEdit, QListView, QMdiArea, QMdiSubWindow,
                                 QMenu, QMenuBar, QMessageBox, QPushButton, QSizePolicy, QSplitter, QTableView, QTabWidget, QTextBrowser, QToolBar,
                                 QTreeWidget, QTreeWidgetItem, QVBoxLayout, QWidget)

from PyQt5.QtWebKitWidgets import (QWebInspector, QWebPage, QWebView)
from PyQt5.QtWebKit import QWebSettings

from qsphere.importexport import (closeFile, saveXml)
from qsphere.qsphere_objmaker import (baseTableModel, ctrlMap, editor, expandableGroupBox, findDialog, findDialogToolSearchFiles, filteredComboBox,
                                      movieCtrl, myMdiSubWindow, myMsgDlgBox, dlgNode, myProgressBar, myPushButton, myPushButtonFunny,
                                      myTextEdit, printerSphere)
from qsphere.qsphere_tools import (correctPath, dicoHasKey, fixeRanges, fileNameWithExtension, fixeLabelsFileDialog,
                                   getComboLayer, getExtension, getOptions, getParamsConnexionQGIS, getThemeIcon, getTypeLayer,
                                   msgBadFileXml, netToHtml, qgisKeyDataSources, saveResultsInFile, sendMsg, windowIcon) 
from qsphere.sql_templates import (GET_ALL_PRIMARYKEYS_DATABASE, GET_ALL_PRIVILEGES_FOR_USER, GET_ALL_SCHEMAS_SYSTEM_DATABASE,
                                   GET_ALL_SCHEMAS_OUTSYSTEM_DATABASE, GET_ALL_SCHEMAS_TABLES_DATABASE,
                                   GET_ALL_SERVER_USERS, GET_PG_KEYWORDS, GET_PG_PROCEDURES, GET_ALL_PUBLIC_TABLES_DATABASE,
                                   GET_SERVER_NUMERIC_TYPES, GET_SERVER_BASES, GET_SERVER_SYSTEM_SCHEMAS,
                                   GET_TABLE_COUNT_RECORDS, GET_TABLE_FIELDS, GET_TABLE_GEOMETRY_COLUMN, GET_TABLE_KEY_COLUMN,
                                   GET_TABLE_JOINS, GET_TABLE_NAMES, GET_TABLE_RASTER_COLUMN, GET_TABLE_RECORDS, GET_TABLE_STATISTICS,
                                   GET_VIEW_SQL, GET_VIEW_SQL_PLUS, TERMINATE_BACKEND_BASE)
from qsphere.ui_editorxml import xmlEditor

from datetime import datetime as thedatetime
from datetime import timedelta
from math import floor
from os import remove as osremove
from os.path import (dirname, exists, join, splitext)
from os import walk
import psycopg2
import re
from psycopg2 import (connect, pool as psycopg2pool, OperationalError)
from psycopg2.extensions import (ISOLATION_LEVEL_AUTOCOMMIT, POLL_OK, POLL_WRITE, POLL_READ)
import textwrap
from urllib.parse import (quote, unquote)
from xml.etree import ElementTree as ET


class uiDlgDatabase(object):
    
    def setupUi(self):
        self.setObjectName("dialogSGDBR")
        self.minWidth, self.minHeight = 900, 660
        self.setMinimumSize(self.minWidth, self.minHeight)
        self.resize(self.minWidth, self.minHeight)
        self.rootWindowTitle = "%s" % self.contextText
        self.setWindowTitle(self.rootWindowTitle)
        
        getOptions(self)

        self.setWindowFlags(Qt.WindowMinimizeButtonHint | Qt.WindowMaximizeButtonHint | Qt.WindowCloseButtonHint)
        self.connexion = None
        self.counterFolders = 0
        self.currentRecord = -1
        self.changeSource, self.oldSource = True, ""
        self.currentLayer = None
        self.startTime = thedatetime.now()
        self.listFiltres = ("%s SQL (*.qry)" %  QApplication.translate("QSphere","File"), "%s SQL (*.sql)" %  QApplication.translate("QSphere","File"))
        self.description, self.numericTypesCodes = [], []
        self.dlgNode = None
        self.titleConfirmation = QApplication.translate("QSphere","Confirmation")
        self.yes = QApplication.translate("QSphere", "Yes")
        self.no = QApplication.translate("QSphere", "No")
        self.sqlCommands = QApplication.translate("QSphere","SQL Commands")
        self.failed = QApplication.translate("QSphere","Failed")
        self.successfull = QApplication.translate("QSphere","Successfull")
        self.completed = QApplication.translate("QSphere","Completed")
        self.lblRecords = QApplication.translate("QSphere","records")
        

        self.pucebase = getThemeIcon("sgbdr/pucebase.png")
        self.pucebaseopen = getThemeIcon("sgbdr/pucebaseopen.png")
        self.ipuceschema = "sgbdr/puceschema.png"
        self.puceschema = getThemeIcon(self.ipuceschema)
        self.ipucetable = "sgbdr/pucetable.png"
        self.pucetable = getThemeIcon(self.ipucetable)
        self.ipuceview = "sgbdr/puceview.png"
        self.puceview = getThemeIcon(self.ipuceview)
        self.pucekey = getThemeIcon("sgbdr/pucekey.png")
        self.pucerecords = getThemeIcon("sgbdr/pucerecords.png")
        self.pucequery = getThemeIcon("sgbdr/sql.png")
        self.ipucesuccessfull = "sgbdr/validate.png"
        self.pucesuccessfull = getThemeIcon(self.ipucesuccessfull)
        self.ipucewarning = "sgbdr/warning.png"
        self.pucewarning = getThemeIcon(self.ipucewarning)
        self.pucearrow = getThemeIcon("sgbdr/arrow.png")
        self.puceparami = getThemeIcon("sgbdr/puceparami.png")
        self.puceparamc =getThemeIcon("sgbdr/puceparamc.png")
        self.ipucestartChrono = "sgbdr/chronoplay.png"
        self.pucestartChrono = getThemeIcon(self.ipucestartChrono)
        self.pucestopChrono = getThemeIcon("sgbdr/chronostop.png")
        self.sphery = getThemeIcon("sphery.gif")
        self.deltaChrono = 0
        self.path = correctPath(dirname(__file__)).replace("\\", "/")
        
        self.path = join(self.path, "icons/sgbdr/")
        self.baseurl = QUrl('file:///')
        
        self.dicSchemas = {}
        self.sqlDictionnary = {}
        self.metadataToDownload = {}
        self.listBases, self.listSchemas, self.listTables, self.listFunctions = [], [], [], []
        self.offset = -1
        self.targetDataBase, self.itemQspMetadata, self.currentItemData = None, None, None
        self.infosChild = False
        self.changedMddFiles = True
        self.mode = "normal"
        self.cursor = QCursor( Qt.ArrowCursor )
        self.styleDlg = None

        self.html = """<HTML><BODY><style>
            table {border-spacing: 0px; width: 100%;}
            table.sql {background-color: white; border-color:black; width: 100%; cellspacing: 0px;}
            th, td {padding: 5px 30px 5px 10px;border-spacing: 0px;font-size: 90%;margin: 0px;}
            th {color: #FFFFFF;text-align: left;background-color: #2C76B4;border-top: 1px solid #f1f8fe;border-bottom: 1px solid #cbd2d8; border-right: 1px solid #cbd2d8;}
            th.title {color: #FFFFFF;text-align: left;background-color: #5D5D5D;border-top: 1px solid #f1f8fe;border-bottom: 1px solid #cbd2d8; border-right: 1px solid #cbd2d8;}
            td {color: #737476;text-align: left;background-color: #e0e9f0;border-top: 1px solid #f1f8fe;border-bottom: 1px solid #cbd2d8; border-right: 1px solid #cbd2d8;}
            tr.comment th {color: #90b4d6;background-color: #FFF; border-bottom: 2px solid #547ca0;border-right: 1px solid #749abe;border-top: 1px solid #90b4d6;text-align: left;text-shadow: -1px -1px 1px #666;letter-spacing: 0.15em;}
            tr.head th {color: #fff;background-color: #90b4d6; border-bottom: 2px solid #547ca0;border-right: 1px solid #749abe;border-top: 1px solid #90b4d6;text-align: left;text-shadow: -1px -1px 1px #666;letter-spacing: 0.15em;}
            tr.head th:first-child {-webkit-border-top-left-radius: 5px;-moz-border-radius-topleft: 5px;border-top-left-radius: 5px;}
            tr.head th:last-child {-webkit-border-top-right-radius: 5px;-moz-border-radius-topright: 5px;border-top-right-radius: 5px;}
            tr.headTitle th {font-size: 150%; color: #fff;background-color: #00b4FF; border-bottom: 2px solid #547ca0;border-right: 1px solid #749abe;border-top: 1px solid #90b4d6;text-align: center;text-shadow: -1px -1px 1px #666;letter-spacing: 0.15em; vertical-align: middle;}
            tr.headTitle th:first-child {-webkit-border-top-left-radius: 10px;-moz-border-radius-topleft: 10px;border-top-left-radius: 10px; -webkit-border-bottom-left-radius: 10px;-moz-border-radius-bottomleft: 10px;border-bottom-left-radius: 10px;}
            tr.headTitle th:last-child {-webkit-border-top-right-radius: 10px;-moz-border-radius-topright: 10px;border-top-right-radius: 10px; -webkit-border-bottom-right-radius: 10px;-moz-border-radius-bottomright: 10px;border-bottom-right-radius: 10px;}

            tr.even td, tr.even th {background-color: #e8eff5;}

            table.browse td.col0 {width: 20%; font-family:Verdana, Geneva, Arial, Helvetica, sans-serif;font-size: 12px;text-align:right;}
            table.browse td.cols {width: 15%; font-family:Verdana, Geneva, Arial, Helvetica, sans-serif;font-size: 10px;}
            table.browse th {font-family:Verdana, Geneva, Arial, Helvetica, sans-serif;font-size: 12px;background-color: #2C76B4;border-bottom: 3px solid #336699;border-right: 1px solid #336699;cursor:pointer;padding: 6px 15px 6px 6px;text-align:left;white-space: nowrap;}
            table.browse td img{padding:0 4px 0 0;margin:0 0 -1px 0;}
            table.browse td.input{border: solid 1px black}
            table.browse td.gel{font-weight:bold; color:336699 ; font-family:Verdana, Geneva, Arial, Helvetica, sans-serif;font-size: 12px; background-color: #E4EDFC;border-bottom: 3px solid #336699;border-right: 1px solid #336699;cursor:pointer;padding: 6px 15px 6px 6px;text-align:left;white-space: nowrap;}
            table.browse td{border-bottom: 1px solid #ddd; border-right: 1px solid #ddd; color: #222222; font-size:.8em; margin:0;padding: 1px 6px 2px 7px; white-space: nowrap; text-align:left;vertical-align:top;}
        """
        self.html+= "table.browse th.descend{background: #2C76B4 95%% 85%% url('file:///%scaret_up.gif') no-repeat;}" % self.path
        self.html+= "table.browse th.ascend{color: black;background: #2C76B4 95%% 85%% url('file:///%scaret_down.gif') no-repeat;}" % self.path
        self.html+= "</style>"
   
        self.groupParams = QgsCollapsibleGroupBox(self)
        self.groupParams.setMinimumHeight(190)
        self.groupParams.setMaximumHeight(190)
        self.groupParams.setMaximumWidth(self.width()-10)
        self.groupParams.setMinimumWidth(self.width()-10)
        self.groupParams.setTitle("%s : " % (QApplication.translate("QSphere","Params")))


        self.groupParamConnexion = QgsCollapsibleGroupBox(self.groupParams)
        self.groupParamConnexion.setMinimumHeight(160)
        self.groupParamConnexion.setMaximumHeight(160)
        self.groupParamConnexion.setMaximumWidth(245)
        self.groupParamConnexion.setMinimumWidth(245)
        self.groupParamConnexion.setTitle("%s : " % (QApplication.translate("QSphere","Connection params")))
        self.groupParamConnexion.setGeometry(10, 20, 245, 160)
        self.groupParamConnexion.setCollapsed(False)

        self.lblQgisConnexion = QLabel(self.groupParamConnexion)
        self.lblQgisConnexion.setGeometry(5, 22, 90, 20)
        self.lblQgisConnexion.setText("%s : " % (QApplication.translate("QSphere","QGIS Con.")))
        self.lblQgisConnexion.setObjectName("lblQgisConnexion")
        self.lblQgisConnexion.setAlignment(Qt.AlignRight)
        self.lblQgisConnexion.setStyleSheet("QLabel {border: 2px solid grey; border-radius: 5px; background-color: #6C96C6; color: white;}")

        self.qgisConnexion = QComboBox(self.groupParamConnexion)
        self.qgisConnexion.setObjectName("qgisConnexion")
        self.qgisConnexion.setGeometry(100, 20, 120, 25)
        self.qgisConnexion.addItems(qgisKeyDataSources())

        icon = getThemeIcon("sgbdr/reload.png")
        toolTip = QApplication.translate("QSphere","Reload QGis connexions")
        self.reloadQgisButton = myPushButton(self.groupParamConnexion) 
        self.reloadQgisButton.initPushButton(24, 24, 5, 0, "reloadQgisButton", "", toolTip, True, icon, 24, 24, True)
        self.reloadQgisButton.setGeometry(220, 20, 24, 24)

        self.lblService = QLabel(self.groupParamConnexion)
        self.lblService.setText("%s : " % (QApplication.translate("QSphere","Service")))
        self.lblService.setObjectName("lblService")
        self.lblService.setAlignment(Qt.AlignRight)
        self.lblService.setStyleSheet("QLabel {border: 2px solid grey; border-radius: 5px; background-color: #6C96C6; color: white;}")

        self.nameService = QLineEdit(self.groupParamConnexion)
        self.nameService.setObjectName("nameService")
        self.nameService.setText("localhost")

        self.lblPort = QLabel(self.groupParamConnexion)
        self.lblPort.setText("%s : " % (QApplication.translate("QSphere","Port")))
        self.lblPort.setObjectName("lblPort")
        self.lblPort.setAlignment(Qt.AlignRight)
        self.lblPort.setStyleSheet("QLabel {border: 2px solid grey; border-radius: 5px; background-color: #6C96C6; color: white;}")

        self.namePort = QLineEdit(self.groupParamConnexion)
        self.namePort.setObjectName("namePort")
        self.namePort.setText("5433")

        self.lblDatabase = QLabel(self.groupParamConnexion)
        self.lblDatabase.setText("%s : " % (QApplication.translate("QSphere","database").capitalize()))
        self.lblDatabase.setObjectName("lblDatabase")
        self.lblDatabase.setAlignment(Qt.AlignRight)
        self.lblDatabase.setStyleSheet("QLabel {border: 2px solid grey; border-radius: 5px; background-color: #6C96C6; color: white;}")

        self.nameDatabase = filteredComboBox(self.groupParamConnexion)
        self.nameDatabase.setObjectName("nameDatabase")
        self.nameDatabase.setEditable(True)
        self.nameDatabase.setModelColumn(0)
        self.nameDatabase.addItem("postgres")

        self.lblUser = QLabel(self.groupParamConnexion)
        self.lblUser.setText("%s : " % (QApplication.translate("QSphere","User")))
        self.lblUser.setObjectName("lblUser")
        self.lblUser.setAlignment(Qt.AlignRight)
        self.lblUser.setStyleSheet("QLabel {border: 2px solid grey; border-radius: 5px; background-color: #6C96C6; color: white;}")

        self.nameUser = QLineEdit(self.groupParamConnexion)
        self.nameUser.setEchoMode(QLineEdit.Password)
        self.nameUser.setObjectName("nameUser")
        self.nameUser.setText("postgres")

        toolTip = QApplication.translate("QSphere","Show or hide the user")
        self.viewUser = myPushButton(self.groupParamConnexion) 
        self.viewUser.initPushButton(20, 20, 95, 0, "viewUser", "", toolTip, True, getThemeIcon("show.png"), 20, 20, True)

        self.lblPwdUser = QLabel(self.groupParamConnexion)
        self.lblPwdUser.setText("%s : " % (QApplication.translate("QSphere","Password")))
        self.lblPwdUser.setObjectName("lblPwdUser")
        self.lblPwdUser.setAlignment(Qt.AlignRight)
        self.lblPwdUser.setStyleSheet("QLabel {border: 2px solid grey; border-radius: 5px; background-color: #6C96C6; color: white;}")

        self.pwdUser = QLineEdit(self.groupParamConnexion)
        self.pwdUser.setEchoMode(QLineEdit.Password)
        self.pwdUser.setObjectName("pwdUser")
        self.pwdUser.setText("postgres")

        toolTip = QApplication.translate("QSphere","Show or hide the user")
        self.viewPwd = myPushButton(self.groupParamConnexion) 
        self.viewPwd.initPushButton(20, 20, 95, 0, "viewPwd", "", "%s [%s]" % (toolTip, QApplication.translate("QSphere","Password")), True, getThemeIcon("show.png"), 20, 20, True)        
       
        icon = getThemeIcon("sgbdr/connect.png")
        toolTip = QApplication.translate("QSphere","Get the connection")
        self.getConButton = myPushButton(self.groupParams) 
        self.getConButton.initPushButton(64, 64, 220, 80, "getConButton", "", toolTip, True, icon, 64, 64, True)

        self.groupOptions = QgsCollapsibleGroupBox(self.groupParams)
        self.groupOptions.setMinimumHeight(140)
        self.groupOptions.setMaximumHeight(140)
        self.groupOptions.setMaximumWidth(245)
        self.groupOptions.setMinimumWidth(245)
        self.groupOptions.setTitle("%s : " % (QApplication.translate("QSphere","Options")))
        self.groupOptions.setGeometry(10, 40, 245, 140)
        self.groupOptions.setCollapsed(False)


        self.checkQSphereAsk = QCheckBox(self.groupOptions)
        self.checkQSphereAsk.setObjectName("checkQSphereAsk")
        text = "%s (QSphere %s)" % (QApplication.translate("QSphere","Silent mode"), QApplication.translate("QSphere","Actions").lower()) 
        self.checkQSphereAsk.setText(text)
        self.checkQSphereAsk.setToolTip(text)
        self.checkQSphereAsk.setChecked(True)

        self.checkSystemObjects = QCheckBox(self.groupOptions)
        self.checkSystemObjects.setObjectName("checkSystemObjects")
        text = QApplication.translate("QSphere","Include system objects")
        self.checkSystemObjects.setText(text)
        self.checkSystemObjects.setToolTip(text)
        self.checkSystemObjects.setChecked(False)

        self.checkSqlComments = QCheckBox(self.groupOptions)
        self.checkSqlComments.setObjectName("checkQSphereAsk")
        self.checkSqlComments.setChecked(False)
        text = "%s (SQL)" % QApplication.translate("QSphere","Comment")
        self.checkSqlComments.setText(text)
        self.checkSqlComments.setToolTip(text)
        
        self.checkResultsInNewTab = QCheckBox(self.groupOptions)
        self.checkResultsInNewTab.setObjectName("checkResultsInNewTab")
        text = QApplication.translate("QSphere","Always results in new tab")
        self.checkResultsInNewTab.setText("%s...(SQL)" % text[0:30])
        self.checkResultsInNewTab.setToolTip("%s (SQL)" % text)
        self.checkResultsInNewTab.setChecked(False)

        self.dynamicalLink = True

        self.checkDynamicalLink= QCheckBox(self.groupOptions)
        self.checkDynamicalLink.setObjectName("checkDynamicalLink")
        text = QApplication.translate("QSphere","Link tab and editor")
        self.checkDynamicalLink.setText("%s...(SQL)" % text[0:30])
        self.checkDynamicalLink.setToolTip("%s (SQL)" % text)
        self.checkDynamicalLink.setChecked(self.dynamicalLink)


        self.dockSql = QDockWidget("SQL", self.groupParams)
        self.dockSql.setWindowTitle("SQL :")
        self.dockSql.setFloating(False)
        self.dockSql.setFeatures(QDockWidget.DockWidgetFloatable | QDockWidget.DockWidgetMovable)
        self.dockSql.setAllowedAreas(Qt.NoDockWidgetArea)
        self.dockSql.setStyleSheet(
                    "QDockWidget { color:rgb(255,255,255); font-size:12px; font-weight: bold; border: 1px solid lightgray;}"
                    "QDockWidget::title {  text-align: center; background: #5D5D5D; padding-left: 5px;}")        

        font = QFont()
        font.setFamily('Courier')
        font.setFixedPitch(True)
        font.setPointSize(10)

        self.editorSql = QgsCodeEditorSQL(self.dockSql)
        self.editorSql.setVerticalScrollBarPolicy(Qt.ScrollBarAsNeeded)
        self.editorSql.setMarginVisible(True)
        self.editorSql.setWrapMode(QsciScintilla.WrapWord)

        self.editorSql.findDialog = findDialog(self.editorSql)

        self.editorSql.setFont(font)
        fontmetrics = QFontMetrics(font)
        self.editorSql.setMarginsFont(font)
        self.editorSql.setMarginLineNumbers(0, True)
        self.editorSql.setMarginWidth(0, fontmetrics.width("000000000"))
        self.editorSql.setMarginLineNumbers(1, True)
        self.editorSql.setMarginWidth(1,1)

        self.widgetEditorSql = myMdiSubWindow(self.editorSql,"widgetEditorSql", "%s :" % QApplication.translate("QSphere","Actions ..."), 300, 80 )
        self.groupEditorSql = QGroupBox(self.widgetEditorSql)
        self.groupEditorSql.setGeometry(10, 30, 280, 35)
        self.editorSql.duration_info = self.duration_info

        self.searchbutton = myPushButton(self.groupEditorSql)
        self.searchbutton.initPushButton(25, 25, 5, 5, "searchbutton", "", QApplication.translate("QSphere", "Search word"), True, getThemeIcon("find.png"), 25, 25, True)
        self.searchbutton.setShortcut(QKeySequence("Ctrl+F"))
        self.BAddParam = myPushButton(self.groupEditorSql)
        self.BAddParam.initPushButton(25, 25, 40, 5, "BAddParam", "", QApplication.translate("QSphere","Insert individual param"), True, self.puceparami, 25, 25, True)
        self.BAddPARAM = myPushButton(self.groupEditorSql)
        self.BAddPARAM.initPushButton(25, 25, 75, 5, "BAddPARAM", "", QApplication.translate("QSphere","Insert common param"), True, self.puceparamc, 25, 25, True)
        self.checkQuote = QCheckBox(self.groupEditorSql)
        self.checkQuote.setObjectName("checkQuote")
        text = QApplication.translate("QSphere","With quote")
        self.checkQuote.setText(text)
        self.checkQuote.setToolTip(text)
        self.checkQuote.setChecked(True)
        self.checkQuote.setGeometry(115, 5, 120, 25)
        self.makeMenuAddParamButton(self.BAddParam)
        self.makeMenuAddParamButton(self.BAddPARAM)

        toolTip = QApplication.translate("QSphere","Start the robot")
        self.chronoSql = myPushButton(self.groupEditorSql) 
        self.chronoSql.initPushButton(25, 25, 235, 5, "chronoSql", "", toolTip, True, self.pucestartChrono, 25, 25, True)
        self.isStartChronoSql = False
        self.chronoSqltoRefresh, self.lastSql = [], []

        self.InitDir = join(dirname(__file__), "qry")

        self.sqlToolBar = QToolBar(self.dockSql)
        self.sqlToolBar.setOrientation(Qt.Vertical)
        self.sqlToolBar.setGeometry(10, 25, 32, 140)
        self.sqlToolBar.setStyleSheet("QToolBar {border: 0px solid grey; border-radius: 1px; color: white; spacing:1px;}")

        icon = getThemeIcon("new.png")
        toolTip = "%s SQL" % QApplication.translate("QSphere","New sheet")
        self.eraseSqlButton = myPushButtonFunny(self.editorSql) 
        self.eraseSqlButton.initPushButton(24, 24, 370, 390, "eraseSqlButton", "", toolTip, True, icon, 24, 24, True)
        self.eraseSqlButton.setShortcut(QKeySequence("Ctrl+N"))
        self.sqlToolBar.addWidget(self.eraseSqlButton)

        icon = getThemeIcon("open.png")
        toolTip = QApplication.translate("QSphere","Open Files")
        self.getSqlButton = myPushButtonFunny(self.groupParams) 
        self.getSqlButton.initPushButton(24, 24, 370, 390, "getSqlButton", "", "SQL", True, icon, 24, 24, True)
        self.getSqlButton.setShortcut(QKeySequence("Ctrl+G"))
        self.sqlToolBar.addWidget(self.getSqlButton)

        self.sqlToolBar.addSeparator()
        
        icon = getThemeIcon("sgbdr/savesql.png")
        toolTip = "%s SQL" % QApplication.translate("QSphere","Save")
        self.saveSqlFilesButton = myPushButtonFunny(self.groupParams) 
        self.saveSqlFilesButton.initPushButton(24, 24, 370, 390, "saveSqlFilesButton", "", toolTip, True, icon, 24, 24, True)
        self.saveSqlFilesButton.setShortcut(QKeySequence("Ctrl+S"))
        self.sqlToolBar.addWidget(self.saveSqlFilesButton)

        icon = getThemeIcon("sgbdr/savesqlas.png")
        toolTip = "%s SQL" % QApplication.translate("QSphere","Save as ...")
        self.saveSqlFilesAsButton = myPushButtonFunny(self.groupParams) 
        self.saveSqlFilesAsButton.initPushButton(24, 24, 370, 390, "saveSqlFilesAsButton", "", toolTip, True, icon, 24, 24, True)
        self.saveSqlFilesAsButton.setShortcut(QKeySequence("Ctrl+Shift+S"))
        self.sqlToolBar.addWidget(self.saveSqlFilesAsButton)

        self.sqlToolBar.addSeparator()
        self.defaultExtension = "qry"
       
        icon = getThemeIcon("sgbdr/sqlplay.png")
        toolTip = QApplication.translate("QSphere","Execute SQL")
        self.getSqlResultsButton = myPushButtonFunny(self.groupParams) 
        self.getSqlResultsButton.initPushButton(24, 24, 370, 390, "getSqlResultsButton", "", toolTip, True, icon, 24, 24, True)
        self.getSqlResultsButton.setShortcut(QKeySequence("F5"))
        self.sqlToolBar.addWidget(self.getSqlResultsButton)

        self.lblService.setGeometry(5, 47, 90, 20) 
        self.nameService.setGeometry(100, 47, 120, 20) 
        self.lblPort.setGeometry(5, 68, 90, 20)
        self.namePort.setGeometry(100, 68, 60, 20)
        self.lblDatabase.setGeometry(5, 90, 90, 20) 
        self.nameDatabase.setGeometry(100, 90, 120, 20) 
        self.lblUser.setGeometry(5, 113, 90, 20) 
        self.nameUser.setGeometry(100, 113, 120, 20) 
        self.lblPwdUser.setGeometry(5, 135, 90, 20) 
        self.pwdUser.setGeometry(100, 135, 120, 20)
        self.reloadQgisButton.setGeometry(220, 20, 25, 25)
        self.viewUser.setGeometry(220, 113, 25, 25)
        self.viewPwd.setGeometry(220, 135, 25, 25)
        
        self.checkQSphereAsk.setGeometry(5, 20, 220, 20)
        self.checkSystemObjects.setGeometry(5, 45, 220, 20)
        self.checkSqlComments.setGeometry(5, 70, 220, 20)
        self.checkResultsInNewTab.setGeometry(5, 95, 220, 20)
        self.checkDynamicalLink.setGeometry(5, 120, 220, 20) 
        self.getConButton.setGeometry(255, 80, 64, 64)


        self.frame = QFrame(self)
        self.masterHLayout = QHBoxLayout(self.frame)
        self.masterHLayout.setObjectName("masterHLayout")
        self.splitter = QSplitter(self.frame)
        self.splitter.setOrientation(Qt.Horizontal)
        self.splitter.setObjectName("splitter")

        self.groupBoxParams = expandableGroupBox(self.frame)
        self.groupBoxParams.initGroupBox(self.frame, "groupBoxParams", self.frame.width()-300, 5, 24, 24, 300, 45, True)

        self.lcdNumber = QLCDNumber(self.groupBoxParams)  
        self.lcdNumber.setObjectName("lcdNumber")
        self.lcdNumber.setSegmentStyle(QLCDNumber.Filled)
        self.lcdNumber.setStyleSheet("QLCDNumber {border: 2px solid grey; border-radius: 5px; background-color: #6C96C6; color: yellow;   text-align: center;}")
        self.lcdNumber.setDigitCount(8)
        self.lcdNumber.display(QTime.currentTime().toString("hh:mm:ss"))
        self.idTimer = self.startTimer(1000)
                 
        self.lcdNumber.setGeometry(155, 2, 120, 40)
        
        fontchrono = QFont()
        fontchrono.setFamily('Courier')
        fontchrono.setFixedPitch(True)
        fontchrono.setPointSize(8)
        fontchrono.setBold(True)

        self.chronoInfo = QLabel(self.groupBoxParams) 
        carIcon = QImage(getThemeIcon("sgbdr/chronoinfo.png")) 
        self.chronoInfo.setPixmap(QPixmap.fromImage(carIcon))
        self.chronoInfo.setVisible(False)
        self.chronoInfo.setToolTip("")

        self.chronoInfo.setGeometry(30, 0, 125, 43)

        self.chronoInfoText = QLabel(self.chronoInfo)
        self.chronoInfoText.setAlignment(Qt.AlignLeft)
        self.chronoInfoText.setFont(fontchrono)
        self.chronoInfoText.setText("0")
        self.chronoInfoText.setGeometry(38, 12, 80, 20)
        self.chronoInfoText.setStyleSheet("QLabel {border: 2px solid grey; border-radius: 5px; background-color: #009900; color: white;}")

        self.stopChronoSQL = myPushButton(self.chronoInfo)
        toolTip = QApplication.translate("QSphere","Stop the robot")
        self.stopChronoSQL.initPushButton(25, 25, 100, 10, "stopChronoSQL", "", toolTip, True, getThemeIcon("sgbdr/stopchrono.png"), 25, 25, True)   

        self.groupButtons = expandableGroupBox(self.frame)
        self.groupButtons.initGroupBox(self.frame, "groupButtons", self.frame.width()-85, self.frame.height()-310, 24, 24, 48, 280, True)
      
        icon = getThemeIcon("closewindow.png")
        toolTip = QApplication.translate("QSphere","Close")
        self.CloseButton = myPushButton(self.groupButtons)  
        self.CloseButton.initPushButton(36, 36, 50, 10, "CloseButton", "", toolTip, True, icon, 36, 36, True)

        icon = getThemeIcon("print.png")
        toolTip = QApplication.translate("QSphere","Print")
        self.printButton = myPushButton(self.groupButtons)  
        self.printButton.initPushButton(36, 36, 50, 10, "printButton", "", toolTip, True, icon, 36, 36, True)


        icon = getThemeIcon("save.png")
        toolTip = QApplication.translate("QSphere","Save")
        self.saveSqlResults = myPushButton(self.groupButtons) 
        self.saveSqlResults.initPushButton(36, 36, 50, 10, "saveSqlResults", "", toolTip, True, icon, 36, 36, True)
        self.saveSqlResults.setEnabled(False)

        icon = getThemeIcon("sgbdr/reload.png")
        toolTip = QApplication.translate("QSphere","Reload page")
        self.reloadButton = myPushButton(self.groupButtons) 
        self.reloadButton.initPushButton(48, 48, 50, 10, "reloadButton", "", toolTip, True, icon, 48, 48, True)

        icon = getThemeIcon("sgbdr/deleteall.png")
        toolTip = QApplication.translate("QSphere","Delete all entries")
        self.deleteLogButton = myPushButton(self.groupButtons)   
        self.deleteLogButton.initPushButton(48, 48, 50, 10, "deleteLogButton", "", toolTip, True, icon, 48, 48, True)

        self.deleteLogButton.setGeometry(5, 30, 48, 48)
        self.reloadButton.setGeometry(5, 80, 48, 48)
        self.printButton.setGeometry(5, 130, 48, 48)
        self.saveSqlResults.setGeometry(5, 180, 48, 48)
        self.CloseButton.setGeometry(5, 230, 48, 48)

        self.dockDataBases = QDockWidget("SQL", self.splitter)
        self.dockDataBases.setWindowTitle("%s :" % QApplication.translate("QSphere","Resources"))
        self.dockDataBases.setFloating(False)
        self.dockDataBases.setFeatures(QDockWidget.DockWidgetFloatable | QDockWidget.DockWidgetMovable)
        self.dockDataBases.setAllowedAreas(Qt.NoDockWidgetArea)
        self.dockDataBases.setStyleSheet(
                    "QDockWidget { color:rgb(255,255,255); font-size:12px; font-weight: bold; border: 1px solid lightgray;}"
                    "QDockWidget::title {  text-align: center; background: #5D5D5D; padding-left: 5px;}")        


        self.groupBoxDataBase = QGroupBox(self.dockDataBases)  
        self.groupBoxDataBase.setObjectName("groupBoxDataBase")
        self.hLayoutgroupBoxDataBase = QHBoxLayout(self.groupBoxDataBase)
        self.hLayoutgroupBoxDataBase.setObjectName("hLayoutgroupBoxDataBase")
        
        self.tabs = QTabWidget(self.dockDataBases) 
        self.tabs.setTabsClosable(False)
        self.tabs.setDocumentMode(True)
        self.tabs.setTabShape(QTabWidget.Rounded)

        self.hLayoutgroupBoxDataBase.addWidget(self.tabs)

        self.groupBoxViewsDataBase = QGroupBox(self.dockDataBases)  
        self.groupBoxViewsDataBase.setObjectName("groupBoxViewsDataBase")

        self.searchBoxDb = QgsFilterLineEdit(self.groupBoxViewsDataBase)
        self.searchBoxDb.setObjectName("searchBoxDb")
        
        self.objDataBaseViewer = QTreeWidget(self.groupBoxViewsDataBase)
        self.objDataBaseViewer.setHorizontalScrollBarPolicy(Qt.ScrollBarAlwaysOff)
        self.objDataBaseViewer.setObjectName("objDataBaseViewer")
        self.objDataBaseViewer.headerItem().setText(0, "1")
        self.objDataBaseViewer.header().setVisible(False)
        self.objDataBaseViewer.header().setDefaultSectionSize(200)
        self.objDataBaseViewer.setStyleSheet("QTreeWidget { font-size:12px; background-color: rgb(255,255,255); padding: 4px ; color: black}"
                                     "QTreeWidget::item:selected {background-color: #5D5D5D; color: rgb(255,255,255); }")
        self.objDataBaseViewer.setContextMenuPolicy(Qt.CustomContextMenu)
        self.objDataBaseViewer.customContextMenuRequested.connect(self.contextMnuShortCuts)

        self.groupBoxViewsSql = QGroupBox(self.dockDataBases)  
        self.groupBoxViewsSql.setObjectName("groupBoxViewsSql")
        
        self.hLayoutgroupBoxViewsSQL = QHBoxLayout(self.groupBoxViewsSql)
        self.hLayoutgroupBoxViewsSQL.setObjectName("hLayoutgroupBoxViewsSQL")

        self.searchBox = QgsFilterLineEdit(self.groupBoxViewsSql)
        self.searchBox.setObjectName("searchBox")

        self.objSqlViewer = QTreeWidget(self.groupBoxViewsSql)
        self.objSqlViewer.setHorizontalScrollBarPolicy(Qt.ScrollBarAlwaysOff)
        self.objSqlViewer.setObjectName("objSqlViewer")
        self.objSqlViewer.headerItem().setText(0, "1")
        self.objSqlViewer.header().setVisible(False)
        self.objSqlViewer.header().setDefaultSectionSize(200)
        self.objSqlViewer.setStyleSheet("QTreeWidget { font-size:12px; background-color: rgb(255,255,255); padding: 4px ; color: black}"
                                     "QTreeWidget::item:selected {background-color: #5D5D5D; color: rgb(255,255,255); }")
        self.objSqlViewer.setVisible(True)
        self.objSqlViewer.setItemsExpandable(True)
        self.objSqlViewer.setExpandsOnDoubleClick(True)

        self.objSqlViewer.setContextMenuPolicy(Qt.CustomContextMenu)
        self.objSqlViewer.customContextMenuRequested.connect(self.contextMnuShortCutsSql)

        self.groupBoxResults = QGroupBox(self.splitter)
        self.groupBoxResults.setObjectName("groupBoxResults")
        self.hLayoutgroupBoxResults = QHBoxLayout(self.groupBoxResults)
        self.hLayoutgroupBoxResults.setObjectName("hLayoutgroupBoxResults")
        
        self.tabsResults = QTabWidget(self.groupBoxResults) 
        self.tabsResults.setTabsClosable(True) 
        self.tabsResults.setDocumentMode(True)
        self.tabsResults.setTabShape(QTabWidget.Rounded)

        self.hLayoutgroupBoxResults.addWidget(self.tabsResults)
        self.masterHLayout.addWidget(self.splitter)

        self.htmlViewer = QWebView(self.groupBoxResults) 
        self.htmlViewer.setObjectName("htmlViewer")
        self.htmlViewer.settings().setAttribute(QWebSettings.JavascriptEnabled, True)
        self.htmlViewer.settings().setAttribute(QWebSettings.DeveloperExtrasEnabled, True)
        self.htmlViewer.settings().setAttribute(QWebSettings.LinksIncludedInFocusChain, True)
        self.htmlViewer.setRenderHints(QPainter.Antialiasing | QPainter.SmoothPixmapTransform)
        self.htmlViewer.page().setLinkDelegationPolicy(QWebPage.DelegateAllLinks)
        self.htmlViewer.page().setForwardUnsupportedContent(True)
        self.htmlViewer.page().linkClicked.connect(self.goToTabResults)

        icon = getThemeIcon("sgbdr/tableinfo.png")
        index = self.tabsResults.addTab(self.htmlViewer, QIcon(icon), "Infos")
        self.tabsResults.setTabToolTip(index, "Infos")

        self.dataViewer = QTableView(self.groupBoxResults) 
        self.dataViewer.setAccessibleDescription("")

        sizePolicy = QSizePolicy(QSizePolicy.Expanding, QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(3)
        sizePolicy.setHeightForWidth(self.dataViewer.sizePolicy().hasHeightForWidth())
        self.dataViewer.setSizePolicy(sizePolicy)
        self.dataViewer.setSelectionBehavior(QAbstractItemView.SelectRows)
        self.dataViewer.setSelectionMode(QAbstractItemView.SingleSelection)
        self.dataViewer.setHorizontalScrollMode(QAbstractItemView.ScrollPerPixel)
        self.dataViewer.setVisible(False)
        self.dataViewer.setAlternatingRowColors(True)
        self.dataViewer.setStyleSheet("QTableView {alternate-background-color: cyan;background-color: white;}")

        icon = getThemeIcon("sgbdr/tabledata.png")
        index = self.tabsResults.addTab(self.dataViewer, QIcon(icon), self.lblRecords.title())
        self.tabsResults.setTabToolTip(index, "Data")

        self.mapViewer = QgsMapCanvas(self.groupBoxResults) 
        self.mapViewer.setVisible(False)
        self.mapViewer.setAccessibleDescription("")

        self.printMapview = QTextDocument() 

        self.widgetMapView = myMdiSubWindow(self.mapViewer, "widgetConsole", "%s :" % QApplication.translate("QSphere","Actions ..."), 280, 80 )
        ctrlMap(self, self.widgetMapView, True, False, False, 5, 5, 340, 35)

        self.toolPan = QgsMapToolPan(self.mapViewer)
        self.toolZoomIn = QgsMapToolZoom(self.mapViewer, False) 
        self.toolZoomOut = QgsMapToolZoom(self.mapViewer, True)

        self.mapViewer.setMapTool(self.toolZoomIn)

        icon = getThemeIcon("sgbdr/tablegeom.png")
        index = self.tabsResults.addTab(self.mapViewer, QIcon(icon), QApplication.translate("QSphere","Location : "))
        self.tabsResults.setTabToolTip(index, "Map")

        self.textBrowserChilds = QWebView(self.groupBoxResults) 
        self.textBrowserChilds.setObjectName("textBrowserChilds")
        self.textBrowserChilds.settings().setAttribute(QWebSettings.JavascriptEnabled, True)
        self.textBrowserChilds.settings().setAttribute(QWebSettings.DeveloperExtrasEnabled, True)
        self.textBrowserChilds.settings().setAttribute(QWebSettings.LinksIncludedInFocusChain, True)
        self.textBrowserChilds.setRenderHints(QPainter.Antialiasing | QPainter.SmoothPixmapTransform)
        self.textBrowserChilds.page().setLinkDelegationPolicy(QWebPage.DelegateAllLinks)
        self.textBrowserChilds.page().setForwardUnsupportedContent(True)
        self.textBrowserChilds.page().linkClicked.connect(self.getLinkedUrl)

        icon = getThemeIcon("windowmdiarea_false.png")
        toolTip = QApplication.translate("QSphere","Sub mainwindow's")
        index = self.tabsResults.addTab(self.textBrowserChilds, QIcon(icon), toolTip)
        self.tabsResults.setTabToolTip(index, toolTip)
        self.tabsResults.setTabEnabled(3, self.isCond()) 

        self.edit = editor(self)
        
        icon = getThemeIcon("exportcat.png")
        index = self.tabsResults.addTab(self.edit, QIcon(icon), QApplication.translate("QSphere","LOG console"))
        self.tabsResults.setTabToolTip(index, "LOG console")

        self.fixedTabs = self.tabsResults.count()

        self.iconBasesOn = getThemeIcon("sgbdr/baseson.png")
        self.iconBasesOff = getThemeIcon("sgbdr/basesoff.png")

        index = self.tabs.addTab(self.groupBoxViewsDataBase, QIcon(self.pucebase), "%s [0]" % QApplication.translate("QSphere","Databases"))
        self.tabs.setTabToolTip(index,  QApplication.translate("QSphere","Databases"))

        index = self.tabs.addTab(self.groupBoxViewsSql, QIcon(getThemeIcon("sgbdr/sql.png")), QApplication.translate("QSphere","SQL"))
        self.tabs.setTabToolTip(index, QApplication.translate("QSphere","SQL"))

        self.loadTheSqlResources()

        self.groupBoxProgressInfos = QGroupBox(self)
        self.groupBoxProgressInfos.setObjectName("groupBoxProgressInfos")
        self.groupBoxProgressInfos.setTitle("")
        self.groupBoxProgressInfos.setStyleSheet("QGroupBox {border: 2px solid grey; border-radius: 5px; background-color: #6C96C6; color: white;}")

        self.imgProgress = QLabel(self.groupBoxProgressInfos)
        self.fixeImage("bigconnect.png")
       
        self.progressBar = myProgressBar(self.groupBoxProgressInfos)
        self.progressBar.initTextEdit(160, 15, 240, 55, "progressBar", True, 0, True, Qt.AlignCenter, 0)

        self.groupBoxProgressInfos.setVisible(False)

        self.movieCtrl = movieCtrl(self, "sgbdr/attente.gif", True)

        self.widgetCounter = myMdiSubWindow(self.dataViewer,"widgetCounter", "%s :" % QApplication.translate("QSphere","View"), 380, 80 )
       
        self.seuils = [10, 50, 100, 200]
        self.comboSqlBoundary = QComboBox(self.widgetCounter)
        self.comboSqlBoundary.addItems(["10", "50","100","200"])
        self.comboSqlBoundary.setStyleSheet("QComboBox {border: 2px solid grey; border-radius: 5px; background-color: #6C96C6; color: white;}")
        self.comboSqlBoundary.setMinimumSize(100, 30)
        self.comboSqlBoundary.setMaximumSize(100, 30)

        self.lblSqlCounter = QLabel(self.widgetCounter)
        self.lblSqlCounter.setText("0")
        self.lblSqlCounter.setAlignment(Qt.AlignCenter)
        self.lblSqlCounter.setStyleSheet("QLabel {border: 2px solid grey; border-radius: 5px; background-color: #6C96C6; color: white;}")
        self.lblSqlCounter.setMinimumSize(100, 30)
        self.lblSqlCounter.setMaximumSize(100, 30)        

        self.findInListResults = myPushButton(self.widgetCounter)
        self.findInListResults.initPushButton(32, 32, 215, 30, "findInListResults", "", QApplication.translate("QSphere","Find in results"), True, getThemeIcon("search.png"), 32, 32, True)
        self.findInListResults.setShortcut(QKeySequence("Ctrl+Shift+F"))
        self.findInListResults.clicked.connect(self.findInResults)


        self.comboSqlBoundary.setGeometry(10, 30, 100, 30)
        self.lblSqlCounter.setGeometry(110, 30, 100, 30)

        self.groupTableView = QGroupBox(self.widgetCounter)
        self.groupTableView.setGeometry(250, 30, 105, 30)

        self.viewAbstractFromBaseButton = myPushButton(self.groupTableView)
        self.viewAbstractFromBaseButton.initPushButton(25, 25, 5, 2, "viewAbstractFromBaseButton", "", QApplication.translate("QSphere","View XML File for the metadata in the base"), True, getThemeIcon("sgbdr/dataabstract.png"), 25, 25, True)

        self.downloadDataFromBaseButton = myPushButton(self.groupTableView)
        self.downloadDataFromBaseButton.initPushButton(25, 25, 40, 2, "downloadDataFromBaseButton", "", QApplication.translate("QSphere","Download metadata from the base"), True, getThemeIcon("sgbdr/datadownload.png"), 25, 25, True)

        self.deleteDataFromBaseButton = myPushButton(self.groupTableView)
        self.deleteDataFromBaseButton.initPushButton(25, 25, 70, 2, "deleteDataFromBaseButton", "", QApplication.translate("QSphere","Delete metadata in the base"), True, getThemeIcon("sgbdr/datadelete.png"), 25, 25, True)

        self.enableQspButtons(False)

        self.barInfo = QgsMessageBar(self)
        self.barInfo.setSizePolicy( QSizePolicy.Minimum, QSizePolicy.Fixed )

        self.dockDataBases.topLevelChanged.connect(self.getDockDataBases)
        self.dockDataBases.resizeEvent = self.setDockDataBasesSize
        self.dockDataBases.moveEvent = self.setDockDataBasesSize

        self.dockSql.topLevelChanged.connect(self.getDock)
        self.dockSql.resizeEvent = self.setDockSize
        self.dockSql.moveEvent = self.setDockSize

        
        self.reloadQgisButton.clicked.connect(self.fixeInfosConnexionQGIS)
        self.checkDynamicalLink.clicked.connect(self.getFrozenLink)
        self.searchBoxDb.textChanged.connect(self.textChanged)
        self.searchBox.textChanged.connect(self.textChanged)

        self.buttonPan.clicked.connect(self.pan)
        self.buttonZoomIn.clicked.connect(self.zoomIn)
        self.buttonZoomOut.clicked.connect(self.zoomOut)
        self.buttonZoomAll.clicked.connect(self.zoomAll)
        self.buttonZoomPrevious.clicked.connect(self.zoomPrevious)
        self.buttonZoomNext.clicked.connect(self.zoomNext)
        self.buttonPropertiesLayer.clicked.connect(self.getPropertiesLayer)

        self.tabsResults.tabCloseRequested.connect(self.closeTabs)
        self.tabsResults.currentChanged.connect(self.viewInfos)
        
        self.qgisConnexion.currentIndexChanged.connect(self.fixeInfosConnexionQGIS)
        self.groupParams.collapsedStateChanged.connect(self.resizeCollapsedGroupBox)
        self.groupParamConnexion.collapsedStateChanged.connect(self.changePosGroupOtions)
        self.getSqlResultsButton.clicked.connect(self.executeSql)

        self.eraseSqlButton.clicked.connect(self.eraseSql)
        self.getSqlButton.clicked.connect(self.openFilesSql)
        self.saveSqlFilesButton.clicked.connect(self.saveFilesSql)
        self.saveSqlFilesAsButton.clicked.connect(self.saveFilesAsSql)
 
        self.viewUser.clicked.connect(self.enabledPwd)
        self.viewPwd.clicked.connect(self.enabledPwd)

        self.getConButton.clicked.connect(self.getInfosBase)
        
        self.objDataBaseViewer.itemDoubleClicked.connect(self.getSchemasBase) 
        self.objDataBaseViewer.itemClicked.connect(self.changeEnableQspButtons)
        self.objSqlViewer.itemDoubleClicked.connect(self.getInfosSql)
        self.comboSqlBoundary.currentIndexChanged.connect(self.getDataViewer)

        self.viewAbstractFromBaseButton.clicked.connect(self.viewAbstractData)
        self.downloadDataFromBaseButton.clicked.connect(self.downloadData)
        self.deleteDataFromBaseButton.clicked.connect(self.deleteData)

        self.deleteLogButton.clicked.connect(self.deleteLog) 
        self.reloadButton.clicked.connect(self.reloadTabWidget) 
        self.printButton.clicked.connect(self.printWidget)
        self.saveSqlResults.clicked.connect(self.saveResultsFiles)
        self.CloseButton.clicked.connect(self.close) 
        self.chronoSql.clicked.connect(self.startStopChronoSql)
        self.stopChronoSQL.clicked.connect(self.startStopChronoSql)
        self.searchbutton.clicked.connect(self.findWord)

        self.fixeInfosConnexionQGIS()
        self.setWindowState(Qt.WindowMaximized)
        self.changeEnableQspButtons() 


    def textChanged(self):
        caseDb = True if (self.sender() == self.searchBoxDb) else False
        treeWidget = self.objDataBaseViewer if caseDb else self.objSqlViewer 
        text = self.sender().text().strip(' ').lower()
        if caseDb :
            for i in range(treeWidget.topLevelItemCount()):
                item = treeWidget.topLevelItem(i)
                for j in range(item.childCount()):
                    childItem = item.child(j)
                    for k in range(childItem.childCount()):
                        itemTable = childItem.child(k)
                        show = (re.search(text, itemTable.text(0), re.IGNORECASE))
                        itemTable.setHidden(not show)
        else :     
            for i in range(treeWidget.topLevelItemCount()):
                item = treeWidget.topLevelItem(i)
                show = (re.search(text, item.text(0), re.IGNORECASE))
                item.setHidden(not show)
        
    def getFrozenLink(self):
        self.dynamicalLink = not self.dynamicalLink
        self.checkDynamicalLink.setChecked(self.dynamicalLink)

    def copySqlInEditorSql(self):
        self.eraseSql()
        self.editorSql.setText(self.tabsResults.tabToolTip(self.tabsResults.currentIndex()))

    def findInResults(self):
          self.findDialog = findDialogToolSearchFiles(self, self.tabsResults.currentWidget())
          ret = self.findDialog.exec_()
          self.findDialog.lineEdit.setFocus()

    def findWord(self):
        ret = self.editorSql.findDialog.exec_()
        self.editorSql.findDialog.lineEdit.setFocus()
        if self.editorSql.findDialog.ret > 0 :
          self.editorSql.find_text = self.editorSql.findDialog.getFindText()
         
    def startStopChronoSql(self):
        if not self.isStartChronoSql :
                self.deltaChrono, ok = QInputDialog.getInt(None, QApplication.translate("QSphere","Params"), "<h2>%s :</h2><br>[%s]" % (QApplication.translate("QSphere","Time interval"),
                                                                                                                                        QApplication.translate("QSphere","minutes")), QLineEdit.Normal)
                if not ok: return
                if self.deltaChrono == 0 : return
                if self.lastSql == []: return

                self.chronoSqltoRefresh = list(self.lastSql)
                self.sendToConsole("sql", self.getTimeStamp(), "%s<br>%s : %s %s." % (self.chronoSqltoRefresh,
                                                                                  QApplication.translate("QSphere","Time interval"),
                                                                                  self.deltaChrono,
                                                                                  QApplication.translate("QSphere","minutes")), self.ipucestartChrono)                
                self.startTimeChrono = thedatetime.now()
                self.deltaChronoInMinutes = self.deltaChrono 
                self.endTimeChrono = self.startTimeChrono + timedelta(minutes=self.deltaChronoInMinutes)
                self.endTimeChronoText = thedatetime.strptime(self.endTimeChrono.replace(microsecond=0).strftime("%d/%m/%Y %Hh:%Mm:%Ss"), "%d/%m/%Y %Hh:%Mm:%Ss")
                
                diff = self.endTimeChronoText - thedatetime.strptime(thedatetime.now().replace(microsecond=0).strftime("%d/%m/%Y %Hh:%Mm:%Ss"), "%d/%m/%Y %Hh:%Mm:%Ss")
                self.chronoInfoText.setText("%s" % diff)
                
                toolTip = ""
                for line in textwrap.wrap("%s" % self.chronoSqltoRefresh, 100) : toolTip+= "%s<br>" % line
                self.chronoInfo.setToolTip(toolTip)
                self.deltaChrono = int(self.deltaChrono)*1000*60
                
               
        self.isStartChronoSql = not(self.isStartChronoSql)    
        self.chronoSql.setIcon(QIcon(self.pucestartChrono)) if not self.isStartChronoSql else self.chronoSql.setIcon(QIcon(self.pucestopChrono))
        self.chronoSql.setToolTip(QApplication.translate("QSphere","Start the robot")) if not self.isStartChronoSql else self.chronoSql.setToolTip(QApplication.translate("QSphere","Stop the robot"))
        self.chronoInfo.setVisible(self.isStartChronoSql)
        
    def enabledPwd(self):
        obj = self.pwdUser if self.sender() == self.viewPwd else self.nameUser
        obj.setEchoMode(QLineEdit.Normal) if obj.echoMode() == QLineEdit.Password else obj.setEchoMode(QLineEdit.Password)
        self.sender().setIcon(QIcon(getThemeIcon("show.png"))) if obj.echoMode() == QLineEdit.Password else self.sender().setIcon(QIcon(getThemeIcon("hide.png")))  


    def loadTheSqlResources(self):
        fileName = join(dirname(__file__),"ressources/mysql.xml")

        if exists(fileName) :
                self.dicoSql = {}

                try : self.tree = ET.parse(fileName)
                except :
                     msgBadFileXml(self)
                     return False

                self.root = self.tree.getroot()
                sqlquery = self.root.findall("sqlquery")

                listSql = []
                for node in sqlquery :
                    dicosqlquery = self.parse_subWindow(node)
                    title = dicosqlquery["title"]

                    index = 1
                    while dicoHasKey(self.dicoSql, title) :
                          title = "%s %s" % (dicosqlquery["title"], index); index+= 1;    

                    listSql.append(title)
                    self.dicoSql[title] = dicosqlquery
                    
                listSql = sorted(listSql)
                self.refreshTreeViewSql(listSql)

    def refreshTreeViewSql(self, listSql):    
        self.getBasesTreeView(listSql, self.objSqlViewer, getThemeIcon("sgbdr/sql.png"), "sql")
        self.objSqlViewer.collapseAll()
        self.objSqlViewer.setCurrentItem(self.objSqlViewer.itemAt(0, 0))
        self.tabs.setTabText(1, "%s [%s]" % (self.tabs.tabToolTip(1), self.objSqlViewer.topLevelItemCount()))

    def makeChildSql(self, parentItem):        
        dicoicones = {"enabled" : "defaulttrue.png", "source" : "copier.png", "run": "sgbdr/sqlplay.png"}
        listChildren = parentItem.takeChildren()
        for child in listChildren : parentItem.removeChild(child)
        dicosqlquery = self.dicoSql[parentItem.text(0)] 
        isEnabled = False
        for key in dicosqlquery.keys() :
                
            if key != "" and not key in ("title", "enabled")  :
               itemSchema = QTreeWidgetItem()
               try : itemSchema.setIcon(0, QIcon(getThemeIcon(dicoicones[key])))
               except : pass
               itemSchema.setText(0, "%s" % dicosqlquery[key].replace("\n", "")) 
               itemSchema.setToolTip(0, "%s" % dicosqlquery[key]) 
               itemSchema.setWhatsThis(0, "%s" % key)
               parentItem.addChild(itemSchema)

            if key == "enabled" : isEnabled = (dicosqlquery[key].title()== "True") #True if dicosqlquery[key].title()== "True" else False    

        if isEnabled == True : 
               itemSchema = QTreeWidgetItem()
               try : itemSchema.setIcon(0, QIcon(getThemeIcon(dicoicones["run"])))
               except : pass
               itemSchema.setText(0, QApplication.translate("QSphere", "run").title())
               itemSchema.setToolTip(0, "%s" % dicosqlquery["source"]) 
               itemSchema.setWhatsThis(0, "run")
               parentItem.addChild(itemSchema)

        parentItem.setDisabled(not isEnabled)       
    

    def getInfosSql(self, parentItem):
        if not parentItem.whatsThis(0) in ("run", "source") or parentItem.isDisabled() : return
        if parentItem.whatsThis(0) in ("run", "source") :
             self.editorSql.setText(parentItem.toolTip(0))
             self.editorSql.setToolTip("")
             self.dockSql.setWindowTitle("SQL : %s" % parentItem.parent().text(0))  
             if parentItem.whatsThis(0) == "run" : self.executeSql()   

    def parse_subWindow(self, node):
            dicosqlquery = {}
            keys = ("title", "source", "enabled")
            for key in keys :
                try : dicosqlquery[key] = node.find(key).text
                except :
                       if key in keys : dicosqlquery[key] = ""
                       elif key == "enabled" : dicosqlquery[key] = "False" 
            return dicosqlquery

    def setDockDataBasesSize(self, obj):
        self.groupBoxDataBase.setGeometry(2, 20, self.dockDataBases.width()-20, self.dockDataBases.height()-22 )
        self.searchBoxDb.setGeometry(5, 5, self.dockDataBases.width()-60, 25 )
        self.objDataBaseViewer.setGeometry(5, 40, self.dockDataBases.width()-55, self.dockDataBases.height()-120 )
        self.searchBox.setGeometry(5, 5, self.dockDataBases.width()-60, 25 )
        self.objSqlViewer.setGeometry(5, 40, self.dockDataBases.width()-55, self.dockDataBases.height()-120 )
        
    def getDockDataBases(self):
        if not self.dockDataBases.isFloating() :
            self.splitter.insertWidget(0, self.dockDataBases)
            self.splitter.blockSignals(False)
            sizes = (70, 340)
            self.splitter.setSizes(sizes)
        else :
            self.splitter.blockSignals(True)
            sizes = (0, 340)
            self.splitter.setSizes(sizes)            
            self.dockDataBases.setParent(self)
            self.dockDataBases.setFloating(True)
            width, height = QDesktopWidget().screenGeometry().width()/2, QDesktopWidget().screenGeometry().height()/2
            self.dockDataBases.setGeometry(200, 200, width, height)
            self.dockDataBases.setVisible(True)
        
    def setDockSize(self, obj):
        self.editorSql.setGeometry(2, 20, self.dockSql.width()-4, self.dockSql.height()-22 )
        self.widgetEditorSql.setGeometry(self.editorSql.width()-320, self.editorSql.height()-90, 100, 35)
        
    def getDock(self):
        if not self.dockSql.isFloating() :
            self.dockSql.setGeometry(320, 10, self.groupParams.width()-340, self.groupParams.height()-15 )
            self.dockSql.setVisible(False) if self.groupParams.isCollapsed() else self.dockSql.setVisible(True)
            self.groupParams.setCollapsed(False)
        else :
            width, height = QDesktopWidget().screenGeometry().width()/2, QDesktopWidget().screenGeometry().height()/2
            self.dockSql.setGeometry(200, 200, width, height)
            self.dockSql.setVisible(True)
            self.groupParams.setCollapsed(True)

    
    def cleanExit(self):
        try :
            if self.currentLayer: QgsProject.instance().removeMapLayers([self.currentLayer.id()])
        except : pass
        
    def closeEvent(self, event):
        event.ignore()
        self.cleanExit()
        if self.dockSql.isFloating() : self.dockSql.hide()
        self.hide() 

    def apply_style_accept(self) :
        if self.styleDlg:
           self.styleDlg.widgetproperties.apply()
           self.mapViewer.refresh()
           self.styleDlg.accept()
           
    def apply_style(self):
        if self.styleDlg:
                if self.mapViewer.layer(0).type()==0 : self.styleDlg.apply()
                else : self.styleDlg.widgetproperties.apply()
                self.mapViewer.refresh()
            
    def getPropertiesLayer(self):
        layer = self.mapViewer.layer(0)
        if not layer : return
        if layer.type() == 0 : dlg = QgsRendererPropertiesDialog(layer, QgsStyle.defaultStyle(), False)
        elif layer.type() == 1 :
            dlg = QDialog()
            dlg.setWindowFlags(Qt.WindowMaximizeButtonHint | Qt.WindowStaysOnTopHint | Qt.WindowCloseButtonHint)
            dlg.setWindowTitle(QApplication.translate("QSphere", "Open properties widget for the layer"))
            layout = QVBoxLayout(dlg)
            dlg.widgetproperties = QgsRendererRasterPropertiesWidget(layer, self.mapViewer, dlg)
            layout.addWidget(dlg.widgetproperties)
        
        else : return
        
        windowIcon(dlg , "layers-database.png")
        if layer.type() == 0 : dlg.setMapCanvas(self.mapViewer)
        dlg.setWindowTitle("%s : %s" % (dlg.windowTitle(), layer.name()))
        buttonBox = None
        for conteneur in dlg.children():
                if conteneur.children()!= []:
                   for child in conteneur.children():
                          for elt in child.children():
                                  if type(elt) == QDialogButtonBox :
                                             buttonBox = elt
                                             break

        if buttonBox and layer.type() == 0 :
                self.applystyle = buttonBox.addButton(QApplication.translate("QSphere", "Apply style"), QDialogButtonBox.ActionRole)
                self.applystyle.clicked.connect(self.apply_style)
                buttonBox.rejected.connect(dlg.close)
        else :
           buttonBox = QDialogButtonBox()
           self.quitButton = buttonBox.addButton(QApplication.translate("QSphere","Cancel"), QDialogButtonBox.RejectRole)
           self.applystyle = buttonBox.addButton(QApplication.translate("QSphere", "Apply style"), QDialogButtonBox.ActionRole)
           self.acceptButton = buttonBox.addButton(QApplication.translate("QSphere","Validate"), QDialogButtonBox.AcceptRole)
           buttonBox.accepted.connect(self.apply_style_accept)
           buttonBox.rejected.connect(dlg.reject)
           layout.addWidget(buttonBox)
           self.applystyle.clicked.connect(self.apply_style)

        self.styleDlg = dlg                          
        dlg.exec_()
        self.mapViewer.refresh()


    def addTag(self):
        textToInsert = {self.act_addparambase : " '%param$' --{votre base}-- ",
                        self.act_addparamschema : " '%param$' --{votre schema}-- ",
                        self.act_addparamschematable : " '%param$' --{votre schema.table}-- ",
                        self.act_addparamtable : " '%param$' --{votre table}-- ",
                        self.act_addparamfunction : " '%param$' --{votre fonction}-- ",
                        self.act_addPARAMbase : " '%PARAM$' --{votre base}-- ",
                        self.act_addPARAMschema : " '%PARAM$' --{votre schema}-- ",
                        self.act_addPARAMschematable : " '%PARAM$' --{votre schema.table}-- ",
                        self.act_addPARAMtable : " '%PARAM$' --{votre table}-- ",
                        self.act_addPARAMfunction : " '%PARAM$' --{votre fonction}-- ",                        
                        }
        self.editorSql.insertText("%s \n" % textToInsert[self.sender()] if self.checkQuote.isChecked() else "%s \n" % textToInsert[self.sender()].replace("'",""))
        
        
    def makeMenuAddParamButton(self, obj):
        menuAddParamButton = QMenu() 
        menuAddParamButton.setStyleSheet("QMenu {color: white; background-color:qlineargradient(x1:0, y1:0, x2:0, y2:1, stop: 0 #cccccc, stop: 1 #5D5D5D);}")

        if obj == self.BAddParam :
                pixpuceparam = QPixmap(self.puceparami)
                self.act_addparambase = menuAddParamButton.addAction(QIcon(pixpuceparam), '{:<45}'.format(QApplication.translate("QSphere","Insert Base tag")), self.addTag)
                self.act_addparamschema = menuAddParamButton.addAction(QIcon(pixpuceparam), '{:<45}'.format(QApplication.translate("QSphere","Insert Schema tag")), self.addTag)
                self.act_addparamschematable = menuAddParamButton.addAction(QIcon(pixpuceparam), '{:<45}'.format(QApplication.translate("QSphere","Insert Schema.Table tag")), self.addTag)
                self.act_addparamtable = menuAddParamButton.addAction(QIcon(pixpuceparam), '{:<45}'.format(QApplication.translate("QSphere","Insert Table tag")), self.addTag)
                self.act_addparamfunction = menuAddParamButton.addAction(QIcon(pixpuceparam), '{:<45}'.format(QApplication.translate("QSphere","Insert function tag")), self.addTag)

        elif obj == self.BAddPARAM:
                pixpuceparam = QPixmap(self.puceparamc)
                self.act_addPARAMbase = menuAddParamButton.addAction(QIcon(pixpuceparam), '{:<45}'.format(QApplication.translate("QSphere","Insert Base tag")), self.addTag)
                self.act_addPARAMschema = menuAddParamButton.addAction(QIcon(pixpuceparam), '{:<45}'.format(QApplication.translate("QSphere","Insert Schema tag")), self.addTag)
                self.act_addPARAMschematable = menuAddParamButton.addAction(QIcon(pixpuceparam), '{:<45}'.format(QApplication.translate("QSphere","Insert Schema.Table tag")), self.addTag)
                self.act_addPARAMtable = menuAddParamButton.addAction(QIcon(pixpuceparam), '{:<45}'.format(QApplication.translate("QSphere","Insert Table tag")), self.addTag)
                self.act_addPARAMfunction = menuAddParamButton.addAction(QIcon(pixpuceparam), '{:<45}'.format(QApplication.translate("QSphere","Insert function tag")), self.addTag)
   
        obj.setMenu(menuAddParamButton)
        
    def saveSqlNodesInFile(self):
        fileName = join(dirname(__file__),"ressources/mysql.xml")

        lang = self.localLangage.lower()

        xml = '<?xml version="1.0" encoding="UTF-8"?>\n'
        xml+= '<gmd:MD_Metadata xsi:schemaLocation="http://www.isotc211.org/2005/gmd http://schemas.opengis.net/iso/19139/20060504/gmd/gmd.xsd" xmlns:gmd="http://www.isotc211.org/2005/gmd" xmlns:gco="http://www.isotc211.org/2005/gco" xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xmlns:gml="http://www.opengis.net/gml" xmlns:xlink="http://www.w3.org/1999/xlink">\n'
        xml+= '\t<gmd:qspheresql>\n'
        xml+= '\t\t<gco:file>\n'
        xml+= '\t\t\t<gco:CharacterString>%s</gco:CharacterString>\n' % fileName
        xml+= '\t\t</gco:file>\n'
        xml+= '\t\t<gmd:language>\n'
        xml+= '\t\t\t<gmd:LanguageCode codeList="http://www.loc.gov/standards/iso639-1/" codeListValue="%s">%s</gmd:LanguageCode>\n' % (lang, lang)
        xml+= '\t\t</gmd:language>\n'
        xml+= '\t</gmd:qspheresql>\n'

        i = 0
        for key in self.dicoSql :
                infosNodeSQL = self.dicoSql[key]
                xml+= '\t<sqlquery>\n'
                xml+= '\t\t<id>%s</id>\n' % i
                xml+= '\t\t<title>%s</title>\n' % infosNodeSQL["title"]
                xml+= '\t\t<source><![CDATA[%s]]></source>\n' % infosNodeSQL["source"]
                xml+= '\t\t<enabled>%s</enabled>\n' % infosNodeSQL["enabled"]
                xml+= '\t</sqlquery>\n'
                i+= 1
        xml+= '\t</gmd:MD_Metadata>'

        with open(fileName, 'w', encoding="utf-8", errors="surrogateescape") as inFile : inFile.write(xml)

        title = QApplication.translate("QSphere", "Information" ) 
        msg = QApplication.translate("QSphere","The file xml was saved as")
        sendMsg(self, title , "%s :<br><u><i>%s</i></ul>" % (msg, fileName), Qgis.Info, self.duration_info)

            
    def addNodeSql(self) :
        self.dlgNode = dlgNode(self, QApplication.translate("QSphere","Add SQL node"), "%s %s" % (QApplication.translate("QSphere","Add a "), QApplication.translate("QSphere","")), "sgbdr/sql.png", "", "", True)
        ret = self.dlgNode.exec_()

        
    def aspirateNodeSql(self) :
        self.dlgNode = dlgNode(self, QApplication.translate("QSphere","Add SQL node"), "%s %s" % (QApplication.translate("QSphere","Add a "), QApplication.translate("QSphere","")), "sgbdr/sql.png",
                                                "", self.editorSql.text(), True)
        ret = self.dlgNode.exec_()

    def editNodeSql(self) :
        item = self.objSqlViewer.currentItem()    
        if not item : return
        if self.objSqlViewer.indexOfTopLevelItem(item) == -1 : self.objSqlViewer.setCurrentItem(item.parent()); item = item.parent();

        infosNode = self.dicoSql[item.text(0)]
        self.dlgNode = dlgNode(self, QApplication.translate("QSphere","Edit SQL node"), "%s %s" % (QApplication.translate("QSphere","Edit SQL node"), QApplication.translate("QSphere","")), "sgbdr/sql.png",
                                                item.text(0), item.child(0).toolTip(0), infosNode['enabled'])
        self.oldtitle = item.text(0)
        ret = self.dlgNode.exec_()
        self.oldtitle = None

        
    def deleteNodeSql(self) :
        item = self.objSqlViewer.currentItem()
        if not item : return
        if self.objSqlViewer.indexOfTopLevelItem(item) == -1 : self.objSqlViewer.setCurrentItem(item.parent()); item = item.parent();
        index = self.objSqlViewer.currentIndex().row()
        if self.askForActions("deletenode", item.text(0), self.objSqlViewer) :
           del self.dicoSql[item.text(0)]     
           self.objSqlViewer.takeTopLevelItem(index)
           self.tabs.setTabText(1, "%s [%s]" % (self.tabs.tabToolTip(1), self.objSqlViewer.topLevelItemCount()))

    def integrateChanges(self, infosNode) :    
           title = infosNode["title"]
           if dicoHasKey(self.dicoSql, title) : del self.dicoSql[self.oldtitle] 
           self.dicoSql[title] = infosNode 
           listSql = sorted(self.dicoSql.keys())
           self.refreshTreeViewSql(listSql)
           self.fixeCurrentItem(title)

    def fixeCurrentItem(self, title):    
        items = self.objSqlViewer.findItems(title, Qt.MatchFixedString | Qt.MatchRecursive)
        if items != [] :
           for item in items :
               self.objSqlViewer.setCurrentItem(item)
               break           
        

    def freeAllOtherButtons(self, senderButton):
        buttons = (self.getSqlButton, self.saveSqlFilesButton, self.saveSqlFilesAsButton, self.getSqlResultsButton)
        for button in buttons :
            if button != senderButton : 
                button.subMenuVisble = False
                button.restoreButton()
   
    def zoomIn(self): self.mapViewer.setMapTool(self.toolZoomIn)
    def zoomOut(self): self.mapViewer.setMapTool(self.toolZoomOut)
    def pan(self): self.mapViewer.setMapTool(self.toolPan)

    def zoomAll(self): self.mapViewer.zoomToFullExtent()
    def zoomPrevious(self):self.mapViewer.zoomToPreviousExtent()
    def zoomNext(self): self.mapViewer.zoomToNextExtent()


    def timerEvent(self, e):
        self.lcdNumber.display(QTime.currentTime().toString("hh:mm:ss"))
        if self.movieCtrl.isVisible() : self.movieCtrl.repaint()
        if self.isStartChronoSql :
           if ((thedatetime.now()-self.startTimeChrono).total_seconds() * 1000) >= self.deltaChrono :
                self.startTimeChrono = thedatetime.now()
                self.endTimeChrono = self.startTimeChrono + timedelta(minutes=self.deltaChronoInMinutes)
                self.endTimeChronoText = thedatetime.strptime(self.endTimeChrono.replace(microsecond=0).strftime("%d/%m/%Y %Hh:%Mm:%Ss"), "%d/%m/%Y %Hh:%Mm:%Ss")
                self.sendToConsole("sql", self.getTimeStamp(), "%s..." % QApplication.translate("QSphere","Execute SQL"), self.ipucestartChrono)
                self.executeSql()

           diff = self.endTimeChronoText - thedatetime.strptime(thedatetime.now().replace(microsecond=0).strftime("%d/%m/%Y %Hh:%Mm:%Ss"), "%d/%m/%Y %Hh:%Mm:%Ss")
           self.chronoInfoText.setText("%s" % diff)

           
    def changePosGroupOtions(self, state) :
        if self.groupParamConnexion.isCollapsed() : self.groupOptions.setGeometry(10, 40, self.groupOptions.width(), self.groupOptions.height())
        else : self.groupOptions.setGeometry(10, 40+self.groupParamConnexion.height(), self.groupOptions.width(), self.groupOptions.height())

        
    def resizeCollapsedGroupBox(self, state) :
        if self.groupParams.isCollapsed() : self.frame.setGeometry(5, 30, self.width()-10 , self.height()-40)
        else : self.frame.setGeometry(5, 200, self.width()-10 , self.height()-220)
        
        self.dockSql.setVisible(True)

        if self.groupParams.isCollapsed() :
           if not self.dockSql.isFloating() : self.dockSql.setVisible(False)
           else : self.dockSql.setVisible(True)
        else : self.dockSql.setVisible(True)


    def resizeEvent(self,ev):
         
         self.groupParams.setMaximumWidth(self.width()-10)
         self.groupParams.setGeometry(5, 0, self.width()-10, 170)

         if not self.dockSql.isFloating() :       
                 self.dockSql.setGeometry(320, 10, self.groupParams.width()-340, self.groupParams.height()-15)
                 self.editorSql.setGeometry(2, 20, self.dockSql.width()-4, self.dockSql.height()-22)

         self.groupBoxProgressInfos.setGeometry(self.width()*2/3-100, self.height()/2-100, 200, 200)
         self.imgProgress.setGeometry(10, 0, self.groupBoxProgressInfos.width()-20, self.groupBoxProgressInfos.height()-20)
         self.progressBar.setGeometry(10, self.groupBoxProgressInfos.height()-40, self.groupBoxProgressInfos.width()-20, 20)

         self.resizeCollapsedGroupBox(self.groupParams.isCollapsed())
         self.movieCtrl.setGeometry(int(self.tabs.width()*2)/3-70, 10, 64, 64)

         if self.groupBoxParams.state == 1: self.groupBoxParams.setGeometry(self.frame.width()-310, 0, 300, 45)
         else : self.groupBoxParams.setGeometry(self.frame.width()-self.groupBoxParams.width()-10, 0, self.groupBoxParams.width(), self.groupBoxParams.height())

         if self.groupButtons.state == 1: self.groupButtons.setGeometry(self.frame.width()-85, self.frame.height()-310, 48, 280)
         else : self.groupButtons.setGeometry(self.frame.width()-self.groupButtons.minimumWidth, self.frame.height()-310, self.groupButtons.width(), self.groupButtons.height())

         self.groupButtons.updateSizes(self.frame.width()-85, self.frame.height()-310)
         self.groupBoxParams.updateSizes(self.frame.width()-300, 0)

         self.barInfo.setGeometry(0, 0, self.width(), 90)

      
    def loadListMddFiles(self):
        try : nameParent = self.parent._objectName
        except :
            try : nameParent = self.parent.objectName()
            except : return

        targetObj = self.parent.mainWindow if nameParent == "qsphere" else self.parent 

        if targetObj == None : return
       
        if self.changedMddFiles :
            listSubWindows = targetObj.mdi.subWindowList(QMdiArea.CreationOrder)
            if len(listSubWindows) == 0 : self.changedMddFiles = False; self.tabsResults.setTabEnabled(3, False) ; return;  #root.childCount() == 0
            else : self.tabsResults.setTabEnabled(3, True)
            
            abstract = "No result"
            html = self.html

            html+= "<script type='text/javascript'>"
            path = correctPath(dirname(__file__)).replace("\\","/")
            with open("%sressources/viewerdata.js" % path, 'r') as javaScriptFile :
                 linesForJavaScript = javaScriptFile.read()
            html+= linesForJavaScript                 
            html+= "</script>"
            
            iconWindowsCount = getThemeIcon("windowcounts.png")
            html+= "<h2><img src='file:///%s'> %s</h2><table width='100%%'>" % (iconWindowsCount,  QApplication.translate("QSphere","Sub mainwindow's"))
            html+= "<tr><th></th><th width='25%%'>%s</th><th>%s</th></tr>" % (QApplication.translate("QSphere","Title : "), QApplication.translate("QSphere","Abstract : "))   
           
            i = 0
            for subWindow in listSubWindows :    
                obj = subWindow.findChild(myTextEdit, "resume")
                abstract = obj.toPlainText().replace("'", "\'") if obj != None else "No result"
                abstract = abstract.replace("\n", "<br>")
                html+= "<tr><td><a href='UPLOAD%s'><img src='file:///%s'></a></td><td><h3>%s<h3></td><td>" % (long(subWindow.winId()), getThemeIcon("copyinlist.png"), textwrap.wrap("%s" % subWindow.windowTitle(), 50)) 
                html+= "<a href='javascript: switchInfoPerso(%s);'><img src='file:///%s'><span id='msg%s'>%s</span></a>" % (i, self.pucearrow, i, QApplication.translate("QSphere", "Hide"))
                html+= "<table><tr><td width='1%%'></td><td bgcolor='white' bordercolor='black'>"
                html+= "<div id='divacacher%s' style='background: #efe; padding: 5px; width: 100%%; height: 100%%;overflow-x: hidden; overflow-y: scroll;'>%s</div></td></tr></table></td></tr>" % (i, abstract)
                i+= 1
            html+= "</table>"

            html+= "<script type='text/javascript'>"
            html+= "function switchInfoPerso(index)"
            html+= "{"
            html+= "divInfo = document.getElementById('divacacher'+index);"
            html+= "divTexte = document.getElementById('msg'+index);"
            html+= "if (divInfo.style.display == 'none'){divInfo.style.display = 'block';divTexte.innerHTML='%s'}" % QApplication.translate("QSphere", "Hide")
            html+= "else{divInfo.style.display = 'none';divTexte.innerHTML='%s'}" % QApplication.translate("QSphere", "View")
            html+= "}\n"                      
            html+= "function init(){"

        
            i = 0
            for subWindow in listSubWindows :  
               html+= "var TableSorter%s = new TSorter;" %(i)
               html+= "TableSorter%s.init('maTable%s');" %(i, i)
               i+= 1
            html+= "}"
            html+= "</script>"
            html+= "<script type='text/javascript'>"    
            html+= "window.onload = init;"
            html+= "</script>"
            html+= "</body></html>"

            self.textBrowserChilds.setHtml(html, self.baseurl) 
            self.infosChild = True
            self.changedMddFiles = False



    def viewInfos(self , index): 
        self.saveSqlResults.setEnabled(index > self.fixedTabs-1)
        self.reloadButton.setEnabled(index != 4)
        self.getInfos()
        

    def deleteLog(self):
            index  = self.tabsResults.currentIndex()
            if index == 0 : self.htmlViewer.setHtml("", self.baseurl)      
            elif index == 4 : self.edit.setText("")
            elif index > self.fixedTabs -1  : self.closeTabs(index)

    def reloadTabWidget(self):
        index  = self.tabsResults.currentIndex()
        if index == 0 :
           if self.htmlViewer.accessibleDescription()!= "sql" : self.getInfos()     
           else :     
                   if self.lastSql != []:
                      self.chronoSqltoRefresh = list(self.lastSql)
                      self.executeSql()
        elif  index == 1 : self.dataViewer.setAccessibleDescription("refresh") ; self.getDataViewer();
        elif index == 2 : self.mapViewer.setAccessibleDescription("refresh") ; self.getMapViewer()
        elif index == 3 : self.changedMddFiles = True ; self.loadListMddFiles();
        elif index > self.fixedTabs -1  : self.reloadExecuteSql(index)


    def reloadExecuteSql(self, index):
        query = self.tabsResults.tabToolTip(index)
        if not self.targetDataBase : self.errorMsgBase(); return;
        
        con, dns, service, port = self.getConnexion(False, False, None, None, None)
        if con != None :
           con.set_isolation_level(ISOLATION_LEVEL_AUTOCOMMIT)
           cur = con.cursor()

           self.movieCtrl.startMovie()

           self.startTime = thedatetime.now()
           records = self.getRecords(cur, query, True)
           fieldNames = [desc[0] for desc in cur.description ]
           model = baseTableModel(fieldNames, records, self)
           self.tabsResults.currentWidget().setModel(model)

           self.movieCtrl.stopMovie()

           title = QApplication.translate("QSphere", "Information" ) 
           msg = "%s : %s<br>%s" % (QApplication.translate("QSphere","Reload page"), QApplication.translate("QSphere","Successfull"), query)
           sendMsg(self, title , msg, Qgis.Info, self.duration_info) 

        else :

           self.errorMsgBase()


    def getImage(self, printer, index):
        fileName = join(dirname(__file__),"xml/tempo.png") 
        self.mapViewer.saveAsImage(fileName, None, "PNG")
        self.printMapview.clear()
        self.printMapview.setPageSize(QSizeF(printer.pageRect().size()))

        try:
            from osgeo.gdal import Open as gdalopen
            from osgeo.gdal import GA_ReadOnly
        except:
            from gdal import Open as gdalopen
            from gdal import GA_ReadOnly

        sourceWidth, sourceHeight = printer.pageRect().width(), printer.pageRect().height()
        dataset = gdalopen( fileName, GA_ReadOnly )
        if dataset :
            sourceWidth = dataset.RasterXSize
            sourceHeight = dataset.RasterYSize
        xscale = (printer.pageRect().width())/ float(sourceWidth)
        yscale = (printer.pageRect().height()-100) / float(sourceHeight)
        scale = min(xscale, yscale)
        width = int(sourceWidth*scale)
        
        title = "%s : %s" % (QApplication.translate("QSphere", "Print"), self.tabsResults.tabText(index))

        html = """
        <head>
            <title>Report</title>
        </head>
        <body>
            <table width="100%">
                <tr><td><h2>{} {}</h2></td></tr>
            </table>
            <hr>
            <p align=right><img src="{}" width="{}"></p>
            <p align=right>QSphere, le {}</p>
        </body>
        """.format(title, self.mapViewer.accessibleDescription(), fileName, width, thedatetime.now().strftime("%d/%m/%Y à %Hh%Mm"))
        self.printMapview.setHtml(html)

    def printWidget(self):
        index  = self.tabsResults.currentIndex()
            
        if  index == 0 : obj = self.htmlViewer
        elif index == 1 : obj = self.printDataTable(index) 
        elif index == 2 :
            printer = QPrinter(); printer.setPageSize(QPrinter.A4); printer.setOrientation(QPrinter.Landscape);
            printer.setPageMargins(5, 5, 5, 5, QPrinter.Millimeter); printer.setOutputFormat(QPrinter.NativeFormat); self.getImage(printer, index); obj = self.printMapview; 
        elif index == 3 : obj = self.textBrowserChilds
        elif index == 4 : obj = self.edit
        else : obj = self.printDataTable(index)  

        orientation = QPrinter.Portrait if index in (0, 4) else QPrinter.Landscape
        dlgPrint = printerSphere(self, obj, "%s : %s" % (QApplication.translate("QSphere", "Print"), self.tabsResults.tabText(index)), orientation)
        

    def getMapViewer(self):
        if self.objDataBaseViewer.currentItem() == None: return      
        item = self.objDataBaseViewer.currentItem()
        if not item.whatsThis(0) in ("table", "view") : return

        newsource = "%s.%s.%s" % (item.parent().parent().text(0), item.parent().text(0), item.text(0))
        if self.mapViewer.accessibleDescription() == newsource : return
        self.mapViewer.setAccessibleDescription(newsource)

        con, dns, service, port = self.getConnexion(False, False, None, None, None)
        if con != None :
           con.set_isolation_level(ISOLATION_LEVEL_AUTOCOMMIT)
           cur = con.cursor()
        else : return           
        
        geomColumn = self.getSpecialColumn(item, True, "geometryColumn")

        if geomColumn == None : geomColumn = self.getSpecialColumn(item, True, "rasterColumn")
        
        if geomColumn == None :
           try :
               if self.currentLayer:
                  QgsProject.instance().removeMapLayers([self.currentLayer.id()])
                  QgsProject.instance().addMapLayers([], False)
                  self.mapViewer.setLayers([]) 
                  self.mapViewer.refresh()
                  self.currentLayer = None
                  self.mapViewer.setAccessibleDescription("%s.%s.%s" %  (item.parent().parent().text(0), item.parent().text(0), item.text(0)))
                  self.tabsResults.setCurrentIndex(1)
           except : pass  
           return

        QApplication.setOverrideCursor(QCursor(Qt.WaitCursor))

        try :
            schema = self.quotedName(item.parent().text(0), False)
            table = self.quotedName(item.text(0), False)
            
            if geomColumn == 'rast' :
                uri = "PG: dbname=%s host=%s user=%s password=%s port=%s mode=2 schema=%s column=rast table=%s" % (self.targetDataBase, self.nameService.text(),
                                                                                                                      self.nameUser.text(), self.pwdUser.text(),
                                                                                                                      self.namePort.text(), schema, table)
                vlayer = QgsRasterLayer(uri, table)
            else :
                keyColumn = self.getSpecialColumn(item, True, "keyColumn")

                uri = QgsDataSourceUri()
                uri.setConnection(self.nameService.text(), self.namePort.text(), self.targetDataBase,  self.nameUser.text(), self.pwdUser.text())
                
                if keyColumn : uri.setDataSource(schema, table, geomColumn, "",  keyColumn)
                else :      
                    req = u"(SELECT row_number() Over() as myid, a.* FROM %s.%s AS a)" % (schema, table) 
                    uri.setDataSource("", req, geomColumn, "",  "myid")

                vlayer = QgsVectorLayer(uri.uri(), item.text(0), "postgres")

            if self.currentLayer == vlayer : return

            if vlayer != None :
               if vlayer.isValid() :
                    self.mapViewer.setLayers([vlayer]) 
                    self.mapViewer.refresh()
                    self.mapViewer.zoomToFullExtent()
                    self.currentLayer = vlayer
                    
                    self.mapViewer.setAccessibleDescription("%s.%s.%s" %  (item.parent().parent().text(0), item.parent().text(0), item.text(0)))
            
        except : pass
        QApplication.restoreOverrideCursor()
            

    def getDataViewer(self):

        if self.objDataBaseViewer.currentItem() == None: return      
        item = self.objDataBaseViewer.currentItem()
        if not item.whatsThis(0) in ("table", "view") : return

        if self.sender() == self.comboSqlBoundary : self.dataViewer.setAccessibleDescription("refresh")

        newsource = "%s.%s.%s" % (item.parent().parent().text(0), item.parent().text(0), item.text(0))
        if self.dataViewer.accessibleDescription() == newsource : return
        self.dataViewer.setAccessibleDescription(newsource)
        
        con, dns, service, port = self.getConnexion(False, False, None, None, None)
        if con != None :
           con.set_isolation_level(ISOLATION_LEVEL_AUTOCOMMIT)
           cur = con.cursor()
        else : return   


        records = None
        try :
                req = GET_TABLE_COUNT_RECORDS % (item.parent().text(0), item.text(0))
                records = self.getRecords(cur, req, True)
                boundary = int("%s" % records[0])
                self.updateComboSqlBoundary(boundary, False, 0)
        except : pass


        try :
             if item.whatsThis(0) == "table" : req = GET_TABLE_RECORDS % (item.parent().text(0), item.text(0), int(self.comboSqlBoundary.currentText()))
             elif item.whatsThis(0) == "view" :
                    records = self.getRecords(cur, GET_VIEW_SQL % (item.text(0)), True)
                    req = records[0][0]
        except : return

        self.movieCtrl.startMovie()
        self.sendToConsole("sql", self.getTimeStamp(), "%s" % req, "sgbdr/minisql.png")
        
        try :
            records = self.getRecords(cur, req, True)
            fieldNames = [desc[0] for desc in cur.description ]
            model = baseTableModel(fieldNames, records, self)
            self.dataViewer.setModel(model)

        except : pass
        
        self.movieCtrl.stopMovie()

        try :
                if self.dataViewer.selectionModel() : self.dataViewer.selectionModel().selectionChanged.connect(self.getCurrentRow)
                else : self.dataViewer.selectionModel().disconnect()
        except : pass

        self.tabsResults.setCurrentIndex(1)



    def quotedName(self, unquotedName, withSimpleQuote):
        if not unquotedName.islower() or unquotedName.find(" ")!= -1 : unquotedName = "\"%s\"" % unquotedName
        if withSimpleQuote : unquotedName = '%s' % unquotedName
        return unquotedName
               
    def getCurrentRow(self):
        self.currentRecord = -1
        if self.dataViewer.selectionModel():
            if self.dataViewer.selectionModel().selectedRows(0)!= [] :
                row = self.dataViewer.selectionModel().selectedRows(0)[0].row()
                self.currentRecord = row
                

    def contextMnuShortCutsSql(self, point):
        if self.objSqlViewer.currentItem() == None: return      
        item = self.objSqlViewer.currentItem()
        if not item : return

        actions = {"expandAllSqlNodes" : ("sgbdr/expandall.png", "Expand all nodes", "" , self.expandTreeAllSqlNodes, True),
                   "closeAllSqlNodes" : ("sgbdr/reduceall.png", "Close all nodes", "" , self.expandTreeAllSqlNodes, True),
                   "~0" : (),
                   "act_editNodeSql" : ("edit.png", "Edit SQL node", "" , self.editNodeSql, True),
                   "act_addNodeSql" : ("add.png", "Add SQL node" , "", self.addNodeSql, True),
                   "act_aspirateNodeSql" : ("savecopy.png", "Aspirate SQL" , "", self.aspirateNodeSql, True),
                   "act_deleteNodeSql" : ("del.png", "Delete SQL node" , "", self.deleteNodeSql, True),
                   "~1" : (),
                   "act_reloadSqlNodes" : ("sgbdr/reload.png", "Reload page", "", self.loadTheSqlResources, True),
                   "saveSqlNodes" : ("save.png", "Save", "" , self.saveSqlNodesInFile, True),
                   }
        self.builderContextMenu(self.sender(), actions, point)

                
    def contextMnuShortCuts(self, point):
        if self.objDataBaseViewer.currentItem() == None: return      
        item = self.objDataBaseViewer.currentItem()
        if not item : return
            
        if item.whatsThis(0) in ("base", "schema") :
            actions = {"expandAllBaseNodes" : ("sgbdr/expandall.png", "Expand all nodes", "" , self.expandTreeBaseNodes, True),
                       "~0" : (),
                       "closeAllBaseNodes" : ("sgbdr/reduceall.png", "Close all nodes" , "", self.expandTreeBaseNodes, True),
                       "~1" : (),
                       "reloadBaseButton" : ("sgbdr/reload.png", "Reload current base", "", self.reloadBase, True)
                       }
            self.builderContextMenu(self.sender(), actions, point)
            
        elif item.whatsThis(0) in ("table", "view"):
        
            actions = {
                       "getLayerInfos" : ("sgbdr/tableinfo.png", "Query a layer : ", "" , self.getInfos, True),
                       "~0" : (), 
                       "addLayerButton" : ("sgbdr/addlayertomapcanas.png", "Add table to project", "" , self.addLayerToMapCanvas, True),
                       "~1" : (),
                       "vacuumTableButton" : ("sgbdr/vacuum.png", "Vacuum table" , "", self.vacuumTable, True),
                       "~2" : (),
                       "deleteTableButton" : ("sgbdr/adeletetable.png", "Delete table", "", self.deleteTable, True)
                       }
            self.builderContextMenu(self.sender(), actions, point)

            

    def builderContextMenu(self, obj, actions, point):
        contextMnu = QMenu()
        
        for key in actions :
            if key.startswith("~") : contextMnu.addSeparator()
            else :   
                icon = getThemeIcon(actions[key][0])
                action = QAction(QIcon(icon), QApplication.translate("QSphere", actions[key][1]).replace(":",""), self)
                if actions[key][2] != "" : action.setShortcut(QKeySequence(actions[key][2]))
                setattr(contextMnu, key, action)
                action.setEnabled(actions[key][4])
                action.setObjectName(key)
                contextMnu.addAction(action)
                action.triggered.connect(actions[key][3])

        contextMnu.exec_(obj.mapToGlobal(point))

        
    def changeEnableQspButtons(self):
        self.currentItemData = None
        
        if self.objDataBaseViewer.currentItem() == None: return      
        item = self.objDataBaseViewer.currentItem()
        index = self.tabsResults.currentIndex()

      
        self.tabsResults.setTabEnabled(1, False)
        self.tabsResults.setTabEnabled(2, False)
       
        if item.whatsThis(0) in ("table", "view"):
            self.tabsResults.setTabEnabled(1, True)
            self.tabsResults.setTabEnabled(2, item.statusTip(0)=="geom")
            cond = True if item.parent().text(0) == "public" and item.text(0)=="qsp_metadata" else False
            self.currentItemData = item
            self.itemQspMetadata = item if cond else None
            self.enableQspButtons(cond)
            
            self.tabsResults.setTabEnabled(3, self.isCond())

            self.viewInfos(index) 

        self.tabsResults.setCurrentIndex(index)
        
        

    def isCond(self):
        try : nameParent = self.parent._objectName
        except :
            try : nameParent = self.parent.objectName()
            except : return False
        try :
            targetObj = self.parent.mainWindow if nameParent == "qsphere" else self.parent
            if targetObj != None :
               if targetObj.listWindowsButtons.topLevelItemCount() > 0 : return True
        except : pass
        return False

    def enableQspButtons(self, cond):
        self.viewAbstractFromBaseButton.setEnabled(cond and self.dataViewer.isVisible() and self.currentRecord != -1)
        self.downloadDataFromBaseButton.setEnabled(cond and self.dataViewer.isVisible() and self.currentRecord != -1)
        self.deleteDataFromBaseButton.setEnabled(cond and self.dataViewer.isVisible() and self.currentRecord != -1)
        
    def viewAbstractData(self):
        self.getCurrentRow()
        if self.currentRecord != -1 :
            idData = self.dataViewer.model().getData(self.currentRecord, 0)
            currentTitle = self.dataViewer.model().getData(self.currentRecord, 1)
            self.getLinkedUrl("VIEW%s|%s" % (idData, currentTitle)) 
            
    def downloadData(self):
        self.getCurrentRow()
        if self.currentRecord != -1 :
            idData = self.dataViewer.model().getData(self.currentRecord, 0)
            self.getLinkedUrl("DOWNLOAD%s" % idData) 


    def deleteData(self):
        self.getCurrentRow()
        if self.currentRecord != -1 :
            idData = self.dataViewer.model().getData(self.currentRecord, 0)
            currentTitle = self.dataViewer.model().getData(self.currentRecord, 1)
            self.getLinkedUrl("DEL%s|%s" % (idData, currentTitle))          

    def goToTabResults(self, iLink):
        if type(iLink) != str : iLink = iLink.toString().replace('file:///','')
        try : index = int(iLink)
        except : pass
        self.tabsResults.setCurrentIndex(index)

    def getLinkedUrl(self, idData):

        if type(idData) != str : idData = idData.toString().replace('file:///','')

        if idData.startswith("UPLOAD"):
            
            if not self.targetDataBase :
                self.errorMsgBase()
                self.sendToConsole("error", self.getTimeStamp(), "%s not defined !" % "targetDataBase", self.ipucewarning)
                self.tabs.setCurrentIndex(1)
                return
            
            item = self.objDataBaseViewer.currentItem()

            if not item : return
            
            if item.whatsThis(0)== "base" : self.targetDataBase = item.text(0) 
            elif item.whatsThis(0) in ("table", "view") : self.targetDataBase = item.parent().parent().text(0) 
            else : self.targetDataBase = item.parent().text(0)
            
            con, dns, service, port = self.getConnexion(False, False, None, None, None)
            if con != None :
                con.set_isolation_level(ISOLATION_LEVEL_AUTOCOMMIT)
                if not "qsp_metadata" in self.dicSchemas["public"] :
                    result = self.createQspMetadataTable(con, self.nameUser.text(), True)
                    if result == False : return

            self.movieCtrl.startMovie()
            idData = int(idData.replace("UPLOAD", ""))
            self.movieCtrl.stopMovie()
            if self.parent :
               if self.parent.mainWindow :
                  for subWindow in self.parent.mainWindow.mdi.subWindowList(QMdiArea.CreationOrder):
                      if long(subWindow.winId()) == idData :
                         for child in subWindow.children():
                             if child.objectName()=="dlgMetadata":
                                self.uploadMetadataInBase(child)
                                self.changeSource = True
                                break
                         break
                      
            self.movieCtrl.stopMovie()       


        elif idData.startswith("DOWNLOAD"):
            idData = idData.replace("DOWNLOAD", "")
            if not self.checkMetadatatodownload(idData, QApplication.translate("QSphere","Download metadata from the base")): return
            self.movieCtrl.startMovie()

            dataXml = self.metadataToDownload[idData]
            dataXml = unquote(dataXml)
                        
            self.sendToConsole("info", self.getTimeStamp(), dataXml, self.ipucesuccessfull)

            from io import StringIO as bufferData
            from qsphere.xmlisoparser import xmlIsoParser

            if self.parent.objectName() != "qsphereMainWindow" :
               self.parent.getMainWindow()
               self.parent = self.parent.mainWindow
               self.tabsResults.setTabEnabled(3, self.isCond())
               self.parent.dlgSgbdr = self
            
            mdiWindow, child  = self.parent.initMetadataDialog()

            buffer = bufferData(dataXml)
            isoParser = xmlIsoParser(None, buffer, 'MEDDE', 'fr') 
            isoParser.createISOdataStructure(True)
            if isoParser.getTagDictionnary() : child.reloadDataFromXml(isoParser, None, 0)

            dns="pgmdd:host=%s port=%s dbname=%s schema=%s table=%s id=%s" % (self.nameService.text(), self.namePort.text(), self.targetDataBase, "public", "qsp_metadata", idData)

            child.setWindowTitle("%s%s" % (child.rootWindowTitle, dns))
            child.connexion, dns, service, port = self.getConnexion(False, False, None, None, None)
            child.show()
            
            self.parent.fixeModeDialog(0, 0)
            self.parent.changeMetadataMode()
            self.parent.tabWidget.setCurrentIndex(1)

            self.changedMddFiles = True
            self.loadListMddFiles()
            self.movieCtrl.stopMovie()


        elif idData.startswith("DEL"):
             idData = idData.replace("DEL", "").split("|")
             if self.askForActions("deleterecord", idData[1], None) :
                 self.deleteMetadataInBase(idData[0])
                 self.changeSource = True

        elif idData.startswith("VIEW"):
             idData = idData.replace("VIEW", "").split("|")
             if not self.checkMetadatatodownload(idData[0], QApplication.translate("QSphere","View a file (XML ISO 19139, HTML ...)")): return
             dataXml = self.metadataToDownload[idData[0]]
             d = xmlEditor(self, "", self)
             windowIcon(d, "editxml.png")
             d.setWindowTitle("%s : %s" % (d.rootWindowTitle, idData[1]))
             d.editor.setText(dataXml)
             d.checkProperties()
             msgBadFileXml(d)             
             d.exec_()

    def checkMetadatatodownload(self, idData, actionTitle):
             if not dicoHasKey(self.metadataToDownload, idData) :
                 msg = "<b>%s</b> <br>%s - %s <br> %s : %s" % (actionTitle,
                                                 QApplication.translate("QSphere","XML Params"),
                                                 QApplication.translate("QSphere","Not found"),
                                                 QApplication.translate("QSphere","Search for"),
                                                 idData)
                 self.sendToConsole("error", self.getTimeStamp(), msg, self.ipucewarning)
                 return False
             return True   
    
    def fixeImage(self, iconName):
        icon = QImage(getThemeIcon(iconName)) 
        self.imgProgress.setPixmap(QPixmap.fromImage(icon))


    def printDataTable(self, index):
        document = QTextDocument()
        cursor = QTextCursor(document)
        model = self.dataViewer.model() if index == 1 else self.tabsResults.currentWidget().model() 
        if model == None : return
        
        borne = min(model.rowCount(), 100)        

        if self.mode == "html" :
                document.setDefaultStyleSheet("body { color : black; background-color : white; }")
                html = self.html
                item = self.objDataBaseViewer.currentItem()
                try :    
                    html+= "<h2><img src='file:///%s'> %s.%s.%s</h2><br><table><tr>" % (self.pucetable, self.targetDataBase, self.currentItemData.parent().text(0), self.currentItemData.text(0))
                except :
                    html+= "<h2><img src='file:///%s'> %s</h2><br><table><tr>" % (self.pucetable, self.dataViewer.toolTip())
                html+= "<table style='background-color: #808080;border-spacing: 0px; width: 100%; padding: 5px 30px 5px 10px; color: white;'><tr>"
                for col in model.columnNames(): html+= "<th>%s</th>" % col
                html+= "</tr>"
                
                for row in range(borne) :
                    html+= "<tr class='comment'>"
                    for column in range(model.columnCount()):
                        data = "%s" % model.getData(row, column)
                        data = netToHtml(data[:100])
                        html+= "<td>%s</td>" % data
                    html+= "</tr>"
                html+= "</table></body>"
                document.setHtml(html)                
        else :
                table = cursor.insertTable(borne+1, model.columnCount())
                for col in model.columnNames(): cursor.insertText("%s" % col);cursor.movePosition(QTextCursor.NextCell)
                for row in range(borne) :
                    for column in range(table.columns()):
                        data = "%s" % model.getData(row, column)
                        cursor.insertText(data[:100])
                        cursor.movePosition(QTextCursor.NextCell)

        return document

    def sendToHtml(self, html): self.htmlViewer.setHtml(html, self.baseurl) 
    
    def sendToConsole(self, nature, timestamp, msg, img):
        img = getThemeIcon(img)
        styles = {
                  "maintitle" : "<table style='border: 2px solid grey; border-radius: 5px; background-color: #1a0dc8; color: white;'>",
                  "title"     : "<table style='border: 2px solid grey; border-radius: 5px; background-color: #6C96C6; color: white;'>",
                  "error"     : "<table style='background-color: #FF0000;border-spacing: 0px; width: 100%; padding: 5px 30px 5px 10px; color: white;'>",
                  "info"      : "<table style='background-color: #e8eff5;border-spacing: 0px; width: 100%; padding: 5px 30px 5px 10px; color: black;'>",
                  "separator" : "<table style='background-color: #e8eff5;border-spacing: 0px; width: 100%; padding: 5px 30px 5px 10px; color: black;'>",
                  "sql"       : "<table style='background-color: #BEBEBE;border-spacing: 0px; width: 100%; padding: 5px 30px 5px 10px; color: black;'>",
                  "sqlplus"   : "<table style='background-color: #808080;border-spacing: 0px; width: 100%; padding: 5px 30px 5px 10px; color: white;'>" 
                  } 
        if dicoHasKey(styles, nature) : style = styles[nature]
        else : style = "<table style='background-color: #e8eff5;border-spacing: 0px; width: 100%; padding: 5px 30px 5px 10px;'>"

        if nature == "maintitle" : self.edit.append("%s<tr><td width='2%%'><img src='%s'></td><td width='10%%'>%s</td><td width='88%%'><h1><font color='white'><b>%s</b></font></h1></td></tr></table>" % (style, img, timestamp, msg))
        elif nature == "title" : self.edit.append("%s<tr><td width='2%%'><img src='%s'></td><td width='10%%'>%s</td><td width='88%%'><h1><font color='blue'><b>%s</b></font></h1></td></tr></table>" % (style, img, timestamp, msg))
        elif nature == "error" : self.edit.append("%s<tr valign='top'><td width='2%%'><img src='%s'></td><td width='10%%'>%s</td><td width='88%%'><h3><b>%s</b></h3></td></tr></table>" % (style, img, timestamp, msg))
        elif nature == "info" : self.edit.append("%s<tr valign='top'><td width='2%%'><img src='%s'></td><td width='10%%'>%s</td><td width='88%%'><h3><b>%s</b></h3></td></tr></table>" % (style, img, timestamp, msg))
        elif nature == "sqlplus" : self.edit.append("%s<tr valign='top'><td width='2%%'><img src='%s'></td><td width='10%%'>%s</td><td width='88%%'><h3><b>%s</b></h3></td></tr></table>" % (style, img, timestamp, msg))
        elif nature == "sql" : self.edit.append("%s<tr valign='top'><td width='2%%'><img src='%s'></td><td width='10%%'>%s</td><td width='88%%'><h3><b>%s</b></h3></td></tr></table>" % (style, img, timestamp, msg))
        elif nature == "separator" : self.edit.append("%s<tr valign='middle'><td width='12%%'></td><td width='88%%'><hr style='height:2px; color:cyan; border:1px;'></td></tr></table>" % style)
        self.edit.repaint()
        
    def getTimeStamp(self):
        self.startTime = thedatetime.now()
        return self.startTime.replace(microsecond=0).strftime("%d/%m/%Y %Hh:%Mm:%Ss")

    def getServerBases(self, cur, terminate):
        records = self.getRecords(cur, GET_SERVER_BASES, True)
        self.sendToConsole("title", self.getTimeStamp(), "%s : %s [%s Port:%s]" % (QApplication.translate("QSphere","Get list of databases"),
                                                 self.qgisConnexion.currentText(), self.nameService.text(), self.namePort.text()), "sgbdr/pgdatabase.png")
        self.sendToConsole("sql", self.getTimeStamp(), "%s" % GET_SERVER_BASES, "sgbdr/minisql.png")
        self.listBases = []

        self.setListBases(sorted(records), True, False)

        self.getBasesTreeView(self.listBases, self.objDataBaseViewer, self.pucebase, "base")
        self.tabs.setTabText(0, "%s [%s]" % (self.tabs.tabToolTip(0), self.objDataBaseViewer.topLevelItemCount()))


    def getBasesTreeView(self, listElts, treeWidget, puce, typeelt):
        treeWidget.clear()
        index = 0
        for elt in listElts :
            itemBase = QTreeWidgetItem()
            itemBase.setIcon(0, QIcon(puce))
            itemBase.setText(0, "%s" % elt)
            itemBase.setWhatsThis(0, typeelt)
            treeWidget.addTopLevelItem(itemBase)
            if treeWidget == self.objSqlViewer : self.makeChildSql(itemBase)
        treeWidget.expandAll()
        treeWidget.resizeColumnToContents(0)
        treeWidget.resizeColumnToContents(1)

    def getTreeViewSchemas(self, parentItem, dictBasesSchemasTables):
        listChildren = parentItem.takeChildren()
        for child in listChildren : parentItem.removeChild(child)
        for key in dictBasesSchemasTables.keys() :
            if key != "" :
               itemSchema = QTreeWidgetItem()
               itemSchema.setIcon(0, QIcon(self.puceschema))
               itemSchema.setText(0, "%s" % key)
               itemSchema.setWhatsThis(0, "schema")
               parentItem.addChild(itemSchema)

               listTables = sorted(dictBasesSchemasTables[key])
               for table in listTables :
                   itemTable = QTreeWidgetItem()
                   itemTable.setIcon(0, QIcon(self.pucetable))
                   schema = self.quotedName(key, True)
                   item = self.quotedName(table, True)
                   value = self.schematable['%s.%s' % (schema, item)][0]
                   geom = self.schematable['%s.%s' % (schema, item)][1]
                   typegeom = self.schematable['%s.%s' % (schema, item)][3]

                   isGeom = True
                   
                   if value.find("VIEW")!=-1 : iconObj, libObj = self.puceview, "view"
                   else : iconObj, libObj = self.pucetable,  "table"
                   
                   if typegeom.lower().find("point")!=-1 : itemTable.setIcon(0, QIcon(getThemeIcon("sgbdr/point.png")))
                   elif typegeom.lower().find("linestring")!=-1 : itemTable.setIcon(0, QIcon(getThemeIcon("sgbdr/linestring.png")))
                   elif typegeom.lower().find("polygon")!=-1 : itemTable.setIcon(0, QIcon(getThemeIcon("sgbdr/polygon.png")))
                   else :
                        if geom == "rast" : itemTable.setIcon(0, QIcon(getThemeIcon("sgbdr/raster.png")))
                        else : itemTable.setIcon(0, QIcon(iconObj)); isGeom = False ;
                   itemTable.setWhatsThis(0, libObj)

       
                   itemTable.setStatusTip(0, "geom") if isGeom else itemTable.setStatusTip(0, "")
                   itemTable.setText(0, "%s" % table)
                   itemSchema.addChild(itemTable)

    def getRecords(self, cursor, req, returnRecords):
            records = None
            try :
                cursor.execute(req)
                if returnRecords : records = cursor.fetchall()
                startTime = self.startTime
                endTime = thedatetime.now() 
                deltaTime = (endTime - startTime).microseconds / 10000
                self.sendToConsole("info", self.getTimeStamp(), "%s<br>%s ! [%s ms]" % (req, self.successfull, deltaTime), self.ipucesuccessfull)
                return records                
            except psycopg2.Error as e:

                if e.pgcode == None and e.pgerror == None :
                          startTime = self.startTime
                          endTime = thedatetime.now() 
                          deltaTime = (endTime - startTime).microseconds / 10000
                          self.sendToConsole("info", self.getTimeStamp(), "%s<br>%s ! [%s ms]" % (req, self.successfull, deltaTime), self.ipucesuccessfull)
                          return True
                else :         
                          self.sendToConsole("error", self.getTimeStamp(), "%s<br><table><tr><td>%s</td><td>%s</td></tr></table>" % (req, e.pgcode, e.pgerror), self.ipucewarning)
                return None


    def getInfosBase(self):
        self.movieCtrl.startMovie()
        targetDataBase = self.nameDatabase.currentText()
        con, dns, service, port = self.getConnexion(True, False, None, None, None)
          
        if con != None :

            self.targetDataBase = None
           
            con.set_isolation_level(ISOLATION_LEVEL_AUTOCOMMIT)
            cur = con.cursor()

            """
            req = TERMINATE_BACKEND_BASE % targetDataBase
            try : cur.execute(req)
            except psycopg2.Error as e:
                self.movieCtrl.stopMovie()
                if e.pgcode == None and e.pgerror == None :
                          startTime = self.startTime
                          endTime = thedatetime.now() 
                          deltaTime = (endTime - startTime).microseconds / 10000
                          self.sendToConsole("info", self.getTimeStamp(), "%s ! :<hr>%s</hr>[%s ms]" % (self.successfull, req, deltaTime), self.ipucesuccessfull)
                          return True
                else :         
                          self.sendToConsole("error", self.getTimeStamp(), "%s<br><table><tr><td>%s</td><td>%s</td></tr></table>" % (req, e.pgcode, e.pgerror), self.ipucewarning)
                return 

            con.commit()
            """
            
            self.oldSource = ""
            self.getServerBases(cur, False)

            records = self.getRecords(cur, GET_PG_PROCEDURES, True)
            self.getSqlDictionnary(records, "functions")

            records = self.getRecords(cur, GET_PG_KEYWORDS, True)
            self.getSqlDictionnary(records, "keywords")

            self.initCompleter()

            records = self.getRecords(cur, GET_SERVER_NUMERIC_TYPES, True)    
            self.numericTypesCodes = [elt[0] for elt in sorted(records)]
            
            cur.close()
            con.close()

            index = self.nameDatabase.findText(targetDataBase, Qt.MatchFixedString)
            self.nameDatabase.setCurrentIndex(index)

            """SELECT CURRENT DATABASE - EMIT DOUBLE CLIC """
            counter = self.objDataBaseViewer.topLevelItemCount()
            for i in range(counter):
                item = self.objDataBaseViewer.topLevelItem(i)
                if item.text(0) == self.nameDatabase.currentText()  : item.setSelected(True); break
            if item :
               self.getSchemasBase(item)
               try :
                   index = self.objDataBaseViewer.selectionModel().selectedIndexes()[0]
                   self.objDataBaseViewer.setExpanded(index, True) 
               except : pass 
        else :    
            msg = "%s : %s [%s Port:%s]" % (QApplication.translate("QSphere","Bad result for"),
                                                 self.qgisConnexion.currentText(), self.nameService.text(), self.namePort.text())
            self.sendToConsole("error", self.getTimeStamp(), msg, "sgbdr/pgdatabase.png")
            self.objDataBaseViewer.clear()
            self.htmlViewer.setHtml("", self.baseurl)
            self.tabs.setTabText(0, "%s [%s]" % (self.tabs.tabToolTip(0), self.objDataBaseViewer.topLevelItemCount()))

        self.movieCtrl.stopMovie()     

    def getSqlDictionnary(self, records, key):
        self.sqlDictionnary[key] = [proc for proc in records] 

    def initLexer(self, lexer):
        white = QColor(255,255,255,255)
        black = QColor(0,0,0,255)
        comment = QColor(150,150,150,255)
  
        lexer.setDefaultColor(black)
        lexer.setColor(black, lexer.Default)
        lexer.setColor(comment, lexer.Comment)
        lexer.setColor(comment, lexer.CommentLine)
        lexer.setColor(comment, lexer.CommentDoc)
        lexer.setColor(QColor(204,33,33,255), lexer.Number)
        lexer.setColor(QColor(0,0,255,255), lexer.Keyword)
        lexer.setColor(QColor(125,125,255,255), lexer.DoubleQuotedString)
        lexer.setColor(QColor(0,255,0,255), lexer.SingleQuotedString)
        lexer.setColor(QColor(252,163,61,255), lexer.PlusKeyword)
        lexer.setColor(QColor(255,0,0,255), lexer.Operator)
        lexer.setColor(black, lexer.Identifier)
        lexer.setColor(comment, lexer.PlusComment)
        lexer.setColor(comment, lexer.CommentLineHash)
        lexer.setColor(comment, lexer.CommentDocKeyword)
        lexer.setColor(comment, lexer.CommentDocKeywordError)
        lexer.setPaper(white)
        lexer.setDefaultPaper(white)

        return lexer

    def initCompleter(self):
        listOfWords = []
        for name, value in self.sqlDictionnary.items(): listOfWords += value   
        listOfWords = list(set(listOfWords))
        lexer = self.initLexer(self.editorSql.lexer())
        api = QsciAPIs(lexer)
        for word in listOfWords:
                try : api.add(word)
                except :
                    try : api.add(word[0])
                    except : pass
        api.prepare()
        self.editorSql.lexer().setAPIs(api)       

    def saveFilesSql(self):
        if self.editorSql.toolTip() != "":
           fileName = self.editorSql.toolTip()       
           inFile = open(fileName, "w")
           inFile.write(self.editorSql.text()) 
           inFile.close()
           self.sendToConsole("info", self.getTimeStamp(), "%s" % fileName, "sgbdr/savesql.png")
           title = QApplication.translate("QSphere", "Information" ) 
           msg = QApplication.translate("QSphere","The session's file was saved as")
           sendMsg(self, title , "%s :<br><u><i>%s</i></ul>" % (msg, fileName), Qgis.Info, self.duration_info)           
        else : self.saveFilesAsSql()   
        
    def saveFilesAsSql(self):
        title = QApplication.translate("QSphere","Save as a file")
        dlgFile = QFileDialog(self, title)
        dlgFile.setNameFilters(self.listFiltres)  
        dlgFile.selectNameFilter(self.listFiltres[0])
        dlgFile.setDefaultSuffix("qry")
        dlgFile.setViewMode(QFileDialog.Detail)
        dlgFile.setDirectory(self.InitDir)
        dlgFile.setAcceptMode(QFileDialog.AcceptSave)

        fixeLabelsFileDialog(self, dlgFile, 1, True)
        
        if dlgFile.exec_():
            fileName = fileNameWithExtension(self, dlgFile.selectedFiles()[0], dlgFile.selectedNameFilter())
            self.InitDir = dirname(fileName)
            inFile = open(fileName, "w")
            inFile.write(self.editorSql.text()) 
            inFile.close()
            self.editorSql.setToolTip(fileName)
            self.dockSql.setWindowTitle("SQL : %s" % fileName)
            self.sendToConsole("info", self.getTimeStamp(), "%s" % fileName, "sgbdr/savesqlas.png")
                

    def eraseSql(self):
        self.editorSql.setText("")
        self.editorSql.setToolTip("")
        self.dockSql.setWindowTitle("SQL :")
              
    
    def openFilesSql(self):
        title = QApplication.translate("QSphere","Load a SQL file")
        dlgFile = QFileDialog(self, title)
        dlgFile.setNameFilters(self.listFiltres)  
        try :
            dlgFile.selectNameFilter(self.listFiltres[0] if self.defaultExtension == "qry" else self.listFiltres[1])
        except:
            dlgFile.selectNameFilter(self.listFiltres[0])
            dlgFile.setDefaultSuffix(self.defaultExtension)
        dlgFile.setViewMode(QFileDialog.Detail)
        dlgFile.setDirectory(self.InitDir)
        dlgFile.setFileMode(QFileDialog.ExistingFiles)  
        dlgFile.setAcceptMode(QFileDialog.AcceptOpen)

        fixeLabelsFileDialog(self, dlgFile, 0, True)
        
        if dlgFile.exec_():
            fileName = "%s" % (dlgFile.selectedFiles()[0])
            if fileName!="" :
                self.InitDir = dirname(fileName)
                self.defaultExtension = getExtension(fileName).replace(".", "")
                inFile = open(fileName, "r")
                sqlQuery = inFile.read().replace("\n\n", "\n")
                inFile.close()
                self.dockSql.setWindowTitle("SQL : %s" % fileName)
                self.editorSql.setToolTip(fileName)
                self.editorSql.setText(sqlQuery)
                self.editorSql.setCursorPosition(self.editorSql.lines() , 1)
                self.editorSql.setFocus()
       
    def executeSql(self):
        if not self.targetDataBase : self.errorMsgBase(); return;
        
        automate = (self.sender() == None or self.sender() == self.reloadButton)
        if self.dlgNode :
            sqlQuery = self.dlgNode.editorSql if self.sender() == self.dlgNode.testSql else self.editorSql
        else :
            sqlQuery =  self.editorSql    

            
        if sqlQuery.text() != "" or automate :
            con, dns, service, port = self.getConnexion(False, False, None, None, None)
            if con != None :
                
                con.set_isolation_level(ISOLATION_LEVEL_AUTOCOMMIT)
                cur = con.cursor()

                if not automate :
                        req = sqlQuery.selectedText() if sqlQuery.hasSelectedText() else sqlQuery.text()
                        
                html = self.html
                html+= "<script type='text/javascript'>"
                path = correctPath(dirname(__file__)).replace("\\","/")
                with open("%sressources/viewerdata.js" % path, 'r') as javaScriptFile :
                        linesForJavaScript = javaScriptFile.read()
                html+= linesForJavaScript                 
                html+= "</script>"
                html+= "<h2><img src='file:///%s'>[%s:%s] %s - %s : %s <img src='file:///%s'></h2>" % (getThemeIcon("mime/file/sql.png"), service, port, QApplication.translate("QSphere","Execute SQL"), QApplication.translate("QSphere", "source"), self.targetDataBase, self.pucebase)
                html+= "<pre>%s %s</pre><br>" % (thedatetime.now().replace(microsecond=0).strftime("%d/%m/%Y %Hh:%Mm:%Ss"), QApplication.translate("QSphere", "Automate") if automate else "")

                if not automate :
                        oldreq = req
                        collectionReq = req.split(";")
                else : collectionReq = self.chronoSqltoRefresh
                        
                idReq, listIdReq, self.lastSql = 0, [], []
                schema, PARAMValue = "", ""

                for req in collectionReq :
    
                    if req != "" :      
                          req = "%s ;" % req
    
                          textSQL = ""
                          for line in textwrap.wrap(req, 100) : textSQL+= "%s<br>" % line
                          textSQL = textSQL.replace("--{", "<b><u>").replace("}--", "</u></b>")
                          newreq = ""
                          
                          if req.find("%param$")!=-1 :

                              listReq = req.split( "%param$")
                              
                              for i in range(len(listReq)-1):
                                     if self.condSql(listReq, i+1):        
                                             paramValue, ok = QInputDialog.getText(None, QApplication.translate("QSphere","Params"), "<h2>SQL :</h2>%s<br><hr>SQL %s : %s" % (textSQL, QApplication.translate("QSphere","Params"), listReq[i+1].replace("\n", "<br>")), QLineEdit.Normal)
                                             if not ok: return
                                             newreq+= "%s%s" % (listReq[i], paramValue)
                                     else :
                                             self.getParamSql(listReq, i+1, textSQL, schema)
                                             if self.dialogInput.exec_() :
                                                 if listReq[i+1].find('schema')!=-1 : schema = self.dialogInput.textValue()
                                                 newreq+= "%s%s" % (listReq[i], self.dialogInput.textValue())
                                             else : return    
                              newreq+= "%s" % listReq[len(listReq)-1]
                              req = newreq 

                                  
                          if req.find("%PARAM$")!=-1 :
                                  
                              listReq = req.split( "%PARAM$")
                             
                              if PARAMValue == "" :
                                  if self.condSql(listReq, 1):        
                                             PARAMValue, ok = QInputDialog.getText(None, QApplication.translate("QSphere","Params"), "<h2>SQL :</h2>%s<br><hr>SQL %s : %s" % (textSQL, QApplication.translate("QSphere","Params"), listReq[1].replace("\n", "<br>")), QLineEdit.Normal)
                                             if not ok: return
                                  else :
                                             self.getParamSql(listReq, 1, textSQL, schema)
                                             if self.dialogInput.exec_() :  PARAMValue = self.dialogInput.textValue()    
                                             else : return                                      

                              req = req.replace("%PARAM$", PARAMValue).strip()

                          if req.strip() != ";" and req.strip() != "":
                                  self.startTime = thedatetime.now()
                                  records = self.getRecords(cur, req, True)
                                  html+= "<table class='sql' cellspacing=0 cellpadding=0>"
                                  html+= "<tr><th class='title' colspan=2><h3>SQL</h3></th></tr>"
                                  html+= "<tr valign='top'><td width='1%%'><img src='file:///%s'></td><td bgcolor='white' bordercolor='black' borderwidth='1px'>%s</td></tr>" % (self.pucequery, self.getColorizedHtml(req))  

                                  if cur.description != None :
                                          if self.checkSqlComments.isChecked() :
                                                  i=0 
                                                  html+= "<tr valign='top'><td width='1%%'><img src='file:///%s'></td><th class='title'><h3>%s %s</h3></th></tr>" % (self.pucetable, QApplication.translate("QSphere", "Structure"), QApplication.translate("QSphere", "Results").lower())
                                                  html+= "<tr><td width='1%%'></td><td bgcolor='white' bordercolor='black'><table>"
                                                  for desc in cur.description : html+= "<tr><th>col%s</th><td>%s</td></tr>" % (i, str(desc)); i+= 1;
                                                  html+= "</table></td></tr>"
    
                                          self.description = list(cur.description)
                                          
                                          if records == True :
                                                  html+= "<tr><td><img src='file:///%s'></td><td><a href='4'><h2>%s %s</h2></a></td></tr>" % (self.pucesuccessfull, self.successfull, QApplication.translate("QSphere","SQL Commands"))
                                                  self.reloadBase()
                                          else :
                                                  """
                                                  html+= "<tr valign='top'><td width='1%%'><img src='file:///%s'></td><th class='title'><h3>%s : %s</h3><a href='javascript: switchInfoPerso(%s);'><img src='file:///%s'><span id='msg%s'>%s</span></a></th></tr>" % (self.pucerecords, self.lblRecords.title(), len(records), idReq, self.pucearrow, idReq, QApplication.translate("QSphere", "Hide"))
                                                  html+= "<tr><td width='1%%'></td><td bgcolor='white' bordercolor='black'>"
                                                  html+= "<div id='divacacher%s' style='background: #efe; padding: 5px; width: 100%%; height: 100%%;overflow-x: hidden; overflow-y: scroll;'>" % idReq
                                                  html+= self.getResultsInHtml(cur, records, True, 'maTable%s' % idReq, req)
                                                  html+= "</div></td></tr>"
                                                  """

                                                  text = self.getResultsInHtml(cur, records, True, 'maTable%s' % idReq, req, idReq)
                                                  if (text): html+= text
                                                  self.movieCtrl.stopMovie()

                                                  listIdReq.append([idReq, len(records)])
                                                  idReq+= 1
                                                  self.lastSql.append(req)
                                  else :
                                       if records == True :
                                           html+= "<tr><td><img src='file:///%s'></td><td><a href='4'><h2>%s %s</h2></a>" % (self.pucesuccessfull, self.successfull, QApplication.translate("QSphere","SQL Commands"))
                                           self.reloadBase()
                                       else :        
                                           html+= "<tr valign='top'><td><img src='file:///%s'></td><td><a href='4'><h2>%s %s</h2></a><br><center><img src='file:///%s'></center>" % (self.pucewarning, self.failed, QApplication.translate("QSphere","SQL Commands"), self.sphery)
                                  html+= "</td></tr></table><br>"


                html+= "<script type='text/javascript'>"
                html+= "function switchInfoPerso(index)"
                html+= "{"
                html+= "divInfo = document.getElementById('divacacher'+index);"
                html+= "divTexte = document.getElementById('msg'+index);"
                html+= "if (divInfo.style.display == 'none'){divInfo.style.display = 'block';divTexte.innerHTML='%s'}" % QApplication.translate("QSphere", "Hide")
                html+= "else{divInfo.style.display = 'none';divTexte.innerHTML='%s'}" % QApplication.translate("QSphere", "View")
                html+= "}\n"                      
                html+= "function init(){"

                
                for i in range(len(listIdReq)):
                    if listIdReq[i][1] <= 500:
                       idReq = listIdReq[i][0]
                       html+= "var TableSorter%s = new TSorter;" %(idReq)
                       html+= "TableSorter%s.init('maTable%s');" %(idReq, idReq)
                html+= "}"
                html+= "</script>"
                html+= "<script type='text/javascript'>"    
                html+= "window.onload = init;"
                html+= "</script>"

                                  
                html+= "<br><br><br><br>"
                
                self.sendToHtml(html)
                self.tabsResults.setCurrentIndex(0)

                if self.dlgNode :
                        if self.sender() == self.dlgNode.testSql :
                           if not sqlQuery.hasSelectedText() and not automate : sqlQuery.setText(oldreq)
                self.oldSource = "sql"
                self.htmlViewer.setAccessibleDescription("sql")
                self.dataViewer.setAccessibleDescription("sql")
                self.mapViewer.setAccessibleDescription("sql")
                
            else :

                self.errorMsgBase()
                            

    def condSql(self, listReq, index):
        if listReq[index].find('schema')==-1 and listReq[index].find('table')==-1 and listReq[index].find('base')==-1 and  listReq[index].find('function')==-1 and  listReq[index].find('fonction')==-1 : return True
        else : return False
            

    def getParamSql(self, listReq, index, textSql, schema):
        self.dialogInput = QInputDialog(self) 
        self.dialogInput.setWindowTitle(QApplication.translate("QSphere","Params"))
        self.dialogInput.setLabelText("<h2>SQL :</h2>%s<br><hr>SQL %s : %s" % (textSql, QApplication.translate("QSphere","Params"), listReq[index].replace("\n", "<br>")))
        self.dialogInput.setOptions(QInputDialog.UseListViewForComboBoxItems)
        if listReq[index].find('schema.table')!=-1 : self.dialogInput.setComboBoxItems(sorted(self.schematable.keys()))
        elif listReq[index].find('base')!=-1 : self.dialogInput.setComboBoxItems(self.listBases)
        elif listReq[index].find('function')!=-1 or listReq[index].find('fonction')!=-1 : self.dialogInput.setComboBoxItems(self.listFunctions)
        elif listReq[index].find('schema')!=-1 : self.dialogInput.setComboBoxItems(self.listSchemas)
        elif listReq[index].find('table')!=-1 and schema != "" : self.dialogInput.setComboBoxItems(self.dicSchemas[schema])
        else : self.dialogInput.setComboBoxItems(self.listTables)
        for child in self.dialogInput.children():
            if type(child) == QListView :
                    child.doubleClicked.connect(self.dialogInput.accept)
  

    def createQspMetadataTable(self, con, user, isSystemAction):
        if self.checkQSphereAsk.isChecked() and not isSystemAction : return   
        
        cur = con.cursor()
        if self.askForActions("createtable", None, None) :

            msg = "%s ..." % QApplication.translate("QSphere","Must create qsp_metadata")
            self.sendToConsole("error", self.getTimeStamp(), msg, self.ipucewarning)
            self.dicSchemas["public"].append("qsp_metadata")
        
            req = """CREATE TABLE qsp_metadata (
                     id SERIAL PRIMARY KEY,
                     title character varying(255),
                     abstract character varying,
                     identificator character varying(255),
                     metadata xml,
                     datasource character varying(255),
                     parentid character varying(100)
                )
                WITH (OIDS=FALSE);
            """
            reqall=req
            cur.execute(req)
            req = """ALTER TABLE public.\"%s\" OWNER TO %s;
                     GRANT ALL ON TABLE public.\"%s\" TO %s;
                     GRANT SELECT ON TABLE public.\"%s\" TO public;
                     """ % ("qsp_metadata", user, "qsp_metadata", user, "qsp_metadata")
            cur.execute(req)
            cur.close()

            reqall+= "<br>%s" % req

            self.successFullMsg(reqall, False)
            self.reloadBase()
            return True
           

        else : return False
            

    def uploadMetadataInBase(self, currentMetadataWindow):
        con, dns, service, port = self.getConnexion(False, False, None, None, None)
        if con != None :
            con.set_isolation_level(ISOLATION_LEVEL_AUTOCOMMIT)
            if "qsp_metadata" not in self.dicSchemas["public"] :
                result = self.createQspMetadataTable(con, self.nameUser.text(), True)
                if result == False : return
                item = self.objDataBaseViewer.currentItem()
                if not item : return
            
            dataXml = saveXml(currentMetadataWindow, None)
            dataXml = dataXml.decode("utf-8").replace("'", "''")

            fileName = currentMetadataWindow.windowTitle().replace(currentMetadataWindow.rootWindowTitle, "")
            obj = currentMetadataWindow.findChild(QLineEdit, "intitule")
            title = obj.text().replace("'", "''") if obj != None else "No result"
            obj = currentMetadataWindow.findChild(myTextEdit, "resume")
            abstract = obj.toPlainText().replace("'", "''") if obj != None else "No result"
            obj = currentMetadataWindow.findChild(QLineEdit, "identificator")
            identificator = obj.text() if obj != None else "-1"

            layer = getComboLayer(currentMetadataWindow, currentMetadataWindow.ComboLayers, True)
            if not layer :  datasource, parentid = None, -1
            else :
               typedatasource = getTypeLayer(currentMetadataWindow, layer.type())
               datasource = layer.source()
 
            msg = title
            self.sendToConsole("info", self.getTimeStamp(), "%s : %s [%s:%s]<br>%s<br>%s" % ("uploadMetadataInBase", self.targetDataBase, service, port, fileName, msg), self.ipuceschema)
      
            con.set_isolation_level(ISOLATION_LEVEL_AUTOCOMMIT)
            cur = con.cursor()

            try :
                    req = """INSERT INTO qsp_metadata 
                            (title, abstract, identificator, metadata, datasource, parentid)
                             VALUES ('%s','%s', '%s','%s', '%s', '%s');
                          """ % (title, abstract, identificator, dataXml, datasource, fileName)
                    cur.execute(req)
                    self.successFullMsg(req, True)

            except psycopg2.Error as e:
                    title = QApplication.translate("QSphere","Warning")
                    msg = "%s : %s [%s]" % (QApplication.translate("QSphere","Error"), e.pgerror, e.pgcode)
                    sendMsg(self, title , "<u><i>%s</i></u>" % msg, Qgis.Warning, self.duration_warning)
                    self.sendToConsole("error", self.getTimeStamp(), "%s<br><table><tr><td>%s</td><td>%s</td></tr></table>" % (req, e.pgcode, e.pgerror), self.ipucewarning)

                

    def deleteMetadataInBase(self, idData):
        con, dns, service, port = self.getConnexion(False, False, None, None, None)
        if con != None :
            msg = QApplication.translate("QSphere","Delete the current line")
            self.sendToConsole("info", self.getTimeStamp(), "%s : %s [%s:%s]<br>%s : %s" % ("deleteMetadataInBase", self.targetDataBase, service, port, msg, idData), self.ipuceschema)
      
            con.set_isolation_level(ISOLATION_LEVEL_AUTOCOMMIT)
            cur = con.cursor()            

            req = "DELETE FROM qsp_metadata WHERE id=%s ;" % int(idData)
            cur.execute(req)
            self.successFullMsg(req, True)
            self.getDataViewer()

    def updateItemIcon(self, refItem):
        counter = self.objDataBaseViewer.topLevelItemCount()
        for i in range(counter):
            item = self.objDataBaseViewer.topLevelItem(i)
            item.setIcon(0, QIcon(self.pucebase)) if item != refItem  else  item.setIcon(0, QIcon(self.pucebaseopen))

    def deleteTable(self):
        if not self.targetDataBase : return
        if not self.currentItemData  : return 

        if self.askForActions("deletetable", None, self.objDataBaseViewer) :
            con, dns, service, port = self.getConnexion(False, False, None, None, None)
            if con :

                base, schema, table = self.currentItemData.parent().parent().text(0), self.currentItemData.parent().text(0), self.currentItemData.text(0)

                req = "DROP TABLE \"%s\".\"%s\" ;" % (schema, table)
                msg = QApplication.translate("QSphere","Delete all entries")
                self.sendToConsole("title", self.getTimeStamp(), "%s %s : %s [%s:%s]" % (msg, "%s.%s" % (schema, table),
                                                     self.targetDataBase, service, port), "sgbdr/deletetable.png")
                        
                con.set_isolation_level(ISOLATION_LEVEL_AUTOCOMMIT)
                cur = con.cursor()
                self.getRecords(cur, req, False)

                cur.close()
                con.close()

                self.htmlViewer.setHtml("", self.baseurl)
                self.objDataBaseViewer.currentItem().parent().removeChild(self.currentItemData)
                self.currentItemData = None               

                try:
                    listTables = self.dicSchemas[schema]
                    newlist = [elt for elt in listTables if elt!=table]
                    del self.dicSchemas[schema]
                    self.dicSchemas[schema] = newlist
                except KeyError: pass

                model = baseTableModel([], None, self)
                self.dataViewer.setModel(model)
                

                if self.mapViewer.accessibleDescription() == "%s.%s.%s" % (base, schema, table) :
                   if self.currentLayer :
                      QgsProject.instance().removeMapLayers([self.currentLayer.id()])
                      QgsProject.instance().addMapLayers([], False)
                      self.mapViewer.setLayers([]) 
                      self.mapViewer.refresh()
                      self.currentLayer = None
                      self.mapViewer.setAccessibleDescription("")

                self.tabsResults.setTabEnabled(1, False)
                self.tabsResults.setTabEnabled(2, False)
                self.tabsResults.setCurrentIndex(0)
                
                self.successFullMsg(req, False)
                self.changeEnableQspButtons()

        
    def errorMsgBase(self):
        title = QApplication.translate("QSphere","Warning")
        msg = QApplication.translate("QSphere","No object focused !")
        sendMsg(self, title , "<u><i>%s</i></u>" % msg, Qgis.Warning, self.duration_warning) 

    def successFullMsg(self, req, reloadData):
        self.sendToConsole("info", self.getTimeStamp(), "%s ! :<hr>%s</hr>" % (self.successfull, req) , self.ipucesuccessfull)
        if reloadData :
           if self.itemQspMetadata != None :
              self.getInfosStructureTable(self.itemQspMetadata, False) 
              if self.tabsResults.currentIndex()!= 1 and self.dataViewer.toolTip().endswith("public.qsp_metadata") :
                      self.dataViewer.setAccessibleDescription("refresh") ; self.getDataViewer();

              
    def vacuumTable(self):
        if not self.targetDataBase : return
        if not self.currentItemData  : return 

        con, dns, service, port = self.getConnexion(False, False, None, None, None)
        if con != None :

            schema, table = self.currentItemData.parent().text(0), self.currentItemData.text(0)
            req = "VACUUM \"%s\".\"%s\" ;" % (schema, table)
            self.sendToConsole("title", self.getTimeStamp(), "%s %s : %s [%s:%s]" % (QApplication.translate("QSphere","Vacuum table"), "%s.%s" % (schema, table),
                                                 self.targetDataBase, service, port), "sgbdr/vacuumtable.png")

            con.set_isolation_level(ISOLATION_LEVEL_AUTOCOMMIT)
            cur = con.cursor()
            self.getRecords(cur, req, False)

            cur.close()
            con.close()

            self.successFullMsg(req, True)
            

    def reloadBase(self): 
        item = self.objDataBaseViewer.currentItem()
        if not item : 
            counter = self.objDataBaseViewer.topLevelItemCount()
            for i in range(counter):
                itemTarget = self.objDataBaseViewer.topLevelItem(i)
                if (self.targetDataBase == itemTarget.text(0)) : item = itemTarget; break ;

        dataBase, item = self.getDataBase()  
        if not item : return
        self.objDataBaseViewer.setCurrentItem(item)
        
        parent = self.objDataBaseViewer.currentIndex()
        model = parent.model()

        stateExpand = self.getExpandChild(item, model, False, False)
        self.oldSource = ""
        self.itemQspMetadata = None
        self.enableQspButtons(False)
        self.getSchemasBase(item) 

        self.objDataBaseViewer.setExpanded(parent, True)
        self.expandChild(self.objDataBaseViewer, parent, model, stateExpand)

    def expandTreeBaseNodes(self): 
        treeWidget = self.objSqlViewer if self.sender().objectName() in ("expandAllSqlNodes", "closeAllSqlNodes") else self.objDataBaseViewer
        item = treeWidget.currentItem()
        if not item :
                try :
                        treeWidget.setCurrentItem(treeWidget.itemAt(0, 0))
                        item = treeWidget.currentItem()
                except : pass
        if not item : return

        cond = True if self.sender().objectName().startswith("expandAll") else False

        if item.whatsThis(0) in ("base", "sql") : pass       
        elif item.whatsThis(0) in ("table", "view")  : item = item.parent().parent()
        else : item = item.parent()  

        if type(self.sender()) == QAction :
                treeWidget.setCurrentItem(item)        
                indexBase = treeWidget.currentIndex()
                model = indexBase.model()
                stateExpand = self.getExpandChild(item, model, True, cond)
                treeWidget.setExpanded(indexBase, cond)
                self.expandChild(treeWidget, indexBase, model, stateExpand)
                self.enableQspButtons(False)
        else :
              treeWidget.expandAll() if cond else treeWidget.collapseAll()

     
    def expandTreeAllSqlNodes(self):
        treeWidget = self.objSqlViewer
        if self.sender().objectName().startswith("expandAll") : treeWidget.expandAll()
        else : treeWidget.collapseAll()
             

    def getExpandChild(self, parent, model, forcestate, state):
            expandChildState = {}
            for i in range( parent.childCount()):
                    child = parent.child(i)
                    expandChildState[child.text(0)] = child.isExpanded() if not forcestate else state
            return expandChildState

    def expandChild(self, treeWidget, parent, model, expand):
            i = 0
            for key in expand.keys():
                    child = treeWidget.findItems(key, Qt.MatchFixedString| Qt.MatchRecursive)
                    if child :
                        indexChild = model.index(i,0, parent)
                        treeWidget.setExpanded(indexChild, expand[key])
                    i+= 1


    def getInfos(self) : 
        if not self.objDataBaseViewer.currentItem() : return
        item = self.objDataBaseViewer.currentItem()
        index = self.tabsResults.currentIndex()

        if item.whatsThis(0) in ("table", "view") :
            if index == 0 :
                if type(self.sender()) == QTreeWidget : self.htmlViewer.setAccessibleDescription("refresh")
                self.getInfosStructureTable(item, False)    
            elif  index == 1 :
                if type(self.sender()) == QTreeWidget : self.dataViewer.setAccessibleDescription("refresh")
                self.getDataViewer()
            elif index == 2 :
                if type(self.sender()) == QTreeWidget : self.mapViewer.setAccessibleDescription("refresh")
                self.getMapViewer()
            elif index == 3 : self.changedMddFiles = True ; self.loadListMddFiles()
            elif index > self.fixedTabs-1 and self.dynamicalLink: self.copySqlInEditorSql()

            self.currentItemData = item
            self.infosChild = False

            if item.whatsThis(0)== "table" : self.changeSource = True


                    
    def getSchemasBase(self, item):
        if type(self.sender()) == QTabWidget : return
        if not item : return
        
        expandState = item.isExpanded()
        self.currentItemData = None
        if item.whatsThis(0)== "base" :
             newSource = "%s" %  item.text(0)  
             if self.oldSource != newSource :
                 self.updateItemIcon(item)
                 self.getInfosBaseSchemas(item)
                 self.setWindowTitle("%s [%s]" % (self.rootWindowTitle, item.text(0)))
                 if self.infosChild == False :
                    if self.htmlViewer.accessibleDescription() != "sqlresults" :
                            self.htmlViewer.setHtml("", self.baseurl) 
                 self.oldSource = newSource
                 
        item.setExpanded(not expandState)
        
    
    def getInfosBaseSchemas(self, item): 
        service, port, user, pwd = self.nameService.text(), self.namePort.text(), self.nameUser.text(), self.pwdUser.text()
        self.targetDataBase = item.text(0) if item.text(0)!="" else self.nameDatabase.currentText() 
        con, dns, service, port = self.getConnexion(False, False, None, None, None)

        if con != None :

            self.sendToConsole("title", self.getTimeStamp(), "%s : %s [%s:%s]" % (QApplication.translate("QSphere","Get list of schemas and tables"),
                                                 self.targetDataBase, service, port), "sgbdr/schemastables.png")
                    
            con.set_isolation_level(ISOLATION_LEVEL_AUTOCOMMIT)
            cur = con.cursor()

            self.listSchemas = [elt[0] for elt in sorted(self.getRecords(cur, GET_ALL_SCHEMAS_OUTSYSTEM_DATABASE, True))]
            self.getSqlDictionnary(self.listSchemas, "schemas")

            if self.checkSystemObjects.isChecked() :
                    listOuterSchemas = str(tuple(self.listSchemas))
                    records = self.getRecords(cur, GET_ALL_SCHEMAS_SYSTEM_DATABASE % listOuterSchemas, True)
                    if records :    
                            systemSchemas = [elt[0] for elt in sorted(records)]
                            for schema in systemSchemas :
                                if schema not in self.listSchemas : self.listSchemas.append(schema)   
            
            tempoListTables = self.getRecords(cur, GET_ALL_SCHEMAS_TABLES_DATABASE, True)

            if tempoListTables :
                    listTables = [elt[2] for elt in sorted(tempoListTables)] if tempoListTables else []
                    self.listTables = list(set(listTables))

            additionalTables = self.getRecords(cur, GET_ALL_PUBLIC_TABLES_DATABASE, True)
            if additionalTables : tempoListTables.extend(additionalTables)

            if self.checkSystemObjects.isChecked() :
                    tempoListSystemSchemas = self.getRecords(cur, GET_SERVER_SYSTEM_SCHEMAS % (self.targetDataBase, listOuterSchemas), True)
                    if tempoListSystemSchemas : tempoListTables.extend(tempoListSystemSchemas)
                        
            if self.listSchemas != []: self.getListTablesForSchemas(tempoListTables)

            self.listFunctions = [elt[0] for elt in sorted(self.getRecords(cur, GET_PG_PROCEDURES, True))]
            self.getSqlDictionnary(self.listFunctions, "functions")


            recordsTables = [elt[0] for elt in sorted(self.getRecords(cur, GET_TABLE_NAMES, True))]

            if dicoHasKey(self.dicSchemas, "public"):
                    if "qsp_metadata" not in self.dicSchemas["public"] : self.createQspMetadataTable(con, user, False)
                    else :

                        req = "SELECT column_name, data_type, character_maximum_length FROM information_schema.columns WHERE table_schema = '%s' AND table_name = '%s';" % ("public", "qsp_metadata") 
                        records = self.getRecords(cur, req, True)

                        def_qsp_metadata = [(u'id', u'integer', None), (u'title', u'character varying', 255), (u'abstract', u'character varying', None), (u'identificator', u'character varying', 255), (u'metadata', u'xml', None), (u'datasource', u'character varying', 255), (u'parentid', u'character varying', 100)]
                        
                        if  records == def_qsp_metadata : self.sendToConsole("info", self.getTimeStamp(), "%s : <b><u>%s</u></b><br>%s" % (QApplication.translate("QSphere","Find"), "qsp_metadata", QApplication.translate("QSphere","Successfull")), self.ipucetable) 
                        else : self.sendToConsole("error", self.getTimeStamp(), "<b><u>%s</u></b> :<br>%s" % ("qsp_metadata", QApplication.translate("QSphere","Bad format !")), self.ipucewarning)


            self.getSqlDictionnary(self.listTables, "tables")
            self.initCompleter()
            self.getTreeViewSchemas(item, self.dicSchemas)

            cur.close()
            con.close()
        
        else :

            if item.parent() != None :
               msg = "%s : %s" % (QApplication.translate("QSphere","Error"), item.text(0))
            else :
               msg = "%s : %s" % (QApplication.translate("QSphere","Bad result for"), self.targetDataBase)
            self.sendToConsole("error", self.getTimeStamp(), msg, self.ipucewarning)


    def askForActions(self, action, idData, treeWidget):
                dictActions = {"createtable" : (QApplication.translate("QSphere","Confirm create table"),
                                                "%s ?" % QApplication.translate("QSphere","Are you sure you want to create the metadata table")),
                               "deletetable" : (QApplication.translate("QSphere","Confirm delete table"),
                                                "%s ? :\n%s" % (QApplication.translate("QSphere","Delete the current line"), treeWidget.currentItem().text(0) )),
                               "deleterecord" : (QApplication.translate("QSphere","Confirm delete record"),
                                                 "%s<br><b>%s</b> ?" % (QApplication.translate("QSphere","Are you sure you want to delete the metadata record"),
                                        idData)),
                               "deletenode" : (QApplication.translate("QSphere","Confirm delete record"),
                                               "%s<br><b>%s</b> ?" % (QApplication.translate("QSphere","Delete the current line"),
                                        idData))
                               }

                if dicoHasKey(dictActions, action) : 

                    title, msg = dictActions[action][0], dictActions[action][1]
                    msgDlgBox = myMsgDlgBox(self, QMessageBox.Question, "%s ..." % title, msg, "qsphere_stop.png", None, self.parent)
                    windowIcon(msgDlgBox , "sgbdr/pucebase.png")
                    ret = msgDlgBox.exec_()    

                    if ret == 1 : return True
                    else : return False
        

    def getInfosStructureTable(self, item, console):
        self.enableQspButtons(False)

        if not item : return
        if not item.whatsThis(0) in ("table", "view") : return

        newsource = "%s.%s.%s" % (item.parent().parent().text(0), item.parent().text(0), item.text(0))
        if self.htmlViewer.accessibleDescription() == newsource : return
        self.htmlViewer.setAccessibleDescription(newsource)

        service, port, user, pwd = self.nameService.text(), self.namePort.text(), self.nameUser.text(), self.pwdUser.text()
        self.targetDataBase = item.parent().parent().text(0) if item.parent().parent().text(0)!= "" and item.parent().parent()!= None else self.nameDatabase.currentText() #"postgres"
        con, dns, service, port = self.getConnexion(False, False, None, None, None)

        self.updateItemIcon(item.parent().parent())

        if item.whatsThis(0) == "table" :
                self.ipuce = self.ipucetable 
                self.puce = self.pucetable 
        else :        
                self.ipuce = self.ipuceview
                self.puce = self.puceview

        self.sendToConsole("info", self.getTimeStamp(), "Function getInfosStructureTable for %s.%s.%s" % (item.parent().parent().text(0), item.parent().text(0), item.text(0)), self.ipuce)
        html = self.html

        if con != None :
            con.set_isolation_level(ISOLATION_LEVEL_AUTOCOMMIT)
            cur = con.cursor()

            req = GET_TABLE_FIELDS % (item.parent().text(0), item.text(0))
            
            records = self.getRecords(cur, req, True)
            html+= "<h2><img src='file:///%s'> %s.%s.%s</h2><table><tr>" % (self.puce, self.targetDataBase, item.parent().text(0), item.text(0))
            for desc in cur.description : html+= "<th>%s</th>" %  desc[0]
            html+= "</tr>"

            listFields = []    
            for record in records :
                html+= "<tr>"
                i = 0
                for elt in record :
                     html+= "<td>%s</td>" %  elt
                     if i == 0 : listFields.append(elt)
                     i+= 1
                html+= "</tr>"
            html+= "</table>"

            self.sqlDictionnary["fieldstable"] = listFields
            self.initCompleter()
            
            
            records = None
            try :
                req = GET_TABLE_COUNT_RECORDS % (item.parent().text(0), item.text(0))
                records = self.getRecords(cur, req, True)
                html+= "<pre><img src='file:///%s'> %s %s</pre>" % (self.pucerecords, records[0][0], self.lblRecords)
            except : pass
            

            self.itemQspMetadata = None
            if item.parent().text(0) == "public" and item.text(0)=="qsp_metadata" :
                self.enableQspButtons(True)
                self.itemQspMetadata = item
                self.movieCtrl.startMovie()
                self.getDataFromBase(item, cur, -1)
                self.movieCtrl.stopMovie()

            if item.whatsThis(0) == "table" :    
                    records = None
                    html+= "<h2><img src='file:///%s'> %s : </h2>" % (getThemeIcon("sgbdr/pucetablestats.png"), QApplication.translate("QSphere","Statistics"))
                    try :
                        req = GET_TABLE_STATISTICS % (item.text(0))
                        records = self.getRecords(cur, req, True)
                    except : pass
                    
                    if records :
                       records = records[0]
                       html+= "<table>"
                       for i in range(len(records)): html+= "<tr><th>%s</th><td>%s</td></tr>" % (cur.description[i][0], records[i])
                       html+= "</table>"
                    else :
                       html+= "%s." % QApplication.translate("QSphere","No statistics")
            else :
                    req = GET_VIEW_SQL_PLUS % (item.text(0))
                    records = self.getRecords(cur, req, True)
                    
                    if records :
                       records = records[0]
                       html+= "<table>"
                       for i in range(len(records)): html+= "<tr><th>%s</th><td>%s</td></tr>" % (cur.description[i][0], records[i])
                       html+= "</table>"
                    else :
                       html+= "%s." % QApplication.translate("QSphere","No definition")

   
            geomColumn = self.getSpecialColumn(item, True, "geometryColumn")
            if geomColumn != None :
                req = """SELECT srid, srtext, proj4text FROM spatial_ref_sys WHERE srid IN (SELECT Find_SRID('%s', '%s', '%s')) ;""" % (item.parent().text(0), item.text(0), geomColumn)
                records = self.getRecords(cur, req, True)
                if records :
                         records = records[0]
                         html+= """<br><h2><img src='file:///%s'> %s</h2><table>""" % (getThemeIcon("sgbdr/pucetablesrid.png"), QApplication.translate("QSphere","Coordinate reference system(s) : "))
                         for i in range(len(records)): html+= "<tr><th>%s</th><td>%s</td></tr>" % (cur.description[i][0], records[i])
                         html+= "</table>"


            req = GET_TABLE_JOINS % (item.parent().text(0), item.text(0))           
            records = self.getRecords(cur, req, True)
            
            if records :
                    html+= "<br><h2><img src='file:///%s'> %s : %s </h2>" % (getThemeIcon("sgbdr/pucetablelinked.png"), QApplication.translate("QSphere","Linked table(s)"), len(records))
                    html+= self.getResultsInHtml(cur, records, False, 0, req, -1)

            html+= "<br><br><br><br></BODY></HTML>"          
               
        
        if not console : self.htmlViewer.setHtml(html, self.baseurl) 
        else : self.sendToConsole("info", self.getTimeStamp(), html, "html.png")

   
    def getSpecialColumn(self, item, console, lblColumn):
        if not item : return
        if item.whatsThis(0) not in ("table", "view") : return
        
        service, port, user, pwd = self.nameService.text(), self.namePort.text(), self.nameUser.text(), self.pwdUser.text()
        self.targetDataBase = item.parent().parent().text(0) if item.text(0)!= "" and item.parent().parent()!= None else self.nameDatabase.currentText() 
        con, dns, service, port = self.getConnexion(False, False, None, None, None)
        
        if con != None :
            con.set_isolation_level(ISOLATION_LEVEL_AUTOCOMMIT)
            cur = con.cursor()
            self.updateItemIcon(item.parent().parent())
            self.sendToConsole("info", self.getTimeStamp(), "Function %s for %s.%s.%s" % (lblColumn, item.parent().parent().text(0), item.parent().text(0), item.text(0)), self.ipucetable)

            if lblColumn == "geometryColumn" : req = GET_TABLE_GEOMETRY_COLUMN % (item.parent().text(0), item.text(0))
            elif lblColumn == "rasterColumn" : req = GET_TABLE_RASTER_COLUMN % (item.parent().text(0), item.text(0))
            elif lblColumn == "keyColumn" : req = GET_TABLE_KEY_COLUMN % (item.parent().text(0), item.text(0))
            else : return None

            records = self.getRecords(cur, req, True)
            self.sendToConsole("sql", self.getTimeStamp(), "%s" % req, "sgbdr/minisql.png")

            if records :
               try :
                   if lblColumn == "keyColumn" : return "%s" % records[0][2]
                   else : return "%s" % records[0][0]
               except : pass 
        return None

    def getDataFromBase(self, item, cur, offset):
        try :
            req = GET_TABLE_RECORDS % (item.parent().text(0), item.text(0), int(self.comboSqlBoundary.currentText()))
        except : return
        
        records = self.getRecords(cur, req, True)
        self.metadataToDownload = {}

        indexXmlData = -1
        i = 0
        for desc in cur.description :
            if desc[0] == "metadata" :
                indexXmlData = i
                break
            i+= 1

        if indexXmlData == -1 : return
     
        for i in range(len(records)): self.metadataToDownload['%s' % records[i][0]] = records[i][indexXmlData].replace("''", "'")
    



    def remoteCtrlSql(self, req, title):
        self.editorSql.setText(req)
        self.editorSql.setToolTip("")
        self.dockSql.setWindowTitle("SQL : %s" % title)        
        self.executeSql()
        
    def getAllUsersForServer(self): self.remoteCtrlSql(GET_ALL_SERVER_USERS, self.sender().text())

    def getAllPrimaryKeysDataBase(self): self.remoteCtrlSql(GET_ALL_PRIMARYKEYS_DATABASE, self.sender().text())           

    def getAllSchemasDataBase(self): self.remoteCtrlSql(GET_ALL_SCHEMAS_OUTSYSTEM_DATABASE, self.sender().text())

    def getAllSchemasTablesDataBase(self): self.remoteCtrlSql(GET_ALL_SCHEMAS_TABLES_DATABASE, self.sender().text())
        
    def getAllPrivilegesForAllUsersDataBase(self):
        self.remoteCtrlSql("SELECT table_schema, table_name, grantee, string_agg(privilege_type, ', ') AS privileges FROM information_schema.role_table_grants GROUP BY table_schema, table_name, grantee ORDER BY table_schema;",
                           self.sender().text())

    def getAllPrivilegesForUserDataBase(self): self.remoteCtrlSql(GET_ALL_PRIVILEGES_FOR_USER % self.nameUser.text(), self.sender().text())           

    def getConnexion(self, test, hasmsg, typemsg, msg, img):
        service = self.nameService.text()
        port = self.namePort.text()
        user = self.nameUser.text()
        pwd = self.pwdUser.text()
        dataBase, item = self.getDataBase()
            
        con, dns = None, ""
        if hasmsg :
            msg = "%s : %s [%s:%s]" % (msg, dataBase, service, port)
            self.sendToConsole(typemsg, self.getTimeStamp(), msg , img)    
        
        if user == "" or pwd == "" :
           self.sendToConsole("error", self.getTimeStamp(), QApplication.translate("QSphere","User or password can't be empty"), self.ipucewarning)
        else : 
           dns = "host=%s dbname=%s user=%s password=%s port=%s" % (service, dataBase, user, pwd, port)
           try : con = connect(dns)
           except OperationalError : pass
        return con, dns, service, port

    def updateComboSqlBoundary(self, boundary, fixeIndex, index):
        steps = fixeRanges(boundary)
        seuils = []
        for seuil in steps :
           elt = "%s" % int(seuil)
           if int(seuil)>0 and elt not in seuils : seuils.append("%s" % int(seuil))
        if seuils == []: seuils = ["%s" % boundary]

        self.comboSqlBoundary.blockSignals(True)
        self.comboSqlBoundary.clear()
        self.comboSqlBoundary.addItems(seuils)
        if fixeIndex :
           if index == -1 : self.comboSqlBoundary.setCurrentIndex(len(seuils)-1)
           else : self.comboSqlBoundary.setCurrentIndex(index)
        self.comboSqlBoundary.blockSignals(False)
        self.lblSqlCounter.setText("%s" % boundary)
              

    def getDataBase(self):
        dataBase, item = "", None
        item = self.objDataBaseViewer.currentItem()
        if not item : dataBase = self.nameDatabase.currentText()
        else :    
            if item.whatsThis(0) in ("table", "view") : item = item.parent().parent()
            elif item.whatsThis(0)== "schema" : item = item.parent()
            dataBase = item.text(0) if item.text(0)!="" else self.nameDatabase.currentText()
        return dataBase, item 

    def saveResultsFiles(self):

        if self.tabsResults.currentIndex() >= self.fixedTabs :

            title = QApplication.translate("QSphere","Save as a file")
            dlgFile = QFileDialog(self, title)
            elt0 = QApplication.translate("QSphere","files").title()
            dlgFile.setNameFilters(("%s CSV (*.csv)" % (elt0),))
            dlgFile.setDefaultSuffix("csv")
            dlgFile.setViewMode(QFileDialog.Detail)
            dlgFile.setDirectory(self.InitDir)
            dlgFile.setAcceptMode(QFileDialog.AcceptSave)

            fixeLabelsFileDialog(self, dlgFile, 1, True)
            
            if dlgFile.exec_():
                fileName = dlgFile.selectedFiles()[0]
                query = self.tabsResults.tabToolTip(self.tabsResults.currentIndex())
                query = query.replace(";", "")

                con, dns, service, port = self.getConnexion(False, False, None, None, None)
                if con != None :

                    con.set_isolation_level(ISOLATION_LEVEL_AUTOCOMMIT)
                    cur = con.cursor()
                    outputquery = "COPY ({0}) TO STDOUT WITH CSV HEADER".format(query)

                    
                    #user = self.nameUser.text()
                    #pwd = self.pwdUser.text()

                    #dataBase, item = self.getDataBase()

                    #pgsql2shp -f <file path to save shape file> -u <username> -h <hostname> -P <password> <database Name> “query to be executed”
                    #"pssql2shp -f %s -u %s -h %s -P %s %s %s" % (fileName, user, service, pwd, targetDataBase, query)
                    """
                    command= ‘pgsql2shp -f “/var/www/html/PHP_pgrouting/public/shpfolder/try_new.shp” -h localhost -u user -P “pass1” postgres “select seq,id1,id2,cost,geom from pgr_dijkstra(‘.$query.’,’.$start1.’,’.$end1.’,false, false) as di JOIN network1 pt ON (di.id2 = pt.gid)”‘;
                    echo shell_exec($command);
                    """

                    try :
                        with open(fileName, "w") as outputFile : cur.copy_expert(outputquery, outputFile)
                        
                        title = QApplication.translate("QSphere","Save as a file") 
                        msg = QApplication.translate("QSphere","The metadata record was saved as")
                        sendMsg(self, title , "%s :<br><u><i>%s</i></ul>" % (msg, fileName), Qgis.Info, self.duration_info)
                    except :
                        title = QApplication.translate("QSphere","Warning")
                        msg = QApplication.translate("QSphere","Error")
                        sendMsg(self, title , "%s : %s" % (msg, fileName), Qgis.Warning, 5)

                    con.close
   

        
    def closeTabs(self, int):
        if int > self.fixedTabs-1 :  self.tabsResults.removeTab(int) 
        elif int in (0, self.fixedTabs-1) :
            self.tabsResults.setCurrentIndex(int)
            msgDlgBox = myMsgDlgBox(self, QMessageBox.Question, QApplication.translate("QSphere","Warning"),
                                                QApplication.translate("QSphere","This tab can't be close.\nWould you like to clear content ?"), "qsphere_stop.png", None, self.parent)
            windowIcon(msgDlgBox , "layers-database.png")
            ret = msgDlgBox.exec_()
            if ret == 1 : self.deleteLog()
               
    def addTabsResults(self, model, req):
        self.updateComboSqlBoundary(model.rowCount(), True, -1)
 
        dataViewer = QTableView(self.groupBoxResults) 
        dataViewer.setAccessibleDescription("")

        sizePolicy = QSizePolicy(QSizePolicy.Expanding, QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(3)
        sizePolicy.setHeightForWidth(dataViewer.sizePolicy().hasHeightForWidth())
        dataViewer.setSizePolicy(sizePolicy)
        dataViewer.setSelectionBehavior(QAbstractItemView.SelectRows)
        dataViewer.setSelectionMode(QAbstractItemView.SingleSelection)
        dataViewer.setHorizontalScrollMode(QAbstractItemView.ScrollPerPixel)
        dataViewer.setVisible(False)
        dataViewer.setAlternatingRowColors(True)
        dataViewer.setStyleSheet("QTableView {alternate-background-color: cyan;background-color: white;}")

        dataViewer.setModel(model)

        index = self.tabsResults.addTab(dataViewer, QIcon(self.pucequery), "")
        title = "%s SQL : %s" % (QApplication.translate("QSphere","Results") , index-self.fixedTabs)
        self.tabsResults.setTabText(index, title)
        self.tabsResults.setTabToolTip(index, req)

        widgetCounter = myMdiSubWindow(dataViewer,"widgetCounter", "%s :" % QApplication.translate("QSphere","Actions"), 140, 80 )
       
        findInListResults = myPushButton(widgetCounter)
        findInListResults.initPushButton(32, 32, 20, 30, "findInListResults", "", QApplication.translate("QSphere","Find in results"), True, getThemeIcon("search.png"), 32, 32, True)
        findInListResults.clicked.connect(self.findInResults)

        copyInEditorSql = myPushButton(widgetCounter)
        copyInEditorSql.initPushButton(32, 32, 60, 30, "copyInEditorSql", "", "%s %s" % (QApplication.translate("QSphere","Copy"), QApplication.translate("QSphere", "SQL Commands")),
                                       True, getThemeIcon("copier.png"), 32, 32, True)
        copyInEditorSql.clicked.connect(self.copySqlInEditorSql)

 
    def getResultsInHtml(self, cur, records, fromSQL, idtable, req, idReq):
        if records == None : return "<img src='file:///%s'><br><h1>%s %s</h1>" % (self.pucewarning, self.failed, QApplication.translate("QSphere","SQL Commands"))
        elif records == True : return "<img src='file:///%s'><br><h1>%s %s</h1>" % (self.pucesuccessfull, self.successfull, QApplication.translate("QSphere","SQL Commands"))

        html = self.html
        boundary = len(records)

        if len(records) <= 500 and not self.checkResultsInNewTab.isChecked() :
            html+= "<tr valign='top'><td width='1%%'><img src='file:///%s'></td><th class='title'><h3>%s : %s</h3><a href='javascript: switchInfoPerso(%s);'><img src='file:///%s'><span id='msg%s'>%s</span></a></th></tr>" % (self.pucerecords, self.lblRecords.title(), len(records), idReq, self.pucearrow, idReq, QApplication.translate("QSphere", "Hide"))
        else :
            html+= "<tr valign='top'><td width='1%%'><img src='file:///%s'></td><th class='title'><h3>%s : %s</h3></th></tr>" % (self.pucerecords, self.lblRecords.title(), len(records))
            
        html+= "<tr><td width='1%%'></td><td bgcolor='white' bordercolor='black'>"
        html+= "<div id='divacacher%s' style='background: #efe; padding: 5px; width: 100%%; height: 100%%;overflow-x: hidden; overflow-y: scroll;'>" % idReq

        if len(records) > 500 or self.checkResultsInNewTab.isChecked() :
                  self.movieCtrl.startMovie()

                  dataBase, item = self.getDataBase()  
                  if not item : return
                  self.objDataBaseViewer.setCurrentItem(item)
                  index = 1 if item.whatsThis(0) in ("table", "view") else self.tabsResults.count()

                  if not self.checkResultsInNewTab.isChecked() :
                      html+= "<h2>%s : <a href='%s'>%s %s</a></h2>" % (QApplication.translate("QSphere","Too many records, sending results in Data viewer."),
                                                                index, QApplication.translate("QSphere","Results"), index-self.fixedTabs)
                  else :
                      html+= "<h2>%s : <a href='%s'>%s %s</a></h2>" % (QApplication.translate("QSphere","Send to"), 
                                                                index, QApplication.translate("QSphere","Results"), index-self.fixedTabs)

                  fieldNames = [desc[0] for desc in cur.description ]
                 
                  model = baseTableModel(fieldNames, records, self)
                  self.addTabsResults(model, req) 

                  self.movieCtrl.stopMovie()

  
        else :
                  if not fromSQL : html+= "<pre><img src='file:///%s'> %s %s</pre><br>" % (self.pucerecords, len(records), self.lblRecords)
                  html+= "<table class='browse' id='%s'><thead><tr>" % idtable
                  k = 0

                  for desc in cur.description :
                         typeField = self.fixeTypeField(desc[0]) 
                         if k == 0 : html+= "<th class='ascend' abbr='%s'>%s</th>" %  (typeField, desc[0])
                         else :  html+= "<th abbr='%s'>%s</th>" %  (typeField, desc[0])
                         k+= 1
                  html+= "</tr></thead><tbody>"
                  for record in records :
                      html+= "<tr>"
                      k = 0
                      for elt in record :
                              if type(elt)==str :
                                  if k == 0 : html+= "<td class='browse.col0'>%s</td>" %  elt.replace("\n", "<br>")
                                  else : html+= "<td class='browse.input'>%s</td>" %  elt.replace("\n", "<br>")
                              else : html+= "<td class='browse.gel'>%s</td>" %  elt
                              k+= 1
                      html+= "</tr>"
                  html+= "</tbody></table><br>"

        html+= "</div></td></tr>"
                  
        return html


    def fixeTypeField(self, nameField):
        if self.description == [] : return ""
        typeField = ""        
        for field in self.description :
            if field[0] == nameField :
               typeColumn = field[1]
               if typeColumn in self.numericTypesCodes : typeField =  "number"
               break     
        return typeField     


    def getListTablesForSchemas(self, listAllInfos):
        self.dicSchemas = {}
        self.schematable = {}
        for elt in self.listSchemas : self.dicSchemas["%s" % elt]= []
        if not listAllInfos : return
        for elt in listAllInfos :
                if not elt[2] in self.dicSchemas["%s" % elt[1]]:
                   self.dicSchemas["%s" % elt[1]].append("%s" %elt[2])
                   schema = self.quotedName(elt[1], True)
                   table = self.quotedName(elt[2], True)
                   self.schematable['%s.%s' % (schema, table)] = ('"%s"' % elt[3], "%s" % elt[4], "%s" % elt[5], "%s" % elt[6], "%s" % elt[7])
        

    def setListBases(self, listBases, inListBases, setIndex):
        self.nameDatabase.clear()
        self.nameDatabase.addItem("postgres")
        
        for elt in listBases :
            if inListBases : self.listBases.append(elt[0])
            
            index = self.nameDatabase.findText(elt[0], Qt.MatchFixedString)
            if index > -1 :
               text = self.nameDatabase.itemText(index) 
               if text != elt[0]: index = -1
            if index == -1 : self.nameDatabase.addItem(elt[0])

        if setIndex :
            index = self.nameDatabase.findText(elt[0], Qt.MatchFixedString)
            self.nameDatabase.setCurrentIndex(index)        


    def fixeInfosConnexionQGIS(self):
        if self.qgisConnexion.currentText()!= "" :
            params = getParamsConnexionQGIS(self.qgisConnexion.currentText())
            if params!= [] and len(params)== 5: 
                self.nameService.setText(params[0])
                self.namePort.setText(params[1])
                self.nameUser.setText(params[2])
                self.pwdUser.setText(params[3])

                self.setListBases([(params[4],)], False, True)



    def getColorizedHtml(self, sql):
        
        keywords = ["ALTER","AND","AS","BEGIN","BY", "CALL","CASE","COMMENT","COMMIT","CONNECT","CREATE","DELETE","DROP","ELSE","END",
                    "EXPLAIN","EXPORT","FROM","GRANT", "GROUP", "IMPORT","IN", "INNER","INSERT","LEFT","LIKE","!=","JOIN","LOAD","LOCK",
                    "MERGE","NOT","NULL","ON","ORDER", "RECURSIVE", "REFRESH","RENAME","REPLACE","REVOKE", "RIGHT","ROLLBACK","SELECT","SET",
                    "THEN","TRUNCATE","UNLOAD","UNSET","UPDATE","UPSERT","WHEN","WHERE", "WITH"]
       
        
        sql = sql.replace("\n", " ").split(" ")
        sqlFormated = ""
        openChr = False
        for word in sql :
            if word.strip().upper() in keywords :  sqlFormated+= " <font color='blue'><b>%s</b></font>" % word.strip().upper()
            elif word.find("'")!= -1 :
                 startPosition = word.find("'")
                 endPosition = word.find("'", startPosition+1)
                 if endPosition!=-1 : sqlFormated+= " %s<font color='#66FF3A'><b>%s</b></font>%s" % (word[0:startPosition], word[startPosition:endPosition+1], word[endPosition+1:])
                 else :
                       if not openChr : sqlFormated+= " %s<font color='#66FF3A'><b>%s" % (word[0:startPosition], word[startPosition:]); openChr = True ;
                       else : sqlFormated+= " %s</b></font>%s" % (word[0:startPosition+1], word[startPosition+1:]); openChr = False ;
                        
            elif  word.find("--{")!= -1 : sqlFormated+= " <font color='#BA96A8'><b>%s" % word
            elif  word.find("}--")!= -1 : sqlFormated+= " %s</b></font>" % word
            else :  sqlFormated+= " %s" % word

        return sqlFormated


    def getDatabasePrivileges(self):
        dataBase, item = self.getDataBase()  
        if not item : return
        self.objDataBaseViewer.setCurrentItem(item)
        
        service, port, user, pwd = self.nameService.text(), self.namePort.text(), self.nameUser.text(), self.pwdUser.text()
        self.targetDataBase = dataBase 
        con, dns, service, port = self.getConnexion(False, False, None, None, None)

        result = False
        
        if con != None :

            self.sendToConsole("title", self.getTimeStamp(), "%s : %s [%s:%s]" % (QApplication.translate("QSphere","Get list of schemas and tables"),
                                                 self.targetDataBase, service, port), "sgbdr/schemastables.png")
                    
            con.set_isolation_level(ISOLATION_LEVEL_AUTOCOMMIT)
            cur = con.cursor()

            req = "SELECT has_database_privilege(current_database(), 'CREATE'), has_database_privilege(current_database(), 'TEMP')"
            records = self.getRecords(cur, req, True)

            if records : result = bool(records[0][0])

        return result    
        
    def addLayerToMapCanvas(self):
        item = self.objDataBaseViewer.currentItem()    
        if not item : return
        if item.whatsThis(0) in ("table", "view") :

            geomColumn = self.getSpecialColumn(item, True, "geometryColumn")
            schema = item.parent().text(0) 
            table = item.text(0) 

            if geomColumn == None : geomColumn = self.getSpecialColumn(item, True, "rasterColumn")
            if geomColumn == 'rast' :
                uri = "PG: dbname=%s host=%s user=%s password=%s port=%s mode=2 schema=%s column=rast table=%s" % (self.targetDataBase, self.nameService.text(),
                                                                                                                      self.nameUser.text(), self.pwdUser.text(),
                                                                                                                      self.namePort.text(), schema, table)
                vlayer = QgsRasterLayer(uri, table)

            else :
            
                keyColumn = self.getSpecialColumn(item, True, "keyColumn")

                uri = QgsDataSourceUri()
                uri.setConnection(self.nameService.text(), self.namePort.text(), self.targetDataBase,  self.nameUser.text(), self.pwdUser.text())

                if keyColumn : uri.setDataSource(schema, table, geomColumn, "", keyColumn)
                else :      
                    req = u"(SELECT row_number() Over() as myid, a.* FROM %s.%s AS a)" % (schema, table) 
                    uri.setDataSource("", req, geomColumn, "",  "myid")

                vlayer = QgsVectorLayer(uri.uri(), item.text(0), "postgres")
                            

            if vlayer != None :
               if vlayer.isValid() :
                   QgsProject.instance().addMapLayer(vlayer, True)
                   self.iface.mapCanvas().refresh()

        return
