# -*- coding: utf-8 -*-
#===================================================
# base on code by : Eli Bendersky (eliben@gmail.com)
# --> qsci_simple_pythoneditor.py
# This code is in the public domain
#===================================================
from qgis.core import Qgis
from qgis.gui import QgsMessageBar

from qgis.PyQt.QtCore import (QEvent, QPoint, Qt)
from qgis.PyQt.QtGui import (QIcon, QKeySequence, QMovie, QStandardItem, QStandardItemModel)
from qgis.PyQt.QtPrintSupport import (QPrintPreviewDialog, QPrinter)
from qgis.PyQt.QtWidgets import (QAbstractItemView, QAction, QApplication, QDialog, QFileDialog, QFrame, QGroupBox, QHBoxLayout,
                                 QLabel, QMenu, QPushButton, QSizePolicy, QSplitter, QTableView, QTextBrowser, QTextEdit, QVBoxLayout)

from qsphere.qsphere_objmaker import (movieCtrl, myPushButton, myPushButtonFunny, myScintilla, myTextEdit, printerSphere)
from qsphere.qsphere_tools import (changeButtonIcon, dicoHasKey, fileNameWithExtension, fixeLabelsFileDialog,
                           windowIcon, sendMsg, getEncodingCar, getThemeIcon, getOptions, getHelp, sortDic, testXmlData)
from qsphere.xmlisoparser import xmlIsoParser

from codecs import open as codecsopen
from io import StringIO as bufferData
from os.path import dirname
from textwrap import fill as textwrapfill

class xmlEditor(QDialog):

  find_text = None
  find_forward = True
  
  def __init__(self, parent, fichier, referent):
   
    self.arrowMarkerNum = 8
    
    try:
      from qgis.PyQt import Qsci
      from qgis.PyQt.Qsci import QsciScintilla, QsciScintillaBase, QsciLexerXML
    except:
      title = QApplication.translate("QSphere", "Error")
      msg = QApplication.translate("QSphere", "QScintilla library not detected or installed !")
      if parent!=None : sendMsg(parent, title , msg, Qgis.Warning, self.duration_warning)
      return

    self.fichier = fichier
    self.parent = parent
    self.referent = referent

    QDialog.__init__(self, parent, Qt.WindowMaximizeButtonHint | Qt.WindowCloseButtonHint | Qt.WindowMinimizeButtonHint)

    self.setObjectName("DialogEditorXML")

    getOptions(self)


    self.listOfRules, self.dictListOfRules = self.referent.listOfRules, self.referent.dictListOfRules
    self.typeData = self.referent.typeData
    self.languesDico = self.referent.languesDico
    self.localLangage = self.referent.localLangage
    self.langs = sortDic(self.languesDico)
    self.listCodecs = self.referent.listCodecs

    self.minWidth, self.minHeight = 800, 520
    self.setMinimumSize(self.minWidth, self.minHeight)
    self.resize(self.minWidth, self.minHeight)
    self.setModal(True)

    self.iconePrint, self.iconeSave = "print.png", "save.png"
    self.iconeSizeUp, self.iconeSizeDown = "sizeup.png", "sizedown.png"

    self.rootWindowTitle = QApplication.translate("QSphere","File")
    self.orderKeys = ("intitule", "resume", "typedata", "tablelocalisator", "identificator", "tablelangues", "tableformats", "tablecarac",
                  "tablecategories", "tablecategories:0", "tablemotsclefsf",
                  "tablescr", "tableemprises",
                  "tableetenduetemporelle", "groups:dates",
                  "genealogie", "coherence", "grouperesolutionscale", "tablespecifications",
                  "groupedroits", "licence", "tableroles",
                  "datemetada", "langmetada")

    self.orderIconkeys = ("identification.png", "identification.png", "identification.png", "identification.png", "identification.png", "identification.png", "identification.png", "identification.png",
                           "classification.png", "classification.png", "classification.png",
                           "location.png", "location.png",
                           "temporality.png", "temporality.png",
                           "quality.png", "quality.png", "quality.png", "quality.png",
                           "rights_and_uses.png", "rights_and_uses.png", "responsible.png",
                           "metadata.png", "metadata.png")


    self.frame = QFrame(self)
    self.masterHLayout = QHBoxLayout(self.frame)
    self.masterHLayout.setObjectName("masterHLayout")
    self.splitter = QSplitter(self.frame)
    self.splitter.setOrientation(Qt.Horizontal)
    self.splitter.setObjectName("splitter")

    self.groupBoxTags = QGroupBox(self.splitter)
    self.groupBoxTags.setMaximumWidth(450)
    
    self.groupBoxTags.setObjectName("groupBoxTags")
    self.vLayoutgroupBoxTags = QVBoxLayout(self.groupBoxTags)
    self.vLayoutgroupBoxTags.setObjectName("vLayoutgroupBoxTags")


    self.tableModel = QStandardItemModel(self)
    self.xmlTags = QTableView(self)
    icon = ""
    for i in range(len(self.orderKeys)):
        key = self.orderKeys[i]
        item = QStandardItem()
        item.setText("%s" % (key))
        if self.orderIconkeys[i]!= icon : icon = getThemeIcon(self.orderIconkeys[i])
        item.setIcon(QIcon(icon))
        item.setCheckable(False)
        item.setEditable(False)
        self.tableModel.setItem(i,0,item)
        
    self.xmlTags.setColumnWidth(0, 240)
    self.xmlTags.horizontalHeader().setDefaultSectionSize(240)
    self.xmlTags.horizontalHeader().setVisible(False) 
    self.xmlTags.verticalHeader().setVisible(False) 
    self.xmlTags.setModel(self.tableModel)
    self.xmlTags.setSelectionMode(QAbstractItemView.SingleSelection)
    self.xmlTags.setSelectionBehavior(QAbstractItemView.SelectRows)
    self.xmlTags.setVisible(True) 
    self.xmlTags.setToolTip(QApplication.translate("QSphere", "Internal controls QSphere."))

    self.vLayoutgroupBoxTags.addWidget(self.xmlTags)

    self.valueElt =  myTextEdit(self)
    self.valueElt.initTextEdit(200, 25, 10, 50, "valueElt", True, False, True, False, True)
    self.valueElt.setStyleSheet("color: black; background-color: #C0C0C0")
    self.valueElt.setToolTip(QApplication.translate("QSphere", "Content for the item."))

    self.vLayoutgroupBoxTags.addWidget(self.valueElt)

    self.groupBoxEditor = QGroupBox(self.splitter) 
    self.groupBoxEditor.setObjectName("groupBoxEditor")
    self.hLayoutgroupBoxEditor = QHBoxLayout(self.groupBoxEditor)
    self.hLayoutgroupBoxEditor.setObjectName("hLayoutgroupBoxEditor")  

    self.editor = myScintilla(self,"xmlWidget", False, (285, 10, 780, 610))
    self.editor.listOfRules, self.editor.dictListOfRules = self.listOfRules, self.dictListOfRules
    self.editor.typeData = self.typeData
    self.editor.languesDico = self.languesDico
    self.editor.langs = sortDic(self.languesDico)
    self.editor.listCodecs = self.listCodecs  


    self.hLayoutgroupBoxEditor.addWidget(self.editor)
    self.masterHLayout.addWidget(self.splitter)

    self.rootMsg = QApplication.translate("QSphere","QSphere XML inspector")

    self.printToolButton = myPushButton(self.valueElt)
    self.printToolButton.initPushButton(48, 24, 5, 5, "printToolButton", "", QApplication.translate("QSphere", "Print ToolTip"), True, getThemeIcon(self.iconePrint), 48, 24, True)
    self.printToolButton.setShortcut(QKeySequence("Ctrl+Shift+P"))

    self.findprevious = myPushButton(self)
    self.findprevious.initPushButton(48, 24, 5, 5, "findprevious", "", QApplication.translate("QSphere", "Find previous"), True, getThemeIcon("leftarrow.png"), 48, 24, True)
    self.findprevious.setShortcut(QKeySequence("F2"))

    self.searchbutton = myPushButton(self)
    self.searchbutton.initPushButton(24, 24, 5, 5, "searchbutton", "", QApplication.translate("QSphere", "Search word"), True, getThemeIcon("find.png"), 24, 24, True)
    self.searchbutton.setShortcut(QKeySequence("Ctrl+F"))

    self.findnext = myPushButton(self)
    self.findnext.initPushButton(48, 24, 5, 5, "findnext", "", QApplication.translate("QSphere", "Find next"), True, getThemeIcon("rightarrow.png"), 48, 24, True)
    self.findnext.setShortcut(QKeySequence("F3"))
    
    self.loadButton = myPushButton(self)
    self.loadButton.initPushButton(24, 24, 5, 5, "loadButton", "", QApplication.translate("QSphere", "Open Files"), True, getThemeIcon("open.png"), 24, 24, True)
    self.loadButton.setShortcut(QKeySequence("Ctrl+O"))

    self.printButton = myPushButton(self)
    self.printButton.initPushButton(24, 24, 5, 5, "printButton", "", QApplication.translate("QSphere", "Print"), True, getThemeIcon(self.iconePrint), 24, 24, True)
    self.printButton.setShortcut(QKeySequence("Ctrl+P"))

    self.saveButton = myPushButton(self)
    self.saveButton.initPushButton(24, 24, 5, 5, "saveButton", "", QApplication.translate("QSphere", "Save"), True, getThemeIcon(self.iconeSave), 24, 24, True)
    self.saveButton.setShortcut(QKeySequence("Ctrl+S"))

    self.actionsSaveButton = myPushButton(self) 
    self.actionsSaveButton.initPushButton(56, 24, 5, 5, "actionsSaveButton", "", QApplication.translate("QSphere","Other saving actions ..."), True, getThemeIcon("saveactions.png"), 56, 24, True)

    self.closeButton = QPushButton(self)
    self.closeButton.setObjectName("closeButton")
    self.closeButton.setText(QApplication.translate("QSphere", "Close"))

    self.zoomInButton = myPushButtonFunny(self)
    self.zoomInButton.initPushButton(32, 32, 5, 5, "zoomInButton", "", QApplication.translate("QSphere", "Zoom in"), True, getThemeIcon(self.iconeSizeUp), 32, 32, True)
    self.zoomInButton.setAutoRepeat(True)
    self.zoomInButton.setShortcut(Qt.Key_Plus)

    self.zoomOutButton = myPushButtonFunny(self)
    self.zoomOutButton.initPushButton(32, 32, 5, 5, "zoomOutButton", "", QApplication.translate("QSphere", "Zoom out"), True, getThemeIcon(self.iconeSizeDown), 32, 32, True)
    self.zoomOutButton.setAutoRepeat(True)
    self.zoomOutButton.setShortcut(Qt.Key_Minus)

    self.movieCtrl = movieCtrl(self, "sablier.gif")

    self.barInfo = QgsMessageBar(self)
    self.barInfo.setSizePolicy(QSizePolicy.Minimum, QSizePolicy.Fixed ) 

    self.xmlTags.selectionModel().currentRowChanged.connect(self.whatItem)
    self.findprevious.clicked.connect(self.findPrevious)
    self.searchbutton.clicked.connect(self.find)
    self.findnext.clicked.connect(self.findNext)
    self.zoomInButton.clicked.connect(self.editor.zoomIn)
    self.zoomOutButton.clicked.connect(self.editor.zoomOut)
    self.loadButton.clicked.connect(self.loadTheFile)
    self.printToolButton.clicked.connect(self.printToolTip)
    self.printButton.clicked.connect(self.printFile)
    self.saveButton.clicked.connect(self.saveChanges)
    self.closeButton.clicked.connect(self.close)

    self.splitter.splitterMoved.connect(self.updateSizeEditor)

    self.contextMnuSaveActions()

    self.setWindowState(Qt.WindowMaximized)

    self.findprevious.setEnabled(False)
    self.findnext.setEnabled(False)

    if fichier == "" : return
    if not self.checkProperties():
            self.badIso()
            return

    self.installEventFilter(self)
    
  

  def eventFilter(self, obj, event):
     if not event : return False
     if event == None : return False
     if event.type() == QEvent.WindowActivate : getOptions(self)
     return QDialog.eventFilter(self, obj, event) 

  def resizeEvent(self, event):
     
     self.frame.setGeometry(5, 5, self.width()-10 , self.height()-40)

     self.updateSizeEditor()
    
     self.movieCtrl.setGeometry(int(self.width()/2)-64, int(self.height()/2)-64, 64, 64)

     self.zoomInButton.setGeometry(self.width()-580, self.height()-40, 32, 32)
     self.zoomOutButton.setGeometry(self.width()-540, self.height()-40, 32, 32)
     self.barInfo.setGeometry(0, 0, self.width(), 90) 

     self.findprevious.setGeometry(self.width()-430, self.height()-30, 48, 25)
     self.searchbutton.setGeometry(self.width()-380, self.height()-30, 25, 25)
     self.findnext.setGeometry(self.width()-350, self.height()-30, 48, 25)
     
     self.loadButton.setGeometry(self.width()-300, self.height()-30, 25, 25)
     self.printButton.setGeometry(self.width()-260, self.height()-30, 25, 25)
     self.saveButton.setGeometry(self.width()-220, self.height()-30, 25, 25)

     self.actionsSaveButton.setGeometry(self.width()-190, self.height()-30, 56, 25)     
     self.closeButton.setGeometry(self.width()-120, self.height()-30, 100, 25)


  def updateSizeEditor(self):
     self.xmlTags.setGeometry(10, 10, self.groupBoxTags.width()-20, self.groupBoxTags.height()-265)
     self.xmlTags.setColumnWidth(0, 200)
     self.xmlTags.horizontalHeader().setDefaultSectionSize(200)

     self.valueElt.setGeometry(12, self.groupBoxTags.height()-250, self.groupBoxTags.width()-20, 198)
     self.printToolButton.setGeometry(self.valueElt.width()-50, self.valueElt.height()-30, 48, 24)
     
     self.editor.setGeometry(10, 10, self.groupBoxEditor.width()-20, self.groupBoxEditor.height()-60)
     self.editor.barInfo.setGeometry(0, 0, self.editor.width()-10, 90)

  def close(self): self.reject()

  def contextMnuSaveActions(self):
        contextMenu = QMenu()

        menuIcon = getThemeIcon("saveas.png")
        text = QApplication.translate("QSphere","Save as ...")
        self.saveAsButton = QAction(QIcon(menuIcon), text, self)
        self.saveAsButton.setObjectName("saveAsButton")
        self.saveAsButton.setShortcut(QKeySequence("Ctrl+Shift+S"))
        contextMenu.addAction(self.saveAsButton)
        self.saveAsButton.triggered.connect(self.saveCopyAs)

        contextMenu.addSeparator()
        
        menuIcon = getThemeIcon("savecopy.png")
        text = QApplication.translate("QSphere","Save as a copy ...")
        self.saveCopyAsButton = QAction(QIcon(menuIcon), text, self)
        self.saveCopyAsButton.setObjectName("saveCopyAsButton")
        self.saveCopyAsButton.setShortcut(QKeySequence("Ctrl+Shift+C"))
        contextMenu.addAction(self.saveCopyAsButton)
        self.saveCopyAsButton.triggered.connect(self.saveCopyAs)

        self.actionsSaveButton.setMenu(contextMenu)


  def enableFindPrevNextButton(self):
        self.findprevious.setEnabled(not self.editor.find_text=="")
        self.findnext.setEnabled(not self.editor.find_text=="")

  def find(self):
        ret = self.editor.findDialog.exec_()
        self.editor.findDialog.lineEdit.setFocus()
        if self.editor.findDialog.ret > 0 :
          self.editor.find_text = self.editor.findDialog.getFindText()
          self.enableFindPrevNextButton()
  
  def findNext(self):
        line, index = self.editor.getCursorPosition()
        bfind = self.editor.findFirst(self.editor.find_text, True, self.editor.findDialog.isCaseSensitive(), self.editor.findDialog.isWholeWord(), True, True, line, index)
        self.isBfind(bfind)
        self.enableFindPrevNextButton()

  def findPrevious(self):
        line, index = self.editor.getCursorPosition()
        index -= len(self.editor.find_text)
        bfind = self.editor.findFirst(self.editor.find_text, True, self.editor.findDialog.isCaseSensitive(), self.editor.findDialog.isWholeWord(), True, False, line, index)
        self.isBfind(bfind)
        self.enableFindPrevNextButton()

  def isBfind(self, bfind):          
      if not bfind:
         title = QApplication.translate("QSphere", "Information")
         msg = QApplication.translate("QSphere","Not found")
         sendMsg(self, title , "%s :<br><u><i>%s</i></ul>" % (msg, self.editor.find_text), Qgis.Info, self.duration_info)
         
  def whatItem(self, newIndex, oldIndex): self.setEltValue(newIndex)
      
  def setEltValue(self, index):
      try :
          if self.isoParser == None : return
      except : return
      
      if index.row()==-1:return    
      key = index.sibling(index.row(), 0).data()
            
      if dicoHasKey(self.widgetValues, key) :
         if type(self.widgetValues[key])== tuple :
            values = [item for item in self.widgetValues[key] if item!= None]
            self.valueElt.setPlainText("%s" % (values))
         else : self.valueElt.setPlainText("%s" % (self.widgetValues[key]))

         if key.find("groups:") != -1: key = key.replace("groups:", "")
         if key.find(":") != -1: key = key.split(":")[0]
         properties, counter = "", 0
         for elt in ("","ax_", "bx_"):
             try :
                 evaluate = eval("self.isoParser.isoModel.%s%s()" % (elt, key))
                 strEvaluate = str(evaluate)
             except : evaluate, strEvaluate = None, ""
             text = QApplication.translate("QSphere", "Alternative mode") if properties!="" else QApplication.translate("QSphere", "First mode")
             text = "<br><i><b><u>%s %s</u></b></i> : <br>" % (text, "" if counter == 0 else counter)
             counter+= 1
             properties+= "%s<br>%s<br>" % (text, textwrapfill(strEvaluate, 100))
         toolTip = "<table bgcolor='#ffffcc'><tr><td><h3>%s </h3><h2>[<b><font color='#0000ff'>%s</b></font>]</h2>%s</td></tr><tr><td><br><hr color='#000099'></td></tr><tr><td><u>%s</u><br><br>%s<br><br></td></tr></table>" % (QApplication.translate("QSphere", "Content for the item."), key, properties,
                                                                                                                                                                                                                                   QApplication.translate("QSphere","Present Value : "), textwrapfill(self.valueElt.toPlainText(), 100))
         self.valueElt.setToolTip(toolTip)

  def loadTheFile(self):
      title = QApplication.translate("QSphere","View a file (XML ISO 19139, HTML ...)")
      if self.InitDir == "" : self.InitDir = dirname(__file__)
      dlgFile = QFileDialog(self, title)
      elt0 = QApplication.translate("QSphere","files").title()
      dlgFile.setNameFilters(("%s eXtensible Markup Language (*.xml *.XML)" % (elt0), )) 
      dlgFile.setViewMode(QFileDialog.Detail)
      dlgFile.setDirectory(self.InitDir)
      dlgFile.setFileMode(QFileDialog.ExistingFile) 
      dlgFile.setAcceptMode(QFileDialog.AcceptOpen)

      fixeLabelsFileDialog(self, dlgFile, 0, True)
      
      if dlgFile.exec_():
          fileName = "%s" % (dlgFile.selectedFiles()[0])
          if fileName!="" :
              self.fichier = fileName
              self.setWindowTitle("%s : %s" % (self.rootWindowTitle, fileName))
              self.loadFile()
              self.InitDir = dirname(fileName)
              if not self.checkProperties():
                    self.badIso()
                    return
              
  def badIso(self):
      if not self.silentMode :
          msg = QApplication.translate("QSphere","Invalid XML document !")
          sendMsg(self, "information" , msg, Qgis.Warning, self.duration_warning)  
    
  def checkProperties(self):
    self.isoParser = None
    self.valueElt.setPlainText("")
    dataXml = self.fichier if self.fichier != "" else (self.editor.text())
    test, data = testXmlData(self, dataXml, False) 
    if test :
       if self.fichier != "": self.isoParser = xmlIsoParser(dataXml, None, 'MEDDE', 'fr')
       else : self.isoParser = xmlIsoParser(None, dataXml, 'MEDDE', 'fr')

       if self.isoParser == None :
          self.badIso()
          self.majAllFalse()
          return False    
       self.isoParser.createISOdataStructure(True)
       if self.isoParser :
           try : cond = self.isoParser.getTagDictionnary()
           except : cond = True
           self.majCheckInfos(cond)
           self.setEltValue(self.xmlTags.currentIndex())
           return True
       else :
           self.majAllFalse()
           return False 

  def majCheckInfos(self, cond):
    if self.isoParser == None : return
    self.isoParser.createISOdataStructure(True)
    if cond :
        self.widgetValues = {"intitule": self.isoParser.title , "resume" : self.isoParser.abstract, "typedata": self.isoParser.typedata, "tablelocalisator" : self.isoParser.localisators,
                         "identificator" : self.isoParser.UUID, "tablelangues" : self.isoParser.languesjdd, "tableformats" : self.isoParser.formatsjdd, "tablecarac": self.isoParser.tablecaracjdd,
                         "tablecategories" : self.isoParser.categories, "tablecategories:0" : self.isoParser.codecategories, "tablemotsclefsf" : self.isoParser.keywordsF,
                         "tablescr" : self.isoParser.scr, "tableemprises" : self.isoParser.boundingboxcoordinates,
                         "tableetenduetemporelle" : self.isoParser.tableetenduetemporelle, "groups:dates" : self.isoParser.dates,
                         "genealogie" : self.isoParser.genealogie, "coherence" : self.isoParser.coherence, "grouperesolutionscale" : self.isoParser.scalesEC,
                         "tablespecifications": self.isoParser.conformities, "groupedroits" : self.isoParser.accessconstraints, "licence" : self.isoParser.legalconstraints,
                         "tableroles" : self.isoParser.pointsofcontact,
                         "datemetada" : (self.isoParser.datemdd, self.isoParser.datetmdd ), "langmetada" : self.isoParser.languemdd
                         }
    else : self.widgetValues = {"tableroles:2": self.isoParser.pointsofcontact}
     
    for row in range(len(self.orderKeys)):  
        key = self.orderKeys[row]
        if dicoHasKey(self.widgetValues, key) :
            if self.widgetValues[key]!=[[]] :
              self.xmlTags.model().item(row, 0).setCheckState(Qt.Checked)
              self.xmlTags.model().item(row, 0).setEnabled(True)          
            else :
              self.xmlTags.model().item(row, 0).setCheckState(Qt.Unchecked)
              self.xmlTags.model().item(row, 0).setEnabled(False)
        else :              
              self.xmlTags.model().item(row, 0).setCheckState(Qt.Unchecked)
              self.xmlTags.model().item(row, 0).setEnabled(False)    

  def majAllFalse(self):
      for row in range(self.xmlTags.model().rowCount()):
          self.xmlTags.model().item(row, 0).setCheckState(Qt.Unchecked)
          self.xmlTags.model().item(row, 0).setEnabled(False)
          
  def loadFile(self):
     if self.fichier != "" :
        self.movieCtrl.startMovie()
        encoding = getEncodingCar(self.fichier, None)
        try : fileInput = codecsopen(self.fichier,'r', encoding, 'replace')
        except : fileInput = codecsopen(self.fichier,'r')
        self.editor.setText(fileInput.read())
        if fileInput : fileInput.close()
        self.movieCtrl.stopMovie()
     else : self.editor.setText(self.fichier)   


  def printToolTip(self):
        editor = QTextBrowser()
        editor.setText(self.valueElt.toolTip())
        dlgPrint = printerSphere(self, editor, QApplication.translate("QSphere", "Print ToolTip"), QPrinter.Landscape)

  def printFile(self):
        editor = QTextEdit()
        editor.setAcceptRichText(True)
        editor.setPlainText(self.editor.text())
        dlgPrint = printerSphere(self, editor, QApplication.translate("QSphere", "Print current file"), QPrinter.Landscape)

  def saveCopyAs(self):
        if self.editor.text()!= "" :
            elt0 = QApplication.translate("QSphere","files").title()
            self.extensions = ("%s TXT (*.txt)" % (elt0), "%s eXtensible Markup Language (*.xml)" % (elt0), "%s (*.*)" % (elt0))
            self.extensionsNames = {"txt": "%s TXT (*.txt)" % (elt0), "xml" : "%s eXtensible Markup Language (*.xml)" % (elt0), "all" : "%s (*.*)" % (elt0)}

            if self.sender() == self.saveAsButton : title = QApplication.translate("QSphere","Save as a file")
            else : title = QApplication.translate("QSphere","Save as a copy ...")
            
            dlgFile = QFileDialog(self, title)
            dlgFile.setNameFilters(self.extensions)
            dlgFile.selectNameFilter(self.extensionsNames["xml"])
            dlgFile.setViewMode(QFileDialog.Detail)
            dlgFile.setDirectory(self.InitDir)
            dlgFile.setAcceptMode(QFileDialog.AcceptSave)
            dlgFile.selectFile("")

            fixeLabelsFileDialog(self, dlgFile, 1, True)
            
            if dlgFile.exec_():
                fichier = fileNameWithExtension(self, dlgFile.selectedFiles()[0], dlgFile.selectedNameFilter())
                with codecsopen(fichier,'w','utf8') as f: 
                  f.write( self.editor.text() )
                if f!= None : f.close()

                if self.sender()== self.saveAsButton : 
                    self.fichier = fichier
                    self.saveChanges()
                    self.setWindowTitle("%s : %s" % (self.rootWindowTitle, self.fichier))
                    self.InitDir = dirname(self.fichier)

                else :
                    title = QApplication.translate("QSphere", "Information")
                    msg = QApplication.translate("QSphere","The file xml was saved as")
                    sendMsg(self, title , "%s :<br><u><i>%s</i></ul>" % (msg, fichier), Qgis.Info, self.duration_info)
            

  def saveChanges(self):
    if self.editor.text()== "" :  return
    result = False
    
    if self.fichier!= "" :
        with codecsopen(self.fichier,'w','utf8') as f: 
          f.write( self.editor.text() )
        if f!= None : f.close()  
        if not self.checkProperties():
            self.badIso()
            title = QApplication.translate("QSphere", "Warning")
            msg = "%s :<br><u><i>%s</i></ul><br>%s : XML." % (QApplication.translate("QSphere","The file xml was saved as"),
                                                              self.fichier, QApplication.translate("QSphere","Error on unsupported content :"))
            sendMsg(self, title , msg, Qgis.Warning, self.duration_warning)
            self.xmlTags.setVisible(True)

        
        if self.parent!=None :
             if self.parent.objectName()== "dlgMetadata" :
                self.parent.listImportCategories = []
                result = self.parent.reloadDataFromXml(self.isoParser, self.fichier, 0)
                    
             elif self.parent.objectName()== "qsphereMainWindow" :
                   dlgMetadata = None
                   if self.parent.mdi.subWindowList(QMdiArea.CreationOrder)!= [] :
                      self.parent.cascadeWindows()
                      dialog = self.parent.mdi.activeSubWindow()
                      if dialog == None : return
                      from doUI import dlgMetadata
                      for child in dialog.children():
                           if type(child) == dlgMetadata :
                              dlgMetadata = child
                              break
                           
                      if dlgMetadata :
                         dlgMetadata.listImportCategories = []
                         result = dlgMetadata.reloadDataFromXml(self.isoParser, self.fichier, 0) 
                            
                   else :
                         title = QApplication.translate("QSphere", "Warning" ) 
                         msg = QApplication.translate("QSphere","No Windows")
                         icon = Qgis.Warning
                         duration = self.parent.duration_warning
                         sendMsg(self, title, "%s !" % (msg), icon, duration)
                         return               

             else : return
             
        title = QApplication.translate("QSphere", "Information")
        msg = QApplication.translate("QSphere","The metadata record was saved as")
        if self.parent!=None :
           if result :
              sendMsg(self.parent, title , "%s :<br><u><i>%s</i></ul>" % (msg, self.fichier), Qgis.Info, self.parent.duration_info)
              sendMsg(self, title , "%s :<br><u><i>%s</i></ul>" % (msg, self.fichier), Qgis.Info, self.duration_info)
        else : sendMsg(self, title , "%s :<br><u><i>%s</i></ul>" % (msg, self.fichier), Qgis.Info, self.duration_info)

    else :
        elt0 = QApplication.translate("QSphere","files").title()
        self.extensions = ("%s TXT (*.txt)" % (elt0), "%s eXtensible Markup Language (*.xml)" % (elt0), "%s (*.*)" % (elt0))
        self.extensionsNames = {"txt": "%s TXT (*.txt)" % (elt0), "xml" : "%s eXtensible Markup Language (*.xml)" % (elt0), "all" : "%s (*.*)" % (elt0)}
        title = QApplication.translate("QSphere","Save as a file")
        
        dlgFile = QFileDialog(self, title)
        dlgFile.setNameFilters(self.extensions)
        dlgFile.selectNameFilter(self.extensionsNames["xml"])
        dlgFile.setViewMode(QFileDialog.Detail)
        dlgFile.setDirectory(self.InitDir)
        dlgFile.setAcceptMode(QFileDialog.AcceptSave)
        dlgFile.selectFile("")

        fixeLabelsFileDialog(self, dlgFile, 1, True)
        
        if dlgFile.exec_():
            self.fichier = fileNameWithExtension(self, dlgFile.selectedFiles()[0], dlgFile.selectedNameFilter())
            self.saveChanges()
            self.setWindowTitle("%s : %s" % (self.rootWindowTitle, self.fichier))
            self.InitDir = dirname(self.fichier)

 

