from qgis.core import Qgis
from qgis.gui import (QgsCollapsibleGroupBox, QgsMessageBar)

from qgis.PyQt.QtCore import (QFileInfo, QSettings, QSize, Qt)
from qgis.PyQt.QtGui import (QFont, QIcon, QImage, QKeySequence, QMovie, QPixmap)
from qgis.PyQt.QtWidgets import (QAction, QApplication, QButtonGroup, QDockWidget, QFileDialog, QFrame, QGroupBox, QHeaderView,
                                 QHBoxLayout, QLabel, QLineEdit, QListWidget, QListWidgetItem, QMainWindow, QMdiArea,
                                 QMenu, QMessageBox, QProgressBar, QPushButton, QSizePolicy, QTabWidget,
                                 QTextEdit, QToolBar, QTreeWidget, QTreeWidgetItem, QWidget) 

from qsphere.cswt_connector import cswtConnection
from qsphere.ui import (dlgContacts, dlgMetadata, dlgSgbdr, loadDlgNavigator) 
from qsphere.qsphere_tools import (addLineWidget, appendNameSpace, convertToUtf8, countItems, dicoHasKey, fixeLabelsFileDialog, fixeClassObj,
                                   getComboLayerIndex, getExtension, getHelp, getKeyInDicoFromValue, getOptions, getSelConnexion,
                                   getThemeIcon, msgBadFileXml, sendMsg, testXmlData, textForMenu, windowIcon, windowProperties)
from qsphere.qsphere_objmaker import (dlgAuthCswt, dlgLustre, filesNavigatorWidget, loadFromLayersDialog, mdiArea, movieCtrl, 
                                      myPushButton, myMsgDlgBox, mySubWindow, roundDockWidget,
                                      toolSearchFilesDialog, windowPlotStatsAndGraphs)
from qsphere.importexport import (closeFile, saveXml, dateStamp)
from qsphere.ui_catalogue import makeLine
from qsphere.ui_editorxml import xmlEditor

from io import StringIO
from math import ceil as ceil 
from os.path import (basename, dirname, exists, splitext)
from textwrap import fill as textwrapfill
from urllib.parse import (quote, unquote)
from webbrowser import open as webbrowseropen
from xml.etree import ElementTree as ET

class myMdiWindow(QMainWindow):
        def __init__(self, parent): 
                QMainWindow.__init__(self) 

                self.setObjectName("qsphereMainWindow")
                self.setWindowFlags(Qt.WindowMaximizeButtonHint | Qt.WindowCloseButtonHint | Qt.WindowMinimizeButtonHint)
                self.setStyleSheet("QMdiSubWindow {color: white; background-color: qlineargradient(x1:0, y1:0, x2:0, y2:1, stop: 0 #cccccc, stop: 1 #5D5D5D);}")

                self.minWidth, self.minHeight = 1100, 800
                self.resize(self.minWidth, self.minHeight)

                self.dicoLabel = {}
                self.extension = "qsp"
                self.counterNew = 0
                self.currentSession = None

                windowProperties(self, parent)
                windowIcon(self, "qspheremainui.png")
                
                getOptions(self)
                self.rootWindowTitle = "QSphere %s" % self.version
                self.setWindowTitle(self.rootWindowTitle)                 
                self.mdi = mdiArea(self) 

                self.parent = parent
                self.tablesWidget = self.parent.tablesWidget
                
                self.qspherePlugin = self.parent.qspherePlugin
                self.connexionCswt = None
                self.webNavigator = None
                self.dialogLUSTRE = None
                self.dialogSearchFiles = None
                self.filesNavigatorWidget = None
                self.listSessions = []
                self.mddFiles = []
                self.dlgSgbdr = self.parent.dlgSgbdr     
 
                self.windowsNoMDIChilds = []

                self.movieCtrl = movieCtrl(self, "sablier.gif")

                self.listNameActionsTabEdition = [ "identificationTab", "classificationTab", "locationTab", "temporalityTab",
                                                   "qualityTab", "rights_and_usesTab", "responsibleTab", "metadataTab", "qgisTab"]

                self.listNameActionsTabWebServices = ["authCSWTButton", "closeCSWTButton", "reportingCSWTButton", "recordsCSWTButton",
                                                      "addCSWTButton", "editCSWTButton", "delCSWTButton"]

                self.styleSheetPushButton = "QPushButton::checked {icon-size: 48px, 48px; border-width: 5px; border-radius: 10px; border-color: black; background-color: #9B9B9B;}"
                self.styleSheetPushButton+= "QPushButton {icon-size: 48px, 48px; border-width: 0px; border-radius: 10px; }"

                self.InitDir = dirname(__file__)

                self.setCentralWidget(self.mdi)

                self.loadInfos("sessions", self.listSessions)
                self.loadInfos("fileshistory", self.mddFiles) 

                self.listActionsTabFile = [
                ("~" , "", "files.png", (10, 0, 70, 40), None, "", True),
                ("toolFiles", "View files navigator", "filesnavigator_true.png", (20, 5, 32, 32), self.getFilesNavigator, "", True),        
                ("~" , "Sessions", "sessions.png", (10, 0, 280, 40), None, "", True),              
                ("newSession", "New session", "new.png", (100, 5, 32, 32), self.newSession, "", True),       
                ("openSession", "Load a session QSphere", "open.png", (125, 5, 50, 32), "self.contextMnuSession(obj)", "", True),
                ("saveSession", "Save current session", "save.png", (180, 5, 32, 32), self.saveSession, "", False),
                ("saveSessionAs", "Actions for the sessions", "saveactions.png", (215, 5, 50, 32), "self.contextMnuSaveSessionActions(obj)", "", True),
                ("-", "-", "", (260, 10, 1, 40), None, "", True),
                ("~" , "Files", "files.png", (10, 0, 330, 40), None, "", True), 
                ("newFile", "New sheet", "new.png", (100, 5, 32, 32), self.newMetadataFile, "", True),
                ("openFile", "Load a metadata file", "open.png", (125, 5, 50, 32), "self.contextMnuFile(obj)", "", True),
                ("saveFile", "Save the current sheet QSphere", "save.png", (180, 5,  32, 32), self.saveMetadataFile, "Ctrl+S", False),
                ("saveFileAs", "Actions for the metadata...", "saveactions.png", (215, 5, 50, 32), "self.contextMnuSaveActions(obj)", "", False),
                ("saveAll", "Save all files", "saveall.png", (280, 5, 32, 32), self.saveAllMetadataFile, "", False),
                ("-", "-", "", (350, 10, 1, 40), None, "", True),
                ("~" , "Close", "files.png", (10, 0, 160, 40), None, "", True),
                ("close", "Close", "close.png", (100, 5, 32, 32), self.close, "ALT+F4", True)            
                ]
                
                self.changeMode = QButtonGroup(self) 
                self.changeMode.setExclusive(True)
                

                self.listActionsTabEdition = [
                ("~" , "Modes", "sessions.png", (10, 0, 230, 40), None, "", True),
                ("formMode", "Form view", "n_project.png", (100, 5, 32, 32), None, "", True),
                ("xmlMode", "rendering XML", "n_xml.png", (135, 5, 32, 32), None, "", True),
                ("htmlMode", "rendering HTML", "n_html.png", (170, 5, 32, 32), None, "", True),
                ("-", "-", "", (780, 10, 1, 40), None, "", True),
                ("~" , "Actions", "sessions.png", (10, 0, 280, 40), None, "", True),
                ("reloadWindow", "Reload the current metadata file", "reload.png", (100, 5, 32, 32), self.reloadWindow, "F5", True),
                ("cloneWindow", "Clone current window", "windowclone.png", (135, 5, 32, 32), self.cloneCurrentWindow, "", True),
                ("printTab", "Print current page", "print.png", (175, 5, 32, 32), self.printMetaTab, "Ctrl+P", True),
                ("radarStats", "Get statistics chart", "histogramchart.png", (215, 5, 32, 32), self.getRadarStats, "Ctrl+R", True),
                ("-", "-", "", (910, 10, 1, 40), None, "", True),
                ("~" , "Wizards", "sessions.png", (10, 0, 220, 40), None, "", True),
                ("wizForm", "Call the form wizard Tables ...", "wizform.png", (100, 5, 32, 32), self.openFormWizardMode, "", False),
                ("wizEditorXML", "Edit XML", "wizeditxml.png", (135, 5, 32, 32), self.openXmlEditorWizardMode, "", False),
                ("wizContacts", "Contacts manager ...", "wizcontacts.png", (170, 5, 32, 32), self.openContactsWizardMode, "", False)
                ]

                self.changeMode.buttonClicked[int].connect(self.changeMetadataMode)
                
                self.listActionsTabWebServices = [
                ("~" , "Web", "sessions.png", (10, 0, 170, 40), None, "", True),                    
                ("navigatorWeb", "Web navigator ...", "servers.png", (100, 5, 50, 32), "self.contextMenuWebViewer(obj)", "", True),
                ("-", "-", "", (65, 10, 1, 40), None, "", True),
                ("~" , "Lustre", "sessions.png", (10, 0, 170, 40), None, "", True), 
                ("openLustreWizard", "Call the LusTRE wizard ...", "lustre_wizard.png", (100, 5, 32, 32), self.openLustreWizard, "", True),
                ("-", "-", "", (130, 10, 1, 40), None, "", True),
                ("~" , "CSWT", "sessions.png", (10, 0, 420, 40), None, "", True),
                ("authCSWTButton", "Authentification for the CSW-T Server ...", "connect.png", (100, 5, 32, 32), self.authCSWT, "", True),
                ("closeCSWTButton", "Close the CSWT session", "disconnect.png", (140, 5, 32, 32), self.disconnectCSWT, "", False),
                ("reportingCSWTButton", "Get all reporting for CSWT session", "reporting.png", (190, 5, 32, 32), self.getAllReporting, "", False),
                ("recordsCSWTButton", "Get records in CSWT server", "records.png", (230, 5, 32, 32), self.recordsInCSWT, "", False),
                ("addCSWTButton", "Add to the CSW-T Server", "cloudadd.png", (280, 5, 32, 32), self.insertInCSWT, "", False),
                ("editCSWTButton", "Update to the CSW-T Server", "cloudupdate.png", (320, 5, 32, 32), self.updateInCSWT, "", False),
                ("delCSWTButton", "Delete to the CSW-T Server", "clouddel.png", (360, 5, 32, 32), self.deleteInCSWT, "", False)
                ]

               
                self.blocProgressBar = QLabel(self)    
                icon = QImage(getThemeIcon("backgroundprogressbar.png"))
                self.blocProgressBar.setPixmap(QPixmap.fromImage(icon))
                self.blocProgressBar.setVisible(False)

                self.blocProgressBarButton = myPushButton(self.blocProgressBar)
                self.blocProgressBarButton.initPushButton(40, 40, 20, 85, "blocProgressBarButton", "", "", True, getThemeIcon(""), 40, 40, True)            

                self.blocProgressBarText = QLabel(self.blocProgressBar)
                self.blocProgressBarText.setStyleSheet("QLabel { font-size: 10px; color : black; background-color: rgba(131, 131, 131, 125); border: 2px solid cyan;  border-radius: 4px;  padding: 2px;}")
                self.blocProgressBarText.setGeometry(5, 140, 390, 80)
                self.blocProgressBarText.setAlignment(Qt.AlignCenter)
                self.blocProgressBarText.setText("")

                self.progressBar = QProgressBar(self.blocProgressBar)
                self.progressBar.setProperty("value", 0)
                self.progressBar.setAlignment(Qt.AlignCenter)
                self.progressBar.setTextVisible(True)
                self.progressBar.setObjectName("progressBar")
                self.progressBar.setStyleSheet("QProgressBar {border: 2px solid grey; border-radius: 5px; text-align: center;}"
                                           "QProgressBar::chunk {background-color: #6C96C6; width: 20px;}")
                self.progressBar.setGeometry(70, 100, 260, 15)
                self.progressBar.setValue(0)

                self.listActionsTabWindows = [
                ("~" , "Organize", "sessions.png", (10, 0, 330, 40), None, "", True),
                ("windowsCount", "Sub mainwindow's", "windowscounts.png", (100, 5, 32, 32), None, "", True),
                ("tileWindows", "Tile windows", "windowstile.png", (190, 5, 32, 32), self.tileWindows, "", True),
                ("cascadeWindows", "Cascade windows", "windowscascade.png", (230, 5, 32, 32), self.cascadeWindows, "", True),
                ("batteriesWindows", "Batteries windows", "windowsbatteries.png", (270, 5, 32, 32), self.batteriesWindows, "", True),
                ("-", "-", "", (190, 10, 1, 40), None, "", True),
                ("~" , "Groups", "sessions.png", (10, 0, 190, 40), None, "", True), 
                ("expandAll", "Expand all", "expandall.png", (100, 5, 32, 32), self.expandReduceCollapsibleGroups, "", False),
                ("reduceAll", "Reduce all", "reduceall.png", (135, 5, 32, 32), self.expandReduceCollapsibleGroups, "", False),
                ("-", "-", "", (190, 10, 1, 40), None, "", True),
                ("~" , "Close", "sessions.png", (10, 0, 190, 40), None, "", True), 
                ("closeWindow", "Close active window", "windowclose.png", (100, 5, 32, 32), self.closeCurrentWindow, "", False),
                ("closeAllWindows", "Close all windows", "windowscloseall.png", (135, 5, 32, 32), self.closeAllMySubWindows, "", False)
                ]
                window = QMainWindow()

                self.barInfo = QgsMessageBar(self.mdi)
                self.barInfo.setSizePolicy(QSizePolicy.Minimum, QSizePolicy.Fixed)

                self.mdi.subWindowActivated.connect(self.fixeIndex)
                self.iface.mapCanvas().layersChanged.connect(self.updateOpenFromLayers)
                try : self.iface.mapCanvas().currentLayerChanged.connect(self.updateOpenFromLayers)
                except : pass
                self.iface.newProjectCreated.connect(self.updateOpenFromLayers)

                self.xnewsession = myPushButton(self)
                self.xnewsession.initPushButton(0, 0, 0, 0, "xnewsession", "", QApplication.translate("QSphere", "New session"), True, getThemeIcon("new.png"), 0, 0, False)
                self.xnewsession.setShortcut(QKeySequence("Ctrl+Shift+N"))
                self.xnewsession.clicked.connect(self.newSession)

                self.xopensession = myPushButton(self)
                self.xopensession.initPushButton(0, 0, 0, 0, "xopensession", "", QApplication.translate("QSphere", "Load a session QSphere"), True, getThemeIcon("new.png"), 0, 0, False)
                self.xopensession.setShortcut(QKeySequence("Ctrl+Shift+F"))
                self.xopensession.clicked.connect(self.openSession)

                self.xsavesession = myPushButton(self)
                self.xsavesession.initPushButton(0, 0, 0, 0, "xsavesession", "", QApplication.translate("QSphere", "Save current session"), True, getThemeIcon("save.png"), 0, 0, False)
                self.xsavesession.setShortcut(QKeySequence("Ctrl+Shift+W"))
                self.xsavesession.clicked.connect(self.saveSession)

                self.xnewfile = myPushButton(self)
                self.xnewfile.initPushButton(0, 0, 0, 0, "xnewfile", "", QApplication.translate("QSphere", "New sheet"), True, getThemeIcon("new.png"), 0, 0, False)
                self.xnewfile.setShortcut(QKeySequence("Ctrl+N"))
                self.xnewfile.clicked.connect(self.newMetadataFile)

                self.xopenfile = myPushButton(self)
                self.xopenfile.initPushButton(0, 0, 0, 0, "xopenfile", "", QApplication.translate("QSphere", "Load a metadata file"), True, getThemeIcon("open.png"), 0, 0, False)
                self.xopenfile.setShortcut(QKeySequence("Ctrl+F"))
                self.xopenfile.clicked.connect(self.openMetadataFile)

                self.xopenFromLayers = myPushButton(self)
                self.xopenFromLayers.initPushButton(0, 0, 0, 0, "xopenFromLayers", "", QApplication.translate("QSphere", "Load metadata file for layers"), True, getThemeIcon("openfromlayers.png"), 0, 0, False)
                self.xopenFromLayers.setShortcut(QKeySequence("Ctrl+L"))
                self.xopenFromLayers.clicked.connect(self.openMetadataFileForLayers)

                self.xopenFromDataBase = myPushButton(self)
                self.xopenFromDataBase.initPushButton(0, 0, 0, 0, "xopenFromDataBase", "", QApplication.translate("QSphere", "Load metadata file from database"), True, getThemeIcon("openfromdatabase.png"), 0, 0, False)
                self.xopenFromDataBase.setShortcut(QKeySequence("Ctrl+D"))
                self.xopenFromDataBase.clicked.connect(self.openMetadataFromBase)

                self.xsaveInDataBase = myPushButton(self)
                self.xsaveInDataBase.initPushButton(0, 0, 0, 0, "xsaveInDataBase", "", "%s ..." % QApplication.translate("QSphere","Save in database"), True, getThemeIcon("openfromdatabase.png"), 0, 0, False)
                self.xsaveInDataBase.setShortcut(QKeySequence("Ctrl+Shift+D"))
                self.xsaveInDataBase.clicked.connect(self.saveMetadataInBase)

                self.xsavefile = myPushButton(self)
                self.xsavefile.initPushButton(0, 0, 0, 0, "xsavefile", "", QApplication.translate("QSphere", "Save the current sheet QSphere"), True, getThemeIcon("save.png"), 0, 0, False)
                self.xsavefile.setShortcut(QKeySequence("Ctrl+S"))
                self.xsavefile.clicked.connect(self.saveMetadataFile)

                self.xsaveallfiles = myPushButton(self)
                self.xsaveallfiles.initPushButton(0, 0, 0, 0, "xsaveallfiles", "", QApplication.translate("QSphere", "Save all files"), True, getThemeIcon("saveall.png"), 0, 0, False)
                self.xsaveallfiles.setShortcut(QKeySequence("Ctrl+A"))
                self.xsaveallfiles.clicked.connect(self.saveAllMetadataFile)

                self.updateOpenFromLayers()

                self.myDock = roundDockWidget(self)
                self.myDock.show()
                
                self.changeStateButtons(None)
                self.translate(False)

        def translate(self, allItems):

            self.dicoLangs = self.parent.dicoLangs
            self.indexLang = self.parent.indexLang
                
            elt0 = QApplication.translate("QSphere","project").title()
            elt1 = QApplication.translate("QSphere","metadata")
            elt2 = QApplication.translate("QSphere","session")

            self.extensionsNames = {"qsp": "%s QSphere (*.qsp)" % elt0, "xml" : "eXtensible Markup Language %s (*.xml)" %  elt1,
                                    "qmd" : "QGis %s (*.qmd)" % elt1, "qss" : "%s QSphere (*.qss)" % (elt2)}

            if not allItems : return

            nameTabs = ("Resources", "Edition", "Web and Services", "Windows", "Tools and Params")

            listToTranslate = (self.listActionsTabFile, self.listActionsTabEdition, self.listActionsTabWebServices, self.listActionsTabWindows)
            for listItems in listToTranslate :
                for item in listItems :
                    if item[0] != "-" :
                       if item[0] == "~" :
                          obj = self.findChild(QLabel, "Label%s_%s" % (item[1], item[2].split(".")[0]))
                          if obj : obj.setText("%s :" % QApplication.translate("QSphere", item[1]))
                       else :
                          obj = self.findChild(myPushButton, item[0])
                          if obj :
                                  text = QApplication.translate("QSphere",item[1])
                                  if item[0] == "newSession" : text = "%s%s" % (text, "Ctrl+Shift+N".rjust(60-len(text)))
                                  if item[0] == "openSession" : text = "%s%s" % (text, "Ctrl+Shift+F".rjust(60-len(text)))
                                  if item[0] == "saveSession" : text = "%s%s" % (text, "Ctrl+Shift+W".rjust(60-len(text)))
                                  if item[0] == "newFile" : text = "%s%s" % (text, "Ctrl+N".rjust(60-len(text)))
                                  if item[0] == "openFile" : text = "%s%s" % (text, "Ctrl+F".rjust(60-len(text)))
                                  if item[0] == "saveFile" : text = "%s%s" % (text, "Ctrl+S".rjust(60-len(text)))
                                  if item[0] == "saveAll" : text = "%s%s" % (text, "Ctrl+A".rjust(60-len(text)))
                                  obj.setToolTip("%s" % QApplication.translate("QSphere", text))
                          else :
                             obj = self.findChild(QLabel, item[0])
                             if obj :
                                text = QApplication.translate("QSphere",item[1])
                                obj.setToolTip(text)
 

            obj = self.findChild(myPushButton, "openSession")    
            if obj : self.contextMnuSession(obj)

            obj = self.findChild(myPushButton, "saveSessionAs")
            if obj : self.contextMnuSaveSessionActions(obj)

            obj = self.findChild(myPushButton, "openFile")
            if obj : self.contextMnuFile(obj)

            obj = self.findChild(myPushButton, "saveFileAs")
            if obj : self.contextMnuSaveActions(obj)

            obj = self.findChild(myPushButton, "navigatorWeb")
            if obj : self.contextMenuWebViewer(obj)

  
        def getFilesNavigator(self):
            if not self.filesNavigatorWidget :
                    child = filesNavigatorWidget(self)
                    self.filesNavigatorWidget = mySubWindow(self, child, QIcon(getThemeIcon("files.png")))
                    self.filesNavigatorWidget.setWindowTitle(QApplication.translate("QSphere","Files navigator"))
                    self.filesNavigatorWidget.setWindowFlags(Qt.Window | Qt.WindowMinimizeButtonHint | Qt.WindowMaximizeButtonHint)
                    self.mdi.addSubWindow(self.filesNavigatorWidget)
                    self.mdi.setActiveSubWindow(self.filesNavigatorWidget)
                    self.filesNavigatorWidget.show()
            else : self.filesNavigatorWidget.setVisible(not self.filesNavigatorWidget.isVisible())
            

        def expandReduceCollapsibleGroups(self):
            dialog, child = self.metadataWindow()
            if child :
                    cond = False if self.sender().objectName() == "expandAll" else True
                    for mddcollapsibleGroupBox in child.mddcollapsibleGroupBox:  mddcollapsibleGroupBox.setCollapsed(cond)

        def metadataWindow(self):
            dialog = None
            if self.mdi.subWindowList(QMdiArea.CreationOrder)!= [] : 
               dialog = self.mdi.activeSubWindow()
               if not dialog : 
                   dialog = self.mdi.subWindowList(QMdiArea.CreationOrder)[0]
                   self.mdi.setActiveSubWindow(dialog)
               if dialog != None :
                  child = dialog.children()[3]
                  if type(child) == dlgMetadata : return dialog, child
                  else : return dialog, None
            return dialog, None    

        def sendMail(self):
            QMessageBox.information(self, "QSphere", "%s :<br><h2>%s</h2><br>" % (QApplication.translate("QSphere","Send a mail to developers"), "pegasus2167@hotmail.fr"))

        def openHelp(self): getHelp(self)
        def helpInspire(self):
            try : webbrowseropen("http://cnig.gouv.fr/?page_id=2916")
            except : pass

        def loadNavigator(self, fileName):
                if self.webNavigator == None :
                        self.webNavigator = loadDlgNavigator(self, self.iface, fileName, False, [], self.localLangage, self, None, "navigatorweb.png", False)
                        self.windowsNoMDIChilds.append(self.webNavigator)
                else :
                        self.webNavigator.addNewTab()
                        self.webNavigator.endLoadTheFile(fileName)
                self.webNavigator.show()
                self.webNavigator.activateWindow()

        def changeMetadataMode(self):
            dialog, child = self.metadataWindow()
            if child :
               idbutton = self.changeMode.id(self.changeMode.checkedButton())
               child.doChangeMode(idbutton)
               child.histoIDChangeMode = idbutton


        def reloadWindow(self):
            dialog, child = self.metadataWindow()
            if child :               
                      self.blocProgressBarButton.setIcon(self.sender().icon())
                      self.blocProgressBarText.setText(textwrapfill("%s :\n %s" % (self.sender().toolTip(), child.windowTitle()), 50))
                      self.blocProgressBar.setVisible(True)     
                      child.reLoadMetadata()
                      self.blocProgressBar.setVisible(False)

        def updateOpenFromLayers(self):
            but = self.findChild(myPushButton, "openFromLayers")
            if but : but.setEnabled(self.iface.mapCanvas().layerCount() > 0)
                
        def fixeIndex(self):
            dialog, child = self.metadataWindow()
            if child :
               self.majCountersSubWindows() 
               self.nb_subwindows.setAlignment(Qt.AlignCenter)
               self.fixeModeDialog(child.histoIDChangeMode)
               if child.indexLang != self.qspherePlugin.indexLang : child.changeLang()
            self.changeStateButtons(child)   

        def majCountersSubWindows(self):
            nb = len(self.mdi.subWindowList(QMdiArea.CreationOrder))
            if self.filesNavigatorWidget : nb-= 1
            self.nb_subwindows.setText("%s" % nb)

        def changeStateButtons(self, child):
            if self.dlgSgbdr :
               self.dlgSgbdr.reloadTabWidget()
               self.dlgSgbdr.tabsResults.setTabEnabled(3, self.dlgSgbdr.isCond())
               self.dlgSgbdr.changedMddFiles = True ; self.dlgSgbdr.loadListMddFiles();


            controlButtonsEnabled = {"saveSession" : 0, "saveFile" : 3, "saveFileAs" : 1, "saveAll" : 1, "reloadWindow" : 3,
                                     "formMode" : 1, "xmlMode" : 1, "htmlMode" : 1,
                                     "cloneWindow" : 1, "printTab" : 1, "radarStats" : 1, "wizForm" : 1, "wizEditorXML" :  1,
                                     "wizContacts" : 1, "addCSWTButton" : 2, "editCSWTButton" : 2, "delCSWTButton" : 2,
                                     "tileWindows" : 1, "cascadeWindows" : 1, "batteriesWindows" : 1,
                                     "expandAll" : 1, "reduceAll" : 1, "closeWindow" : 1, "closeAllWindows" : 1}
            
            borne = len(self.mdi.subWindowList(QMdiArea.CreationOrder)) 
            for key in controlButtonsEnabled :
                but = self.myDock.findChild(myPushButton, key)         
                if but :
                      if controlButtonsEnabled[key] == 0: but.setEnabled(self.windowTitle() != self.rootWindowTitle)  
                      elif controlButtonsEnabled[key] == 1: but.setEnabled(child != None and borne > 0)
                      elif controlButtonsEnabled[key] == 2: but.setEnabled(child != None and borne > 0 and self.connexionCswt != None)
                      else :
                           if child : but.setEnabled(child.windowTitle() != child.rootWindowTitle and child.windowTitle().find("~")== -1)
                           else : but.setEnabled(False)

                           

        def fixeModeDialog(self, idbutton):
            nameObj = ("formMode", "xmlMode", "htmlMode")[idbutton]
            obj = self.myDock.findChild(myPushButton, nameObj)
            if obj :
                    obj.blockSignals(True)    
                    obj.setChecked(True)
                    obj.blockSignals(False)    

        def printMetaTab(self):
            dialog, child = self.metadataWindow()
            if child : child.printMetadata()


        def getRadarStats(self):
            dialog, child = self.metadataWindow()
            if child :
                listNameAxes = ("",
                                QApplication.translate("QSphere","Identification"),
                                QApplication.translate("QSphere","Classification & Keywords"),
                                QApplication.translate("QSphere","Geographic"),
                                QApplication.translate("QSphere","Temporal"),
                                QApplication.translate("QSphere","Quality & validity"),
                                QApplication.translate("QSphere", "Constraints"),
                                QApplication.translate("QSphere","Responsible party"),
                                QApplication.translate("QSphere", "Metadata"))
                data = child.analyzeData()
                self.myDock.setVisible(False)                                
                self.dlgStats = windowPlotStatsAndGraphs(child, "bar", data, listNameAxes, 1.0) 
                self.dlgStats.exec_()
                self.myDock.setVisible(True)

             
            

        #========================
        # BLOC ACTIONS FILES MDD
        #========================                
        def initMetadataDialog(self): 
            self.blocProgressBar.setVisible(True)
            self.blocProgressBar.repaint()

            try : icon = self.sender().icon()
            except :
               try : icon = self.sender().currentItem().icon()
               except : icon = QIcon(getThemeIcon("sgbdr/metadatadownload.png"))
            self.blocProgressBarButton.setIcon(icon)
            
            try : self.blocProgressBarText.setText(textwrapfill("%s" % self.sender().toolTip(), 50))
            except :
                try : self.blocProgressBarText.setText(textwrapfill("%s" % self.sender().mainFrame().url().toString(),50))
                except : self.blocProgressBarText.setText("")

            child =  dlgMetadata(self.iface, self.langue, self.languageIndex, self.langs, self.languesDico,
                                    self.localLangage, self.formats, self.listCodecs, self.listTemporalSystem,
                                    self.listTypeRessources, self.listCountries, self.indexCountry, self.listCountriesCode, self.localeFullName, self)
            child.setWindowTitle("%s ~ %s" % (child.windowTitle(), self.counterNew))
           
            mdiWindow = mySubWindow(self, child, QIcon(getThemeIcon("openmeta.png")))
            self.mdi.addSubWindow(mdiWindow)
            self.mdi.setActiveSubWindow(mdiWindow)
            self.blocProgressBar.setVisible(False)
            return mdiWindow, child

        def newSession(self):
            self.currentSession = None
            self.setWindowTitle(self.rootWindowTitle)
            self.butsavesession.setEnabled(self.currentSession != None)
            self.closeAllMySubWindows()

        def openLastSession(self) :
            icon = self.sender().icon()
            text = QApplication.translate("QSphere","Load a session file")
            if self.sender().text()!= "" :
                    textSender = self.sender().text()
                    result = self.loadTheSession(textSender, icon, text)
                    if not result :
                            self.listSessions.remove(textSender)
                            self.ajustListInfos("", self.listSessions)

        def openLastFile(self):
            icon = self.sender().icon()
            text = QApplication.translate("QSphere","Load a session file")
            fileName = self.sender().text()

            if exists(fileName) :
                inputFile = QFileInfo(fileName)
                extension = inputFile.suffix().lower()    
                test, data = testXmlData(self, fileName, False) if extension == "xml" else True, None     
                if test :     
                        self.loadMetadataFile(fileName, False, -1)     
                        self.InitDir = dirname(fileName)
                else :
                        self.mddFiles.remove(fileName)
                        self.ajustListInfos("", self.mddFiles)                        
            else :   
                self.mddFiles.remove(fileName)
                self.ajustListInfos("", self.mddFiles)

        def openSession(self):
            extension = "qss"
            icon = self.sender().icon()
           
            text = QApplication.translate("QSphere","Load a session file")
            dlgFile = QFileDialog(self, text)
            dlgFile.setNameFilters([items[1] for items in self.extensionsNames.items() if items[0]==extension]) 
            dlgFile.selectNameFilter(self.extensionsNames[extension])
            dlgFile.setDefaultSuffix(extension)
            dlgFile.setViewMode(QFileDialog.Detail)
            dlgFile.setDirectory(self.InitDir)
            dlgFile.setFileMode(QFileDialog.ExistingFile)  
            dlgFile.setAcceptMode(QFileDialog.AcceptOpen)

            fixeLabelsFileDialog(self, dlgFile, 0, True)
            self.myDock.setVisible(False)    
            if dlgFile.exec_():
               fileName = "%s" % (dlgFile.selectedFiles()[0])
               if fileName!="" :
                  result = self.loadTheSession(fileName, icon, text)
                  if result : self.ajustListInfos(fileName, self.listSessions)
            self.myDock.setVisible(True)      


        def loadTheSession(self, fileName, icon, text):       
                try :
                        self.tree = ET.parse(fileName)
                except :
                        msgBadFileXml(self)
                        return False
                self.counterNew = 0
                child = None
                self.currentSession = fileName
                self.setWindowTitle("%s : %s" % (self.rootWindowTitle, fileName))
                self.InitDir = dirname(fileName)

                self.blocProgressBarButton.setIcon(icon)
                self.blocProgressBarText.setText(textwrapfill("%s :\n %s" % (text,fileName), 50))
                self.blocProgressBar.setVisible(True)  

                self.closeAllMySubWindows()
                
                self.root = self.tree.getroot()
                subWindows = self.root.findall("subWindow")
                dialogFocused = None
                counterTemp = 0

                for node in subWindows :
                      dicoSubWindow = self.parse_subWindow(node)
                      try : source = QApplication.translate("QSphere", dicoSubWindow["source"])
                      except :
                             self.blocProgressBar.setVisible(False) 
                             msgBadFileXml(self)
                             return False


                      if source !="" and source :
                              inputFile = QFileInfo(fileName)
                              extension = inputFile.suffix().lower()
                              test, data = testXmlData(self, fileName, False) if extension == "xml" else True, None
                              if test : mdiWindow, child = self.loadMetadataFile(source, False, -1)
                              else :
                                   child = None   
                                   pass    
                              
                      else : mdiWindow, child = self.newMetadataFile()

                      if not source :
                         dataQSP = dicoSubWindow["projecttemp"]

                         from io import StringIO as bufferData
                         from qsphere.xmlisoparser import xmlIsoParser
                         buffer = bufferData(dataQSP) 
                         isoParser = xmlIsoParser(None, buffer , 'MEDDE', 'fr') 
                         isoParser.createISOdataStructure(True)

                         try :       
                            if isoParser.getTagDictionnary() :
                              child.reloadDataFromXml(isoParser, None, 0)
                              child.setWindowTitle("%s ~ %s" % (child.windowTitle(), counterTemp))
                              counterTemp+= 1
                         except :
                            try : result = child.loadQsp(None, dataQSP)
                            except : pass
                      
                      if child :
                         posx = dicoSubWindow["posx"] 
                         posy = dicoSubWindow["posy"] 
                         width = dicoSubWindow["width"]
                         height = dicoSubWindow["height"]
                         visible = dicoSubWindow["visible"]       

                         try : mdiWindow.setGeometry(int(posx), int(posy), int(width), int(height))
                         except : pass                           
                         if dicoSubWindow["activate"]=="True" : dialogFocused = mdiWindow

                         try :
                               child.ComboLayers.setCurrentIndex(int(dicoSubWindow["ComboLayers"]))
                               child.primaryXslt.setCurrentIndex(int(dicoSubWindow["PrimaryXSLT"]))  
                         except :
                                 pass
                         
                         try :        
                               idbutton = int(dicoSubWindow["histoIDChangeMode"])
                               child.doChangeMode(idbutton)
                               child.histoIDChangeMode = idbutton
                         except : pass
                         
                if dialogFocused :       
                   self.mdi.setActiveSubWindow(dialogFocused)      
                   dialogFocused.setFocus()
                self.butsavesession.setEnabled(self.currentSession != None)
                self.blocProgressBar.setVisible(False)
                self.fixeIndex()

                return True
                  
        def parse_subWindow(self, node):
            dicoSubWindow = {}
            keys = ("title", "source", "posx", "posy", "width", "height", "ComboLayers", "PrimaryXSLT", "activate", "visible", "projecttemp",  "histoIDChangeMode")
            for key in keys :
                try : dicoSubWindow[key] = node.find(key).text
                except :
                       if key in ("title", "source", "activate", "projecttemp") : dicoSubWindow[key] = ""
                       elif key in ("posx", "posy", "width", "height", "ComboLayers", "PrimaryXSLT", "histoIDChangeMode"): dicoSubWindow[key] = 0 
                       elif key == "activate" : dicoSubWindow[key] = "False" 
                       elif key == "visible" : dicoSubWindow[key] = "True" 
            return dicoSubWindow

               
        def saveSession(self) :
            extension = "qss"
            if self.sender() == self.saveSessionAsButton : title = QApplication.translate("QSphere","Save session as file")
            elif self.sender() == self.saveSessionCopyAsButton : title = QApplication.translate("QSphere","Save session as copy file")
            else : title = QApplication.translate("QSphere","Save session file")        

            fileName = ""        
            if self.sender() in (self.saveSessionAsButton, self.saveSessionCopyAsButton) or self.currentSession == None :
                    dlgFile = QFileDialog(self, title)
                    dlgFile.setNameFilters([items[1] for items in self.extensionsNames.items() if items[0]==extension]) 
                    dlgFile.selectNameFilter(self.extensionsNames[extension])
                    dlgFile.setDefaultSuffix(extension)
                    dlgFile.setViewMode(QFileDialog.Detail)
                    dlgFile.setDirectory(self.InitDir)
                    dlgFile.setAcceptMode(QFileDialog.AcceptSave)

                    fixeLabelsFileDialog(self, dlgFile, 1, True)
                    self.myDock.setVisible(False)    
                    if dlgFile.exec_(): fileName = "%s" % (dlgFile.selectedFiles()[0])
                    self.myDock.setVisible(True)
            else :
                    fileName = self.currentSession    
                                      
            if fileName!="" :
                   
               self.doSaveSession(fileName)
               
               if self.sender() != self.saveSessionCopyAsButton :
                  self.ajustListInfos(fileName, self.listSessions)
                  self.InitDir = dirname(fileName)
                  self.currentSession = fileName
                  self.setWindowTitle("%s : %s" % (self.rootWindowTitle, fileName))

               title = QApplication.translate("QSphere", "Information" ) 
               msg = QApplication.translate("QSphere","The session's file was saved as")
               sendMsg(self, title , "%s :<br><u><i>%s</i></ul>" % (msg, fileName), Qgis.Info, self.duration_info)


        def doSaveSession(self, fileNameQss):                
            xml = '<?xml version="1.0" encoding="UTF-8"?>\n'
            xml+= '<gmd:MD_Metadata xsi:schemaLocation="http://www.isotc211.org/2005/gmd http://schemas.opengis.net/iso/19139/20060504/gmd/gmd.xsd" xmlns:gmd="http://www.isotc211.org/2005/gmd" xmlns:gco="http://www.isotc211.org/2005/gco" xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xmlns:gml="http://www.opengis.net/gml" xmlns:xlink="http://www.w3.org/1999/xlink">\n'
            xml+= '\t<gmd:qspheresession>\n'
            xml+= '\t\t<gco:file>\n'
            xml+= '\t\t\t<gco:CharacterString>%s</gco:CharacterString>\n' % (fileNameQss)
            xml+= '\t\t</gco:file>\n'
            xml+= '\t\t<gmd:language>\n'
            xml+= '\t\t\t<gmd:LanguageCode codeList="http://www.loc.gov/standards/iso639-1/" codeListValue="%s">%s</gmd:LanguageCode>\n' % (self.localLangage, self.localLangage)
            xml+= '\t\t</gmd:language>\n'
            xml+= dateStamp(self, 2, False,  None)
            xml+= '\t</gmd:qspheresession>\n'
            
            xml+= '\t<formMainUI name="%s">\n' % (self.objectName())
            xml+= '\t\t<postab>0</postab>\n'# % self.tabWidget.currentIndex())
            xml+= '\t</formMainUI>\n'
            
            counter=0

            for subWindow in self.mdi.subWindowList(QMdiArea.CreationOrder):
                xml+= '\t<subWindow>\n'
                xml+= '\t\t<id>%s</id>\n' % counter           
                for child in subWindow.children():
                      if type(child)==dlgMetadata :
                         fileName = ""     
                         if child.windowTitle() != child.rootWindowTitle and child.windowTitle().find("~")== -1 : fileName = child.windowTitle().replace(child.rootWindowTitle, "")
                         xml+= '\t\t<source>%s</source>\n' % QApplication.translate("QSphere", fileName.encode("utf-8")) 
                         if fileName == "" :
                            xml+= '\t\t<projecttemp><![CDATA[%s]]></projecttemp>\n' % saveXml(child, None)      
                         else : xml+= '\t\t<projecttemp></projecttemp>\n'
                         xml+= '\t\t<tabwidgetcurrentindex>0</tabwidgetcurrentindex>\n'
                         xml+= '\t\t<histoIDChangeMode>%s</histoIDChangeMode>\n' % child.histoIDChangeMode
                         xml+= '\t\t<ComboLayers>%s</ComboLayers>\n' % child.ComboLayers.currentIndex()
                         xml+= '\t\t<PrimaryXSLT>%s</PrimaryXSLT>\n' % child.primaryXslt.currentIndex()
                         break     
                xml+= '\t\t<posx>%s</posx>\n' % subWindow.x()
                xml+= '\t\t<posy>%s</posy>\n' % subWindow.y()
                xml+= '\t\t<width>%s</width>\n' % subWindow.width()
                xml+= '\t\t<height>%s</height>\n' % subWindow.height()
                xml+= '\t\t<activate>%s</activate>\n' % (subWindow == self.mdi.activeSubWindow())    
                xml+= '\t\t<visible>%s</visible>\n' % subWindow.isVisible()
                xml+= '\t</subWindow>\n'
                counter+= 1

            xml+= "</gmd:MD_Metadata>\n"
                
            with open(fileNameQss, 'w', encoding="utf-8", errors="surrogateescape") as inFile : inFile.write(xml)
            

        def initChildGroups(self, child):
            for mddcollapsibleGroupBox in child.mddcollapsibleGroupBox:
                mddcollapsibleGroupBox.setCollapsed(True) 
                if mddcollapsibleGroupBox.objectName() == "tab1" : mddcollapsibleGroupBox.setCollapsed(False);
                
        def newMetadataFile(self):
            mdiWindow, child = self.initMetadataDialog()
            if child :
                    child.show()
                    self.initChildGroups(child)   
            self.counterNew+= 1
            return mdiWindow, child

        def openMetadataFileForLayers(self): 
            dialog = loadFromLayersDialog(self.iface, self)
            windowIcon(dialog, "layers-icon.png")
            self.myDock.setVisible(False)
            if dialog.exec_():
               for elt in dialog.qgisLayersList():
                   infos = elt.split("|")    
                   fileName = infos[0]
                   layerid  = infos[1]
                   if fileName!="" :
                        inputFile = QFileInfo(fileName)
                        extension = inputFile.suffix().lower()    
                        test, data = testXmlData(self, fileName, False) if extension == "xml" else True, None  
                        if test :                            
                              self.loadMetadataFile(fileName, True, layerid)     
                              self.InitDir = dirname(fileName)
            self.myDock.setVisible(True)                  

        def saveMetadataInBase(self):
            if self.mdi.subWindowList(QMdiArea.CreationOrder) != [] :
                    self.loadUIMetadataFromBase(0, QApplication.translate("QSphere", "Save"))
                    self.dlgSgbdr.loadListMddFiles()
            
        def openMetadataFromBase(self):
            self.loadUIMetadataFromBase(1, QApplication.translate("QSphere", "Load a metadata file"))
            
        def loadUIMetadataFromBase(self, context, title):
            if self.dlgSgbdr == None : self.dlgSgbdr = dlgSgbdr(self.iface, self, context, title)
            self.dlgSgbdr.show()
            self.dlgSgbdr.getDock()
            self.dlgSgbdr.activateWindow()


        def openMetadataFile(self):
                extension = self.extension 
                title = QApplication.translate("QSphere","Load a metadata file")
                dlgFile = QFileDialog(self, title)
                dlgFile.setNameFilters([items[1] for items in self.extensionsNames.items() if items[0]!="qss"])  
                dlgFile.selectNameFilter(self.extensionsNames[extension])
                dlgFile.setDefaultSuffix(extension)
                dlgFile.setViewMode(QFileDialog.Detail)
                dlgFile.setDirectory(self.InitDir)
                dlgFile.setFileMode(QFileDialog.ExistingFiles)  
                dlgFile.setAcceptMode(QFileDialog.AcceptOpen)

                fixeLabelsFileDialog(self, dlgFile, 0, True)
                self.myDock.setVisible(False)
                
                if dlgFile.exec_():
                    if  dlgFile.selectedFiles() != [] :   
                        for fileName in dlgFile.selectedFiles() :
                            if fileName!="" :
                               inputFile = QFileInfo(fileName) 
                               extension = inputFile.suffix().lower()    
                               test, data = testXmlData(self, fileName, False) if extension == "xml" else True, None      
                               if test : 
                                       self.loadMetadataFile(fileName, False, -1)
                                       self.InitDir = dirname(fileName)
                                       self.ajustListInfos(fileName, self.mddFiles)
                self.myDock.setVisible(True)


        def loadMetadataFile(self, fileName, fromLayer, layerid):
                mdiWindow, child = self.initMetadataDialog()
                self.fixeModeDialog(0)
                extension = getExtension(fileName)
                self.extension = extension.replace(".","")
                try : icon = self.sender().icon()
                except :
                     try : icon = self.sender().currentItem().icon()
                     except : icon = QIcon(getThemeIcon("unknow.png"))
                self.blocProgressBarButton.setIcon(icon)

                try :
                        if self.sender().toolTip() != fileName : self.blocProgressBarText.setText(textwrapfill("%s :\n %s" % (self.sender().toolTip(), fileName), 50))
                        else : self.blocProgressBarText.setText(textwrapfill("%s" % fileName, 50))
                except : self.blocProgressBarText.setText(textwrapfill("%s :\n %s" % ("Drag and drop", fileName), 50))
                self.blocProgressBar.setVisible(True)

                self.result = False      
                if extension == "" and fileName.startswith("pgmdd:"): child.mddFomDataBase(fileName)
                else :
                        if extension == ".qsp" : self.result = child.loadQsp(fileName, None)
                        elif extension == ".xml" : self.result = child.loadXml(fileName)
                        elif extension == ".qmd" : self.result = child.loadQmd(fileName)

                if fromLayer :
                   index = getComboLayerIndex(child, child.ComboLayers, layerid)
                   child.ComboLayers.setCurrentIndex(index)
                   
                   if child.ComboLayers.currentIndex() > 0 : 
                      child.isVisibleRestoreDico(True)
                      child.getMetaData()
                      if child.autoComment and extension == ".qsp" : child.initComments()
                      child.isVisibleRestoreDico(extension == ".qsp")
                
                self.blocProgressBar.setVisible(False)

                if child :
                        child.show()
                        self.initChildGroups(child)
                
                return mdiWindow, child

        def loadMetadata(self):
                icon = "navigatorweb.png"
                url = self.sender().text() if self.sender().objectName()!= "mActionNavigator" else "#blank"
                self.loadNavigator(url)

        def makeMajGetOptions(self):
            listSubWindows = self.mdi.subWindowList(QMdiArea.CreationOrder)    
            if listSubWindows!= [] :
               for subWindow in listSubWindows :     
                       for child in subWindow.children():
                           if type(child) == dlgMetadata : getOptions(child); break;

        def saveAllMetadataFile(self):
            listSubWindows = self.mdi.subWindowList(QMdiArea.CreationOrder)    
            if listSubWindows!= [] :
               for subWindow in listSubWindows :     
                       for child in subWindow.children():
                           if type(child) == dlgMetadata :
                                  if child.windowTitle() != child.rootWindowTitle and child.windowTitle().find("~")== -1 :                
                                          child.saveData()
                                          break
                                
        def saveMetadataFile(self):
            dialog, child = self.metadataWindow()
            if child : child.saveData()

               
        def saveAsMetadataFile(self):
            index = 2 if self.sender().objectName() == "saveCopyAsButton" else 1
            self.myDock.setVisible(False)
            dialog, child = self.metadataWindow()
            if child :
               child.InitDir = self.InitDir  
               child.saveDataAs(index)
            self.myDock.setVisible(True)   

        def saveXmlCatFile(self):
            dialog, child = self.metadataWindow()
            self.myDock.setVisible(False)
            if child : child.saveXmlCatalog()
            self.myDock.setVisible(True)


        def openModalDialog(self, dialog):
                     self.myDock.setVisible(False)
                     dialog.exec_()
                     self.myDock.setVisible(True)
                     
        def openFormWizardMode(self):
            dialog, child = self.metadataWindow()
            if child :
                    self.myDock.setVisible(False)
                    child.OpenFormTable(child.focusWidget())
                    self.myDock.setVisible(True)
                      
        def openXmlEditorWizardMode(self):
            dialog, child = self.metadataWindow()
            if child :
                if child.windowTitle().startswith("%spgmdd:" % child.rootWindowTitle) or child.windowTitle() == child.rootWindowTitle or child.windowTitle().find("~")!= -1:
                     dataXml = saveXml(child, None)
                     d = xmlEditor(child, "", child)
                     windowIcon(d, "editxml.png")
                     d.setWindowTitle("%s : %s" % (d.rootWindowTitle, child.windowTitle()))
                     d.editor.setText(dataXml.decode("utf-8")) 
                     d.checkProperties()
                     msgBadFileXml(d)
                     self.openModalDialog(d)
                else :        
                        if child.windowTitle() != child.rootWindowTitle and child.windowTitle().find("~")== -1 :
                            fileName = child.windowTitle().replace(child.rootWindowTitle, "")
                            extension = getExtension(fileName)
                            if extension == ".xml" :
                               d = xmlEditor(child, fileName, child)
                               windowIcon(d, "wizeditxml.png")
                               d.setWindowTitle("%s : %s" % (d.rootWindowTitle, fileName))
                               d.loadFile()
                               self.openModalDialog(d)
                            else :
                               dataXml = saveXml(child, None)
                               self.getReporting(dataXml.decode("utf-8"), False, "wizeditxml.png", True)


        def openContactsWizardMode(self):
            dialog, child = self.metadataWindow()
            if child : 
               for mddcollapsibleGroupBox in child.mddcollapsibleGroupBox:
                   mddcollapsibleGroupBox.setCollapsed(True)
                   if mddcollapsibleGroupBox.objectName() == "tab7" :
                      mddcollapsibleGroupBox.setCollapsed(False)
                      d = dlgContacts(child, True)
                      d.importContactsFromSheet()
                      self.openModalDialog(d)
                      break

        #========================
        # BLOC ACTIONS WINDOWS
        #========================             
        def tileWindows(self):
            dialog, child = self.metadataWindow()    
            self.mdi.tileSubWindows()    
            self.mdi.setActiveSubWindow(dialog)
            dialog.setFocus()
            
        def cascadeWindows(self):
            dialog, child = self.metadataWindow()        
            self.mdi.cascadeSubWindows()
            self.mdi.setActiveSubWindow(dialog)
            dialog.setFocus()

        def batteriesWindows(self):
            subWindows = self.mdi.subWindowList(QMdiArea.CreationOrder)
            N = len(subWindows)
            if N == 0 : return    

            from math import pi, sin ,cos, sqrt    
            self.cascadeWindows()
            dialog, child = self.metadataWindow()

            centerX = subWindows[0].width()
            dialog.setGeometry(0, 0, dialog.width(), dialog.height())
            elt = 0
            for itheta in range(N):
                if subWindows[itheta]!=dialog :
                   subWindows[itheta].setGeometry(centerX, 100*elt, subWindows[itheta].width(), subWindows[itheta].height())
                   elt+= 1
                
            self.mdi.setActiveSubWindow(dialog)
            dialog.setFocus()
            self.mdi.horizontalScrollBar().setValue(0)
            self.mdi.verticalScrollBar().setValue(0)
                

        def closeAllMySubWindows(self):
                self.counterNew = 0
                listSubWindows = self.mdi.subWindowList(QMdiArea.CreationOrder)    
                if listSubWindows!= [] :
                       for subWindow in listSubWindows :
                               if subWindow != self.filesNavigatorWidget : self.mdi.removeSubWindow(subWindow)
                self.majCountersSubWindows()
                self.changeStateButtons(None)

        
        def cloneCurrentWindow(self):
            refdialog, refchild = self.metadataWindow()

            if not refchild : return
            
            idbutton = self.changeMode.id(self.changeMode.checkedButton())
            dialog, child = self.newMetadataFile()
 
            windowProperties(child, refchild)
            child.paramsLineWidget = refchild.paramsLineWidget
            child.show()

            for elt in child.mddcollapsibleGroupBox:
                mddcollapsibleGroupBox = refchild.findChild(QgsCollapsibleGroupBox, elt.objectName())
                if mddcollapsibleGroupBox :
                        objs = mddcollapsibleGroupBox.children()
                        elt.setCollapsed(mddcollapsibleGroupBox.isCollapsed())
                        for j in range(len(objs)):
                            objName = "%s" % (objs[j].objectName())
                            objClassName = "%s" % (objs[j].metaObject().className())
                            
                            if not objClassName in ("QCommandLinkButton", "myPushButton", "QLabel") and objName != "" :
                               tableWidgetToCopy = fixeClassObj(objClassName, objName, child) 
                               tableWidget = objs[j]

                               if tableWidgetToCopy == None : pass
                               else : 
                                       if objClassName in ("QTableWidget", "MyTableWidget", "myTableWidget") and objName != "tablemotsclefso" :
                                          for row in range(tableWidget.rowCount()):
                                             if row > tableWidgetToCopy.rowCount()-1 : makeLine(child, tableWidgetToCopy, True, False, row, False)
                                             for col in range(tableWidget.columnCount()):
                                                   OriginalWidget = tableWidget.cellWidget(row, col) if tableWidget.cellWidget(row, col)!= None else tableWidget.item(row, col)
                                                   TargetObjWidget = tableWidgetToCopy.cellWidget(row, col) if tableWidgetToCopy.cellWidget(row, col)!= None else tableWidgetToCopy.item(row, col)  
                                                   try :  
                                                           if OriginalWidget.metaObject().className() in ("QLineEdit", "MySimpleWidgetLineEdit", "MyWidgetLineEdit", "MySimpleWidgetLineEditST") : TargetObjWidget.setText(OriginalWidget.text())
                                                           elif OriginalWidget.metaObject().className() in ("QCheckBox", "MyCheckBox", "myCheckBox") : TargetObjWidget.setChecked(OriginalWidget.isChecked())
                                                           elif OriginalWidget.metaObject().className() in ("QComboBox", "MyComboBox", "myComboBox") :
                                                               TargetObjWidget.setCurrentIndex(OriginalWidget.currentIndex())
                                                               TargetObjWidget.setEnabled(OriginalWidget.isEnabled())
                                                           elif OriginalWidget.metaObject().className() == "mySpinBox" : TargetObjWidget.setValue(OriginalWidget.value())
                                                   except :
                                                           try : TargetObjWidget.setText(OriginalWidget.text())
                                                           except : pass
                                                           
                                          countItems(dialog, objName, tableWidgetToCopy)

                                       elif objClassName in ("QTableWidget", "MyTableWidget", "myTableWidget") and objName == "tablemotsclefso" :
                                           for row in range(tableWidget.rowCount()):
                                               addLine = False    
                                               for col in range(tableWidget.columnCount()):
                                                   OriginalWidget = tableWidget.cellWidget(row, col) if tableWidget.cellWidget(row, col)!= None else tableWidget.item(row, col)
                                                   if OriginalWidget.isEnabled() :
                                                      if not addLine : makeLine(child, tableWidgetToCopy, True, False, row, False); addLine = True;
                                                      TargetObjWidget = tableWidgetToCopy.cellWidget(row, col) if tableWidgetToCopy.cellWidget(row, col)!= None else tableWidgetToCopy.item(row, col)
                                                      try :  
                                                           if OriginalWidget.metaObject().className() in ("QLineEdit", "MySimpleWidgetLineEdit", "MyWidgetLineEdit", "MySimpleWidgetLineEditST") : TargetObjWidget.setText(OriginalWidget.text())
                                                           elif OriginalWidget.metaObject().className() in ("QCheckBox", "MyCheckBox", "myCheckBox") : TargetObjWidget.setChecked(OriginalWidget.isChecked())
                                                           elif OriginalWidget.metaObject().className() in ("QComboBox", "MyComboBox", "myComboBox") :
                                                               TargetObjWidget.setCurrentIndex(OriginalWidget.currentIndex())
                                                               TargetObjWidget.setEnabled(OriginalWidget.isEnabled())
                                                           elif OriginalWidget.metaObject().className() == "mySpinBox" : TargetObjWidget.setValue(OriginalWidget.value())
                                                      except :
                                                           try : TargetObjWidget.setText(OriginalWidget.text())
                                                           except : pass                                                  
                                               
                                             
                                       elif objClassName == "QTableView" :
                                           for row in range(tableWidget.model().rowCount()):
                                               for col in range(tableWidget.model().columnCount()):
                                                   item = tableWidget.model().item(row, 0)
                                                   tableWidgetToCopy.model().item(row, 0).setCheckState(item.checkState())

                                       elif objClassName in ("QCalendarWidget", "QgsDateTimeEdit") :
                                               try : tableWidgetToCopy.setDate(tableWidget.date())
                                               except :        
                                                        try : tableWidgetToCopy.setSelectedDate(tableWidget.selectedDate())
                                                        except : reinitDate(tableWidgetToCopy)

                                               
                                       elif objClassName in ("QTextEdit", "MyTextEdit", "myTextEdit") :  tableWidgetToCopy.setPlainText(tableWidget.toPlainText()) 
                                       elif objClassName in ("QLineEdit", "MySimpleWidgetLineEdit", "MyWidgetLineEdit") : tableWidgetToCopy.setText(tableWidget.text()) 
                                       elif objClassName in ("QComboBox", "MyComboBox", "myComboBox") : tableWidgetToCopy.setCurrentIndex(tableWidget.currentIndex())
                                       elif objClassName == "QRadioButton" : tableWidgetToCopy.setChecked(tableWidget.isChecked())
                                       elif objClassName == "QGroupBox" :

                                            for ic in range(len(tableWidget.children())) :
                                                childGroup = tableWidget.children()[ic]
                                                childClone = tableWidgetToCopy.children()[ic]
                                                
                                                if childGroup.metaObject().className()== "QTableView" :
                                                   for row in range(childGroup.model().rowCount()):
                                                       for col in range(childGroup.model().columnCount()):
                                                           item = childGroup.model().item(row, 0)
                                                           childClone.model().item(row, 0).setCheckState(item.checkState())
                                                elif childGroup.metaObject().className() == "QRadioButton" : childClone.setChecked(childGroup.isChecked())           
                                                elif childGroup.metaObject().className() in ("QTableWidget", "MyTableWidget", "myTableWidget") :
                                                       childClone.clearContents()
                                                       for row in range(childClone.rowCount()): childClone.removeRow(0)
                                                       for row in range(childGroup.rowCount()):
                                                           childClone.insertRow(row)
                                                           addLineWidget(child, childClone, row,  0, 4, 0, float(childGroup.cellWidget(row, 0).value()))     
                                                elif childGroup.metaObject().className() in ("QComboBox", "MyComboBox", "myComboBox") : childClone.setCurrentIndex(childGroup.currentIndex())

            
            child.primaryXslt.setCurrentIndex(refchild.primaryXslt.currentIndex())
            index =  0 
            self.fixeModeDialog(child.histoIDChangeMode)
            child.doChangeMode(idbutton)
            child.changeMode.button(idbutton).setChecked(True)
            child.histoIDChangeMode = idbutton
            
            nameObj = self.listNameActionsTabEdition[0] 
            obj = self.findChild(myPushButton, nameObj)
            if obj : obj.setChecked(True)   
            
            
            
        def closeCurrentWindow(self):
            dialog, child = self.metadataWindow()
            if dialog :
                    dialog.showNormal()
                    if child : child.killWindows()
                    self.mdi.removeSubWindow(dialog)
                    self.majCountersSubWindows()
                    dialog, child = self.metadataWindow()
                    self.changeStateButtons(child)

               
            
        #========================
        # BLOC ACTIONS WEB
        #========================                       
        def openLustreWizard(self):
            if self.dialogLUSTRE == None :    
                self.dialogLUSTRE = dlgLustre(self, self.serverLustre, False) 
                self.windowsNoMDIChilds.append(self.dialogLUSTRE)
            self.dialogLUSTRE.show()
            self.dialogLUSTRE.activateWindow() 


        #CSW-T BLOC FUNTIONS
        def setEnabledCswtActions(self):
                cond = True if (self.connexionCswt != None and self.connexionCswt.connected()) else False
                for item in self.listNameActionsTabWebServices :
                    obj = self.findChild(myPushButton, item)
                    if obj :
                       obj.setEnabled(not cond) if item == "authCSWTButton" else obj.setEnabled(cond)
                       dialog, child = self.metadataWindow()
                       self.changeStateButtons(child)
                       
        def authCSWT(self):
                nameCswt = getSelConnexion(self)
                settings = QSettings()
                serverCswt = (settings.value("/qsphere/connections/%s/host" % (nameCswt)))
                authServerCswt = (settings.value("/qsphere/connections/%s/authhost" % (nameCswt)))
                userCswt = (settings.value("/qsphere/connections/%s/username" % (nameCswt)))
                pwdUserCswt = (settings.value("/qsphere/connections/%s/password" % (nameCswt)))

                dialog = dlgAuthCswt(userCswt, pwdUserCswt, serverCswt, False)
                windowIcon(dialog, "connect.png")
                self.myDock.setVisible(False)
                if dialog.exec_():
                   self.movieCtrl.startMovie()

                   self.connexionCswt = cswtConnection(self, nameCswt, serverCswt, authServerCswt)
                   responseCswt = self.connexionCswt.initSession(dialog.authInfos()[0], dialog.authInfos()[1], True)

                   if responseCswt != None :
                           self.setEnabledCswtActions()

                           obj = self.findChild(myPushButton, "authCSWTButton")

                           if obj :     
                                   if not obj.isEnabled():
                                      title = QApplication.translate("QSphere", "Information" ) 
                                      msg = QApplication.translate("QSphere","Connected with the server")
                                      icon = Qgis.Info
                                      duration = self.duration_info
                                   else :
                                      title = QApplication.translate("QSphere", "Warning" ) 
                                      msg = QApplication.translate("QSphere","Error")
                                      icon = Qgis.Warning
                                      duration = self.duration_warning
                          
                                   sendMsg(self, title, "%s" % (msg), icon, duration)
                           
                   if responseCswt == None : self.getAllReporting()

                   self.movieCtrl.stopMovie()
                   
                self.myDock.setVisible(True)

        def disconnectCSWT(self):
                if self.connexionCswt != None :
                    if self.connexionCswt.connected() :
                       self.connexionCswt.closeSession()
                       self.connexionCswt = None
                       self.setEnabledCswtActions()
                       
                       obj = self.findChild(myPushButton, "authCSWTButton")
                       if obj :
                               if obj.isEnabled():
                                  title = QApplication.translate("QSphere", "Information" ) 
                                  msg = QApplication.translate("QSphere","Disconnected with the server")
                                  icon = Qgis.Info
                                  duration = self.duration_info
                               else :
                                  title = QApplication.translate("QSphere", "Warning" ) 
                                  msg = QApplication.translate("QSphere","Error")
                                  icon = Qgis.Warning
                                  duration = self.duration_warning
                      
                               sendMsg(self, title, "%s" % (msg), icon, duration)   

        def getAllReporting(self):
                if self.connexionCswt != None : self.getReporting(self.connexionCswt.allReporting(), False, "editxml.png", True)
        
        def getReporting(self, responseCswt, isReporting, iconpath, isModal):
                d = xmlEditor(self, "", self)
                windowIcon(d, iconpath)
                d.setWindowTitle("%s : %s" % (d.rootWindowTitle, ""))
                d.show() if isModal else self.openModalDialog(d) 

                if isReporting :
                    try : charset = (responseCswt.headers["Content-Type"].split("=")[1].strip())
                    except : charset = "utf-8"  
                    try : d.editor.setText("%s%s" % (self.connexionCswt.reporting(), responseCswt.read().decode(charset)))
                    except : d.editor.setText("%s%s" % (self.connexionCswt.reporting(), responseCswt))
                else :
                    d.editor.setText(responseCswt)
                    
                d.checkProperties()
                msgBadFileXml(d)


        def recordsInCSWT(self): self.actionsInCswt(6)                
        def insertInCSWT(self): self.actionsInCswt(1)
        def updateInCSWT(self): self.actionsInCswt(2)
        def deleteInCSWT(self): self.actionsInCswt(-1)

        def actionsInCswt(self, useCase):
                dialog, child = self.metadataWindow()
                if child or useCase == 6 :        
                        self.movieCtrl.startMovie()
                        if useCase in (1,2) :
                            if child.windowTitle() != child.rootWindowTitle and child.windowTitle().find("~")== -1 :        
                                child.saveData()
                                fileName = child.windowTitle().replace(child.rootWindowTitle, "")
                                extension = getExtension(fileName)
                                if extension == ".xml" :
                                    f = open(fileName, "r", encoding='utf-8')
                                    dataXml = f.read()
                                    f.close()
                                elif extension == ".qsp" : dataXml = saveXml(child, None) .decode("utf-8")
                            else :  dataXml = saveXml(child, None).decode("utf-8") 
  
                            dataXml = convertToUtf8(unquote(dataXml))
                            
                            pos = dataXml.find("<gmd:MD_Metadata")
                            dataXml = dataXml[pos:]
                            
                        elif useCase == -1 :
                            obj = child.findChild(QLineEdit, "identificator")
                            if obj : dataXml = obj.text()
                            else : return
                        elif useCase == 6: dataXml = None    
                        else : return

                        if self.connexionCswt != None :
                           responseCswt = self.connexionCswt.metadataCswtPublication(useCase, dataXml, True) 
                           if useCase != 6 : self.connexionCswt.analyseResponse(useCase, responseCswt)

                           if self.reportingCSWT : self.getReporting(responseCswt, True, "editxml.png", True)

                        self.movieCtrl.stopMovie()

        #END BLOC CSWT
        #===============================
        # BLOC ACTIONS MAINWINDOW ITEMS
        #===============================
        def makeMenuCentralWidget(self, tabTarget, icon, title, listActions, onTab):
                toolBar = QToolBar(tabTarget)
                toolBar.setObjectName(title)
                toolBar.setAllowedAreas(Qt.TopToolBarArea | Qt.BottomToolBarArea)
                toolBar.setWindowTitle(title)
                toolBar.setSizePolicy(QSizePolicy.Expanding, QSizePolicy.Expanding)
                toolBar.setStyleSheet("QToolBar {spacing:5px;}")             
                conteneur = None

                for item in listActions :
                        if item[0]=="-":  toolBar.addSeparator(); conteneur = None;
                        elif item[0]=="~":
                                
                          obj = QGroupBox(tabTarget)       
                          obj.setGeometry(item[3][0], item[3][1], item[3][2], item[3][3])
                          obj.setMinimumSize(item[3][2], item[3][3])
                          obj.setMaximumSize(item[3][2], item[3][3])
                          obj.setStyleSheet("QGroupBox { font-size: 12px;  text-align: center; color : black; border: 0px;}"
                                            "QGroupBox::title {top: -2px; left: 10px;}")

                          background = QLabel(obj)
                          background.setGeometry(10, 10, item[3][2]-20, 26)
                          background.setStyleSheet(" QLabel {background-color: qlineargradient(x1: 0, y1: 0, x2: 0, y2: 1, stop: 0 #E1E1E1, stop: 0.4 #9B9B9B, stop: 0.5 #D8D8D8, stop: 1.0 #D3D3D3);  border-width: 2px; border-radius: 10px; border-color: beige;}")

                          trumbail = QLabel(obj)
                          trumbail.setGeometry(0, 10, 90, 25)
                          trumbail.setText("%s :" % QApplication.translate("QSphere",item[1]))
                          trumbail.setObjectName("Label%s_%s" % (item[1], item[2].split(".")[0]))
                          trumbail.setAlignment(Qt.AlignRight)
                          if item[1]!="": trumbail.setStyleSheet(" QLabel { font-size: 12px; color : black; background-color: rgba(160, 160, 160, 125); border: 0.5px solid black;  border-radius: 4px;  padding: 2px;}")

                          toolBar.addWidget(obj)
                          conteneur = obj

                          
                        elif item[0] in ("childsCount", "windowsCount"):
                          obj = QLabel() if not conteneur else QLabel(conteneur)      
                          obj.setGeometry(item[3][0], item[3][1], item[3][2], item[3][3])
                          obj.setObjectName("%s" % item[0])
                          icon = QImage(getThemeIcon(item[2])) 
                          obj.setPixmap(QPixmap.fromImage(icon))
                          obj.setToolTip(QApplication.translate("QSphere",item[1]))

                          if not conteneur : toolBar.addWidget(obj)

                          font = QFont()
                          font.setFamily('Courier')
                          font.setFixedPitch(True)
                          font.setPointSize(12)
                          font.setBold(True)

                          nbObj = QTextEdit() if not conteneur else QTextEdit(conteneur)
                          nbObj.setGeometry(item[3][0]+item[3][2]+5, item[3][1], item[3][2], item[3][3])
                          nbObj.setMinimumSize(item[3][2], item[3][3])
                          nbObj.setMaximumSize(item[3][2], item[3][3])
                          nbObj.setStyleSheet("QTextEdit { font-size: 14px;  text-align: center; color : white; background-color: rgba(160, 160, 160, 125); border: 2px solid cyan;  border-radius: 4px;  padding: 2px;}")
                          nbObj.setText("0")
                          nbObj.setFont(font)
                          nbObj.setAlignment(Qt.AlignCenter)
                          nbObj.setEnabled(False)
                          
                          if not conteneur : toolBar.addWidget(nbObj)
                          if item[0] == "windowsCount" : self.nb_subwindows = nbObj                 
                         
                        else:
                          obj = myPushButton() if not conteneur else myPushButton(conteneur)

                          text = QApplication.translate("QSphere",item[1])
                          if item[0] == "newSession" : text = "%s%s" % (text, "Ctrl+Shift+N".rjust(60-len(text)))
                          if item[0] == "openSession" : text = "%s%s" % (text, "Ctrl+Shift+F".rjust(60-len(text)))
                          if item[0] == "saveSession" : text = "%s%s" % (text, "Ctrl+Shift+W".rjust(60-len(text)))
                          if item[0] == "newFile" : text = "%s%s" % (text, "Ctrl+N".rjust(60-len(text)))
                          if item[0] == "openFile" : text = "%s%s" % (text, "Ctrl+F".rjust(60-len(text)))
                          if item[0] == "saveFile" : text = "%s%s" % (text, "Ctrl+S".rjust(60-len(text)))
                          if item[0] == "saveAll" : text = "%s%s" % (text, "Ctrl+A".rjust(60-len(text)))
                         
                          obj.initPushButton(item[3][2], item[3][3], item[3][0], item[3][1], item[0], "", text, True, getThemeIcon(item[2]), item[3][2], item[3][3], True)
                          if item[4]!= None : obj.clicked.connect(item[4]) if type(item[4])!=str else eval(item[4])
                          if item[5]!="" : obj.setShortcut(QKeySequence(item[5]))
                          obj.setEnabled(item[6])

                          if item[0] =="toolFiles" : self.butToolFiles = obj       
                          elif item[0] =="openSession" : self.butopensession = obj
                          elif item[0] =="openFile" : self.butopenfile = obj
                          elif item[0] == "saveSession" : self.butsavesession = obj
                          elif item[0] == "navigatorWeb" : self.butnavigatorweb = obj
           
                          if item[0] in self.listNameActionsTabEdition:
                                obj.setCheckable(True)
                                obj.setStyleSheet(self.styleSheetPushButton)
                                if item[0] == "identificationTab" : obj.setChecked(True)
                                self.positionTab.addButton(obj, self.listNameActionsTabEdition.index(item[0]))

                          if item[0] in ("formMode", "xmlMode", "htmlMode"):
                                obj.setCheckable(True)
                                obj.setStyleSheet(self.styleSheetPushButton)
                                if item[0] == "formMode" : obj.setChecked(True)
                                self.changeMode.addButton(obj, ("formMode", "xmlMode", "htmlMode").index(item[0]))

                          if not conteneur : toolBar.addWidget(obj)
                
                i = len(listActions)-1
                item = listActions[i]

                toolBar.setGeometry(0, 0, (i+1)*38, 25) 
                toolBar.setStyleSheet("QToolBar {color: white; background-color:qlineargradient(x1:0, y1:0, x2:0, y2:1, stop: 0 #cccccc, stop: 1 #5D5D5D);}")

                return toolBar

           
        def resizeEvent(self,ev):
            self.blocProgressBar.setGeometry((self.width()/2)-205, (self.height()/2)-120, 400, 230)
            self.movieCtrl.setGeometry((self.width()/2)-64, (self.height()/2)-64, 64, 64)
            self.barInfo.setGeometry(0, 0, self.width(), 90)
            

        def closeEvent(self,event):
                self.myDock.setVisible(False)
                msgDlgBox = myMsgDlgBox(self, QMessageBox.Question, QApplication.translate("QSphere","Confirm Exit..."),
                                                QApplication.translate("QSphere","Are you sure you want to exit ?"), "qsphere_stop.png", None, self.parent)
                windowIcon(msgDlgBox , "qspheremainui.png")
                ret = msgDlgBox.exec_()
                if ret == 1 :
                   self.saveInfos("sessions", self.listSessions)
                   self.saveInfos("fileshistory", self.mddFiles)
                   self.parent.mainWindow = None
                   if self.dialogLUSTRE : self.dialogLUSTRE.close()
                   if self.dlgSgbdr :
                                
                                msgDlgBox = myMsgDlgBox(self, QMessageBox.Question, QApplication.translate("QSphere","Confirm Exit..."),
                                                "%s :\n%s" % (QApplication.translate("QSphere","Are you sure you want to exit ?"), self.dlgSgbdr.windowTitle()), "qsphere_stop.png", None, self.parent)
                                windowIcon(msgDlgBox , "qspheremainui.png")
                                ret = msgDlgBox.exec_()
                                if ret == 1 : self.dlgSgbdr.close(); self.parent.dlgSgbdr = None;

                   self.myDock.isClosable = True    
                   self.myDock.close()
                   if self.filesNavigatorWidget : self.filesNavigatorWidget.close()
                   event.accept()
                   
                else :
                        self.myDock.setVisible(True)
                        event.ignore()
                        
                   
        def contextMnuSaveSessionActions(self, obj):
                contextMenu = QMenu()

                icon = getThemeIcon("saveas.png")
                text = QApplication.translate("QSphere","Save session as")
                self.saveSessionAsButton = QAction(QIcon(icon), "%s ..." % text, self)
                self.saveSessionAsButton.setObjectName("saveAsButton")
                contextMenu.addAction(self.saveSessionAsButton)
                self.saveSessionAsButton.triggered.connect(self.saveSession)

                contextMenu.addSeparator()

                icon = getThemeIcon("savecopy.png")
                text = QApplication.translate("QSphere","Save as a copy ...")
                self.saveSessionCopyAsButton = QAction(QIcon(icon), text, self)
                self.saveSessionCopyAsButton.setObjectName("saveCopyAsButton")
                contextMenu.addAction(self.saveSessionCopyAsButton)
                self.saveSessionCopyAsButton.triggered.connect(self.saveSession)

                obj.setMenu(contextMenu)


        def contextMnuSaveActions(self, obj):
                contextMenu = QMenu()

                icon = getThemeIcon("saveas.png")
                text = QApplication.translate("QSphere","Save as ...")
                self.saveAsButton = QAction(QIcon(icon), text, self)
                self.saveAsButton.setObjectName("saveAsButton")
                self.saveAsButton.setShortcut(QKeySequence("Ctrl+Shift+S"))
                contextMenu.addAction(self.saveAsButton)
                self.saveAsButton.triggered.connect(self.saveAsMetadataFile)

                icon = getThemeIcon("layers-database.png")
                text = "%s ..." % QApplication.translate("QSphere","Save in database")
                self.saveInDataBase = QAction(QIcon(icon), text, self)
                self.saveInDataBase.setObjectName("saveInDataBase")
                self.saveInDataBase.setShortcut(QKeySequence("Ctrl+Shift+D"))
                contextMenu.addAction(self.saveInDataBase)
                self.saveInDataBase.triggered.connect(self.saveMetadataInBase)                

                contextMenu.addSeparator()

                icon = getThemeIcon("savecopy.png")
                text = QApplication.translate("QSphere","Save as a copy ...")
                self.saveCopyAsButton = QAction(QIcon(icon), text, self)
                self.saveCopyAsButton.setObjectName("saveCopyAsButton")
                self.saveCopyAsButton.setShortcut(QKeySequence("Ctrl+Shift+C"))
                contextMenu.addAction(self.saveCopyAsButton)
                self.saveCopyAsButton.triggered.connect(self.saveAsMetadataFile)

                contextMenu.addSeparator()

                icon = getThemeIcon("saveattribut.png")
                text = QApplication.translate("QSphere","Save as the Fields Map ...")
                self.saveXmlCatButton = QAction(QIcon(icon), text, self)
                self.saveXmlCatButton.setObjectName("saveXmlCatButton")
                contextMenu.addAction(self.saveXmlCatButton)
                self.saveXmlCatButton.triggered.connect(self.saveXmlCatFile)

                obj.setMenu(contextMenu)

        def fixeMaxDim(self, listItems):
                maxDim = 60
                if listItems != []:
                   for item in listItems: maxDim = max(maxDim,len(item))
                maxDim = int(maxDim * 1.8)
                return maxDim


                
        def contextMnuFile(self, obj):
                contextMenu = QMenu()

                maxDim, pattern = self.fixeMaxDim(self.mddFiles), "%s ... %s"

                icon = getThemeIcon("openmeta.png")
                text = textForMenu(pattern, QApplication.translate("QSphere","Load a metadata file"), maxDim, "Ctrl+F")
                self.openFile = QAction(QIcon(icon), text, self)
                self.openFile.setObjectName("mActionOpenFile")
                contextMenu.addAction(self.openFile)
                self.openFile.triggered.connect(self.openMetadataFile)

                contextMenu.addSeparator()

                icon = getThemeIcon("layers-icon.png")
                text = textForMenu(pattern, QApplication.translate("QSphere","Load metadata file for layers"), maxDim, "Ctrl+L")
                self.openFromLayers = QAction(QIcon(icon), text, self)
                self.openFromLayers.setObjectName("mActionOpenFromLayers")
                contextMenu.addAction(self.openFromLayers)
                self.openFromLayers.triggered.connect(self.openMetadataFileForLayers)

                icon = getThemeIcon("layers-database.png")
                text = textForMenu(pattern, QApplication.translate("QSphere","Load metadata file from database"), maxDim, "Ctrl+D")
                self.openFromDataBase = QAction(QIcon(icon), text, self)
                self.openFromDataBase.setObjectName("mActionOpenFromDataBase")
                contextMenu.addAction(self.openFromDataBase)
                self.openFromDataBase.triggered.connect(self.openMetadataFromBase)                

                
                icon = getThemeIcon("openmeta.png")
                
                if self.mddFiles != []:
                        contextMenu.addSeparator()

                        i = 0
                        for mddfile in self.mddFiles :
                            if mddfile :    
                                    self.actionLastFile = QAction(QIcon(icon), mddfile, self)
                                    self.actionLastFile.setObjectName("actionLastFile%s" % (i) )
                                    contextMenu.addAction(self.actionLastFile)
                                    self.actionLastFile.triggered.connect(self.openLastFile)
                                    i+= 1


                contextMenu.addSeparator()

                icon = getThemeIcon("poubelle.png")
                text = "%s ..." % QApplication.translate("QSphere","Clear history")
                self.delAllSavedFiles = QAction(QIcon(icon), text, self)
                self.delAllSavedFiles.setObjectName("delAllSavedFiles")
                contextMenu.addAction(self.delAllSavedFiles)
                self.delAllSavedFiles.triggered.connect(self.delInfos)

                obj.setMenu(contextMenu)
                

        def contextMnuSession(self, obj):
                contextMenu = QMenu()

                icon = getThemeIcon("opensession.png")
                text = "%s ..." % QApplication.translate("QSphere","Open session")

                maxDim = self.fixeMaxDim(self.listSessions)
                
                text = "%s%s" % (text, "Ctrl+Shift+F".rjust(maxDim-len(text)))
                self.openSessionAction = QAction(QIcon(icon), text, self)
                self.openSessionAction.setObjectName("mActionOpenSession")
                contextMenu.addAction(self.openSessionAction)
                self.openSessionAction.triggered.connect(self.openSession)
                
                if self.listSessions != []:
                        contextMenu.addSeparator()

                        i = 0
                        for session in self.listSessions :
                            if session :    
                                    self.actionLastSession = QAction(QIcon(icon), session, self)
                                    self.actionLastSession.setObjectName("actionLastSession%s" % (i) )
                                    contextMenu.addAction(self.actionLastSession)
                                    self.actionLastSession.triggered.connect(self.openLastSession)
                                    i+= 1

                contextMenu.addSeparator()

                icon = getThemeIcon("poubelle.png")
                text = "%s ..." % QApplication.translate("QSphere","Clear history")
                self.delAllSavedSession = QAction(QIcon(icon), text, self)
                self.delAllSavedSession.setObjectName("delAllSavedSession")
                contextMenu.addAction(self.delAllSavedSession)
                self.delAllSavedSession.triggered.connect(self.delInfos)

                obj.setMenu(contextMenu)


        def contextMenuWebViewer(self, obj):
                self.contextMenuWebViewer = QMenu()

                icon = getThemeIcon("navigatorweb.png")
                text = QApplication.translate("QSphere","Web navigator ...")
                self.OpenNavigatorWeb = QAction(QIcon(icon), text, self)
                self.OpenNavigatorWeb.setObjectName("mActionNavigator")
                self.contextMenuWebViewer.addAction(self.OpenNavigatorWeb)
                self.OpenNavigatorWeb.triggered.connect(self.loadMetadata)

                self.contextMenuWebViewer.addSeparator()

                icon = getThemeIcon("formatactivate.png")
                text = QApplication.translate("QSphere","Url Server for metadata : ")
                self.contextMnu_serverMetadata = QMenu(text)
                self.contextMnu_serverMetadata.setIcon(QIcon(icon))
                i = 0
                for elt in self.serverMetadata :
                    self.OpenServerMDD = QAction(QIcon(icon), elt, self)
                    self.OpenServerMDD.setObjectName("OpenServerMDD%s" % (i) )
                    self.contextMnu_serverMetadata.addAction(self.OpenServerMDD)
                    self.OpenServerMDD.triggered.connect(self.loadMetadata)
                    i+= 1

                self.contextMenuWebViewer.addMenu(self.contextMnu_serverMetadata)
                self.contextMenuWebViewer.addSeparator()

                text = QApplication.translate("QSphere","Url Server for keywords : ")
                self.contextMnu_serverKeywords = QMenu(text)
                self.contextMnu_serverKeywords.setIcon(QIcon(icon))
                i = 0
                for elt in self.serverKeywords :
                    self.OpenServerKW = QAction(QIcon(icon), elt, self)
                    self.OpenServerKW.setObjectName("OpenServerKW%s" % (i) )
                    self.contextMnu_serverKeywords.addAction(self.OpenServerKW)
                    self.OpenServerKW.triggered.connect(self.loadMetadata)
                    i+= 1

                self.contextMenuWebViewer.addMenu(self.contextMnu_serverKeywords)

                self.contextMenuWebViewer.addSeparator()
                
                icon = getThemeIcon("info.png")
                text = QApplication.translate("QSphere","Get informations about metadata and INSPIRE")
                self.helptopic = QAction(QIcon(icon), text, self)
                self.helptopic.setObjectName("helptopic")
                self.contextMenuWebViewer.addAction(self.helptopic)
                self.helptopic.triggered.connect(self.helpInspire)
                
                obj.setMenu(self.contextMenuWebViewer)

        def ajustListInfos(self, historyFile, listinfos):
            if historyFile in listinfos : return    
            newHistory = []
            newHistory.append(historyFile)
            for i in range(min(len(listinfos), 9)) :
                if listinfos[i] != "" : newHistory.append(listinfos[i])
            if listinfos is self.listSessions :
               self.listSessions = list(newHistory)
               self.contextMnuSession(self.butopensession)
            elif listinfos is self.mddFiles :
               self.mddFiles = list(newHistory)
               self.contextMnuFile(self.butopenfile)
                
        def saveInfos(self, flagInfo, listinfos):
            settings = QSettings()
            pos = 0
            self.delSavedInfos(flagInfo, listinfos, False)
            for i in range(min(len(listinfos), 10)) : 
                historyFile = listinfos[i]
                if historyFile :
                   settings.setValue("/qsphere/%s/" % flagInfo, pos)
                   settings.setValue("/qsphere/%s/%s/file" % (flagInfo, pos), "%s" % historyFile)
                   pos+= 1
            settings.sync()   

        def loadInfos(self, flagInfo, listinfos):
            settings = QSettings()
            settings.beginGroup("/qsphere/%s/" % flagInfo)
            keys = settings.childGroups()
            if keys != []:
                   valueSettings = QSettings() 
                   for key in keys :
                       historyFile = valueSettings.value("/qsphere/%s/%s/file" % (flagInfo, key))
                       if historyFile : listinfos.append(historyFile)

        def delInfos(self):
            if self.sender().objectName()== "delAllSavedSession" : self.delSavedInfos("sessions", self.listSessions, True)
            elif self.sender().objectName()== "delAllSavedFiles" : self.delSavedInfos("fileshistory", self.mddFiles, True)
            
        def delSavedInfos(self, flagInfo, listinfos, askFor):
                self.myDock.setVisible(False)
                if askFor :
                        msgDlgBox = myMsgDlgBox(self, QMessageBox.Question, QApplication.translate("QSphere","Warning"),
                                QApplication.translate("QSphere", "Delete all the entries ?"), "qsphere_stop.png", None, self.parent)
                        windowIcon(msgDlgBox , "qspheremainui.png")
                        ret = msgDlgBox.exec_()
                        if ret !=  1 : self.myDock.setVisible(True); return
                self.myDock.setVisible(True)    
                settings = QSettings()
                settings.beginGroup("/qsphere/%s/" % flagInfo)
                keys = settings.childGroups()
                valueSettings = QSettings()
                if keys != []:
                  for key in keys : valueSettings.remove("/qsphere/%s/%s" % (flagInfo, key))
                valueSettings.sync()

                if askFor :   
                        if listinfos is self.listSessions :
                          self.listSessions = []
                          self.contextMnuSession(self.butopensession)
                        elif listinfos is self.mddFiles :
                          self.mddFiles = []     
                          self.contextMnuFile(self.butopenfile)

               
            
