from qgis.core import Qgis
from qgis.gui import (QgsCollapsibleGroupBox, QgsColorButton, QgsMessageBar)

from qgis.PyQt.QtCore import (QSettings, Qt)
from qgis.PyQt.QtGui import (QColor, QIcon, QImage, QKeySequence, QMovie, QPixmap)
from qgis.PyQt.QtPrintSupport import (QPrintPreviewDialog, QPrinter)
from qgis.PyQt.QtWidgets import (QAction, QApplication, QCheckBox, QComboBox, QFileDialog, QGroupBox,
                                 QHBoxLayout, QLabel, QLineEdit, QListWidget, QListWidgetItem, QMenu, QMessageBox,
                                 QPushButton, QRadioButton, QSizePolicy, QTabWidget, QTextBrowser, QWidget)

from qsphere.cswt_connector import cswtConnection
from qsphere.qsphere_objmaker import (movieCtrl, myMsgDlgBox, myPushButton, myPushButtonFunny, myPushButtonMappable, mySlider, myTextEdit,
                                      myUserComboBox, myWebComboBox, MyWidgetLineEdit, printerSphere)
from qsphere.qsphere_tools import (countItems, correctPath, fixeLabelsFileDialog, fileNameWithExtension,
                                   getHelp, getOptions, getSelConnexion, getThemeIcon, getWidget,
                                   markerText, msgBadFileXml, multiGetRequest, prettifyXml, sendMsg, windowIcon, windowProperties)
from qsphere.ui_editorxml import xmlEditor

from configparser import ConfigParser
from datetime import datetime
from os.path import (abspath, dirname, join, exists)
from xml.etree import ElementTree as etree

class uiDlgOptions(object):
  def setupUi(self):

    self.minWidth, self.minHeight = 835, 580
    self.setMinimumSize(self.minWidth, self.minHeight)
    self.setMaximumSize(self.minWidth*1.5, self.minHeight)
    self.setObjectName("dlgOptions")
    self.resize(self.minWidth, self.minHeight)
    self.setModal(True)
    self.setWindowFlags(Qt.WindowMaximizeButtonHint | Qt.WindowCloseButtonHint)
    
    self.labelXmlFiles = "eXtensible Markup Language (*.xml *.XML)"
    self.InitDir = ""
    self.webNavigator = None

    self.title = ("Servers","Time and Folders", "Parser and Editor", "CSW-T", "Ergonomy")
              
    self.toolTips = ("Server parameters : ", "Fix duration for process (Web queries, QgsMessageBox)",
                     "Parser autocorrection parameters", "CSW-T parameters","Ergonomical parameters")

    self.images = {"groupBoxServer" : "links", "groupBoxDuration" : "folders",
                   "groupEditorInfo" : "xml", "groupConnexionInfo" : "cswt",
                   "groupErgonomicalInfo" : "ergonomy"} 

    posH, self.mddcollapsibleGroupBox, self.heights, delta =  5, [], [], 95
    
    for i in range(len(self.title)):
          titleGroup = "collaspiblegroup%s" % i
          collapsibleGroupBox = QgsCollapsibleGroupBox(self)
          collapsibleGroupBox.setCollapsed(True)
          collapsibleGroupBox.setObjectName(titleGroup)
          collapsibleGroupBox.setMinimumHeight(self.minHeight - delta)
          collapsibleGroupBox.setMaximumHeight(self.minHeight - delta)
          collapsibleGroupBox.setGeometry(205, posH, 615, self.minHeight - delta)

          posH+= 15
          self.mddcollapsibleGroupBox.append(collapsibleGroupBox)
           
          setattr(self, titleGroup, collapsibleGroupBox)

          collapsibleGroupBox.collapsedStateChanged.connect(self.changePosGroups)


          if i == 0 :
            self.groupBoxServer = QGroupBox(collapsibleGroupBox)
            self.groupBoxServer.setObjectName("groupBoxServer")
          
            #Metadata
            self.lblServerMetadata = QLabel(self.groupBoxServer)
            self.lblServerMetadata.setObjectName("lblserverMetadata")
            self.lblServerMetadata.setAlignment(Qt.AlignLeft)    

            self.serverMetadata = myWebComboBox(self.groupBoxServer)
            self.serverMetadata.setEditable(True)
            self.serverMetadata.setObjectName("serverMetadata")
            self.serverMetadata.currentTextChanged.connect(self.serverMetadata.checkRegExp)

            icon = getThemeIcon("actions.png")
            toolTip = QApplication.translate("QSphere","Actions ...")
            self.serverMetadataButton = myPushButtonFunny(self.groupBoxServer) 
            self.serverMetadataButton.initPushButton(40, 24,  0, 20, "serverMetadataButton", "", toolTip, True, icon, 40, 24, True)

            #keywords
            self.lblServerKeywords = QLabel(self.groupBoxServer)
            self.lblServerKeywords.setObjectName("lblserverKeywords")
            self.lblServerKeywords.setAlignment(Qt.AlignLeft)

            self.serverKeywords = myWebComboBox(self.groupBoxServer)
            self.serverKeywords.setEditable(True)
            self.serverKeywords.setObjectName("serverKeywords")
            self.serverKeywords.currentTextChanged.connect(self.serverKeywords.checkRegExp)

            icon = getThemeIcon("actions.png")
            toolTip = QApplication.translate("QSphere","Actions ...")
            self.serverKeywordsButton = myPushButtonFunny(self.groupBoxServer) 
            self.serverKeywordsButton.initPushButton(40, 24,  0, 20, "serverKeywordsButton", "", toolTip, True, icon, 40, 24, True)
            
            #URL navigator
            self.lblServerNavigator = QLabel(self.groupBoxServer)
            self.lblServerNavigator.setObjectName("lblserverNavigator")
            self.lblServerNavigator.setAlignment(Qt.AlignLeft)

            self.serverNavigator = myWebComboBox(self.groupBoxServer)
            self.serverNavigator.setObjectName("serverNavigator")
            self.serverNavigator.setEditable(True)
            self.serverNavigator.currentTextChanged.connect(self.serverNavigator.checkRegExp)

            icon = getThemeIcon("actions.png")
            self.serverNavigatorButton = myPushButtonFunny(self.groupBoxServer) 
            self.serverNavigatorButton.initPushButton(40, 24,  0, 20, "serverNavigatorButton", "", "", True, icon, 40, 24, True)

            self.drawImg(self.groupBoxServer)

          elif i == 1 :
            self.groupBoxFolders = QGroupBox(collapsibleGroupBox)
            self.groupBoxFolders.setObjectName("groupBoxFolders")


            self.lblFolder = QLabel(self.groupBoxFolders)
            self.lblFolder.setObjectName("lblFolder")
            self.lblFolder.setAlignment(Qt.AlignLeft)    

            self.folder = myTextEdit(self.groupBoxFolders)
            self.folder.initTextEdit(self.minWidth-310, 25, 10, 50, "folder", True, False, True, False, True)
            self.folder.setStyleSheet("color: black; background-color: #C0C0C0")

            self.folderButton = myPushButton(self.groupBoxFolders)
            self.folderButton.initPushButton(24, 24, 660, 32 , "folderButton", "", "", True, getThemeIcon("folder.png"), 24, 24, True)

            self.checkAutoComment = QCheckBox(self.groupBoxFolders)
            self.checkAutoComment.setObjectName("checkAutoComment")
            
            #Group Informations Duration
            self.groupBoxDuration = QGroupBox(collapsibleGroupBox)
            self.groupBoxDuration.setObjectName("groupBoxDuration")

            self.checkShowWarning = QCheckBox(self.groupBoxDuration)
            self.checkShowWarning.setObjectName("checkShowWarning")
                                  
            self.lblSliderInfo = QLabel(self.groupBoxDuration)
            self.lblSliderInfo.setObjectName("lblSliderInfo")
            self.lblSliderInfo.setAlignment(Qt.AlignRight)
            self.lblSliderInfo.setGeometry(10, 90, 300, 25)
            

            self.sliderInformation = mySlider(self.groupBoxDuration)
            self.sliderInformation.initSlider(200, 20, 320, 90, "sliderInformation", "", True, False, Qt.Horizontal, 1, 10, 1)

            self.txtSliderINFO = QLabel(self.groupBoxDuration)
            self.txtSliderINFO.setGeometry(540, 90, 25, 25) 
            self.txtSliderINFO.setStyleSheet("color: black; background-color: #C0C0C0")
            self.txtSliderINFO.setAlignment(Qt.AlignCenter)

            self.lblSliderWarning = QLabel(self.groupBoxDuration)
            self.lblSliderWarning.setObjectName("lblSliderWarning")
            self.lblSliderWarning.setAlignment(Qt.AlignRight)
            self.lblSliderWarning.setGeometry(10, 130, 300, 25)

            self.sliderWarning = mySlider(self.groupBoxDuration)
            self.sliderWarning.initSlider(200, 20, 320, 130, "sliderWarning", "", True, False, Qt.Horizontal, 1, 10, 1)

            self.txtSliderWARNING = QLabel(self.groupBoxDuration)
            self.txtSliderWARNING.setGeometry(540, 130, 25, 25) 
            self.txtSliderWARNING.setStyleSheet("color: black; background-color: #C0C0C0")
            self.txtSliderWARNING.setAlignment(Qt.AlignCenter)

            self.lblSliderTimeOut = QLabel(self.groupBoxDuration)
            self.lblSliderTimeOut.setObjectName("lblSliderTimeOut")
            self.lblSliderTimeOut.setAlignment(Qt.AlignRight)
            self.lblSliderTimeOut.setGeometry(10, 170, 300, 25)
            
            self.sliderTimeOut = mySlider(self.groupBoxDuration)
            self.sliderTimeOut.initSlider(200, 20, 320, 170, "sliderTimeOut", "", True, False, Qt.Horizontal, 1, 10, 1)

            self.txtSliderTIMEOUT = QLabel(self.groupBoxDuration)
            self.txtSliderTIMEOUT.setGeometry(540, 170, 25, 25) 
            self.txtSliderTIMEOUT.setStyleSheet("color: black; background-color: #C0C0C0")
            self.txtSliderTIMEOUT.setAlignment(Qt.AlignCenter)

            self.drawImg(self.groupBoxDuration)

          elif i == 2 :
            self.groupParserInfo = QGroupBox(collapsibleGroupBox)
            self.groupParserInfo.setObjectName("groupParserInfo")

            self.checkAutoCorrect = QCheckBox(self.groupParserInfo)
            self.checkAutoCorrect.setObjectName("checkAutoCorrect")

            self.radioStream = QRadioButton(self.groupParserInfo)
            self.radioStream.setObjectName("radioStream")

            self.radioFile = QRadioButton(self.groupParserInfo)
            self.radioFile.setObjectName("radioFile")

            self.lblFolderXml = QLabel(self.groupParserInfo)
            self.lblFolderXml.setObjectName("lblFolderXml")
            self.lblFolderXml.setAlignment(Qt.AlignLeft)    

            self.folderXml = myTextEdit(self.groupParserInfo)
            self.folderXml.initTextEdit(self.minWidth-310, 25, 10, 50, "folderXml", True, False, True, False, True)
            self.folderXml.setStyleSheet("color: black; background-color: #C0C0C0")

            self.buttonFolderXml = myPushButton(self.groupParserInfo)
            self.buttonFolderXml.initPushButton(24, 24, 660, 32 , "buttonFolderXml", "", "", True, getThemeIcon("folder.png"), 24, 24, True)

            self.groupEditorInfo = QGroupBox(collapsibleGroupBox)
            self.groupEditorInfo.setObjectName("groupEditorInfo")
            self.groupEditorInfo.setTitle("%s : " % (QApplication.translate("QSphere","Editor parameters")))

            self.checkSilentMode = QCheckBox(self.groupEditorInfo)
            self.checkSilentMode.setObjectName("checkSilentMode")
          
            self.checkReportingCswt = QCheckBox(self.groupEditorInfo)
            self.checkReportingCswt.setObjectName("checkReportingCswt")

            self.drawImg(self.groupEditorInfo)


          elif i == 3 :

            self.groupCswtInfo = QGroupBox(collapsibleGroupBox)
            self.groupCswtInfo.setObjectName("groupCswtInfo")

            self.lblListConnexionCswt = QLabel(self.groupCswtInfo)
            self.lblListConnexionCswt.setObjectName("LblListConnexionCSWT")
            self.lblListConnexionCswt.setAlignment(Qt.AlignRight)

            icon = getThemeIcon("actionsleft.png")
            self.listConnexionCswtButton = myPushButtonFunny(self.groupCswtInfo) 
            self.listConnexionCswtButton.initPushButton(40, 24,  0, 20, "listConnexionCswtButton", "", "", True, icon, 40, 24, True)
            
            self.connexionCswt = QComboBox(self.groupCswtInfo)
            self.connexionCswt.setEditable(False)
            self.connexionCswt.setObjectName("connexionCswt")

            self.connexionCswt.currentIndexChanged.connect(self.loadConnexion)
            
            self.defaultConnexionCswtButton = myPushButtonFunny(self.groupCswtInfo)
            icon = getThemeIcon("defaulttrue.png")
            self.defaultConnexionCswtButton.initPushButton(24, 24,  0, 20, "defaultConnexionCswtButton", "", "", True, icon, 24, 24, True)
            self.defaultConnexionCswtButton.clicked.connect(self.selConnexion)

            icon = getThemeIcon("new.png")
            self.newConnexionCswtButton = myPushButton(self.groupCswtInfo) 
            self.newConnexionCswtButton.initPushButton(24, 24,  0, 20, "newConnexionCswtButton", "", "", True, icon, 24, 24, True)
            self.newConnexionCswtButton.clicked.connect(self.newConnexion)

            icon = getThemeIcon("edit.png")
            self.editConnexionCswtButton = myPushButton(self.groupCswtInfo) 
            self.editConnexionCswtButton.initPushButton(24, 24,  0, 20, "editConnexionCswtButton", "", "", True, icon, 24, 24, True)
            self.editConnexionCswtButton.clicked.connect(self.loadConnexion)

            icon = getThemeIcon("del.png")
            self.deleteConnexionCswtButton = myPushButton(self.groupCswtInfo) 
            self.deleteConnexionCswtButton.initPushButton(24, 24,  0, 20, "deleteConnexionCswtButton", "", "", True, icon, 24, 24, True)
            self.deleteConnexionCswtButton.clicked.connect(self.delConnexion)

            self.groupConnexionInfo = QGroupBox(self.groupCswtInfo)
            self.groupConnexionInfo.setObjectName("groupConnexionInfo")

            #CSW-T Service
            self.lblNameConnexionCswt = QLabel(self.groupConnexionInfo)
            self.lblNameConnexionCswt.setObjectName("lblNameConnexionCswt")
            self.lblNameConnexionCswt.setAlignment(Qt.AlignLeft)

            self.nameConnexionCswt = MyWidgetLineEdit(self.groupConnexionInfo)
            self.nameConnexionCswt.initType(6)
            self.nameConnexionCswt.setObjectName("nameConnexionCswt")
            self.nameConnexionCswt.textChanged.connect(self.nameConnexionCswt.checkRegExp)

            icon = getThemeIcon("saveactions.png")
            self.buttonAddConnexionCswt = myPushButton(self.groupConnexionInfo) 
            self.buttonAddConnexionCswt.initPushButton(40, 40,  0, 20, "buttonAddConnexionCswt", "", "", True, icon, 40, 40, True)
            self.buttonAddConnexionCswt.clicked.connect(self.saveConnexion)

            self.lblServerCswt = QLabel(self.groupConnexionInfo)
            self.lblServerCswt.setObjectName("lblServerCswt")
            self.lblServerCswt.setAlignment(Qt.AlignLeft)    

            self.serverCswt = MyWidgetLineEdit(self.groupConnexionInfo)
            self.serverCswt.initType(40)
            self.serverCswt.setObjectName("serverCswt")
            self.serverCswt.textChanged.connect(self.serverCswt.checkRegExp)

            #BUTTON GETCAPABILITIES
            icon = getThemeIcon("cswtgetcapabilities.png")
            self.getCapabilitiesServerCswtButton = myPushButton(self.groupConnexionInfo) 
            self.getCapabilitiesServerCswtButton.initPushButton(40, 40,  0, 20, "getCapabilitiesServerCswtButton", "", "", True, icon, 40, 40, True)
            self.getCapabilitiesServerCswtButton.clicked.connect(self.getCapabilitiesCSWT)

            self.lblAuthentificationServerCswt = QLabel(self.groupConnexionInfo)
            self.lblAuthentificationServerCswt.setObjectName("lblAuthentificationServerCswt")
            self.lblAuthentificationServerCswt.setAlignment(Qt.AlignLeft)    

            self.authentificationServerCswt = MyWidgetLineEdit(self.groupConnexionInfo)
            self.authentificationServerCswt.initType(41)
            self.authentificationServerCswt.setObjectName("authentificationServerCswt")
            self.authentificationServerCswt.textChanged.connect(self.authentificationServerCswt.checkRegExp)

            icon = getThemeIcon("cswtusersrights.png")
            self.getAuthentificationServerCswtButton = myPushButton(self.groupConnexionInfo) 
            self.getAuthentificationServerCswtButton.initPushButton(40, 40,  0, 20, "getAuthentificationServerCswtButton", "", "", True, icon, 40, 40, True)
            self.getAuthentificationServerCswtButton.clicked.connect(self.loadUrl) 

            #User CSW-T Service
            self.lblUserServerCswt = QLabel(self.groupConnexionInfo)
            self.lblUserServerCswt.setObjectName("lblUserServerCswt")
            self.lblUserServerCswt.setAlignment(Qt.AlignLeft)    

            self.userCswt = MyWidgetLineEdit(self.groupConnexionInfo)
            self.userCswt.initType(6)
            self.userCswt.setObjectName("userCswt")
            self.userCswt.textChanged.connect(self.userCswt.checkRegExp)

            self.lblPwdUserCswt = QLabel(self.groupConnexionInfo)
            self.lblPwdUserCswt.setObjectName("lblPwdUserCswt")
            self.lblPwdUserCswt.setAlignment(Qt.AlignLeft)  

            self.pwdUserCswt = MyWidgetLineEdit(self.groupConnexionInfo)
            self.pwdUserCswt.initType(9)
            self.pwdUserCswt.setObjectName("pwdUserCswt")
            self.pwdUserCswt.textChanged.connect(self.pwdUserCswt.checkRegExp)

            self.checkViewPwdUserCswt = QCheckBox(self.groupConnexionInfo)
            self.checkViewPwdUserCswt.setObjectName("checkViewPwdUserCswt")

            self.drawImg(self.groupConnexionInfo)
            

          elif i == 4 :

            self.groupErgonomicalInfo = QGroupBox(collapsibleGroupBox)
            self.groupErgonomicalInfo.setObjectName("groupErgonomicalInfo")
            self.groupErgonomicalInfo.setStyleSheet("QGroupBox{background-color:qlineargradient(x1: 0, y1: 0, x2: 0.5, y2: 0.5, stop: 0 #FFFFBF, stop: 1 #FEB164); border:1px;}")

            self.lblStartColor = QLabel(self.groupErgonomicalInfo)
            self.lblStartColor.setObjectName("lblStartColor")
            self.lblStartColor.setAlignment(Qt.AlignRight)
            self.lblStartColor.setGeometry(10, 10, 200, 25)

            self.startColor = QgsColorButton(self.groupErgonomicalInfo, '', None)
            self.startColor.setObjectName("startColor")
            self.startColor.setGeometry(220, 10, 100, 25)
            self.startColor.colorChanged.connect(self.changeColor)

            self.lblStopColor = QLabel(self.groupErgonomicalInfo)
            self.lblStopColor.setObjectName("lblStopColor")
            self.lblStopColor.setAlignment(Qt.AlignRight)
            self.lblStopColor.setGeometry(10, 50, 200, 25)

            self.stopColor = QgsColorButton(self.groupErgonomicalInfo, '', None)
            self.stopColor.setObjectName("stopColor")
            self.stopColor.setGeometry(220, 50, 100, 25)
            self.stopColor.colorChanged.connect(self.changeColor)

            self.indexLang = self.parent.qspherePlugin.indexLang
            self.indexLang = 0
            
            self.langs_gui = myPushButtonMappable(self.groupErgonomicalInfo)
            icon = getThemeIcon("langs_gui_%s.png" % (self.parent.qspherePlugin.dicoLangs[self.parent.qspherePlugin.indexLang]))
            self.langs_gui.initPushButton(self, 278, 15, 220, 140, "langs_gui", "", "", True, icon, 278, 15, True)
            self.langs_gui.isMappable = True
            self.langs_gui.qspherePlugin = self.parent.qspherePlugin

            self.drawImg(self.groupErgonomicalInfo)
            
          else : pass  


    self.myPathPrint, self.myPathSave = "print.png", "save.png"
    self.myPathSizeUp, self.myPathSizeDown = "sizeup.png", "sizedown.png"
    
    self.printButton = myPushButton(self)
    self.printButton.initPushButton(24, 24, 5, 5, "printButton", "", "", True, getThemeIcon(self.myPathPrint), 24, 24, True)
    self.printButton.setShortcut(QKeySequence("Ctrl+P"))

    self.saveButton = myPushButton(self)
    self.saveButton.initPushButton(24, 24, 5, 5, "saveButton", "", "", True, getThemeIcon(self.myPathSave), 24, 24, True)
    self.saveButton.setShortcut(QKeySequence("Ctrl+S"))

    icon = getThemeIcon("qspherehelp.png")
    self.HelpButton = myPushButton(self) 
    self.HelpButton.initPushButton(48, 48, -50, -50, "HelpButton", "", "", True, icon, 48, 48, True)
    self.HelpButton.setShortcut(QKeySequence("F1")) 

    self.barInfo = QgsMessageBar(self)
    self.barInfo.setSizePolicy( QSizePolicy.Minimum, QSizePolicy.Fixed )

    self.movieCtrl = movieCtrl(self, "sablier.gif")

    self.closeButton = QPushButton(self)
    self.closeButton.setObjectName("closeButton")

    self.sliderInformation.valueChanged.connect(self.setVisibleValue)
    self.sliderWarning.valueChanged.connect(self.setVisibleValue)
    self.sliderTimeOut.valueChanged.connect(self.setVisibleValue)

    self.sliderInformation.setValue(2)
    self.sliderWarning.setValue(5)
    self.sliderTimeOut.setValue(5)

    self.checkAutoCorrect.clicked.connect(self.enabledCtrl)
    self.checkViewPwdUserCswt.clicked.connect(self.enabledPwd)
    self.printButton.clicked.connect(self.printFileOptions)
    self.folderButton.clicked.connect(self.getFolderSearchFiles)
    self.buttonFolderXml.clicked.connect(self.getFolderSearchFiles)
    self.saveButton.clicked.connect(self.saveOptions)
    self.closeButton.clicked.connect(self.close)
    self.connexionCswt.currentIndexChanged.connect(self.setImageDefault)
    self.HelpButton.clicked.connect(self.clickHelp)


    self.changeLang()
    self.loadOptions()
    self.enabledCtrl()
    self.enabledPwd()

    key = getSelConnexion(self) 
    self.FixeCurrentConnexion(key)


  def drawImg(self, obj):
      name  = obj.objectName()
      icon = getThemeIcon("options_%s.png" % self.images[name])
      icon = QImage(abspath(icon).replace("\\", "/"))
      img = QLabel(obj)
      img.setPixmap(QPixmap.fromImage(icon))
      titleImg = "img%s" % name
      img.setObjectName(titleImg)

      setattr(self, titleImg, img)
      

  def changeLang(self):
      self.qspherePlugin.changeLang(True)
      lang = self.qspherePlugin.dicoLangs[self.qspherePlugin.indexLang]
      self.indexLang = self.qspherePlugin.indexLang
      self.langs_gui.setIcon(QIcon(getThemeIcon("langs_gui_%s.png" % (lang))))
      windowProperties(self, self.parent)

      self.setWindowTitle("%s" % (QApplication.translate("QSphere", "QSphere options")))
      self.msgItem = QApplication.translate("QSphere","Item")
    
      i = 0
      for collapsibleGroupBox in self.mddcollapsibleGroupBox :
        collapsibleGroupBox.setTitle("%s : " % (QApplication.translate("QSphere", self.title[i])))
        i+= 1

      self.groupBoxServer.setTitle(QApplication.translate("QSphere","Server parameters : "))
      self.lblServerMetadata.setText("%s" % (QApplication.translate("QSphere","Url Server for metadata : ")))
      self.lblServerMetadata.setAccessibleDescription(self.lblServerMetadata.text())
      self.serverMetadataButton.setToolTip(QApplication.translate("QSphere","Actions ..."))
      self.lblServerKeywords.setText("%s" % (QApplication.translate("QSphere","Url Server for keywords : ")))
      self.lblServerKeywords.setAccessibleDescription(self.lblServerKeywords.text())
      self.serverKeywordsButton.setToolTip(QApplication.translate("QSphere","Actions ..."))
      self.lblServerNavigator.setText("%s" % (QApplication.translate("QSphere","Url for navigator : ")))
      self.lblServerNavigator.setAccessibleDescription(self.lblServerNavigator.text())
      self.serverNavigatorButton.setToolTip(QApplication.translate("QSphere","Actions ..."))

      self.groupBoxFolders.setTitle(QApplication.translate("QSphere","Init Directory (open, save files) : "))
      self.lblFolder.setText(QApplication.translate("QSphere","Folder : "))
      self.folderButton.setToolTip(QApplication.translate("QSphere","Select a Folder"))
      self.checkAutoComment.setText(QApplication.translate("QSphere","Don't erase project's comments with associative mode"))
      self.checkAutoComment.setToolTip(QApplication.translate("QSphere","Don't erase project's comments with associative mode"))
      self.groupBoxDuration.setTitle(QApplication.translate("QSphere","Fix duration for process (Web queries, QgsMessageBox)"))
      self.checkShowWarning.setText("%s." % QApplication.translate("QSphere","Always show warning messages"))
      self.checkShowWarning.setToolTip(QApplication.translate("QSphere","Always show warning messages"))
      self.lblSliderInfo.setText(QApplication.translate("QSphere","Duration for information message : "))
      self.lblSliderWarning.setText(QApplication.translate("QSphere","Duration for warning message : "))
      self.lblSliderTimeOut.setText(QApplication.translate("QSphere","TimeOut for testing URL : "))

      self.groupParserInfo.setTitle("%s : " % (QApplication.translate("QSphere","XML Parser parameters")))            
      self.checkAutoCorrect.setText(QApplication.translate("QSphere","Auto-correction mode"))
      self.radioStream.setText(QApplication.translate("QSphere","working with stream"))
      self.radioFile.setText(QApplication.translate("QSphere","working with file"))
      self.lblFolderXml.setText("%s : " % (QApplication.translate("QSphere","Folder")))
      self.buttonFolderXml.setToolTip(QApplication.translate("QSphere","Select a Folder"))
      self.checkSilentMode.setText(QApplication.translate("QSphere","Silent mode (unrespected metadata iso XML standard)"))
      self.checkReportingCswt.setText(QApplication.translate("QSphere","Opening report for CSW-T transactions"))
      self.groupCswtInfo.setTitle("%s : " % (QApplication.translate("QSphere","Service CSW-T parameters")))
      self.lblListConnexionCswt.setText("%s : " % (QApplication.translate("QSphere","List of connections")))
      self.listConnexionCswtButton.setToolTip("%s ..." % (QApplication.translate("QSphere","Actions")))

      self.defaultConnexionCswtButton.setToolTip(QApplication.translate("QSphere","Select this connection as active connection"))
      self.newConnexionCswtButton.setToolTip(QApplication.translate("QSphere","New connection"))
      self.editConnexionCswtButton.setToolTip(QApplication.translate("QSphere","Edit the connection"))
      self.deleteConnexionCswtButton.setToolTip(QApplication.translate("QSphere","Delete the connection"))
      self.lblNameConnexionCswt.setText("%s : " % (QApplication.translate("QSphere","Name for connection to CSW-T server")))
      self.buttonAddConnexionCswt.setToolTip(QApplication.translate("QSphere","Add or update the connection"))
      self.lblServerCswt.setText("%s : " % (QApplication.translate("QSphere","Url Server for CSW-T")))
      self.getCapabilitiesServerCswtButton.setToolTip("%s %s" % (QApplication.translate("QSphere","Get capabilities for the server"), "CSW-T"))
      self.lblAuthentificationServerCswt.setText("%s : " % (QApplication.translate("QSphere","Url Server for CSW-T Authentification")))
      self.getAuthentificationServerCswtButton.setToolTip(QApplication.translate("QSphere","Test URL for authentification to the server CSW-T"))
      self.lblUserServerCswt.setText("%s : " % (QApplication.translate("QSphere","User for CSW-T")))
      self.lblPwdUserCswt.setText("%s : " % (QApplication.translate("QSphere","Password for the connection")))
      self.checkViewPwdUserCswt.setText(QApplication.translate("QSphere","View the password"))


      self.lblStartColor.setText("%s :" % QApplication.translate("QSphere","Start color"))
      self.lblStopColor.setText("%s :" % QApplication.translate("QSphere","Stop color"))
      self.groupErgonomicalInfo.setTitle("%s : " % (QApplication.translate("QSphere","Color and lang parameters")))

      self.printButton.setToolTip(QApplication.translate("QSphere", "Print"))
      self.saveButton.setToolTip(QApplication.translate("QSphere", "Save"))
      self.closeButton.setText(QApplication.translate("QSphere", "Close"))

      self.contextMnuActionsConnexions("connexionCswt", self.listConnexionCswtButton)
      self.contextMnuActions("ServerMetadata", self.serverMetadataButton)
      self.contextMnuActions("ServerKeywords", self.serverKeywordsButton)
      self.contextMnuActions("ServerNavigator", self.serverNavigatorButton)
    

  def changeColor(self):
     self.groupErgonomicalInfo.setStyleSheet("QGroupBox{background-color:qlineargradient(x1: 0, y1: 0, x2: 0.5, y2: 0.5, stop: 0 %s, stop: 1 %s); border:1px;}" % (self.startColor.color().name(), self.stopColor.color().name())) 

  def changePosGroups(self):
         posH, newHeight, i = 0, 0, 0
         refCollapsibleGroupBox = self.sender()
         for mddcollapsibleGroupBox in self.mddcollapsibleGroupBox:
              mddcollapsibleGroupBox.blockSignals(True)
              if mddcollapsibleGroupBox != refCollapsibleGroupBox : mddcollapsibleGroupBox.setCollapsed(True)
              else : mddcollapsibleGroupBox.setCollapsed(False)
              mddcollapsibleGroupBox.blockSignals(False)
              mddcollapsibleGroupBox.setMaximumWidth(self.width()-20)
              mddcollapsibleGroupBox.setMinimumWidth(self.width()-20)                         
              mddcollapsibleGroupBox.setGeometry(5, posH, self.width()-20, mddcollapsibleGroupBox.height())

              posH+= mddcollapsibleGroupBox.height() if not mddcollapsibleGroupBox.isCollapsed() else 15
              newHeight+= 15 + mddcollapsibleGroupBox.height() if not mddcollapsibleGroupBox.isCollapsed() else 15
              i+= 1
       
         
  def clickHelp(self): getHelp(self)

  def enabledCtrl(self):
      cond = self.checkAutoCorrect.isChecked()
      self.radioStream.setEnabled(cond)
      self.radioFile.setEnabled(cond)
      self.buttonFolderXml.setEnabled(cond)
      
  def enabledPwd(self):
      cond = self.checkViewPwdUserCswt.isChecked()
      self.pwdUserCswt.setEchoMode(QLineEdit.Normal) if cond else self.pwdUserCswt.setEchoMode(QLineEdit.Password)

  def setImageDefault(self):
      cond  = (getSelConnexion(self) == self.connexionCswt.currentText())
      iconName = "default%s.png" % (cond)
      iconName = getThemeIcon(iconName.lower())
      icon = QIcon(iconName)
      self.defaultConnexionCswtButton.setIcon(icon) 


  def contextMnuActionsConnexions(self, nameObj, actionButton):
      contextMnu_Actions = QMenu()

      menuIcon = getThemeIcon("importmetasearch.png")
      toolTip = "%s ..." % (QApplication.translate("QSphere","Import from MetaSearch"))
      self.importFromMetaSearch = QAction(QIcon(menuIcon), toolTip, self)
      self.importFromMetaSearch.setObjectName("import%s" % (nameObj))
      contextMnu_Actions.addAction(self.importFromMetaSearch)
      self.importFromMetaSearch.triggered.connect(self.selImportConnexion)
      
      menuIcon = getThemeIcon("open.png")
      toolTip = "%s ..." % (QApplication.translate("QSphere","Load from XML File"))
      self.loadFromXMLFile = QAction(QIcon(menuIcon), toolTip, self)
      self.loadFromXMLFile.setObjectName("load%s" % (nameObj))
      contextMnu_Actions.addAction(self.loadFromXMLFile)
      self.loadFromXMLFile.triggered.connect(self.getConnexionsFromXMLFile)

      contextMnu_Actions.addSeparator()

      menuIcon = getThemeIcon("save.png")
      toolTip = "%s ..." % (QApplication.translate("QSphere","Save to XML File"))
      self.saveToXMLFile = QAction(QIcon(menuIcon), toolTip, self)
      self.saveToXMLFile.setObjectName("save%s" % (nameObj))
      contextMnu_Actions.addAction(self.saveToXMLFile)
      self.saveToXMLFile.triggered.connect(self.saveConnexionsInXMLFile)

      contextMnu_Actions.addSeparator()
      
      menuIcon = getThemeIcon("deleteall.png")
      toolTip = "%s ..." % (QApplication.translate("QSphere","Delete all entries"))
      self.deleteAllItems = QAction(QIcon(menuIcon), toolTip, self)
      self.deleteAllItems.setObjectName("deleteallitems%s" % (nameObj))
      contextMnu_Actions.addAction(self.deleteAllItems)
      self.deleteAllItems.triggered.connect(self.deleteAll)      

      actionButton.setMenu(contextMnu_Actions)


  def contextMnuActions(self, nameObj, actionButton):
      contextMnuActions = QMenu()

      menuIcon = getThemeIcon("add.png")
      action = QAction(QIcon(menuIcon), QApplication.translate("QSphere","Add the server"), self)
      action.setObjectName("add%s" % (nameObj))
      contextMnuActions.addAction(action)
      action.triggered.connect(self.addItem)

      menuIcon = getThemeIcon("del.png")
      action = QAction(QIcon(menuIcon), QApplication.translate("QSphere","Delete the server"), self)
      action.setObjectName("del%s" % (nameObj))
      contextMnuActions.addAction(action)
      action.triggered.connect(self.delItem)

      contextMnuActions.addSeparator()

      menuIcon = getThemeIcon("moveup.png")
      action = QAction(QIcon(menuIcon), QApplication.translate("QSphere","Move the line up"), self)
      action.setObjectName("moveup%s" % (nameObj))
      contextMnuActions.addAction(action)
      action.triggered.connect(self.moveItemUp)

      menuIcon = getThemeIcon("movedown.png")
      action = QAction(QIcon(menuIcon), QApplication.translate("QSphere","Move the line down"), self)
      action.setObjectName("movedown%s" % (nameObj))
      contextMnuActions.addAction(action)
      action.triggered.connect(self.moveItemDown)


      contextMnuActions.addSeparator()

      menuIcon = getThemeIcon("urlvalid.png")
      action = QAction(QIcon(menuIcon), QApplication.translate("QSphere","Test the URL"), self)
      action.setObjectName("testor%s" % (nameObj))
      contextMnuActions.addAction(action)
      action.triggered.connect(self.testUrl)

      menuIcon = getThemeIcon("urllocalisator.png")
      action = QAction(QIcon(menuIcon), QApplication.translate("QSphere","Go to the URL"), self)
      action.setObjectName("open%s" % (nameObj))
      contextMnuActions.addAction(action)
      action.triggered.connect(self.loadUrl)
         
      actionButton.setMenu(contextMnuActions)
     

  def deleteAll(self):
      msgDlgBox = myMsgDlgBox(self, QMessageBox.Question, QApplication.translate("QSphere","Warning"),
                                                QApplication.translate("QSphere","Delete all the connections ?"), "qsphere_stop.png", None, self.parent)
      windowIcon(msgDlgBox , "qsphereoptions.png")
      ret = msgDlgBox.exec_()
      if ret == 1 :
          settings = QSettings()
          settings.beginGroup("/qsphere/connections/")
          keys = settings.childGroups()
          if keys != []:
             settings.setValue("/qsphere/connections/selected", "")
             for key in keys :
                 settings = QSettings()
                 settings.remove("/qsphere/connections/%s" % (key))
             self.connexionCswt.clear()
             countItems(self, self.connexionCswt.objectName(), self.connexionCswt)
          
          
  def setVisibleValue(self):
      value = "%s" % (self.sender().value())
      self.sender().setToolTip(value)
      if self.sender() == self.sliderInformation : self.txtSliderINFO.setText(value)
      elif self.sender() == self.sliderWarning : self.txtSliderWARNING.setText(value)
      elif self.sender() == self.sliderTimeOut : self.txtSliderTIMEOUT.setText(value)
      else : pass
     
  def loadOptions(self):
      optionsFile = join(dirname(__file__),"ressources/options.ini")
      if exists(optionsFile):

         config = ConfigParser()
         config.read(optionsFile)
         sections = config.sections()

         for section in sections :
             try : obj = getWidget(self, section)
             except : obj = None
             
             if obj !=None :
                if type(obj) in (myWebComboBox, myUserComboBox) :
                    infos = config.get(section,'index')
                    objIndex = int(infos.rstrip())
                    itemsInfos = config.get(section,'items')
                    itemsCount = int(itemsInfos.rstrip())
                    obj.clear()
                    for k in range(itemsCount): obj.addItem(config.get(section,'item_%s' % (k)))
                    obj.setCurrentIndex(objIndex)
                    countItems(self, obj.objectName(), obj)

                elif type(obj) == mySlider :
                    infos = config.get(section,'value')
                    obj.setValue(int(infos))
                    
                elif type(obj) in (QRadioButton, QCheckBox) :
                     infos = config.get(section,'state').lower().strip()
                     obj.setChecked(infos=='true')

                elif type(obj) == myTextEdit :
                    initFolder = config.get(section,'initFolder')
                    obj.setPlainText(dirname(__file__)) if initFolder == "" or not (exists(initFolder)) else obj.setPlainText(initFolder)

                elif type(obj) == QgsColorButton :
                    color = config.get(section,'name')
                    obj.setColor(QColor(color))   


  def newConnexion(self):
      self.nameConnexionCswt.setText("")
      self.serverCswt.setText("")
      self.authentificationServerCswt.setText("")
      self.userCswt.setText("")
      self.pwdUserCswt.setText("")

  def selImportConnexion(self):
      try :
        metaSearchSettings = QSettings()
        metaSearchSettings.beginGroup('/MetaSearch/')
        keys = metaSearchSettings.childGroups()
        addkeys = []
        if keys != []:
           for key in keys :
               if self.connexionCswt.findText("%s" % (key),  Qt.MatchFixedString) == -1 : 
                  self.newConnexion()
                  self.nameConnexionCswt.setText(key)
                  self.serverCswt.setText(metaSearchSettings.value('%s/url' % key))
                  self.saveConnexion(True)
                  addkeys.append(key)

           title = QApplication.translate("QSphere","Information")
           msg = QApplication.translate("QSphere","Added URL")
           sendMsg(self, "%s :" % (title), "%s (%s)<br>%s" % (msg, len(addkeys), addkeys), Qgis.Info, self.sliderInformation.value())
      except :
           title = QApplication.translate("QSphere","Information")
           msg = QApplication.translate("QSphere","Import stopped.")
           sendMsg(self, "%s :" % (title), "<h1>%s</h1>" % (msg), Qgis.Info, self.sliderInformation.value())
      
  def saveConnexion(self, *args):
      if self.nameConnexionCswt.text() == "" :
          title = QApplication.translate("QSphere","Information")
          msg = QApplication.translate("QSphere","The connection must have a name !")
          sendMsg(self, title , "<b>%s</b>" % (msg), Qgis.Info, self.sliderInformation.value())
          return

      settings = QSettings()
      key = self.nameConnexionCswt.text()        
      index, newItem = self.findkey(key)

      msg =  QApplication.translate("QSphere","Add the connection") if newItem else QApplication.translate("QSphere","Update the connection")
   
      settings.setValue("/qsphere/connections/", key) 
      settings.setValue("/qsphere/connections/%s/host" % (key), self.serverCswt.text())
      settings.setValue("/qsphere/connections/%s/authhost" % (key), self.authentificationServerCswt.text())
      settings.setValue("/qsphere/connections/%s/username" % (key), self.userCswt.text())
      settings.setValue("/qsphere/connections/%s/password" % (key),  self.pwdUserCswt.text())

      if args[0] == False :
        title = QApplication.translate("QSphere","Information")
        msg = "%s<br>%s<br>%s" % (msg, key, QApplication.translate("QSphere","Successfull"))
        sendMsg(self, "%s :" % (title), msg, Qgis.Info, self.sliderInformation.value())

      self.FixeCurrentConnexion(key)
      countItems(self, self.connexionCswt.objectName(), self.connexionCswt)
      if newItem : self.connexionCswt.setCurrentIndex(self.connexionCswt.count()-1)

  def FixeCurrentConnexion(self, key):
      self.loadConnexions()
      try : index = self.connexionCswt.findText(key)
      except : index = self.connexionCswt.currentIndex()
      try : self.connexionCswt.setCurrentIndex(index)
      except : pass

  def loadConnexions(self):
      self.connexionCswt.clear()
      settings = QSettings()
      try : 
          settings.beginGroup("/qsphere/connections/")
          keys = settings.childGroups()
          if keys != []:
             for key in keys : self.connexionCswt.addItem(key)
          countItems(self, self.connexionCswt.objectName(), self.connexionCswt)   
      except : pass

  def loadConnexion(self):
      if self.connexionCswt.currentIndex()==-1: return
      settings = QSettings()
      key = self.connexionCswt.currentText()
      self.nameConnexionCswt.setText(key)
      self.serverCswt.setText(settings.value("/qsphere/connections/%s/host" % (key)))
      self.authentificationServerCswt.setText(settings.value("/qsphere/connections/%s/authhost" % (key)))
      self.userCswt.setText(settings.value("/qsphere/connections/%s/username" % (key)))
      self.pwdUserCswt.setText(settings.value("/qsphere/connections/%s/password" % (key)))

  def getCapabilitiesCSWT(self):
      connexion = cswtConnection(self, self.nameConnexionCswt.text(), self.serverCswt.text(), self.authentificationServerCswt.text())
      try :
        responseCswt = connexion.getCapabilities()

        if responseCswt != None and self.checkReportingCswt.isChecked() :
            d = xmlEditor(self, "", self)
            windowIcon(d, "editxml.png")
            d.setWindowTitle("%s : %s" % (d.rootWindowTitle, ""))
            d.show()
            d.move(self.x()+100, self.y()+100)

            try : charset = (responseCswt.headers["Content-Type"].split("=")[1].strip())
            except : charset = "utf-8"  
            
            d.editor.setText("%sDATA RESPONSE :\n%s" % (connexion.reporting(), responseCswt.text)) 
            d.checkProperties()
            msgBadFileXml(d)
      except :
                title = QApplication.translate("QSphere","Warning")
                msg = "%s :<br>%s [%s] " % (QApplication.translate("QSphere","Unable to establish connection to"),
                                            self.serverCswt.text(), QApplication.translate("QSphere","HTTP error"))
                sendMsg(self, title , msg, Qgis.Warning, self.sliderWarning.value())

  def selConnexion(self):
      if self.connexionCswt.currentText()=="" : return
      if getSelConnexion(self)==self.connexionCswt.currentText() : return
      settings = QSettings()
      settings.setValue("/qsphere/connections/selected", self.connexionCswt.currentText())
      self.setImageDefault()
      if self.parent != None :
          try :
            if self.parent.objectName() == "dlgMetadata" :
                  self.parent.disconnectCSWT()
          except : pass

  def delConnexion(self):
      oldkey = self.connexionCswt.currentText()
      msgDlgBox = myMsgDlgBox(self, QMessageBox.Question, QApplication.translate("QSphere","Warning"),
                              "%s<br><b>%s</b>" % (QApplication.translate("QSphere","Delete the current connection ?"), self.connexionCswt.currentText()), "qsphere_stop.png", None, self.parent)
      windowIcon(msgDlgBox , "qsphereoptions.png")
      ret = msgDlgBox.exec_()
      if ret == 1 :
          settings = QSettings()
          settings.remove("/qsphere/connections/%s" % (oldkey))
          newsel = (getSelConnexion(self)==oldkey)
          index = self.connexionCswt.currentIndex()-1 if self.connexionCswt.currentIndex()>0 else 0
          self.loadConnexions()
          if newsel : settings.setValue("/qsphere/connections/selected", self.connexionCswt.currentText())
          countItems(self, self.connexionCswt.objectName(), self.connexionCswt)
          self.connexionCswt.setCurrentIndex(index)
          

  def saveConnexionsInXMLFile(self):

      title = QApplication.translate("QSphere","Save to XML File")
      self.InitDir = dirname(__file__) if self.InitDir == "" else self.InitDir
      
      dlgFile = QFileDialog(self, title)
      dlgFile.setDefaultSuffix("xml")
      dlgFile.setNameFilters((self.labelXmlFiles,))
      dlgFile.selectNameFilter(self.labelXmlFiles)
      dlgFile.setViewMode(QFileDialog.Detail)
      dlgFile.setDirectory(self.InitDir)
      dlgFile.setAcceptMode(QFileDialog.AcceptSave)

      fixeLabelsFileDialog(self, dlgFile, 1, True)
      
      if dlgFile.exec_():
          fileName = fileNameWithExtension(self, dlgFile.selectedFiles()[0], dlgFile.selectedNameFilter())
          self.initDir = dirname(fileName)

          doc = etree.Element('qgsCSWConnections')
          doc.attrib['version'] = '1.0'

          settings = QSettings()
          settings.beginGroup("/qsphere/connections/")
          keys = settings.childGroups()
          if keys != []:
             for key in keys :
                url = settings.value("%s/host" % (key))
                
                if url is not None :
                   connection = etree.SubElement(doc, 'csw')
                   connection.attrib['name'] = key
                   connection.attrib['url'] = url
                   connection.attrib['authurl'] = settings.value("%s/authhost" % (key))

          with open(fileName, 'w') as fileobj: fileobj.write(prettifyXml(etree.tostring(doc)))
          
          title = QApplication.translate("QSphere","Information")
          msg = QApplication.translate("QSphere","The file xml was saved as")
          sendMsg(self, title , "%s :<br>%s" % (msg, fileName), Qgis.Info, self.sliderInformation.value())          

  def getConnexionsFromXMLFile(self):
        title = QApplication.translate("QSphere", "Load from XML File")
        self.InitDir = dirname(__file__) if self.InitDir == "" else self.InitDir
        
        dlgFile = QFileDialog(self, title)
        dlgFile.setNameFilters((self.labelXmlFiles,))  
        dlgFile.setViewMode(QFileDialog.Detail)
        dlgFile.setDirectory(self.InitDir)
        dlgFile.setFileMode(QFileDialog.ExistingFile) 
        dlgFile.setAcceptMode(QFileDialog.AcceptOpen)

        fixeLabelsFileDialog(self, dlgFile, 0, True)        
        
        if dlgFile.exec_():
           fileName = "%s" % (dlgFile.selectedFiles()[0])
           error = 0
           try:
               doc = etree.parse(fileName).getroot()
               if doc.tag != 'qgsCSWConnections':
                  error = 1
                  msg = QApplication.translate("QSphere", "Invalid CSW connections XML")
                  err = QApplication.translate("QSphere","Root qgsCSWConnections not found")
           except (etree.ParseError, err):
               error = 1
               msg = QApplication.translate("QSphere", "Cannot parse XML file") 
           except (IOError, err):
               error = 1
               msg = QApplication.translate("QSphere", "Cannot open file") 

           if error == 1:
               title = QApplication.translate("QSphere","Warning")
               sendMsg(self, "%s :" % (title), "%s :<br>%s." % (msg, err), Qgis.Warning, self.sliderWarning.value())
               return

           addkeys = []
           for csw in doc.findall('csw'):
               key = csw.attrib.get('name')
               if self.connexionCswt.findText("%s" % (key),  Qt.MatchFixedString) == -1 : 
                  self.newConnexion()
                  self.nameConnexionCswt.setText(key)
                  try : self.serverCswt.setText(csw.attrib.get('url'))
                  except : self.serverCswt.setText("")
                  try : self.authentificationServerCswt.setText(csw.attrib.get('authurl'))
                  except : self.authentificationServerCswt.setText("")
                  self.saveConnexion(True)
                  addkeys.append(key)
           if len(addkeys) > 0 :     
               title = QApplication.translate("QSphere","Information")
               msg = QApplication.translate("QSphere","Added URL")
               sendMsg(self, "%s :" % (title), "%s (%s)<br>%s" % (msg, len(addkeys), addkeys), Qgis.Info, self.sliderInformation.value())
          
         
  def findkey(self, targetKey):
      settings = QSettings()
      settings.beginGroup("/qsphere/connections")
      keys = settings.childGroups()
      i, newitem = 0, True
      if keys != []:
         for key in keys :
             if targetKey == key :
                newitem = False
                break
             i+= 1 
      return i, newitem

  def saveOptions(self):
      optionsFile = join(dirname(__file__),"ressources/options.ini")
      inFile = open(optionsFile, "w")
      config = ConfigParser()
      config.add_section("options")
      config.set("options",'date', datetime.now().strftime("%d/%m/%Y %Hh%Mm%Ss"))
      config.set("options",'mandatoryPlugin',"QSphere")
      config.set("options",'version', self.version)


      for mddcollapsibleGroupBox in self.mddcollapsibleGroupBox:
             objs = mddcollapsibleGroupBox.children()          
             for child in objs :
                if type(child) == QGroupBox :
                  
                   for subchild in child.children() :
                       if  type(subchild) in (myWebComboBox, myUserComboBox, myTextEdit, mySlider, QRadioButton, QCheckBox, QgsColorButton) :
                           
                           objName = subchild.objectName()
                           config.add_section(objName)
                           objClassName = "%s" % (subchild.metaObject().className())
                           config.set(objName, 'type', objClassName)

                           if type(subchild) in (myWebComboBox, myUserComboBox) :
                               config.set(objName,'items',"%s" % subchild.count())
                               config.set(objName,'index',"%s" % subchild.currentIndex())
                               for k in range(subchild.count()): config.set(objName,'item_%s' % (k), "%s" % (subchild.itemText(k)))
                               
                           elif type(subchild) == mySlider : config.set(objName,'value',"%s" % subchild.value())

                           elif type(subchild) in (QRadioButton, QCheckBox) : config.set(objName, "state","%s" % (subchild.isChecked()))

                           elif type(subchild) == myTextEdit :
                                config.set(objName,'initFolder',subchild.toPlainText())

                           elif type(subchild) == QgsColorButton :  config.set(objName,'name',"%s" % subchild.color().name())   

      config.write(inFile)
      if inFile != None : inFile.close()

      title = QApplication.translate("QSphere","Information")
      msg = QApplication.translate("QSphere","The file ini was saved as")
      sendMsg(self, title , "%s :<br>%s" % (msg, optionsFile), Qgis.Info, self.sliderInformation.value())

      if self.parent != None :
         if self.parent.objectName() == "qsphere" :
            getOptions(self.parent)
            if self.parent.mainWindow != None :
               getOptions(self.parent.mainWindow)
               self.parent.mainWindow.makeMajGetOptions()


  def getFolderSearchFiles(self):
      if self.sender() == None : return
      if self.sender()== self.folderButton : obj = self.folder
      elif self.sender()== self.buttonFolderXml : obj = self.folderXml
      else : return
      
      InitDir = dirname(__file__) if obj.toPlainText()=="" else dirname(obj.toPlainText())
      title = QApplication.translate("QSphere", "Select the folder")
      
      dlgFile = QFileDialog(self, title)
      dlgFile.setDirectory(InitDir)
      dlgFile.setFileMode(QFileDialog.DirectoryOnly)

      fixeLabelsFileDialog(self, dlgFile, 0, False)      

      if dlgFile.exec_():
         inputDir = dlgFile.selectedFiles()[0] 
         obj.setPlainText(correctPath(inputDir.replace("\\","/")))

  def fixeItem(self, txtProperty):   
      if not txtProperty :
         if self.sender().objectName().find("ServerMetadata")!=-1 : comboBox = self.serverMetadata
         elif self.sender().objectName().find("ServerKeywords")!=-1 : comboBox = self.serverKeywords
         elif self.sender().objectName().find("ServerNavigator")!=-1 : comboBox = self.serverNavigator
         else : comboBox == None
         return comboBox
      else:
         if self.sender().objectName().find("ServerMetadata")!=-1 : url = self.serverMetadata.currentText()
         elif self.sender().objectName().find("ServerKeywords")!=-1 : url = self.serverKeywords.currentText()
         elif self.sender().objectName().find("ServerNavigator")!=-1 : url = self.serverNavigator.currentText()
         elif self.sender().objectName().find("GetAuthentificationServerCswt")!=-1 : url = self.authentificationServerCswt.text()
         elif self.sender().objectName().find("GetCapabilitiesCswt")!=-1 : url = self.serverCswt.currentText()
         else : url = "#blank"
         return url 

  def addItem(self):
      comboBox = self.fixeItem(False)
      if comboBox == None : return
      if comboBox.findText(comboBox.currentText()) == -1:
          comboBox.addItem(comboBox.currentText())
          title = QApplication.translate("QSphere","Information")
          msg = QApplication.translate("QSphere","added to the list !")
          sendMsg(self, title , "%s :<br><u><i>%s</i></u><br>%s" % (self.msgItem, comboBox.currentText(), msg), Qgis.Info, self.sliderInformation.value())
          countItems(self, comboBox.objectName(), comboBox)
          comboBox.setCurrentIndex(comboBox.count()-1)
      else :
          title = QApplication.translate("QSphere","Warning")
          msg = QApplication.translate("QSphere","already found in the list !")
          sendMsg(self, title , "%s :<br><u><i>%s</i></u><br>%s" % (self.msgItem, comboBox.currentText(), msg), Qgis.Warning, self.sliderWarning.value())

  def delItem(self):
      comboBox = self.fixeItem(False)
      if comboBox == None : return
      if comboBox.currentIndex()!=-1 :
         item = comboBox.currentText()
         index = comboBox.currentIndex()-1 if comboBox.currentIndex()>0 else 0
         comboBox.removeItem(comboBox.currentIndex())   
         title = QApplication.translate("QSphere","Information")
         msg = QApplication.translate("QSphere","removed the list !")
         sendMsg(self, title , "%s :<br><u><i>%s</i></u><br>%s" % (self.msgItem, item, msg), Qgis.Info, self.sliderInformation.value())
         countItems(self, comboBox.objectName(), comboBox)
         comboBox.setCurrentIndex(index)

  def moveItemDown(self):
      comboBox = self.fixeItem(False)
      if comboBox == None : return
      index = comboBox.currentIndex()
      if index <= comboBox.count()-1:
         text = comboBox.itemText(index)
         comboBox.insertItem(index+2, text)
         comboBox.removeItem(index)
         index = comboBox.findText(text)
         comboBox.setCurrentIndex(index)
         comboBox.showPopup()

  def moveItemUp(self):
      comboBox = self.fixeItem(False)
      if comboBox == None : return
      index = comboBox.currentIndex()
      if index >= 1 :
         text = comboBox.itemText(index)
         comboBox.insertItem(index-1, text)
         comboBox.removeItem(index+1)
         index = comboBox.findText(text)
         comboBox.setCurrentIndex(index)
         comboBox.showPopup()

  def loadUrl(self):
      from qsphere.ui import loadDlgNavigator
      url = self.fixeItem(True)
      self.webNavigator = loadDlgNavigator(self, self.iface, url, False, [], self.localLangage, self.parent, None, "navigatorweb.png", True)
      
  def testUrl(self):
      self.movieCtrl.startMovie()
      url = self.fixeItem(True)
      sites = [('%s' % (url), 'GET', {'User-Agent': 'Mozilla/5.0', 'Accept': '*/*'}, 'None')] 
      try : isValidUrl = multiGetRequest(self.movieCtrl.movie, self.movieCtrl, sites, self.sliderTimeOut.value(), 0)[0][1]
      except : isValidUrl = None

      if isValidUrl != None :
         title = QApplication.translate("QSphere", "Information" ) 
         textIsValid = QApplication.translate("QSphere", "Valid")
         img = Qgis.Info
         duration = self.sliderInformation.value()
      else :
         title = QApplication.translate("QSphere", "Warning" ) 
         textIsValid = QApplication.translate("QSphere", "Not valid")
         img = Qgis.Warning
         duration = self.sliderWarning.value()
      msg = "%s :<br><u><i>%s</i></u><br>%s." % (QApplication.translate("QSphere", "Current URL"), url, textIsValid)
      sendMsg(self, title, msg, img, duration)
      self.movieCtrl.stopMovie()

      return isValidUrl

    
  def resizeEvent(self,ev):
     self.changePosGroups()
     
     self.printButton.setGeometry(self.width()-240, self.height()-30, 25, 25) 
     self.saveButton.setGeometry(self.width()-190, self.height()-30, 25, 25)
     self.closeButton.setGeometry(self.width()-120, self.height()-30, 100, 25)
     
     self.groupBoxServer.setGeometry(10, 20, self.width()-40, self.height()-120)
     
     self.lblServerMetadata.setGeometry(10, 25, self.width()-165, 25)
     self.serverMetadata.setGeometry(10, 50, self.width()-100, 25)
     self.serverMetadataButton.setGeometry(self.width()-95, 50, 40, 24)

     self.lblServerKeywords.setGeometry(10, 100, self.width()-165, 25)
     self.serverKeywords.setGeometry(10, 125, self.width()-100, 25)
     self.serverKeywordsButton.setGeometry(self.width()-95, 125, 25, 25)

     self.lblServerNavigator.setGeometry(10, 175, self.width()-165, 25)
     self.serverNavigator.setGeometry(10, 200, self.width()-100, 25)
     self.serverNavigatorButton.setGeometry(self.width()-95, 200, 25, 25)

     self.groupBoxFolders.setGeometry(10, 20, self.width()-40, 120)

     self.lblFolder.setGeometry(10, 25, self.width()-165, 25)
     self.folder.setGeometry(10, 50, self.width()-100, 25)
     self.folderButton.setGeometry(self.width()-80, 50, 25, 25)
     self.checkAutoComment.setGeometry(10, 80, self.width()-310, 25)

     self.groupBoxDuration.setGeometry(10, 150, self.width()-40, self.height()-260)
     
     self.checkShowWarning.setGeometry(10, 30, self.width()-165, 25)

     self.groupParserInfo.setGeometry(10, 20, self.width()-40, self.height()-400)
     
     self.checkAutoCorrect.setGeometry(10, 15, self.width()-165, 25)
     self.radioStream.setGeometry(30, 45, self.width()-165, 25)
     self.radioFile.setGeometry(30, 75, self.width()-165, 25)

     self.lblFolderXml.setGeometry(10, 110, self.width()-165, 25)
     self.folderXml.setGeometry(10, 135, self.width()-100, 25)
     self.buttonFolderXml.setGeometry(self.width()-80, 135, 25, 25)

     self.groupEditorInfo.setGeometry(10, self.height()-380, self.width()-40, self.height()-320)
     
     self.checkSilentMode.setGeometry(10, 20, self.width()-165, 25)
     self.checkReportingCswt.setGeometry(10, 50, self.width()-165, 25)
     
     self.groupCswtInfo.setGeometry(10, 20, self.width()-40, self.height()-60)

     self.lblListConnexionCswt.setGeometry(10, 30, 140, 20)

     self.listConnexionCswtButton.setGeometry(20, 55, 25, 25)
     self.connexionCswt.setGeometry(65, 55, self.width()-435, 25)
     self.defaultConnexionCswtButton.setGeometry(self.width()-365, 55, 25, 25)
     self.newConnexionCswtButton.setGeometry(self.width()-335, 55, 25, 25)
     self.editConnexionCswtButton.setGeometry(self.width()-310, 55, 25, 25)
     self.deleteConnexionCswtButton.setGeometry(self.width()-285, 55, 25, 25)
     
     self.groupConnexionInfo.setGeometry(10, 110, self.width()-60, self.height()-240)

     self.lblNameConnexionCswt.setGeometry(10, 15, 250, 20)
     self.nameConnexionCswt.setGeometry(10, 35, 300, 25)
     self.buttonAddConnexionCswt.setGeometry(315, 28, 48, 48)

     self.lblServerCswt.setGeometry(10, 70, 250, 20)
     self.serverCswt.setGeometry(10, 90, self.width()-150, 25)
     self.getCapabilitiesServerCswtButton.setGeometry(self.width()-130, 80, 25, 25)

     self.lblAuthentificationServerCswt.setGeometry(10, 145, 250, 20)
     self.authentificationServerCswt.setGeometry(10, 165, self.width()-150, 25)
     self.getAuthentificationServerCswtButton.setGeometry(self.width()-130, 155, 25, 25)

     self.lblUserServerCswt.setGeometry(10, 200, self.width()-165, 20)
     self.userCswt.setGeometry(10, 220, self.width()-610, 25)
     
     self.lblPwdUserCswt.setGeometry(10, 250, 250, 20)
     self.pwdUserCswt.setGeometry(10, 270, self.width()-610 , 25)
    
     self.checkViewPwdUserCswt.setGeometry(20, 300, 200, 20)

     self.groupErgonomicalInfo.setGeometry(10, 20, self.width()-40, self.height()-140)

     self.movieCtrl.setGeometry(int(self.width()/2)-64, int(self.height()/2)-64, 64, 64)
     self.barInfo.setGeometry(0, 0, self.width(), 90)

     i = 0
     listImg = [key for key in self.images.keys()]
     for mddcollapsibleGroupBox in self.mddcollapsibleGroupBox :
        img = listImg[i]
        objGroupBox = mddcollapsibleGroupBox.findChild(QGroupBox, img)
        if objGroupBox :
              label = objGroupBox.findChild(QLabel, "img%s" % img)
              if label : label.setGeometry( objGroupBox.width()-130, objGroupBox.height()-130, 128, 128)
        i+= 1
     

  def close(self): self.reject()

  def printFileOptions(self): 
      optionsFile = join(dirname(__file__),"ressources/options.ini")
      if exists(optionsFile):
          with open(optionsFile, 'r') as myfile : myfileText = myfile.readlines()
          lineText = ""
          for k in range(len(myfileText)): lineText+= "%s" % (markerText(self, myfileText[k]))
          editor = QTextBrowser()
          editor.setText(lineText)
          dlgPrint = printerSphere(self, editor, QApplication.translate("QSphere", "Print ini file"), QPrinter.Portrait)


