from io import StringIO
from xml.etree import ElementTree as ET
from qsphere.qsphere_tools import appendNameSpace
from urllib.parse import unquote

class xmlIsoParser:
        def __init__(self, filenameIP, dataXml, isoFormat, langue):				
                self.inputXmlfile = filenameIP
                self.dataXml = dataXml
                self.isoFormatToExtract = isoFormat
                self.localLangage = langue
                self.isoModel = None
                self.properties = {}

        def getTagDictionnary(self):
                if self.dataXml == None and self.inputXmlfile == None : return False
                else : self.root = self.getIsoXML(self.inputXmlfile, self.dataXml)

                if self.root is None : return False

                self.importIsoxmlDefinition(self.isoFormatToExtract)
                self.dictionnary = self.getElementValue(self.isoModel.dictionnary())

                return (self.dictionnary==[[]])


        def createQMDdataStructure(self, isMetaData):
                if self.isoModel == None :
                        self.importIsoxmlDefinition('QMD')
                        self.root = self.getIsoXML(self.inputXmlfile, self.dataXml)

                dicoTags = {'title' : self.isoModel.title, 'abstract' : self.isoModel.abstract,
                            'identifier': self.isoModel.identifier, 'parentidentifier': self.isoModel.parentidentifier,
                            'language' : self.isoModel.language, 'type' : self.isoModel.type, 'localisators' : self.isoModel.links,
                            'tableemprises' : self.isoModel.tableemprises, "tablescr" : self.isoModel.tablescr,
                            'tableetenduetemporelle' : self.isoModel.tableetenduetemporelle} 

                for key in dicoTags :
                        try : setattr(self, key, self.getBranchValue(dicoTags[key]()))
                        except : setattr(self, key, [[]])

                self.pointsofcontact = []         
                try : self.pointsofcontact = self.getPointsOfContactMDD(self.pointsofcontact, True)
                except : self.pointsofcontact = [[]]

                self.codecategories, self.keywordsF = self.getQmdCategories()


        def getQmdCategories(self):
                try : rootEl = self.root.findall("keywords")
                except : return [[]], [{}]
                
                codecategories, keywordsF = [], []
                
                for elVal in rootEl:
                    categorie = False    
                    if elVal.attrib["vocabulary"] != "" :
                       if elVal.attrib["vocabulary"].find("gmd:topicCategory")>-1 : categorie = True
                            
                    for childItem in elVal :
                         if categorie :   
                            if not childItem.text in codecategories : codecategories.append(childItem.text)
                         else :
                            keywordsF.append({'keyword': '%s' % childItem.text, 'thesaurus': '', 'date': '', 'typedate': ''})
                
                return [codecategories], keywordsF
        
        def getBranchValue(self, keyMethod):
                returnValList, returnVal, dataStruct, counter =[], 'None', {}, 0
                valueList, ordering, data = [], False, ""
                
                for i in keyMethod:
                        if type(i) is dict:
                           for j in i.keys():
                               dataStruct[counter]=j
                               if j == 'order': ordering = True
                        if type(i) is str:
                                dataStruct[counter] = i
                                if i == 'order': ordering = True
                        counter = counter + 1
                        
                listKeys = self.extractListKeys(dataStruct)

                for key in listKeys:
                    thisData = keyMethod[key][dataStruct[key]]
                   
                    if 'xpath' in thisData.keys():
                        data = thisData['xpath']    
                        returnVal =  self.simpleEltValue(data, False)
                        valueList.append(returnVal)

                    if 'basilexpathvalue' in thisData.keys():
                        data = thisData['basilexpathvalue']    
                        valueList =  self.baliseElts(data)

                    if 'linkedpath' in thisData.keys() and key == 1 :
                        returnVal = self.simpleMultiEltValue(keyMethod, dataStruct, True)
                        return returnVal
                        

                    if ordering : orderingList = thisData
                    else: pass
                    
                if ordering:
                        checkCompLnth = len(valueList[0])
                        index=0

                        for list in valueList:
                                if len(list) != checkCompLnth:
                                        if len(list)> checkCompLnth:
                                           elts = ""
                                           for elt in list : elts+= " %s" % (elt)
                                           list=[]
                                           list.append(elts)
                                           valueList[index] = list
                                        else :
                                           while len(list)< checkCompLnth:
                                                 list.append('')          
                                           valueList[index]=list      

                                if data.startswith('gmd:contact/gmd:CI_ResponsibleParty/gmd:contactInfo') or data.startswith('gmd:identificationInfo/gmd:MD_DataIdentification/gmd:pointOfContact'):
                                   pos=0
                                   for item in list :
                                       if item==None :
                                          if index==0 : valueList[index][pos] = "pointOfContact"
                                          elif index==3 and self.localLangage == "fr" : valueList[index][pos] = "France"
                                          else : valueList[index][pos] = ""
                                       pos+= 1   
                                index+=1

                        
                        outer = []

                              
                        for localPos in range(0,checkCompLnth):
                                inner=[]
                                for listPos in range(0,len(valueList)):
                                    try :
                                         inner.append(valueList[listPos][localPos]) if valueList[listPos][localPos]!=None else inner.append(" ")
                                    except :
                                         inner.append(" ") 
                                outer.append(inner)
                
                        for returnedList in outer:
                                orderedValsSub = {}
                                if len(returnedList) != len(orderingList): returnValList.append('None')
                                else:		
                                    for key in orderingList.keys(): orderedValsSub[key] = returnedList[orderingList[key]-1] 
                                    returnValList.append(orderedValsSub)
                        return returnValList
                        
                else:
                        return valueList

                
        def baliseElts(self, xPath):
                explode = xPath.split("|")
                rootPath = explode[0]

                matchingTag = {"miny" : "north", "maxy": "south", "minx": "east" , "maxx": "west"}
               
                valueList = []
                try: rootEl = self.root.findall(rootPath)
                except: return ['None']
                for elVal in rootEl:
                    resElementVal = {}    
                    if elVal is None: resElementVal.append('None')
                    else:
                       if elVal.attrib == {}:
                          for childItem in elVal :
                              if childItem.tag.find("Boolean")!=-1 : resElementVal["Boolean"] = unquote(childItem.text)
                       else :
                          i = 0     
                          for tag in explode :
                              if i>0 :
                                      targetTag = tag
                                      if (tag in matchingTag) : targetTag = matchingTag[tag]  
                                      resElementVal[targetTag] = unquote(elVal.attrib[tag])
                              i+= 1
                    valueList.append(resElementVal)
                return valueList

                
        def createISOdataStructure(self, isMetaData):
                if self.isoModel == None : self.importIsoxmlDefinition('MEDDE')
                if isMetaData :

                        dicoTags = {'UUID' : self.isoModel.identificator, 'title' : self.isoModel.intitule, 'abstract' : self.isoModel.resume,
                                    'typedata' : self.isoModel.typedata, 'localisators' : self.isoModel.tablelocalisator,
                                    'tablecaracmdd' : self.isoModel.tablecaracmdd, 'tablecaracjdd' : self.isoModel.tablecaracjdd,
                                    'rs_identifier' : self.isoModel.rs_identifier, 'languesjdd' : self.isoModel.tablelangues,
                                    'categories' : self.isoModel.tablecategories, 'codecategories' : self.isoModel.ax_tablecategories,
                                    'keywordsF' : self.isoModel.tablemotsclefsf, 'keywordsFNC' : self.isoModel.ax_tablemotsclefsf,
                                    'tableetenduetemporelle' : self.isoModel.tableetenduetemporelle, 'dates' : self.isoModel.dates,
                                    'formatsjdd' : self.isoModel.tableformats, 'boundingboxcoordinates' : self.isoModel.tableemprises,
                                    'scr' : self.isoModel.tablescr, 'genealogie' : self.isoModel.genealogie, 'coherence' : self.isoModel.coherence,
                                    'scalesEC' : self.isoModel.grouperesolutionscale, 'scalesUM' : self.isoModel.ax_grouperesolutionscale, 'UnitsScalesUM' : self.isoModel.UnitsScalesUM,
                                    'conformities' : self.isoModel.tablespecifications, 'legalconstraints' : self.isoModel.licence,
                                    'accessconstraints' : self.isoModel.groupedroits, 'otherconstraints' : self.isoModel.otherconstraints,
                                    'authors' : self.isoModel.authors, 'languemdd' : self.isoModel.langmetada, 'datetmdd' : self.isoModel.ax_datemetada,
                                    'datemdd' : self.isoModel.datemetada
                                    }


                        for key in dicoTags :
                                try :  setattr(self, key, self.getElementValue(dicoTags[key]())) 
                                except : setattr(self, key, [[]])
                       
                        
                        self.pointsofcontact = []         
                        try : self.pointsofcontact = self.getPointsOfContactMDD(self.pointsofcontact, False)
                        except : self.pointsofcontact = [[]]
                        
                else :

                        self.pointsofcontact = [] 
                        try : self.pointsofcontact = self.getPointsOfContactMDD(self.pointsofcontact, False)
                        except : self.pointsofcontact = [[]]


                return True

        def getPointsOfContactMDD(self, contacts, isQmd):
            roleKeys = self.isoModel.tableroles()[1]

            if not isQmd :    
                    xmlKeys = ('gmd:identificationInfo/gmd:MD_DataIdentification/gmd:pointOfContact', 
                                    'gmd:identificationInfo/gmd:MD_DataIdentification/gmd:citation/gmd:CI_Citation/gmd:citedResponsibleParty', 
                                    'gmd:identificationInfo/gmd:MD_DataIdentification/gmd:resourceSpecificUsage/gmd:MD_Usage/gmd:userContactInfo', 
                                    'gmd:distributionInfo/gmd:MD_Distribution/gmd:distributor/gmd:MD_Distributor/gmd:distributorContact', 
                                    'gmd:contact/gmd:CI_ResponsibleParty', 
                                    'gmd:contact'                
                                    )
            else : xmlKeys = ('contact','')

                    
            for xmlKey in xmlKeys :
                   xmlKeyNs = appendNameSpace(self, xmlKey) if not isQmd else xmlKey
                   for contactType in self.root.findall(xmlKeyNs):
                        contact = []
                        if contactType != None :
                           for key in roleKeys :
                               value = contactType.findall(appendNameSpace(self, roleKeys[key])) if not isQmd else contactType.findall(roleKeys[key])
                               try :
                                       if len(value) == 1 : valueContact = unquote(value[0].text); contact.append("%s" % valueContact);
                                       else : contact.append("")
                               except : contact.append("")         
                           if contact!=['', '', '', '', '', '', '', '', '']: contacts.append(contact)
            return contacts 


        def boundingDateRange(self, boundingDatesList):
                allDates, returnDates = [], {}
                for outer in boundingDatesList:
                        for key in outer.keys():
                                if outer[key] is not None:
                                        if outer[key] != 'None': allDates.append(outer[key])
                if allDates != []:
                    returnDates['start'] = min(allDates)
                    returnDates['end'] = max(allDates)
                return returnDates


        def importIsoxmlDefinition(self, format):
                if format == 'MEDDE' :
                   from qsphere.xmlisoreadertag import xmlIsoReaderTag as isoModel
                elif format == 'QMD':   
                   from qsphere.xmlisoreadertag import qmdIsoReaderTag as isoModel 
                self.isoModel = isoModel()


        def extractListKeys(self, dataStruct):
                listKeys = []
                refvalue = None
                for item in dataStruct.items():
                    if not refvalue : refvalue = item[1]
                    if item[1] != refvalue :
                       listKeys.append(item[0])
                return listKeys       
			
        def getElementValue(self, keyMethod):
                returnValList, returnVal, dataStruct, counter =[], 'None', {}, 0
                valueList, ordering, data = [], False, ""
                
                for i in keyMethod:
                        if type(i) is dict:
                           for j in i.keys():
                               dataStruct[counter] = j
                               if j == 'order': ordering = True
                        if type(i) is str:
                                dataStruct[counter] = i
                                if i == 'order': ordering = True
                        counter = counter + 1
                        
                listKeys = self.extractListKeys(dataStruct)
                
                for key in listKeys:
                        thisData = keyMethod[key][dataStruct[key]]
                      
                        if 'basilexpath' in thisData.keys():
                                returnVal =  self.baliseElt(thisData['basilexpath'])
                                valueList.append(returnVal)                                

                        if 'basilexpathvalue' in thisData.keys():
                                returnVal =  self.baliseEltValue(thisData['basilexpathvalue'])
                                valueList.append(returnVal)
                        
                        if 'baseXpath' in thisData.keys():
                                returnVal = self.dependantEltValue(thisData['baseXpath'],thisData['elValXpath'],thisData['depValXpath'],thisData['depVal'])
                                valueList.append(returnVal)
                                                                
                        if 'xpath' in thisData.keys():
                                returnVal =  self.simpleEltValue(thisData['xpath'], True)
                                data = thisData['xpath']
                                if returnVal == [] and data == "gmd:contact/gmd:CI_ResponsibleParty/gmd:role/gmd:CI_RoleCode" :
                                        if 'baseXpath' in thisData.keys():
                                                returnVal = self.dependantEltValue(thisData['baseXpath'],thisData['elValXpath'],thisData['depValXpath'],thisData['depVal'])
                                                valueList.append(returnVal)
                                else : valueList.append(returnVal)

                        if 'linkedpath' in thisData.keys() and key == 1 :
                                returnVal = self.simpleMultiEltValue(keyMethod, dataStruct, True)
                                return returnVal
                                
                        if ordering : orderingList = thisData
                        else: pass

                if [] in valueList : return valueList

                if ordering:
                        checkCompLnth = len(valueList[0])
                        index=0

                        for list in valueList:
                                if len(list) != checkCompLnth:
                                        if len(list)> checkCompLnth:
                                           elts = ""
                                           for elt in list : elts+= " %s" % (elt)
                                           list=[]
                                           list.append(elts)
                                           valueList[index] = list
                                        else :
                                           while len(list)< checkCompLnth:
                                                 list.append('')          
                                           valueList[index]=list      

                                if data.startswith('gmd:contact/gmd:CI_ResponsibleParty/gmd:contactInfo') or data.startswith('gmd:identificationInfo/gmd:MD_DataIdentification/gmd:pointOfContact'):
                                   pos=0
                                   for item in list :
                                       if item==None :
                                          if index==0 : valueList[index][pos] = "pointOfContact"
                                          elif index==3 and self.localLangage == "fr" : valueList[index][pos] = "France"
                                          else : valueList[index][pos] = ""
                                       pos+= 1   
                                index+=1

                        
                        outer = []

                              
                        for localPos in range(0,checkCompLnth):
                                inner=[]
                                for listPos in range(0,len(valueList)):
                                    try :
                                         inner.append(valueList[listPos][localPos]) if valueList[listPos][localPos]!=None else inner.append(" ")
                                    except :
                                         inner.append(" ") 
                                outer.append(inner)
                
                        for returnedList in outer:
                                orderedValsSub = {}
                                if len(returnedList) != len(orderingList): returnValList.append('None')
                                else:		
                                    for key in orderingList.keys(): orderedValsSub[key] = returnedList[orderingList[key]-1] 
                                    returnValList.append(orderedValsSub)
                        return returnValList
                        
                else:
                        return valueList
			
			

        def dependantEltValue(self,baseXpath,elXpath,depXpath,depValReqd):
                baseXpath = appendNameSpace(self, baseXpath)
                resDependantVal = []
                try: rootEl = self.root.findall(baseXpath)
                except: return 'None'

                for el in rootEl:
                        thisElXpth = appendNameSpace(self, elXpath)
                        thisEl = self.doFindall(el,thisElXpth)
                        if thisEl != 'None':
                                elVal = thisEl[0].text 
                                thisEldepXpth = appendNameSpace(self, depXpath)					
                                thisDepEl = self.doFindall(el,thisEldepXpth)								
                                if thisDepEl != 'None':
                                        depVal = thisDepEl[0].text
                                        resDependantVal.append(unquote(elVal))
                return resDependantVal

        def baliseElt(self, xPath):
                xPathNs = appendNameSpace(self, xPath)
                resElementVal = []
                try: rootEl = self.root.findall(xPathNs)
                except: return ['None']
                for elVal in rootEl:
                    if elVal is None: resElementVal.append('None')
                    else:
                       if elVal.attrib == {}:
                          for childItem in elVal :
                              if childItem.tag.find("Boolean")!=-1 : resElementVal.append(unquote(childItem.text))
                       else : resElementVal.append(elVal.attrib)
                return resElementVal        

        def baliseEltValue(self, xPath):
                xPathNs = appendNameSpace(self, xPath)
                resElementVal = []
                try: rootEl = self.root.findall(xPathNs)
                except: return ['None']
                for elVal in rootEl:
                    if elVal is None: resElementVal.append('None')
                    else:
                       if elVal.attrib == {}:
                          for childItem in elVal :
                              if childItem.tag.find("Boolean")!=-1 : resElementVal.append(unquote(childItem.text))
                       else : resElementVal.append(elVal.attrib['codeListValue'])
                return resElementVal
                       
                
        def simpleEltValue(self, xPath, withNameSpace):
                xPathNs = appendNameSpace(self, xPath) if withNameSpace else xPath
                resElementVal = []
                try: rootEl = self.root.findall(xPathNs)
                except: return ['None']
                for elVal in rootEl:
                        if not elVal is None : resElementVal.append(unquote("%s" % elVal.text))
                return resElementVal

        def simpleMultiEltValue(self, keyMethod, dataStruct, withNameSpace):
               
                listKeys = self.extractListKeys(dataStruct)
                dicoKeyWords = []
                
                xPath = "gmd:identificationInfo/gmd:MD_DataIdentification/gmd:descriptiveKeywords/gmd:MD_Keywords"
                xPathNs = appendNameSpace(self, xPath) if withNameSpace else xPath
                try: rootEl = self.root.findall(xPathNs)
                except: pass

                counter = len(rootEl)

                for i in range(0, counter, 1):
                        rootEl = self.root.findall(xPathNs)
                        index = 0
                        for elVal in rootEl :
                            if index == i :    
                                    if elVal :
                                            dictKeyword = self.extractKeyword(elVal, keyMethod, dataStruct, listKeys, True)
                                            dicoKeyWords.append(dictKeyword)
                                            break
                            index+= 1    
                return dicoKeyWords

        def extractKeyword(self, element, method, data, keys, withNameSpace):
                
                keywordInfos = {'keyword': None,'thesaurus': None,'date': None,'typedate': None}
              
                for i in range(1, len(keys), 1):
                    subPath = method[i][data[i]]['linkedpath']
                    subPathNs = appendNameSpace(self, subPath) if withNameSpace else subPath
                                              
                    subEl = element.findall(subPathNs)
                    for elVal in subEl:
                        if not elVal is None : 
                           if   i == 1 : keywordInfos['keyword']= unquote("%s" % elVal.text)
                           elif i == 2 : keywordInfos['thesaurus']= unquote("%s" % elVal.text)
                           elif i == 3 : keywordInfos['date']= unquote("%s" % elVal.text)
                           elif i == 4 : keywordInfos['typedate']= unquote("%s" % elVal.text)                    
                          
                return keywordInfos

        def doFindall(self, el, thisElXpth):
                try:
                        thisElXpthEl = el.findall(thisElXpth)
                        if len(thisElXpthEl) == 0: thisElXpthEl = 'None'
                except: thisElXpthEl = 'None'
                return thisElXpthEl

        def getXmlVal(self, paths):
                xmlVals = []
                for path in paths:
                        try: xmlVals.append(self.root.find(path).text)			
                        except:	xmlVals.append('null')
                return xmlVals					
                
        def defaultIsoNamespace(self):  return 'gmd'

        def getIsoXML(self, inputFile, dataXml):
                etree, root = None, None

                if inputFile :
                        try : etree = ET.parse(inputFile)
                        except : return None
                else :
                        try : etree = ET.parse(StringIO(dataXml))
                        except :
                                try : etree = ET.parse(dataXml)
                                except : return None
                if etree :
                   root = etree.getroot()
                   try :
                           if not root.tag in ('qgis', '{http://www.isotc211.org/2005/gmd}MD_Metadata') : return None
                   except :
                           pass

                return root
